/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");

dojo.require("jazz.app.context");
dojo.require("dojo/json");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var getCurrentPA = jazz.app.context.get;
var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;

com.ibm.asq.common.web.client.internal.FiltersQueriesClient = {

    privateScopeId: "USER",
    publicScopeId: "PROJECT_AREA",
    autoSaveScopeId: "AUTO_SAVE",
    MODELROOT: "SQ-MODEL-ROOT",
    
    // query keys for supported artifact types
    //
    REQUIREMENT_QUERY_KEY: "REQUIREMENT_QUERIES",
    TEST_PLAN_QUERY_KEY: "TEST_PLAN_QUERIES",
    TEST_CASE_QUERY_KEY: "TEST_CASE_QUERIES",
    TEST_SCRIPT_QUERY_KEY: "TEST_SCRIPT_QUERIES",
    TEST_SUITE_QUERY_KEY: "TEST_SUITE_QUERIES",
    TEST_DATA_QUERY_KEY: "TEST_DATA_QUERIES",
    KEYWORDS_QUERY_KEY: "KEYWORDS_QUERIES",
    BUILD_REC_QUERY_KEY: "BUILD_REC_QUERIES",
    BUILD_DEF_QUERY_KEY: "BUILD_DEF_QUERIES",
    TER_QUERY_KEY: "TEST_EXE_RECORD_QUERIES",
    SER_QUERY_KEY: "SUITE_EXE_RECORD_QUERIES",
    TER_TC_RESULT_QUERY_KEY: "TER_TC_RESULT_QUERIES",
    TER_TS_RESULT_QUERY_KEY: "TER_TS_RESULT_QUERIES",
    EXE_SCHEDULE_QUERY_KEY: "EXE_SCHEDULE_QUERIES",
    EXE_CONSOLE_QUERY_KEY: "EXE_CONSOLE_QUERIES",
    ADAPTER_CONSOLE_QUERY_KEY: "ADAPTER_CONSOLE_QUERIES",
	JOB_SCHEDULER_QUERY_KEY: "JOB_SCHEDULER_QUERIES",
    PRINT_JOB_QUERY_KEY: "PRINT_JOB_QUERIES",
    
    serviceInterface: "com.ibm.rqm.planning.common.service.rest.IQueryFilterRestService",
        
    _invokeService : function (responseHandler, method, args) {
        args = args ? args : {};
        if (!args.projectAreaItemId) {
            args.projectAreaItemId = getCurrentPA().itemId;
        }
        var serviceRequest = new ServiceRequest(this.serviceInterface, method,  args);
        TeamServerClient.invokeService(serviceRequest, responseHandler);
        return serviceRequest;
    },
    
    getQueries: function(responseHandler, artifactType, projectAreaItemId){
        var attributes = {
            "artifactType": artifactType,
            "projectAreaItemId": projectAreaItemId
        };
        return this._invokeService(responseHandler, "getQueryFilters", attributes);
    },

	getQueries2: function(responseHandler, params){
		//Send itemIds of queries here to get back queryObject
        return this._invokeService(responseHandler, "getQueryFilters", params);
    },
    getMenuQueries: function(responseHandler, artifactType){
        var attributes = artifactType ? { "artifactType": artifactType } : {};
        return this._invokeService(responseHandler, "getMenuQueryFilters", attributes);
    },

    saveQueries: function(responseHandler, queries){
        var attributes = {
            "uiFilterQueries": dojo.toJson(queries)
        };
        return this._invokeService(responseHandler, "postCreateFilter", attributes);
    },

    updateQueries: function(responseHandler, queries){
        var attributes = {
            "uiFilterQueries": dojo.toJson(queries)
        };
        return this._invokeService(responseHandler, "postUpdateFilter", attributes);
    },

    deleteQueries: function(responseHandler, queries){
        var attributes = {
            "uiFilterQueries": dojo.toJson(queries)
        };
        return this._invokeService(responseHandler, "postRemoveFilter", attributes);
    },

    autoSaveQueries: function(responseHandler, queries){
        var attributes = {
            "uiFilterQueries": dojo.toJson(queries)
        };
        return this._invokeService(responseHandler, "postAutoSaveFilter", attributes);
    },
    
    getAutoSavedQueries: function(callback, artifactType){
        var self = this;
        
        var responseHandler = new ServiceResponseHandler({
            success: function(response){
                if (dojo.isArray(response) && response.length > 0) {
                    dojo.forEach(response, function(query) {
                        query.filters = self._getFilters(query.values);
	                    callback(query);
                    });
                } else {
                    self.inMenuQueries = {};
                    callback(null);
                }
            },
            error: function(e){
                    console.error(e);
                    callback(null);
                }
            },
	        "success",
    	    "error");

        var attributes = {
            "artifactType": artifactType
        };
        return this._invokeService(responseHandler, "getAutoSavedFilters", attributes);
    },
    
    // parse the values of the filters in the response
    //
    _getFilters: function(filtersArray) {
        var filters = {};
        dojo.forEach(filtersArray, function(filter) {
            // the values is wrapped in an array
            var field = filter.field;
            var value = filter.values[0];
            
            if (dojo.isObject(value) && value.string) {
            	var len = filter.values.length;
            	for(var x=1; x < len; x++)
                    //string filters larger than 250 chars, are split in multiple values, we need here to append the strings before processing json data.
    	            value.string += filter.values[x].string;

                try {
                	//if value.string is a plain text, it could be evaluated to a reference if any exists with the same name than the text. That's a undesired behavior here.
                	//That problem is constant when we are grouping by any category. So let's check for group key and fix it closing it in an array.
                    var stringValue = value ? value.string : "";
                    if (field=='QueryGroupBy' && stringValue && stringValue[0]!='[' && stringValue[0]!='{') {
                        // padded the JSON string as array in separate variable not the actual value
                        stringValue = "[\"" + stringValue + "\"]";
                    }
                    value = JSON.parse(stringValue, true) || value.string;      	
                } catch (e) {
                    value = value.string;
                }
            }
            filters[field] = value;
        });
        return filters;
    },
    
    // cache the inMenu queries for performance
    inMenuQueries: {},
    
    // Consumer should invoke this on startup without queryKey to get all inMenu queries
    // and use getInMenuQueries() to get the artifact type inMenu queries from the cache.
    // Specify queryKey to invalidate/refresh the cache of the specified artifact type.
    //
    getAllMenuQueries: function(callback, queryKey){
        var self = this;
        var handler = new ServiceResponseHandler({
            success: function(response){
                if (dojo.isArray(response) && response.length > 0) {
                    if (queryKey) {
                        // clear the cache for the query key of the artifact type
                        self.inMenuQueries[queryKey] = [];
                    }
                    dojo.forEach(response, function(query) {
                        var key = query.type;
                        var filters = self._getFilters(query.values);
                        var menuQuery = {
                            "type": query.type,
                            "itemId": query.itemId,
                            "stateId": query.stateId,
                            "title": query.name,
                            "inMenu": query.menu,
                            "folder": query.folder,
                            "scope": query.scope,
                            "filters": filters
                        };
                        // group them by the query key, artifact type
                        if (self.inMenuQueries[key]) {
                            self.inMenuQueries[key].push(menuQuery);
                        } else {
                            self.inMenuQueries[key] = [menuQuery];
                        }
                    });
                    // sort the queries for each type
                    var sorter = function(q1, q2){
                        var name1 = q1.title ? q1.title.toLowerCase() : q1.title;
                        var name2 = q2.title ? q2.title.toLowerCase() : q2.title;
                        return (name1 > name2) ? 1 : (name1 == name2) ? 0 : -1;
                    };
                    for (var typeKey in self.inMenuQueries) {
                        self.inMenuQueries[typeKey].sort(sorter);
                    }
                    callback(self.inMenuQueries);
                } else {
                    // reset the in menu queries
                    self.inMenuQueries = {};
                    callback({});
                }
            },
            error: function(e){
                    console.error(e);
                    callback({});
                }
            },
            "success",
            "error");

        this.getMenuQueries(handler, queryKey);
    },
    
    // Return the inMenu queries of the specified queryKey of artifact type from the cache
    //
	getInMenuQueries: function(queryKey){
        var queriesInMenu = this.inMenuQueries[queryKey];
        if (queriesInMenu) {
            return queriesInMenu;
        } else {
            return [];
        }
    },
	
	getQueryTypeIconURL: function(queryKey){
        switch (queryKey) {
            case this.REQUIREMENT_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-req.gif");
            case this.TEST_PLAN_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-plan.gif");
            case this.TEST_CASE_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-case.gif");
            case this.TEST_SCRIPT_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-script.gif");
            case this.TEST_SUITE_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-suite.gif");
            case this.KEYWORDS_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-keyword.gif");
            case this.BUILD_REC_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-build-rec.gif");
            case this.BUILD_DEF_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-build-def.gif");
            case this.TER_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-ter.gif");
            case this.SER_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-ser.gif");
            case this.TER_TC_RESULT_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-tc-result.gif");
            case this.TER_TS_RESULT_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-ts-result.gif");
            case this.EXE_SCHEDULE_QUERY_KEY:
                 return ImageCache._getIconUri("obj16/query-exec-sched.gif");
            case this.TEST_DATA_QUERY_KEY:
            case this.EXE_CONSOLE_QUERY_KEY:
            case this.ADAPTER_CONSOLE_QUERY_KEY:
			case this.JOB_SCHEDULER_QUERY_KEY:
            case this.PRINT_JOB_QUERY_KEY:
		    default:
                 return ImageCache._getIconUri("obj16/query.gif");
        }
    }
};

})();
