/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("dojo.date");
dojo.require("dijit._Widget");
dojo.require("jazz.app.i18n");
dojo.require("jazz.ui.ImagePreview");
dojo.require("jazz.ui.ErrorPage");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.core.loader");
dojo.require("jazz.util.html");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");              
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("jazz.app.context");
dojo.require("jazz.app.viewHistory");
dojo.require("jazz.app.internal.LoginDialog");
dojo.require("jazz.util.internal.login");
dojo.require("net.jazz.ajax.BootstrapProperties");

(function() 	
{
// added any new supported languages to this list
//
var SUPPORTED_LANG = [
    "en",
    "de",
    "es",
    "fr",
    "it",
    "ja",
    "ko",
    "pt",
    "zh",
    "cs",
    "hu",
    "pl",
    "ru"
];

// add any new supported languages of specific countries to this list
var SUPPORTED_COUNTRIES = [
    "en-us",
    "pt-br",
    "zh-hk",
    "zh-tw",
    "en-zz"
];

	var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	var Constants  = com.ibm.asq.common.web.client.internal.Constants;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var ACTION_PREFIX = "#action=com.ibm.rqm.planning.home.actionDispatcher";
    var _Widget = dijit._Widget.prototype;
	var CommonPreferenceClient=com.ibm.asq.common.web.client.internal.CommonPreferenceClient;
    var getCurrentProjectArea = jazz.app.context.get;
    var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
    var viewHistory = jazz.app.viewHistory;
    var ImagePreview = jazz.ui.ImagePreview;
	var _dojo = dojo;
	
    // Unassigned is a special user like ADMIN and GUEST.  The item id is fixed according to the file
    // com.ibm.team.workitem.common.internal.model.DefaultModel.java from where QMContributorRestService
    // gets the Unassigned user to send to the UI:
    //
    // public final static String NULL_CONTRIBUTOR_NAME= Messages.getString("DefaultModel.UNASSIGNED_CONTRIBUTOR_NAME"); //$NON-NLS-1$
    // public final static UUID NULL_CONTRIBUTOR_ITEM_ID= UUID.valueOf("_YNh4MOlsEdq4xpiOKg5hvA"); //$NON-NLS-1$
    var UNASSIGNED_ITEMID = "_YNh4MOlsEdq4xpiOKg5hvA";
	
    // viewHistory entries can have same uri from renamed label or different type (identical entry in Construction and Execution)
    // work around the defect 67550 from the Foundation to add a delay to avoid db insert error with back-to-back entries with same uri
    var lastHistoryUri = null;
    var lastHistoryDelayCount = 0;
    
	com.ibm.asq.common.web.ui.internal.util.Utility = 
	{
        PLANNING_PERSPECTIVE: 1,
        CONSTRUCTION_PERSPECTIVE: 2,
        EXECUTION_PERSPECTIVE: 3,

        _planningPrefix: ACTION_PREFIX + "&subAction=",
        _constructionPrefix: ACTION_PREFIX + "&subAction=",
        _executionPrefix: ACTION_PREFIX + "&subAction=",
         
        viewDashboardSubaction: "viewDashboard",
        viewAllDashboardsSubaction: "viewAllDashboards",
        
		/******************************************************************************************
		 * Admin URI functions
		 * 
		*****************************************************************************************/        
		getProjectTimeLinesActionUri: function(){		
			return this._getBaseProjectAdminUri()+ "tabId=com.ibm.team.process.web.Timelines";
		},
		
		_getBaseProjectAdminUri: function(){
			var adminUri = dojo.baseUrl + "../../admin#";
			return adminUri + "action=com.ibm.team.process.editProjectArea&itemId=" + getCurrentProjectArea().itemId + "&";
		},
		
		/******************************************************************************************
		 * Generic URI functions
		 * 
		 *****************************************************************************************/
		_getPlanningActionUri: function(action, kwArgs)
		{
			var str = this._planningPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getConstructionActionUri: function(action, kwArgs)
		{
			var str = this._constructionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getExecutionActionUri: function(action, kwArgs)
		{
			var str = this._executionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getActionUri: function(str, kwArgs) 
		{
		    for (var key in kwArgs) 
			{
	        	str += "&" + key + "=" + kwArgs[key];
	        }
			return str;
		},

		_buildUri: function(method, args, perspective)
		{
			var result;
			
			switch(perspective)
			{
				case this.PLANNING_PERSPECTIVE:
					result = this._getPlanningActionUri(method, args);
					break;
				case this.CONSTRUCTION_PERSPECTIVE:
					result = this._getConstructionActionUri(method, args);
					break;
				case this.EXECUTION_PERSPECTIVE:
					result = this._getExecutionActionUri(method, args);
					break;
			}
			
			return(result);
		},
		
        // Compose the uri of the subAction and additional params
        //
        getDispatcherSubActionUri: function(subAction, kwArgs) {
            var uri = ACTION_PREFIX + "&subAction=" + subAction;
            return this._getActionUri(uri, kwArgs);
        },
        
        // Get the uri of an artifact list view.
        //
        // The artifact list view default uri uses one "updateAction" param of "clear-filter-selection"
        // while saved queries of the artifact list view will have different "updateAction"
        //
        getArtifactListViewUri: function(subAction) {
            return this.getDispatcherSubActionUri(subAction, {'updateAction': "clear-filter-selection"});
        },
        
        getViewAllDashboardsUri: function() {
            var uri = ACTION_PREFIX + "&subAction=" + this.viewAllDashboardsSubaction;
            return uri;
        },
        
		/******************************************************************************************
		 * Action URI methods
		 * 
		 * @param {Object} id
		 *****************************************************************************************/
		getGenericSearchUri: function(str)
		{
			return(this._getPlanningActionUri("genericSearch", {searchStr: str}));
		},

		getNewQueryUri: function()
		{
			return(this._getPlanningActionUri("newQuery"));
		},

		getViewWorkItemURI: function(id)
		{
			return this._getPlanningActionUri("viewWorkItem", {id: id});
		},
		     // DATAPOOL URLs
        getViewDatapoolURI: function(itemId)
        {
            return this._getPlanningActionUri("viewDatapool", {itemId: itemId});
        },	
        
        getViewKeywordURI: function(itemId)
        {
            return this._getActionUri(this._planningPrefix + "viewKeyword", {itemId: itemId});
        },
		
		getNewTestCaseURI: function(name) 
		{
			return this._getPlanningActionUri("newTestCase", {name: name});
		},
			
		getTestCaseURI: function(testId)
		{
			var args = {};
			if (testId != null)
			{
				args = {id: testId};
			}
			return this._getPlanningActionUri("viewTestCase", args);
		},
				
		getTestCasesURI: function(queryName)
		{
			var args = {};
			if (queryName != null)
			{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestCases", args);
		},
		
		getTestSuiteURI: function(suiteId, perspective){
			var args = {};
			args.id = suiteId;
			return this._buildUri("viewTestSuite", args, perspective ? perspective : this.CONSTRUCTION_PERSPECTIVE);
		},
		
		getNewTestScriptURI: function(name, perspective) 
		{
			if (perspective == null)
			{
				perspective = this.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("newTestScript", {name: name}, perspective));
		},
			
		getTestScriptURI: function(scriptId, perspective)
		{
			var args = {};
		
			if (scriptId != null)
			{
				args = {id: scriptId};
			}
		
			if (perspective == null)
			{
				perspective = this.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("viewTestScript", args, perspective));
		},
				
		getTestScriptsURI: function(queryName, perspective) 
		{
			var args = {};

			if (queryName != null) 
			{
				args = {name: queryName};
			}

			if (perspective == null)
			{
				perspective = this.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("viewTestScripts", args, perspective));
		},
		
		getNewTestPlanURI: function(name) 
		{
			return this._getPlanningActionUri("newTestPlan", {name: name});
		},
			
		getTestPlanURI: function(kwargs) 
		{
			var args = {};
			if ( kwargs.itemId != null )
			{
				args = {itemId: kwargs.itemId};
			}
			else if (kwargs.id != null)
			{
				args = {id: kwargs.id};
			}
			else if (kwargs.name != null)
			{
				args = {name: kwargs.name};
			}
			return this._getPlanningActionUri("viewTestPlan", args);
		},
		
        // get the args from the uri as the args in actionDispatcher()
        getArgsFromUri: function(uri) {
            var params = uri ? uri.split('&') : [];
            var args = {};
            dojo.forEach(params, function(param){
                var parts = param.split('=');
                if (parts[0] == '#action') {
                    args['action'] = parts[1];
                } else {
                    args[parts[0]] = parts[1];
                }
            });
            return args;
        },
        
        // given the array of param keywords, remove the params (key and value) from the specified uri
        //
        removeUriParams: function(uri, paramKeys) {
            var newUri = uri;
            if (paramKeys) {
                dojo.forEach(paramKeys, function(key) {
                    // create a regular exporession for & separated params in uri
                    var exp = new RegExp("&" + key + "=[^&]+");
                    newUri = newUri.replace(exp, "");
                });
            }
            return newUri;
        },
        
		getPDFDownloadLink: function(itemId){
			return net.jazz.ajax._contextRoot + "/secure/service/" + "com.ibm.rqm.print.common.service.IPrintService" + "/download?uuid="+itemId;			
		},
		getTestPlansURI: function(queryName) 
		{
			var args = {};
			if (queryName != null) 
			{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestPlans", args);
		},
		
		getNewEwiURI: function(name, testCaseId) 
		{
			var testId = testCaseId != null ? testCaseId : null; 
			return this._getPlanningActionUri("newEwi", {name: name, testCaseId: testId});
		},
			
		getEwiURI: function(planName, perspective) 
		{
			var args = {};
			
			if (planName != null) 
			{
				args = {name: planName};
			}

			if (perspective == null)
			{
				perspective = this.PLANNING_PERSPECTIVE;
			}
			
			return(this._buildUri("viewEwi", args, perspective));
//			return this._getPlanningActionUri("viewEwi", args);
		},
				
		getEwisURI: function(queryName, perspective) 
		{
			var args = {};
			if (queryName != null) {
				args.name = queryName;
			}

			if (perspective == null)
			{
				perspective = this.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("viewEwis", args, perspective));
//			return this._getPlanningActionUri("viewEwis", args);
		},
		
		getSizingRecordURI: function(sizingRecordId) {
			var args = {};
			if (sizingRecordId != null)
			{
				args = {uid: sizingRecordId};
			}
			return this._getPlanningActionUri("viewTestSizing", args);
		},
		
		startExecution: function(ewi, scriptId)
		{
			var args = {};
			
			if (ewi != null)
			{
				args = {
				ewi: ewi,
				runScript: scriptId
				};
			}
			return this._getExecutionActionUri("startExecution", args);
		},
	
		executedItemsURI: function (queryName)
		{
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}
			return this._getExecutionActionUri("viewExecutedItems", args);
		},
	
		getEWIResultsURI: function(ewiItemId)
		{
			var args = {};			
			if (ewiItemId != null) {
				args.ewiItemId = ewiItemId;
			}			
			return this._getExecutionActionUri("viewEWIResults", args);
		},
		
		showResultsURI: function(queryName)
		{
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}			
			return this._getExecutionActionUri("viewResults", args);
		},
		
		getResultURI: function(args)
		{
			// args.editable = true means it is scriptless execution
			if(args.editable){
				return this._getExecutionActionUri("startScriptlessExecution", args );
			}else{
				return this._getExecutionActionUri("viewResult", args );
			}
		},
		
		getCurrentProjectAreaAlias: function()
		{
			var currentProjectArea = getCurrentProjectArea();
			var userProjectAreas = dojo.getObject(Constants.USER_PROJECT_AREAS_OBJECT_ID);
			if (userProjectAreas!=null) {
				var length=userProjectAreas.length;
				for (var i=0; i<length; i++) {
					var userProjectArea = userProjectAreas[i];
					if (userProjectArea.itemId == currentProjectArea.itemId) {
						return userProjectArea.alias;
					}
				}
			}
			console.debug("failed to get the project area alias for current project area " + currentProjectArea.name);
			return null;
		},
		
        getCurrentProjectAreaId: function()
        {
            var currentProjectArea = getCurrentProjectArea();
            return currentProjectArea.itemId;
        },
        
		/******************************************************************************************
		 * Generate Unique URL
		 * 
		 * @param {Object} url
		 *****************************************************************************************/
		reuniquifyUrl: function(url)
		{
			return(url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId());
		},
		
		getUniqueId: function()
		{
			return Date.parse(new Date().toString());
		},

		/******************************************************************************************
		 * Set the window location to an action with keywords
	 	 *****************************************************************************************/
		setLocationToActionWithKeywords: function(action, kwArgs)
		{
			//TODO: fix this for all perspectives
			this._setLocationHash(this._getPlanningActionUri(action, kwArgs));
		},
	
		/******************************************************************************************
		 * Set the window location hash.
		 * This is pushed to a method so all changes to the URI can be traced if needed 
		 *****************************************************************************************/
		_setLocationHash: function(locationHash)
		{
			window.location.hash = locationHash;
		},

        /******************************************************************************************
         * Get the window location hash in original form not decoded.
         * window.location.hash returns the decoded URI while window.location.href does not
         * Attributes containing characters such as & and ? must be encoded
         *****************************************************************************************/
        getDecodedLocationHash: function()
        {
            var hash = window.location.hash;
            var href = window.location.href;
            var hashStart = -1;
            if (hash && hash.length > 8) {
                hashStart = href.indexOf(hash.substring(0, 8));
                if (hashStart > 0) {
                    return href.substring(hashStart);
                }
            }
            return window.location.hash;
        },
        
        /******************************************************************************************
         * window.location.href returns the encoded URI including the single quote to %27.
         * However, encodedComponent() does not encode the single quote.  It caused mismatch
         * when comparing uri in the actionDispatcher for maintaining tabs and menu items.
         *****************************************************************************************/
        encodeURIComponent: function(component)
        {
            var encodedComponent = encodeURIComponent(component);
            return encodedComponent.replace(/'/g, "%27");
        },
        
		/******************************************************************************************
		 * QSort functions
		 * 
		 * @param {Object} list
		 * @param {Object} a
		 * @param {Object} b
		 *****************************************************************************************/
		swap: function(list, a, b)
		{
			var tmp = list[a];
			list[a] = list[b];
			list[b] = tmp;
		},
		
		encodeHTMLTags: function(value) {
	    	if(value){
	            // sanitize the label. But in case the label (a name
	            // of an artifact) already has '&' in the name, we want
	            // to keep the '&'.
	            var sanitized = jazz.util.html.escape(value);
	            value = sanitized.replace(/\&amp;/gi, "&");
	    	}
	    	return value; 
	    },
		
		partition: function(array, begin, end, pivot)
		{
			var piv = array[pivot];
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix) 
			{
				if(array[ix] <= piv)
				{
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort: function(list, begin, end)
		{
			if(end-1>begin)
			{
				var pivot = begin+Math.floor(Math.random()*(end-begin));
	
				pivot = this.partition(list, begin, end, pivot);
	
				this.qsort(list, begin, pivot);
				this.qsort(list, pivot+1, end);
			}		
		},
		
		partition_obj: function(array, index, begin, end, pivot)
		{
			var piv = array[pivot].elements[index].data;
			if (typeof piv == "string")
			{
				piv = piv.toLowerCase();
			}
	
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix) 
			{
				data = array[ix].elements[index].data;
				if (typeof data == "string")
				{
					data = data.toLowerCase();
				}
				
				if(data <= piv)
				{
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort_obj: function(list, index, begin, end)
		{
			if(end-1>begin)
			{
				var pivot = begin+Math.floor(Math.random()*(end-begin));
	
				pivot = this.partition_obj(list, index, begin, end, pivot);
	
				this.qsort_obj(list, index, begin, pivot);
				this.qsort_obj(list, index, pivot+1, end);
			}		
		},
		
		/******************************************************************************************
		 * Get Icon/Image functions
		 *  
		 * @param {Object} path
		 *****************************************************************************************/
		getArtifactIconURL: function(artifactType)
		{
			switch(artifactType) {
				case Constants.TYPE_PLAN:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_PLAN_SIZING:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP_SIZING:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE_SIZING:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT_SIZING:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_EWILIST:
					return ImageCache.EWI;	
				default: 
					return "";
					
			};		
		},
		
		// Summary:
        //   Returns the URI for building a query to the artifact type
        //   The following are the supported types.
        TYPE_CASE: Constants.TYPE_CASE,
        TYPE_PLAN:Constants.TYPE_PLAN,
        TYPE_EWI: Constants.TYPE_EWI,
        TYPE_EXECUTION_RESULT: "TestCaseExecutionResult",
		TYPE_SCRIPT: Constants.TYPE_SCRIPT,
		TYPE_SUITE: Constants.TYPE_SUITE,
		TYPE_SUITE_RECORD: Constants.TYPE_SUITE_RECORD,
		TYPE_SUITE_RESULT: Constants.TYPE_SUITE_RESULT,
		TYPE_SCRIPT_STEP: Constants.TYPE_SCRIPT_STEP,

        getQmResourceQueryUri: function(artifactType, /*optional*/webUrl, projectAreaItemId) {
            var typeStr = null;
            switch(artifactType) {
                case this.TYPE_PLAN:
                    typeStr = "com.ibm.rqm.planning.VersionedTestPlan";
                    break;
                case this.TYPE_CASE:
                    typeStr = "com.ibm.rqm.planning.VersionedTestCase";
                    break;
                case this.TYPE_EWI:
                    typeStr = "com.ibm.rqm.execution.TestcaseExecutionRecord";
                    break;
                case this.TYPE_EXECUTION_RESULT:
                    typeStr = "com.ibm.rqm.execution.ExecutionResult";
                    break;
				case this.TYPE_SCRIPT:
					typeStr = "com.ibm.rqm.planning.VersionedExecutionScript";
					break;
				case this.TYPE_SUITE:
					typeStr = "com.ibm.rqm.planning.VersionedTestSuite";
					break;
				case this.TYPE_SUITE_RECORD:
					typeStr = "com.ibm.rqm.execution.TestSuiteExecutionRecord";
					break;
				case this.TYPE_SUITE_RESULT:
					typeStr = "com.ibm.rqm.execution.ExecutionResultGroup";
					break;
				case this.TYPE_SCRIPT_STEP:
					typeStr = "com.ibm.rqm.planning.ExecutionElement2";
					break;
                default:                     
            };      
            if(!typeStr) {
                console.error("artifact type does not support resource links: " + artifactType);
                return null;
            }
            var urlRoot = webUrl || (location.protocol + "//" + location.host + net.jazz.ajax._contextRoot);
            return urlRoot + "/oslc_qm/contexts/" + (projectAreaItemId || getCurrentProjectArea().itemId) + "/resources/" + typeStr + "/";           
        },
        
        // Summary:
        //   Returns the URI for the specified artifact item id to be used in a ResourceLink
        //   widget. The URI provides the rich hover content for the link. Non-supported artifact
        //   types would generate an error and null is returned.
        //
        getQmResourceLinkUri: function(artifactType, versionableItemId, /*optional*/webUrl, projectAreaItemId) {
        
            if(!versionableItemId) {
                console.error("a versionableItemId is required for resource link URIs");
                return null;
            }

            return this.getQmResourceQueryUri(artifactType, webUrl, projectAreaItemId) + versionableItemId;           
        },

		/******************************************************************************************
		 * Returns the Constant representing the artifactType
		 *  
		 * @param {Object} artifact - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getArtifactType: function(artifact) {
			var fullType = artifact._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch(type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.TYPE_SCRIPT;
				case Constants.CLASSNAME_EWI:
					return Constants.TYPE_EWI;	
				case Constants.CLASSNAME_PLAN_SIZING:
					return Constants.TYPE_PLAN_SIZING;
				case Constants.CLASSNAME_GROUP_SIZING:
					return Constants.TYPE_GROUP_SIZING;
				case Constants.CLASSNAME_CASE_SIZING:
					return Constants.TYPE_CASE_SIZING;
				case Constants.CLASSNAME_SCRIPT_SIZING:
					return Constants.TYPE_SCRIPT_SIZING;
				default:
					return null;											
			}			
		},		
		
		/******************************************************************************************
		 * Returns the Constant representing the item's model type
		 *  
		 * @param {Object} item - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getModelType: function(item) {
			var fullType = item._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch (type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.MODEL_TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.MODEL_TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.MODEL_TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.MODEL_TYPE_SCRIPT;
				default:
					return Constants.MODEL_TYPE_UNKNOWN;
			}											
		},
		
		openPrintQueueTable : function(){
			window.location.hash = this._getPlanningActionUri("viewPrintJob");
		},
		
		/******************************************************************************************
		 * Get time as a string
		 *****************************************************************************************/
		getTimeString: function(time) {
			var now = new Date();
			var option = {selector: "time"};
			if(time)
				now.setTime(time);
			return jazz.app.i18n.date.locale.format(now, option);
		},

        /******************************************************************************************
         * Get date/time as a string
         *****************************************************************************************/
        getDateTimeString: function(date) {
            var dateStyle = CommonPreferenceClient.getDateFormatStyle();   
            var style = dojo.clone(dateStyle);
            
            // use both date & time.
            if(style.selector) {
                delete style.selector;
            }         
            return jazz.app.i18n.date.locale.format(date, style);
        },
		
		/******************************************************************************************
		 * Get Date object from ISO string.  Need to wrap call to dojo.date.stamp.fromISOString.
		 *****************************************************************************************/		
		dateFromISOString: function(timestamp) {
			//Example:  2003-10-06T06:46:00.000-0400
			// dojo.date.stamp.fromISOString wants:
			//          2005-06-30T08:05:00-07:00
			if(timestamp.length == 28) {
				timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
			}
			return dojo.date.stamp.fromISOString(timestamp);
		
		},		
	
        /******************************************************************************************
         * Function: formatDate
         * @param {Object} modified
         * @param {Object} isIso
         *****************************************************************************************/
        formatDateFromLong: function(modified) {
            var d = new Date();
            d.setTime(modified);
            return this.formatDate(d, false);
        },
            
		/******************************************************************************************
		 * Function: formatDate
		 * @param {Object} modified
		 * @param {Object} isIso
		 *****************************************************************************************/
		formatDate: function(modified, isIso)
		{
			var now = new Date();
			var modDate = modified;
			if(isIso) {
				modDate = dojo.date.fromIso8601(modified);
			}
            var showRelative = CommonPreferenceClient.getIsDateRelativeStyle();
            
            if (showRelative) {
                var deltaMinutes = dojo.date.difference(modDate, now, "minute");
                
                // if less than an hour ago, display in minutes
                if (deltaMinutes >= 0 && deltaMinutes < 60) {
                    if (deltaMinutes === 0) {
                        deltaMinutes = 1;
                    }
                    return (deltaMinutes === 1) ? Constants.getMessage("labelOneMinuteAgo") : Constants.getMessage("labelMinutesAgo", deltaMinutes);
                }
                
                //			var curDayOfYear= dojo.date.getDayOfYear(now);
                //			var modDateDayOfYear= dojo.date.getDayOfYear(modDate);
                
                // var yearOffset = dojo.date.difference(modDate, now, "year");
                
                //			var yearOffset= now.getYear() - modDate.getYear();
                //			var dayOffset= curDayOfYear - modDateDayOfYear;
                
                //			var totalDayOffset= dayOffset > 0 ?
                //					yearOffset * 365 + dayOffset :
                //					(yearOffset - 1) * 365 + curDayOfYear + 365 - modDateDayOfYear;
                
                var totalDayOffset = dojo.date.difference(modDate, now, "day");
                
                // if same day or less than 12 hours ago, display in hours
                if (((totalDayOffset === 0) && (deltaMinutes >= 0)) || (deltaMinutes >= 0 && deltaMinutes < 720)) {
                    var hours = Math.round(deltaMinutes / 60);
                    return (hours === 1) ? Constants.getMessage("labelOneHourAgo") : Constants.getMessage("labelHoursAgo", hours);
                }
                
                // if previous calendar date, display "Yesterday"
                if (totalDayOffset === 1) {
					var option = {
						selector: "time",
						formatLength: "medium"
					};
					var time = jazz.app.i18n.date.locale.format(modDate, option);
                    return Constants.getMessage("labelYesterdayAt", time);
                }
            }
            var dateStyle = CommonPreferenceClient.getDateFormatStyle();            
			return jazz.app.i18n.date.locale.format(modDate, dateStyle);
		},
		
		/******************************************************************************************
		 * Function: decodeHTMLEntities
		 * @param {String} value
		 * Returns: The string value with any HTML entities transformed back to characters.
		 *****************************************************************************************/
		decodeHTMLEntities: function(value){
            var rawText = value;
            rawText = rawText.replace(/\&gt;/gi, ">");
            rawText = rawText.replace(/\&lt;/gi, "<");
            rawText = rawText.replace(/\&amp;/gi, "&");
            rawText = rawText.replace(/\&tilde;/gi, "~");
            rawText = rawText.replace(/\&quot;/gi, "\"");
            rawText = rawText.replace(/\&circ;/gi, "^");
            rawText = rawText.replace(/\&nbsp;/gi, " ");
            rawText = rawText.replace(/<br\/>/gi, "\n");
            rawText = rawText.replace(/\&#09;/g, "\t");
            return rawText;
        },

        // Summary:
        //    Handler for failed to load an artifact.  If parentNode is specified, the jazz.ui.ErrorPage
        //    will be placed in the parentNode.  Otherwise an alert dialog will be presented with the
        //    with the same info.
        //
        //    @param type - localized artifact type
        //    @param reason - optional detail reason of the failure
        //    @param parentNode - optional DOM node to place the error widget
        //                        Note that everything under the specified parentNode will be deleted!!!!!!!!
        //    @param skipClose - optional if parentNode is specified to not use the "Close Page" action link
        //    @param actions - optional if parentNode is specified an array of action link with these supported
        //                     properties for each action
        //                         iconClass
        //                         icon
        //                         onClick
        //                         href
        //                         label
        //
        // Note: all strings are in HTML format
        handleArtifactLoadError: function(type, reason, parentNode, skipClose, actions) {
            var messageKey = parentNode ? "msgLoadArtifactFail" : "msgLoadArtifactFailAndClose";
            var message = MessageUtil.getMessage(MessageBundle.COMMON, messageKey, type);
            var cleanedReason = null;
            
            if(reason){
          
            	//Note: Remove all script text from the reason to eliminate eliminate cross-site scripting (XSS) attacks and remove all HTML text from the reason to eliminte un-validated redirects.
            	cleanedReason = jazz.util.html.escape(jazz.util.html.removeScript(reason));  
            }
            
            if (parentNode) {
                // remove everything under the parentNode 
                this.destroyWidgetsFromNode(parentNode);
                dojo.empty(parentNode);
                
                var errorPageArgs = {
                    summary: message,
                    details: cleanedReason || ""
                };
                var pageActions = null;
                
                // do not use the standard Go Back action link
                errorPageArgs.includeBackLink = false;
                
                if (!skipClose) {
                    // use a close tab action link 
                    pageActions = [{
                        iconClass: "delete-icon-image",
                        label: MessageUtil.getMessage(MessageBundle.COMMON, "msgCloseThisPage"),
                        onClick: function(e){
                            e.preventDefault();
                            dojo.publish(GlobalEvents.CloseTab, [window.location.hash]);
                        }
                    }];
                }
                if (actions && dojo.isArray(actions)) {
                    if (pageActions) {
                        pageActions = pageActions.concat(actions);
                    } else {
                        pageActions = actions;
                    }
                }
                if (pageActions) {
                    errorPageArgs.actions = pageActions;
                }
                
                var errorPage = new jazz.ui.ErrorPage(errorPageArgs);
                dojo.place(errorPage.domNode, parentNode);
            } else {
                // use a alert dialog if no parent node is specified
                if (cleanedReason) {
                    message += "<br><br>" + MessageUtil.getMessage(MessageBundle.COMMON, "labelDetailLabel") + "<br>" + cleanedReason;
                }
                var args = {
                    okCallback: function(){
                        dojo.publish(GlobalEvents.CloseTab, [window.location.hash]);
                    },
                    width: "40%",
                    primaryTitle: MessageUtil.getMessage(MessageBundle.COMMON, "labelAlert"),
                    message: message
                };
                var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);
                alertDialog.open(true);
            }
        },
        
        // Summary:
        //    Handler for artifact failed to load because it has been deleted.  This is mostly used
        //    for deleted artifacts that have recently viewed entries
        //
        handleArtifactDeleted: function(type, parentNode, skipClose, actions) {
            this.handleArtifactLoadError(type, MessageUtil.getMessage(MessageBundle.COMMON, "msgArtifactDeleted", type), parentNode, skipClose, actions);
        },
        
        //Summary:
        //    Wrapper method for loader.load_async to detect module not loaded from the server.
        //      className - the class module to load
        //      callback - method to call back when done.
        //
        loadClassAsync: function(className, callback){
            if (!dojo.exists(className)) {
                var handler = {
                        'loadModule': className,
                        'callback': callback
                    };
                    var func = dojo.hitch(handler, function() {
                        // if async load done callback and the module doesn't exist, this
                        // could be a server down. Post a message.
                        if(!dojo.exists(this.loadModule)) {
                            alert(MessageUtil.getMessage(MessageBundle.COMMON, "msgJSModuleNotLoaded"));
                        }
                        if(this.callback) {
                            this.callback();
                        }
                    });
                try {
                    jazz.core.loader.load_async(className, func);
                } 
                catch (e) {
                    com.ibm.asq.common.web.ui.internal.util.Utility.printException("Utility", "batchLoadClassAsync", e);
                }
            }
            else if(callback){
                setTimeout(callback, 0);
            }
        },
        
        // Summary:
        //    Returns true if the userId is the unassigned user.
        isUnassignedUserId: function(userId) {
            return userId == UNASSIGNED_ITEMID;
        },
		getUnassignedUserId:function(){
			return UNASSIGNED_ITEMID;
		},
        isCJKLocale: function() {
            return (dojo.locale.indexOf("zh") == 0 || dojo.locale.indexOf("ja") == 0 || dojo.locale.indexOf("ko") == 0);
        },
        
        // Summary:
        //    Return the client locale with default to English if the current locale
        //    is not one of the supported languages.
        //
        getSupportedLocale: function() {
            var languagesCount = SUPPORTED_LANG.length;
            var currentLocale = dojo.locale;
            // iterate through the list of supported language
            for (var i = 0; i < languagesCount; i++) {
                if (currentLocale.indexOf(SUPPORTED_LANG[i]) == 0) {
                    // it is a supported language, check if it is one of
                    // the country specific language
                    var countriesCount = SUPPORTED_COUNTRIES.length;
                    for (var j = 0; j < countriesCount; j++) {
                        if (currentLocale.indexOf(SUPPORTED_COUNTRIES[j]) == 0) {
                            // found exact match, return the locale
                            return currentLocale;
                        }
                    }
                    // it is a supported language but not country specific, return language code only
                    // except if it is English from other countries we default to en-us
                    // or Portuguese we only support from Brazil
                    var language = currentLocale.substring(0, 2);
                    if (language == "en") {
                        language = "en-us";
                    } else if (language == "pt") {
                        language = "pt-br";
                    }
                    return language;
                }
            }
            // it is not a supported language, default to English
            return "en-us";
        },
        
        // Summary:
        //    Given a dom node, find all dijit widgets that are currently attached to
        //    it and its children nodes, and destroy all found widgets. This is useful
        //    when an UI is updating its display by recreating certain part of the dom
        //    tree and ensures that no widget's leaked due to dynamic removal.
        // 
        destroyWidgetsFromNode: function(node) {
            var list = dojo.query('[widgetId]', node);
            var removeWidgets = list.map(dijit.byNode);
            this.destroyWidgetList(removeWidgets);
        },
        
        // Summary:
        //    Given an array of widgets, call destroy on the each of the widgets.
        //    This method handles Jazz widgets as well, if the widget has a 'destroyWidget'
        //    defined, it will call that instead of the plain destroy method. it also
        //    handles the case where certain Jazz widgets doesn't remove itself from 
        //    the dijit registry when it is destroyed.
        //
        //    NOTE: When Jazz clean up there widget destroy code, we might not need to do
        //    some of the extraneous handling to cleanup Jazz widgets.
        //
        destroyWidgetList: function(removeWidgets) {
            var i = removeWidgets.length;
            var registry = dijit.registry;
            
            while (i--) {
                var widget = removeWidgets[i];
                if (widget) {
                    var destroyed = false;
                    var wid = widget.id;
                    //console.debug("Utility.destroyWidgetList - processing : " + widget.id);
                    // handle Jazz widget destroy method. Call this if
                    // it exists.
                    if(widget.destroyWidget) {
                        widget.destroyWidget();
                        destroyed = true;
                        //console.debug("Utility.destroyWidgetList - destroyWidget : " + widget.id);
                    }
                    else if (widget.destroyRecursive) {
                        // Always call 'destroyRecursive' on the widget. Since dojo 1.4.1, container widget's
                        // destroy would attempt to destroy its descendants. If we only call destroy
                        // on a widget, that widget could be partially destroyed. However, if its parent
                        // is a container widget, it would call the widget's destroyRecursive later (within this list). 
                        // destroy & destroyRecursive are two different code paths. destroy could just partially
                        // destroy the widget. And if destroyRecursive is called later, it could get into
                        // accessing null objects. If we always call the destroyRecursive on the widget, 
                        // all of its contents are marked destroyed, calling destroyRecursive on them would
                        // be a no-op (guarded by the widget's destroy flags). 
                        try{
                        	widget.destroyRecursive();
                        	destroyed = true;
                        }
                        catch (e){
							try {
								widget.destroy();
							} catch (ex) {
// uncomment only when debugging
//                        		console.debug("Widget already destroyed in destroyRecursive");
							}
//                        	console.debug("Widget destroyed without destroyRecursive: " + wid);
                        }
                        //console.debug("Utility.destroyWidgetList - destroyRecursive : " + widget.id);
                    }
                    // Jazz widgets may override the destroy method and never
                    // call the superclass's impl. This would leak in the dijit
                    // registry. Manually call the base _Widget class's destroy to
                    // properly dispose of the widget.
                    if(destroyed && registry.byId(wid)) {
                        //console.debug("Utility.destroyWidgetList - widget destroy NOT removed from dijit.registry! - " + wid);
                        _Widget.destroy.apply(widget, []);
                    }
                }
            }            
        },
        preUninitialize: function(cleanupWidgets) {
        	//var start = new Date();
            var i = cleanupWidgets.length;
            while (i--) {
                var widget = cleanupWidgets[i];
                if (widget && widget._magicPreUninitialize) {
                	widget._magicPreUninitialize();
                }
			}        
			//console.debug(">>>>>>>>>>>>>>preUninitialize time = "+((new Date())-start)/1000);
        },
        
        printException: function(className, methodName, e) {
            className = className || "undefined class";
            methodName = methodName || "undefined method";
            
            console.debug("Exception caught in " + className + "#" + methodName);
            if(e) {
                if (dojo.isObject(e)) {
                    for (var i in e) {
                        console.debug(i + ": " + e[i]);
                    }
                }
                else if(dojo.isString(e)) {
                    console.debug(e);
                }
            }
        },
		requireLogin:function(url,callback){
			var _self = this;
			var credentialProvider = new jazz.app.internal.LoginDialog();
			credentialProvider.getCredentials({
				onSubmit: function(userId, password) {
					var dfd = jazz.client.xhr("POST", _self.getJEEFormAuthArgs(userId, password, url), true);
					dfd.addBoth(function(result) {
						if(jazz.util.internal.login.getUserIdCookie()) {
							jazz.util.internal.login.setUserIdCookie(userId);
						}
						if(callback)
							callback();
						
						// publish LOGIN event; include userId so auth framework can tell when user changed
						dojo.publish(jazz.app.events.LOGIN, [userId]);
					});
				},
				onCancel: function() {
				},
				state: null
			});
		},
		
		isCSVEnabled:function(){
			var settingsData = 	InitializationData.get("com.ibm.rqm.planning.service.settingsWebUIInitializer");
			if(settingsData && settingsData.webUISettings){
				return  settingsData.webUISettings.isCSVExportEnabled;
			}
			return false;
		},
		
		isIterationSupportActivated:function(){
			var settingsData = 	InitializationData.get("com.ibm.rqm.planning.service.settingsWebUIInitializer");
			if(settingsData && settingsData.webUISettings){
				return  settingsData.webUISettings.isIterationSupportEnabled;
			}
			return false;
		},
		
		isTeamAreaEnabled: function() {
			return InitializationData.get("teamarea.enabled");
		},

		getJEEFormAuthArgs:function(userId, password, requestUrl) {
			var urlPrefix = requestUrl.substring(0, requestUrl.lastIndexOf("/") + 1);
			return {
				contentType: "application/x-www-form-urlencoded; charset=utf-8",
				url: urlPrefix + "j_security_check",
				content: {
		            j_username: userId,
		            j_password: password
		        }
			};
		},
		
		/**
         * Display an application level warning message across the top of the main app window content area.
         * Changing tabs will not hide the message.  This message is not closable.
         *  
         * @param status  - status message
         * @param isError - optional boolean to indicate message is an error, not just warning
         *
		 * @returns {Boolean} true if message is posted
		 */
        setAppWarning: function(status, isError) {
            var contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
            if (contentArea && dojo.isFunction(contentArea._setAppStatus)) {
                var type = isError ? jazz.ui.MessageBox.STATUS_TYPES.ERROR : jazz.ui.MessageBox.STATUS_TYPES.WARNING;
                contentArea._setAppStatus(status, type);
                return true;
            }
            return false;
        },

      /***
       * Compose the query object for jazz.app.viewHistory containing scope information
       * required to get the correct history. Accepted query properties are:
       *    - 'application' - a value normally specified as application:about
       *    - 'project' - name of the current project area
       *    - 'type' - type of the entry (e.g. 'com.ibm.team.dashboard').
       *    - 'size' - number of items to return (if not set, default is 5)
       *    - 'startIndex' - where to start (if not set, default is 0)
       *    - 'exclude' - an Array of URIs that will be used to exclude the matching items.
       *                  This feature should be used to exclude unsaved items from the list.
       */
      _getViewHistoryQueryObj: function(type) {
        var application = net.jazz.ajax._viewletWebUIRoot + "/application-about";  // match the services.xml doc
        var projectArea = getCurrentProjectArea();
        var queryObj = {
            application: application,
            project: projectArea.itemId,
            type: type
        };
        return queryObj;
      },
      
      /***
       * Get the viewHistory of the specified type.
       */
      getViewHistory: function(type, size, callback, errback, startIndex) {
        var queryObj = this._getViewHistoryQueryObj(type);
        queryObj.size = size;
        if (startIndex) {
            queryObj.startIndex = startIndex;
        }
        var deferred = viewHistory.get(queryObj);
        deferred.then(callback, errback ? errback : function() {
            callback({
                count: 0,
                item: []
            });
        });
      },
      
	  /*
	   * Adds an rich hover over the images for preview
	   */
	  attachImagePreview: function(anchor, href, fileName){
	  	var imagePreviewNode = new ImagePreview( {around:[anchor], src: href, filename: fileName}, document.createElement('div'));
		dojo.connect(imagePreviewNode, "position", this, function(){
			var m = jazz.ui._getMasterPopup();
			var mContent = m.content;
			if(mContent && dojo.hasClass(mContent, "content")){
				dojo.removeClass(mContent, "content");
			}
			});
		return imagePreviewNode;
	  },
	  
	  /***
	   * An implementer must call this function to link image nodes for Preview
	   * Pass on the parent domNode to this function, Ensure that the innerHTML does not change after you have 
	   * called this function else hover will not show the preview
	   * 
	   * This attaches a preview popup for links that points to an image file.
	   */
	  linkImageNodes: function(span, cleanupCallback){
	  	if(!cleanupCallback || !dojo.isFunction(cleanupCallback)){
			console.debug("no cleanupCallback found for the node, add a callback else there will be memory leak!!!");
		}
		//Break the thread, recursive operation may take a while.
		setTimeout(dojo.hitch(this, function(){
			this._linkImageNodes(span, cleanupCallback);
		}), 0);
	},
	
	//Internal function, should not be called directly
	_linkImageNodes: function(span, cleanupCallback){
		//ensure that the span innerHTML does not change after this function is called, else the hover will not show up
		if(span){
            // find link nodes and if it has a filename and the filename's extension
            // is of the type image file, create an image preview popup.
            var cNodes = dojo.query('a', span);
			if (cNodes && cNodes.length > 0) {
                dojo.forEach(cNodes, dojo.hitch(this, function(link){
                    //Process the node
                    if (link.href) {
                    	var isFullSizeImg = false;
                        var filename = link.textContent || link.innerText;
                        if(!filename) {
                            var img = dojo.query('img', link);
                            if(img && img.length == 1) {
                                filename = img[0].alt;
                                if((img[0].width != 50) && (img[0].height != 50)){
                                	// no need to associate preview node to full size image
                                	isFullSizeImg = true;
                                }
                            }
                        }
                        filename = dojo.trim(filename);
                        if (filename && !isFullSizeImg) {
                            if ((/\.(gif|jpg|jpeg|tiff|png|bmp|thm|tif)$/i).test(filename)) {
                                var href = link.href;
                                var richToolTip = this.attachImagePreview(link, href, "");
                                cleanupCallback(richToolTip);
                            }
                        }
                    }
                }));
            }
		}	
	},
	  
	_sameAsLastHistory: function(type, label, uri, iconUri, data) {
	    if (!this.lastHistory) {
	        this.lastHistory = {};
	    }
	    // get the last added history entry of the type
        var lastHistory = this.lastHistory[type];
        var sameHistory = false;

        // compare new entry to the last entry
        if (lastHistory) {
            if ((lastHistory.label == label) && (lastHistory.uri == uri)) {
                // required label and uri params are the same
                if ((lastHistory.iconUri && iconUri && (lastHistory.iconUri == iconUri)) ||
                    (!lastHistory.iconUri && !iconUri)) {
                    // optional iconUri is same or not defined
                    if ((lastHistory.data && data && (lastHistory.data == data)) ||
                        (!lastHistory.data && !data)) {
                        // optional data is same or not defined
                        sameHistory = true;
                    }
                }
            }
        }
        if (!sameHistory) {
            // save this history request as the last entry
            var lastItem = {
                label: label,
                uri: uri
            };
            if (iconUri) {
                lastItem.iconUri = iconUri;
            }
            if (data) {
                lastItem.data = data;
            }
            this.lastHistory[type] = lastItem;
        }
        return sameHistory;
	},

	/***
       * Add an entry to the viewHistory of the specified type.  Adding an entry
       * with existing uri will update the entry and makes it more recent.
       */
      addViewHistory: function(type, label, uri, iconUri, data) {
        var queryObj = this._getViewHistoryQueryObj(type);
        var projectArea = getCurrentProjectArea();
        var root = new dojo._Url(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
        if(root) {
            // QM uri is only the href, need to use fully qualified uri
            uri = root.toString() + "/web/console/" + projectArea.name + uri;
            
            if (iconUri) {
                // locate the path of the context root ("/qm" is the default)
                var pathIndex = iconUri.indexOf(root.path);
                if (pathIndex >= 0) {
                    // the uri can be absolute from viewHistory or relative, keep just the plugin path
                    var pluginPath = iconUri.substr(pathIndex + root.path.length);
                    // iconUri must be absolute if it is used
                    iconUri = root.toString() + pluginPath;
                }
            }
        }
        
        if (this._sameAsLastHistory(type, label, uri, iconUri, data)) {
            // skip if the history is the same as the last one added for the same type
            return;
        }

        var item = {
            label: label,
            uri: uri
        };
        if (iconUri) {
            item.iconUri = iconUri;
        }

        if (data) {
            item.data = data;
        }
        if (lastHistoryUri && uri == lastHistoryUri) {
            // entry with same uri as last request
            lastHistoryDelayCount++; // in case there is delayed request pending, delay more with this request
            setTimeout(function(){
                viewHistory.put(queryObj, item);
                lastHistoryDelayCount--;
            }, 500 * lastHistoryDelayCount);
        } else {
            lastHistoryDelayCount = 0;
            viewHistory.put(queryObj, item);
        }
        lastHistoryUri = uri;
      },
        
      /***
       * Get the href of the uri that QM use for uri from the uri attribute of the viewHistory 
       */
      getViewHistoryUri: function(uri){
          if (uri) {
              var url = new dojo._Url(uri);
              if (url.fragment) {
                  return "#" + url.fragment;
              }
          }
          return uri;
      },

      /***
       * Check if the specified uri maps to one of the known subActions that use versionableItemId
       * instead of just the id as param in the uri for plans, cases, suites and scripts.  This is 
       * used in the editor page to update the uri to the standard one using id.
       * 
       * The subAction must match the definition in the ActionBarEntry.
       */ 
      subActionWithVersionableItemId: function(uri) {
          var uriArgs = this.getArgsFromUri(uri);
          var subAction = uriArgs.subAction;
          var verId = uriArgs.versionableItemId;
          
          if (subAction && verId) {
              switch (subAction) {
                  // list of subActions that use versionableItemId, editors that can create
                  // work item that implicitly create a back link to the artifact
                  //
                  case "viewTestPlan":
                  case "viewTestCase":
                  case "viewTestSuite":
                  case "viewTestScript":
                  case "viewKeyword":
                      return true;
                  default:
                      return false;
              }
          }
          return false;
      },
      
      /***
       * Check if the processing of the page with the specified uri should be skipped on selected.
       * This is used in page selected handler such as viewHistory to skip adding a history entry,
       * For now we skip uri with subAction that use the versionableItemId instead of the standard
       * id for test plans, cases, suites and scripts.  Additional "rules" to skip can be added
       * here instead of modifying the the individual page selected handlers.
       */ 
      skipProcessSelectedPage: function(uri) {
          return this.subActionWithVersionableItemId(uri);
      },
      
      isNavigationKey: function(keyEvent) {
          var key = keyEvent.keyCode || keyEvent.charCode || keyEvent.charOrCode;
          if (key == dojo.keys.TAB || key == dojo.keys.RIGHT_ARROW || key == dojo.keys.LEFT_ARROW || key == dojo.keys.DOWN_ARROW || key == dojo.keys.UP_ARROW ||
              key == dojo.keys.PAGE_DOWN || key == dojo.keys.PAGE_UP || key == dojo.keys.HOME || key == dojo.keys.END)
              return true;
          else
              return false;
      },
      
      /**
       * Removes the script and the style elements from the given HTML text. The style defined in the style tag
       * will be transformed to inline style when possible.
       * @param html
       * @returns sanitized html text.
       */
      removeScriptAndInlinifyStyle: function(html) {
          return jazz.util.html.removeStyleAndScript(html);
          
          /*
           * Disable inlinify styles for 4.0.4
           * 
          var content = jazz.util.html.removeScript(html);
          
          // IE 8 ignores the style tags, no need to do the extra
          // process for nothing.
          if(_dojo.isIE < 9) {
              return content;
          }
          return this.inlinifyStyle(content);
          */
      },
      
    encodeHTMLTags: function(value) {
    	if(value){
            // sanitize the label. But in case the label (a name
            // of an artifact) already has '&' in the name, we want
            // to keep the '&'.
            var sanitized = jazz.util.html.escape(value);
            value = sanitized.replace(/\&amp;/gi, "&");
    	}
    	return value; 
    },
		

      
      /**
       * Given a html text, remove any style element from it and process the style rules to apply as
       * inline styles in the html elements.
       */
      inlinifyStyle: function(html) {
          _tempCss = [];
          _tempRules = {};

          var div = _dojo.create("div", {
              style: {
                  "position":  "absolute",
                  "left": "-10000px"
              }
          });
          var sanitizeDiv = document.createElement("div");

          // we need to add the content to our page to see the rule elements.
          _dojo.place(div, _dojo.body());
          div.innerHTML = html;

          // copy the soource content to the sanitize div. During the
          // copy, the style elements are removed and any css styles matches
          // the nodes will be converted to inline styles
          _copyNode(div, sanitizeDiv);

          var sanitizedHtml = sanitizeDiv.innerHTML;

          // clean up the temp nodes
          _dojo.body().removeChild(div);
          _dojo.destroy(sanitizeDiv);
          _dojo.destroy(div);
          _tempCss = [];
          _tempRules = {};

          return sanitizedHtml;
      }      
	};
	
	// temporary objects used during inlinify style process
    var _tempCss = null;
    var _tempRules = null;
    var CssRulesAttrName = _dojo.isIE ? "rules" : "cssRules";
    
    // Copies a source element's content to an destination element. The destination element
    // will have all attributes of the source but minus the style tag elements.
    //
    var _copyNode = function(element, toElement) {      
        var children = element.childNodes;
        
        for (var i = 0; i < children.length; i++) {
            var node = children[i];
            if (node.nodeType == 1) {
                // Found a style tag object, save it for later applying its rules.
                var nodeName = node.nodeName.toLowerCase();
                if (nodeName == "style") {
                    _tempCss.push(node);
                    continue;
                }
            }
            // copy all attributes to the new node, but don't create it's descendants
            // we will copy its descendants
            var dupNode = node.cloneNode(false);
            _applyRule(dupNode);
            _copyNode(node, dupNode);
            toElement.appendChild(dupNode);         
        }
    };
    
    // Given a node, apply any cssRules that matches it as inline styles
    //
    var _applyRule = function(node) {
        if(node.style && _tempCss && _tempCss.length) {
            
            for(var i = 0; i < _tempCss.length; i++) {
                var rules = _tempCss[i].sheet ? _tempCss[i].sheet[CssRulesAttrName] : null;
                if(rules) {
                    for(var j = 0; j < rules.length; j++) {
                        // the node matches the rule, transform the
                        // rules into a style object and apply it as an
                        // inline style to the node.
                        if(_matchedRule(node, rules[j])) {
                            var styleObj = _transformRules(rules[j]);
                            if(styleObj != "empty") {
                                _dojo.setStyle(node, styleObj);
                            }
                        }
                    }
                }
            }
        }
    };
    
    // Transform a cssRule object's cssText string into an associative array that we can
    // pass into dojo.setStyle directly. We temporarily cache the processed rule in our temp
    // object
    //
	var _transformRules = function(rule) {
	    var ruleId = rule.selectorText;
	    
	    // we have processed this rule, just return it.
	    if(_tempRules[ruleId]) return _tempRules[ruleId];
	    
	    // remove the rule tag & { }
	    var ruleStr = rule.cssText.replace(ruleId, '').replace('{', '').replace('}', '').trim();
        var stylesObj = {};
        var hasStyle = false;
	    
	    if(ruleStr && ruleStr.length) {
	        var styles = ruleStr.split(';');

	        for(var i = 0; i < styles.length; i++){
	            var styleStr = styles[i].trim();
	            
	            if(styleStr.length) {
	                var aStyle = styles[i].split(":");
	                if(aStyle.length == 2) {
	                    var styleName = aStyle[0].trim();
	                    var styleVal = aStyle[1].trim();

	                    if(styleName == "" || styleVal == "") continue;
	                    stylesObj[styleName] = styleVal;
	                    hasStyle = true;
	                }
	            }
	        }
	    }
        // cache it for the run.
        _tempRules[ruleId] = hasStyle ? stylesObj : "empty";
        return  _tempRules[ruleId];
	};
	
	// returns whether the css rule applies to the given node. We match the
	// selector and the class name.
	//
	var _matchedRule = function(node, rule) {
	    var nodeName = node.nodeName.toLowerCase();
	    
	    // match node type selector exactly
	    var selector = rule.selectorText;
	    if(selector == nodeName) return true;
	    
	    var className = node.className;
	    if(className && className.length) {
	        var classes = className.split(" ");
	        for(var i = 0; i < classes.length; i++) {
	            var aClass = "." + classes[i];
	            if(selector == aClass) {
	                return true;
	            }
	        }
	    }
	    return false;
	};
	
}
)();
