/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014. All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
	dojo.provide("com.ibm.rdm.web.links.LinksListWidget");
	
	dojo.require("dojo.i18n");
	dojo.require("dojo.string");
	dojo.require("dijit._Templated");
	dojo.require("dijit.Dialog");
	dojo.require("dijit._Widget");
	dojo.require("dijit._base");
	dojo.require("dijit.layout.ContentPane");
	dojo.require("dijit.form.Button");
	dojo.require("dijit.form.Form");
	dojo.require("dijit.Menu");
	dojo.require("dijit.MenuItem");
	dojo.require("dijit.MenuSeparator");
	dojo.require("dijit.Toolbar");
	dojo.require("dojox.collections.Dictionary");
	
	dojo.require("jazz.ui.MessageBox");
	
	dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactConstants");
	dojo.require("com.ibm.rdm.web.calm.CalmLinkUtils");
	dojo.require("com.ibm.rdm.web.calm.DerivedLinkUtils");
	dojo.require("com.ibm.rdm.web.calm.CalmConstants");
	dojo.require("com.ibm.rdm.web.commenting.EventTopics");
	dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
	dojo.require("com.ibm.rdm.web.common.MessageForm");
	dojo.require("com.ibm.rdm.web.common.MessageHandler");
	dojo.require("com.ibm.rdm.web.common.MimeTypes");
	dojo.require("com.ibm.rdm.web.common.ProjectCache");
	dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
	dojo.require("com.ibm.rdm.web.common.ScrollableMenu");	
	dojo.require("com.ibm.rdm.web.common.StatusDisplay");
	dojo.require("com.ibm.rdm.web.grid.Grid");
	dojo.require("com.ibm.rdm.web.grid.GridModel");
	dojo.require("com.ibm.rdm.web.grid.Operation");
	dojo.require("com.ibm.rdm.web.links.FilterCheckedMenuItem");
	dojo.require("com.ibm.rdm.web.links.Link");
	dojo.require("com.ibm.rdm.web.links.LinkDialog");
	dojo.require("com.ibm.rdm.web.links.LinkItemWidget");
	dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
	dojo.require("com.ibm.rdm.web.links.linkutils");
	dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
	dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
	dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
	dojo.require("com.ibm.rdm.web.project.SavedFilter");
	dojo.require("com.ibm.rdm.web.query.SavedFilterUtils");
	dojo.require("com.ibm.rdm.web.review.ReviewCache");
	dojo.require("com.ibm.rdm.web.review.reviewTypes");
	dojo.require("com.ibm.rdm.web.server.cache.ProcessOperationsCache");
	dojo.require("com.ibm.rdm.web.server.client.utils.LinkUtils");
	dojo.require("com.ibm.rdm.web.suspicion.ClearSuspicionForm");
	dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
	dojo.require("com.ibm.rdm.web.suspicion.SuspicionStatus");
	dojo.require("com.ibm.rdm.web.util.domutils");
	
	// Loaded via load_async
	//dojo.require("com.ibm.rdm.web.linksexplorer.LinksExplorerDialog");
	
	dojo.requireLocalization("com.ibm.rdm.web.search", "common");
	dojo.requireLocalization("com.ibm.rdm.web.links", "LinksListWidget");
	
	var ArtifactConstants = com.ibm.rdm.web.artifactgrid.ArtifactConstants;
	var ProcessOperationsCache = com.ibm.rdm.web.server.cache.ProcessOperationsCache;
	var LinkTypeUtils = com.ibm.rdm.web.links.LinkTypeUtils;
	var LinkUtils = com.ibm.rdm.web.server.client.utils.LinkUtils;
	var domutils = com.ibm.rdm.web.util.domutils;
	var CalmLinkUtils = com.ibm.rdm.web.calm.CalmLinkUtils;
	var DerivedLinkUtils = com.ibm.rdm.web.calm.DerivedLinkUtils;
	var CalmConstants = com.ibm.rdm.web.calm.CalmConstants;
	var linkutils = com.ibm.rdm.web.links.linkutils;
	var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
	var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
	var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
	var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
	var MessageHandler = com.ibm.rdm.web.common.MessageHandler;
	var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
	var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
	var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
	var SuspectLinksUtil = com.ibm.rdm.web.suspicion.SuspectLinksUtil;
	var MessageBox = jazz.ui.MessageBox;
	var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
	var ResourceManager = com.ibm.rdm.web.model.resources.ResourceManager;
	var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
	var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
	var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
	
	var GROUP_BY_LINK_TYPES    = "linkTypesGrouping";
	var GROUP_BY_PROJECT 	   = "linkProjectGrouping";
	var GROUP_BY_DATE_MODIFIED = "linkDateGrouping";
	
	var RESOURCE_ROW = "resourceRow";
	var EXTERNAL_TYPE_NAME_COMPARE = "EXTERNAL_LINK";
	var EXTERNAL_URL_TYPE = "EXTERNAL_URL";
	
	var PAGE_SIZE=15;
	
	var LinkFilterTypes = {
		LINK_TYPES: 0,
		ARTIFACT_TYPE: 1,
		PROJECT_TYPE: 2
	};

	// revisionDate: Date object
	//		When provided, the list of links to display will be filtered to show
	//		only those that are not newer than this revision date.
	var revisionDate = null;

	
	/**
	 * Main links widget.
	 */
	dojo.declare("com.ibm.rdm.web.links.LinksListWidget", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.rdm.web", "links/templates/LinksListWidget.html"),
		
		widgetsInTemplate: true,
		
		///////////////////////////////////////////////////////////////////////////////////
		// Public fields
		
		viewer: null,
		resourceURI: null,
		projectURI: null,
		baselineURI: null,
		reviewId: null,
		selectLinkURI: null, // if specified, select this link in the widget
		boundArtifact: null, // specifies the base URI for the current resource (will be defined if resource is a modulefact)
		modulefactToModuleHostURIMap : [],
		isInModuleBaseline: null,
		
		///////////////////////////////////////////////////////////////////////////////////
		// NLS Fields
		
		sortByName: null,
		sortByType: null,
		sortByLastModifiedDate: null,
		
		///////////////////////////////////////////////////////////////////////////////////
		// Private fields
		
		_messages: null,
		_commonMessages: null,
		
		_viewerResourceURIInfo: null,
		_projectOperations: null,
		_linksLoaded: false,
		_linkTypes: null,
		_foundLinkTypes: null,
		_links: null,
		_externalLinks: [],
		_visibleLinkCount: 0,
		_groupOrder: null,
		_uriToResourceInfos: [],
		_resourceInfoTypeToObjectType: {},
		_totalProjectsByResource: [],
		_projectInfoMap: {},
		_fromElementSelect: false,
		// Used to indicate if uri is forbidden from access or not found.
		_uriToResourceStatus: {},
		
		_linkItemWidgets: null,
		
		_resultsSortBy: null,
		
		_showAllLinkTypeWidget: null,
		_showAllArtifactTypeWidget: null,
		_showAllProjectTypeWidget: null,
		widgetStarted: false,
		// _listenerArr: array
		//		List of listeners within this class
		_listenerArr: null,
		// All subscribed events
		_subs: [],
		
		grid: null,
		gridSelection: null,
		MENU_OPERATIONS: null,
		MENU_LAYOUT: null,
		_contentPane: null,
		_contentPaneSize: null,
		_resultSetGridView: null,
		_arrayListOfResources: [],
		_resourcesMarkedForDeletion: null,
		_toDestroy:[],
		_createLinksMenu: null,
		_createLinksMenuNode: null,

		_linkToolbarFilterDropDownButton: null,
		_linkToolbarSortDropDownButton: null,
		_filterTypesMenu: null,
		_linkTypesMenu: null,
		_artifactTypesMenu: null,
		_projectTypesMenu: null,
		_selectedFilterFromMenu:null,
		_sortMenu: null,
		
		_suspicionListener: null,
		_suspicionEntriesMap: null,
		_savedViewsNamesMap: null,
		
		_RM2RMOrDMBaselineMsgShown: false,
		_rm2rmBaselineMsgHandler: null,
		_messageBox: null,
			
		// Interface to container
		// callbacks.  An object containing methods setTitle, requestActivate, and isActive
		//		 	setTitle: a method that takes a string and sets the title.
		_setTitle: null,
		//			requestActivate: called to request that this widget is activated.
		_requestActivate: null,
	
		// set true on the very first activation.
		_hasBeenActivated: false,
		// set true when activated, false when deactivated.
		_active: false,
		
		//should this widget should be activated by default.
		activateOnLoad: false,

		postCreate: function(){
			var self = this;
			this._listenerArr = [];
			this._resultsSortBy = com.ibm.rdm.web.links.LinksListWidget.sorts.NAME;
			this._rm2rmBaselineMsgHandler = new MessageBox();
			this._messageBox = new MessageBox();
			this.rm2rmBaselineMsgBox.appendChild(this._rm2rmBaselineMsgHandler.domNode);
			this.rm2rmBaselineMsgBox.appendChild(this._messageBox.domNode);
			this.inherited("postCreate", arguments);
			this._linkItemWidgets = [];
			this._toDestroy = [
				this._rm2rmBaselineMsgHandler,
				this._messageBox
			];
			this._subs = [];
			this._selectedFilterFromMenu=[];
			this._suspicionEntriesMap = {};
			this._savedViewsNamesMap = {};
			this.connect(this._linksWidget, "onclick", dojo.hitch(this, function(evt) {
				this.publish(EventTopics.LINK_SELECTED_CLICK, []);
			}));
			this.connect(this._sortByTypeNode, "onClick", this.handleSortByType);
			this.connect(this._sortByNameNode, "onClick", this.handleSortByName);
			this.connect(this._sortByDateNode, "onClick", this.handleSortByDate);
			this.connect(this._groupByRelationShipNode,"onClick", this.handleGroupByType);
			this.connect(this._groupByProjectNode	  ,"onClick", this.handleGroupByProject);
			this.connect(this._groupByDateNode	      ,"onClick", this.handleGroupByDate);
			
			if (this.artifactSuspicionModel) {
				this._suspicionListener = dojo.subscribe(
						SuspectLinksUtil.EVENT_TOPICS.entriesChanged, this, this.handleRefresh);
			}
			
			this._groupOrder = GROUP_BY_LINK_TYPES;
			//Remove selection from other choices, select Group by Type or Relationship
			dojo.removeClass(this._groupByProjectNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._groupByDateNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._groupByRelationShipNode.iconNode, "commentingToolbarDropdownBullet");
			
			this._resultSetGridView = new dijit.layout.ContentPane({}, this._linkGroupsNode);
			this._contentPane = this._resultSetGridView;
			
			// Let's see if the links explorer is installed
			// need to make sure we aren't calling post create from the RelatedResource
			// LinksListWidget
			if (this.declaredClass == "com.ibm.rdm.web.links.LinksListWidget") {
				dojo.style(this._showLinksExplorerNode.domNode, 'visibility', 'visible');
				this.connect(this._showLinksExplorerNode, "onClick", this.handleLinksExplorer);
			}
		},
		
		startup: function() {
			// Seems that we must do this before drop down buttons in the template
			// are started
			this._createLinksMenu = new com.ibm.rdm.web.common.ScrollableMenu(
				null, this._createLinksMenuNode);
			if (this.viewer.isModuleViewer()) {
				this._setTitle(this._messages.moduleLinks);
			} else {
				this._setTitle(this._messages.links);
			}
				
			this.inherited(arguments);
		},
		
		_doFirstActivation: function(){
			// This defines the operations for display on the menu.
			this._menuStartup();
		 
			this.grid = new com.ibm.rdm.web.grid.Grid({
				doTrackFocusForCells: true
			}, this._gridDiv);

			this._totalProjectsByResource = [];
			this._projectInfoMap = {};
			this.grid.startup();
			this._setupGridModelData();
			
			this._loadLinkTypesAndLinks();
			this._setupGridGroups();
			this.grid.showArtifacts({});
			
			// Hide the highlight-all links button when not a diagram editor
			// Hide the highlight-all links button when a Collection or Wrapped Resource type
			var isCollection = this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, this.viewer.getProjectURI()) 
			|| this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, this.viewer.getProjectURI());
			var isWrapperResource = false;
			if (!isCollection)
			{
				// If it is a Collection, then there is no need to check if it is a Wrapped Resource
				isWrapperResource = this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, this.viewer.getProjectURI());
			}
			if ((isCollection || isWrapperResource || this.viewer._resourceViewerWidget && this.viewer._resourceViewerWidget._editorHandle == null) 
					&& this._linkToolbarHighlightLinksNode) {
					var node = this._linkToolbarHighlightLinksNode.domNode;
					node.parentNode.removeChild(node);
			}
			this._linkToolbarFilterDropDownButton.startup();
			this._linkToolbarSortDropDownButton.startup();
		},
		
		// =========== subView lifecycle methods ============
	
		setContainerEnvironment: function(params) {
			// summary:
			//		Called by our container to set up callbacks.
			// params:
			//		An object with the following fields:
			//		 	setTitle: a method that takes a string and sets the title.
			//			requestActivate: called to request that this widget is activated.
			this._setTitle = params.setTitle;
			this._requestActivate = params.requestActivate;
		},
		
		activate: function() {
			// summary:
			//		Called by container when this sub-view is to be activated.
			if (this._active) {
				// shouldn't be called - already active
				return;
			}
			if (!this._hasBeenActivated) {
				this._doFirstActivation();
				this._hasBeenActivated = true;
			}
			this._active = true;
		},
		
		deactivate: function() {
			// summary:
			//		Called by container when this sub-view is to be deactivated.
			//		At the moment we don't do anything when deactivated, but we could
			//		probably avoid doing screen updates other than for title. 
			this._active = false;
		},
		
		handleRefresh: function(){
			if (this._active) {
				this._destroyLinkItemWidgets();
				this._totalProjectsByResource = [];
				this._projectInfoMap = {};
				this._suspicionEntriesMap = {};
				this._linksLoaded = false;
				this._loadLinkTypesAndLinks();
			}
		},
				
		containerDimensionsHaveChanged: function(){
			if (this.domNode.parentNode) {
				var height = this.domNode.parentNode.clientHeight;
				var width = this.domNode.parentNode.clientWidth;
				var contentPaneTop = (this._contentPane.domNode.offsetTop -
				this._contentPane.domNode.parentNode.offsetTop);
				// TODO: Find out where this 8px is coming from and measure the
				// node that uses it.
				var scrollHeightToUse = (height - contentPaneTop - 8);
				
				this.resize({
					h: scrollHeightToUse,
					w: width
				});
			}
		},

	
		// =========== end subView lifecycle methods ============
		
		postMixInProperties: function(){
			this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
			this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinksListWidget");
		},
		
		uninitialize: function(){
			for(var i=0; i < this._toDestroy.length; i++){
				if (this._toDestroy[i].destroyRecursive){
					this._toDestroy[i].destroyRecursive();
				}
				this._toDestroy[i] = null;
			}	
			this._toDestroy = [];
			
			//Destroy the grid
			if(this.grid){
				this.grid.destroyRecursive();
			}
					
			for (var i = 0; i < this._subs.length; i++) {
				dojo.unsubscribe(this._subs[i]);
				this._subs[i] = null;
			}
			this._subs = [];
			
			this._destroyLinkItemWidgets();
			
			if (this._listenerArr) {
				for (var i = 0; i < this._listenerArr.length; i++) {
					var listener = this._listenerArr[i];
					ChangeNotifier.removeListener(listener);
				}
			}
			
			if (this._createLinksMenu) {
				this._createLinksMenu.destroyRecursive();
			}

			if (!this._hasBeenActivated) {
				// Note: A DropDownButton will ONLY destroy its associated menu
				// if its startup function has previously been called.  However,
				// the startup function for this LinksListWidget will ONLY be
				// called when/if the associated sidebar section is expanded.
				// This means that we must explicitly destroy the menu found
				// in each DropDownButton if the activate function found in this
				// class has not yet been called ...
				if (this._linkTypesMenu) {
					this._linkTypesMenu.destroyRecursive();
				}
				if (this._artifactTypesMenu) {
					this._artifactTypesMenu.destroyRecursive();
				}
				if (this._projectTypesMenu) {
					this._projectTypesMenu.destroyRecursive();
				}
				if (this._filterTypesMenu) {
					this._filterTypesMenu.destroyRecursive();
				}
				if (this._sortMenu) {
					this._sortMenu.destroyRecursive();
				}
			}
			if (this._linkToolbarHighlightLinksNode
				&& this._linkToolbarHighlightLinksNode.domNode
				&& !this._linkToolbarHighlightLinksNode.domNode.parentNode
				) {
				// Note: It _may_ be the case that this class has explicitly
				// performed a removeChild for the DOM node associated with the
				// ToggleButton represented by this._linkToolbarHighlightLinksNode.
				// To prevent leaking this we take care here to explicitly
				// destroy it.
				this._linkToolbarHighlightLinksNode.destroyRecursive();
			}
			
			if (this._suspicionStatusWidget) {
				this._suspicionStatusWidget.destroyRecursive();
			}
			if (this._suspicionListener) {
				dojo.unsubscribe(this._suspicionListener);
			}
			
			this._selectedFilterFromMenu=null;
		},
		
		destroy: function(){
			this.inherited(arguments);
		},
		
		_destroyLinkItemWidgets: function(){
			dojo.forEach(this._linkItemWidgets, function(widget){
				if (widget){
					widget.destroyRecursive();
				}
			});
			this._linkItemWidgets.length = 0;
		},
		
		_menuStartup: function() {
			var that = this;
			
			this.subscribe(EventTopics.COMMENT_SELECTED_TOPIC, "_resourceLocationSelected");
			this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC, "_commentPaneSelected");
		
			var viewModeListener = ChangeNotifier.addListener({
				eventTopics: [com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_VIEW_MODE],
				callbackFunction: this._onViewModeNotify,
				callbackScope: this,
				resourceUriScope: this.resourceURI
			});
			this._listenerArr.push(viewModeListener);
			
			var elementSelectionListener = ChangeNotifier.addListener({
				eventTopics: [com.ibm.rdm.web.notifications.EventTopics.ELEMENT_SELECTED],
				callbackFunction: this._onElementSelected,
				callbackScope: this,
				resourceUriScope: this.resourceURI
			});
			this._listenerArr.push(elementSelectionListener);
			
			//Check the links and permissions and determine whether to allow deletion of links
			
			//Get Project URI
			var deferred = ResourceManager.getResourceDescribe({
					resourceURI: this.viewer.resourceURI,
					revision: this.viewer.revision,
					projectSnapshotURI: this.baselineURI,
					ignoreErrors: true
				});
			deferred.addCallback(this, function(resourceDescription){
			
				this._viewerResourceURIInfo = resourceDescription;
				var projectURI = (resourceDescription) ? resourceDescription.projectURI : null;
				this.setProjectURI(projectURI);
				
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_RENAMED,
					callbackFunction: this._onRenamed,
					callbackScope: this
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_DELETED,
					callbackFunction: this._onDeleted,
					callbackScope: this
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_CREATED,
					callbackFunction: this._onLinkCreate,
					callbackScope: this
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_DELETED,
					callbackFunction: this._onLinkDelete,
					callbackScope: this
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_TYPE_ADDED,
					callbackFunction: this._onLinkTypesChanged,
					callbackScope: this,
					projectURI: projectURI
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_TYPE_REMOVED,
					callbackFunction: this._onLinkTypesChanged,
					callbackScope: this,
					projectURI: projectURI
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_TYPE_UPDATED,
					callbackFunction: this._onLinkTypesChanged,
					callbackScope: this,
					projectURI: projectURI
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_PREFERENCES_UPDATED,
					callbackFunction: this._onLinkPreferencesChanged,
					callbackScope: this,
					projectURI: projectURI
				}));
				this._listenerArr.push(ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.DM_LINK_ACTIONS,
					callbackFunction: this._onDMLinkCreatedOrDeleted,
					callbackScope: this
				}));
				this._listenerArr.push(ChangeNotifier.addListener({					
					eventTopic: [com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_SHOW_BASE_PROPERTIES],
					callbackFunction: this._onShowBasePropertiesNotify,	        	
					resourceUriScope: this.resourceURI,
					callbackScope: this
				}));
			});
			
			this.MENU_OPERATIONS = {
				".remove": // Params: _id, _iconClass, _displayText,
				// _isEnabledMethod, _executeMethod
				new com.ibm.rdm.web.grid.Operation(".remove", "removeCommand", 
					function(numSelected){return that.removeText(numSelected);}, 
					function(rowIds, count){return that.isRemoveEnabled(rowIds, count);}, 
					function(rowIds){that._executeRemove(rowIds);}, 
					null,
					function(rowIds, count) {return that.getRemoveEnablementDeferred(rowIds, count);
				}),
				".editLinkLabel": new com.ibm.rdm.web.grid.Operation(".editLinkLabel", "editLinkCommand", 
					function(numSelected){return that._messages.editLinkLabel;}, 
					function(rowIds, count){return that.isEditLinkLabelEnabled(rowIds, count);}, 
					function(rowIds){that._executeEditLinkLabel(rowIds);},
					null,
					function(rowIds, count) {return that.getEditEnablementDeferred(rowIds, count);
				}),
				".clearSuspicion": new com.ibm.rdm.web.grid.Operation(".clearSuspicion", "clearSuspicionCommand", 
					function(numSelected){return that._messages.clearSuspicion;}, 
					function(rowIds, count){return that.isClearSuspicionEnabled(rowIds, count);}, 
					function(rowIds){that._executeClearSuspicion(rowIds);},
					function(rowIds, count){return that.isClearSuspicionVisible(rowIds, count);},
					function(rowIds, count) {return that.getClearSuspicionEnablementDeferred(rowIds, count);
				})
			};
			
			// This defines how the menu is laid out - ie the grouping etc.
			this.MENU_LAYOUT = [{
				groupName: "EditGroup",
				isShown: false,
				separatorAbove: false,
				operations: [{
					id: ".editLinkLabel",
					seperatorAbove: false
				}, {
					id: ".remove",
					separatorAbove: true
				}, {
					id: ".clearSuspicion",
					separatorAbove: true
				}]
			}];
		},
				
		handleLinksExplorer: function() {
			// summary: 
			//		Open the links explorer window used to show
			//		a graphical representation of the artifacts
			//		links.
			
			try {
				jazz.core.loader.load_async("com.ibm.rdm.web.linksexplorer.LinksExplorerDialog", dojo.hitch(this, function() {
			        var constructor = dojo.getObject("com.ibm.rdm.web.linksexplorer.LinksExplorerDialog.createNewInstance");
			        if(!constructor) { throw "No constructor found"; }
			        var linksExplorerDialog = constructor({
						_resourceURI: this.resourceURI,
						_projectURI: this.projectURI,
						_baselineURI: this.baselineURI,
						viewer: {
							getProcessAreaURI: dojo.hitch(this, function() {
								return this.viewer.getResourceObject().getProcessAreaURI();
							}),
							getProjectURI: dojo.hitch(this, function() {
								return this.projectURI;
							}),
							projectSnapshotURI: this._baselineURI
						}
					});
				
					linksExplorerDialog.show();
					// as of 17/9/12 we no longer need .startup()?
					//linksExplorerDialog.startup(); 	
				
				}));
			} catch (e) {
			    console.error("Failed to load LinksExplorer : " + e.message);
			}
		},	
				
		_highlightAllLinks: function(val) {
			var result = {};
			var links = this.getLinks();
			if (this._linkToolbarHighlightLinksNode.checked && links) {
				for (var i = 0; i < links.length; i++) {
					var link = links[i];
					if (link.isElementLink()) {
						var locationId = null;
						var resourceURIToCompare = this._getResourceURIToCompare(link);
						if (link.getSubject() == resourceURIToCompare && link.getSubjectLocationId() != null) {
							locationId = link.getSubjectLocationId();
						} else if (link.getObject() == resourceURIToCompare && link.getObjectLocationId() != null) {
							locationId = link.getObjectLocationId();
						}
						if (locationId) {
							if (result[locationId]) {
								result[locationId]++;
							} else {
								result[locationId] = 1;
							}
						}
					}
				}
			}
			this.publish(EventTopics.LOCATE_LINKED_LOCATIONS_IN_RESOURCE, [result]);
		},
		
		_setDisabledHighlightlinksAttr: function(val) {
			var isRichTextType = this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.REQUIREMENT_NEW_NAME, this.viewer.getProjectURI());
			if (isRichTextType) {
				this._linkToolbarHighlightLinksNode.set("disabled", val);
			}
		},
		
		_onElementSelected: function(params) {
			this._fromElementSelect = true;
			var links = this.getLinks();
			if (links) {
				var elementId = params.notifierParams.elementId;
				this.grid.module.selectAll(false);
				var keys = [];
				for (var i = 0; i < links.length; i++) {
					var link = links[i];
					var suffix = null;
					if (link.getSubjectLocationId() == elementId) {
						suffix = "/object";
					} else if (link.getObjectLocationId() == elementId) {
						suffix = "/subject";
					}
					if (suffix) {
						keys.push(link.getResourceURI() + suffix);
					}
				}
				this.grid.module.addToSelection(keys);
			}
		},
		
		showArtifacts: function(){
		
			this._visibleLinkCount = 0;
			
			switch (this._resultsSortBy) {
				case com.ibm.rdm.web.links.LinksListWidget.sorts.NAME:
					this.grid.selectGroup(this._groupOrder, this._sortLinksByName);
					break;
				case com.ibm.rdm.web.links.LinksListWidget.sorts.TYPE:
					this.grid.selectGroup(this._groupOrder, this._sortLinksByArtifactType);
					break;
				case com.ibm.rdm.web.links.LinksListWidget.sorts.DATE:
					this.grid.selectGroup(this._groupOrder, this._sortLinksByDate);
					break;
			}
			
			domutils.createDefaultCursor();
		},
			
		_onViewModeNotify: function() {
			// highlight the associated element in the artifact if a link is selected
			if (this._gridSelection) {
				this._onSelectionChange(this._gridSelection);
			}
		},
		
		_resourceLocationSelected: function(selectedLocationId, unselectedLocationId) {
			// an element in the artifact has been selected, clear the selected link
			// we only do highlighting from links list widget to element, not both ways
			
			if (this.grid && this.grid.module) {
				this.grid.module.selectAll(false);
			}
		},
		
		_commentPaneSelected: function(selectedLocationId, resourceURI) {
			// a comment has been selected, clear the selected link since the current
			// artifact selection will represent what is valid for the selected comment
			
			// protect against this event coming before we are ready (can happen when comment
			// pane gets going before we are fully initialised).
			if (this.grid && this.grid.module) {
				this.grid.module.selectAll(false);
			}
		},
		
		_onRenamed: function(/*notifierParams*/eventParams){
		
			var self = this;
			var renamedResource = eventParams.notifierParams.resourceURI;
			//Get the modified resource info
			var notifierResourceInfo = eventParams.notifierParams.notifyResource;
			var refresh = [];
			
			//Check if the renamed resource is found in the links
			dojo.forEach(self._arrayListOfResources, function(item){
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(item.link)];
				if (resourceInfo) {
					if (resourceInfo.resourceURI == renamedResource) {
						//Update the current stored resource info with the data from the new one.
						var wrappedResourceURI = "";
						if (notifierResourceInfo.artifactFormat == RRMTypes.getArtifactFormats(notifierResourceInfo.projectURI)["WrapperResource"]) {
							wrappedResourceURI = notifierResourceInfo.getWrappedResourceURI();
						}
						var modifiedResourceURI = self._uriToResourceInfos[resourceInfo.resourceURI];
						modifiedResourceURI.identifier = notifierResourceInfo.getID();
						modifiedResourceURI.name = notifierResourceInfo.getTitle();
						modifiedResourceURI.displayName = notifierResourceInfo.getTitle();
						modifiedResourceURI.contentType = (wrappedResourceURI) ? notifierResourceInfo.getMimeType() : notifierResourceInfo.getEntityType();
						modifiedResourceURI.wrappedResourceURI = wrappedResourceURI;
						modifiedResourceURI.type = notifierResourceInfo.getEntityType();
						modifiedResourceURI.lastModifiedDate = notifierResourceInfo.getRevisionDate();
						modifiedResourceURI.lastModifiedBy = notifierResourceInfo.getLastModifiedBy();
						modifiedResourceURI.projectSnapshotURI = notifierResourceInfo.getSnapshotURI();
						modifiedResourceURI.etag = notifierResourceInfo.getRevision();
						
						self._uriToResourceInfos[resourceInfo.resourceURI] = modifiedResourceURI;
						var linkType = (item.link.getSubject() == resourceInfo.resourceURI) ? "subject" : "object";
						refresh.push({
							key: item.id + "/" + linkType,
							rowtype: RESOURCE_ROW
						});
					}
				}
			});
			
			if (refresh.length > 0) {
				self.grid.refreshRows(refresh);
			}
		},
		
		getLinks: function() {
			return this._links;
		},
		
		_onDeleted: function(/*notifierParams*/eventParams){
		
			var self = this;
			var deletedResource = eventParams.notifierParams.resourceURI;
			var refresh = [];
			
			//Check if the deleted resource is found in the links
			dojo.forEach(self._arrayListOfResources, function(item){
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(item.link)];
				if (resourceInfo) {
					if (resourceInfo.resourceURI == deletedResource) {
						//Void the removed resource in our the list of resourceInfos and refresh the row 
						self._uriToResourceInfos[resourceInfo.resourceURI] = null;
						var linkType = (item.link.getSubject() == resourceInfo.resourceURI) ? "subject" : "object";
						refresh.push({
							key: item.id + "/" + linkType,
							rowtype: RESOURCE_ROW
						});
					}
				}
			});
			
			if (refresh.length > 0) {
				self.grid.refreshRows(refresh);
			}
		},
		
		_populateFilterButtons: function(){
			this._createLinkTypesMenu();
			this._createProjectAndArtifactTypeMenu();
			
		},
		
		_getTargetURIFromLink: function(link) {
			var resourceURIToCompare = this._getResourceURIToCompare(link);
			return link.getSubject() == resourceURIToCompare ? link.getObject() : link.getSubject();
		},
		
		_compareTypes: function(resourceInfo1, resourceInfo2){
		
			var self = this;
			var resourceContentType1 = null;
			var resourceContentType2 = null;
			
			if (resourceInfo1 && self._resourceInfoTypeToObjectType[resourceInfo1.ofType]) {
				resourceContentType1 = self._resourceInfoTypeToObjectType[resourceInfo1.ofType].getTitle();
			}
			else {
				resourceContentType1 = EXTERNAL_TYPE_NAME_COMPARE;
			}
			if (resourceInfo2 && self._resourceInfoTypeToObjectType[resourceInfo2.ofType]) {
				resourceContentType2 = self._resourceInfoTypeToObjectType[resourceInfo2.ofType].getTitle();
			}
			else {
				resourceContentType2 = EXTERNAL_TYPE_NAME_COMPARE;
			}
			
			if (resourceContentType1.toLowerCase() == resourceContentType2.toLowerCase()) {
				//Artifact Types are case sensitive. Return the results in 'A' to 'z' order				
				if (resourceContentType1 == resourceContentType2) {//Same type, same case
					return 0;
				}
				if (resourceContentType1 < resourceContentType2) {
					return -1;
				}
				else {
					return 1;
				}
				
			}
			else {
				if (resourceContentType1.toLowerCase() < resourceContentType2.toLowerCase()) {
					return -1;
				}
				else {
					return 1;
				}
			}
		},
		
		_sortLinksByName: function(artifacts){
			var self = this;
			
			artifacts.sort(function(art1, art2){
				var resourceInfo1 = self._uriToResourceInfos[self._getTargetURIFromLink(art1.link)];
				var resourceInfo2 = self._uriToResourceInfos[self._getTargetURIFromLink(art2.link)];
				var resourceName1 = null;
				var resourceName2 = null;
				
				if (resourceInfo1) {
					resourceName1 = resourceInfo1.displayName;
					if (resourceName1) {
						resourceName1 = resourceName1.toLowerCase();
					}
				}
				else {
					resourceName1 = art1.link.getRationale().toLowerCase();
					if (resourceName1 == "") {
						resourceName1 = art1.link.getTitle().toLowerCase();
					}
				}
				if (resourceInfo2) {
					resourceName2 = resourceInfo2.displayName;
					if (resourceName2) {
						resourceName2 = resourceName2.toLowerCase();
					}
				}
				else {
					resourceName2 = art2.link.getRationale().toLowerCase();
					if (resourceName2 == "") {
						resourceName2 = art2.link.getTitle().toLowerCase();
					}
				}
				if (resourceName1 == resourceName2) {
					if (linkutils.sameLocation(art1.id) && linkutils.sameLocation(art2.id)) {
						if (resourceInfo1 && resourceInfo2) {
							return self._compareTypes(resourceInfo1, resourceInfo2);
						}
						else {
							return 0;
						}
					}
					else {
						return 0;
					}
				}
				else {
					return (resourceName1 < resourceName2) ? -1 : 1;
				}
			});
			
			dojo.removeClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
		},
		
		_sortLinksByArtifactType: function(artifacts){
		
			var self = this;
			artifacts.sort(function(art1, art2){
				var resourceInfo1 = self._uriToResourceInfos[self._getTargetURIFromLink(art1.link)];
				var resourceInfo2 = self._uriToResourceInfos[self._getTargetURIFromLink(art2.link)];
				return self._compareTypes(resourceInfo1, resourceInfo2);
				
			});
			
			dojo.removeClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
		},
		
		_sortLinksByDate: function(artifacts){
		
			var self = this;
			artifacts.sort(function(art1, art2){
			
				var nameComparator = function(linkObj1, linkObj2){
					var resourceInfo1 = self._uriToResourceInfos[self._getTargetURIFromLink(linkObj1)];
					var resourceInfo2 = self._uriToResourceInfos[self._getTargetURIFromLink(linkObj2)];
					var resourceName1 = null;
					var resourceName2 = null;
					
					if (resourceInfo1) {
						resourceName1 = resourceInfo1.displayName;
						if (resourceName1) {
							resourceName1 = resourceName1.toLowerCase();
						}
					}
					else {
						resourceName1 = art1.link.getRationale().toLowerCase();
						if (resourceName1 == "") {
							resourceName1 = art1.link.getTitle().toLowerCase();
							if (!resourceName1 || resourceName1 == "") {
								resourceName1 = art1.link.getTitle().toLowerCase();
							}
						}
					}
					if (resourceInfo2) {
						resourceName2 = resourceInfo2.displayName;
						if (resourceName2) {
							resourceName2 = resourceName2.toLowerCase();
						}
					}
					else {
						resourceName2 = art2.link.getRationale().toLowerCase();
						if (resourceName2 == "") {
							resourceName2 = art2.link.getTitle().toLowerCase();
						}
					}
					return (resourceName1 < resourceName2) ? -1 : 1;
				};
				
				var linkDate1;
				var linkDate2;
				var resourceInfo1 = self._uriToResourceInfos[self._getTargetURIFromLink(art1.link)];
				var resourceInfo2 = self._uriToResourceInfos[self._getTargetURIFromLink(art2.link)];
				
				if (resourceInfo1) {
					linkDate1 = resourceInfo1.lastModifiedDate;
				}
				else {
					linkDate1 = art1.link.getLastModifiedDate();
				}
				if (resourceInfo2) {
					linkDate2 = resourceInfo2.lastModifiedDate;
					
				}
				else {
					linkDate2 = art2.link.getLastModifiedDate();
				}
				
				var compare = dojo.date.compare(linkDate1, linkDate2);
				
				if (compare == 0) {
					return nameComparator(art1.link, art2.link);
				}
				else {
					return compare;
				}
				
			});
			artifacts.reverse();
			dojo.removeClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
		},
		
		setResults: function(results){
			var self = this;
			this._arrayListOfResources = [];
			dojo.forEach(results, function(item){
				self._arrayListOfResources.push({
					id: item.getResourceURI(),
					link: item
				});
			});
		},
		
		_setupGridGroups: function(){
		
			var that = this;
			
			//Group by Link Type
			this.grid.addGrouping({
				groupingId: GROUP_BY_LINK_TYPES,
				hideEmptyGroups: true,
				getKeysFunction: function(groupingId){				
					return that.getLinkTypeKeys(groupingId);
				},
				getHeaderDataFunction: function(groupingId, key, count){
					return that.getLinkTypeNames(groupingId, key, count);
				},
				getRowsFunction: function(groupingId, key, sortFunction){
					return that.getLinkTypeRows(groupingId, key, sortFunction);
				}
			});			
			//Group by Project
			this.grid.addGrouping({
				groupingId: GROUP_BY_PROJECT,
				hideEmptyGroups: true,
				getKeysFunction: function(groupingId){
					//Here we return our array of unique project URIS (extracted from the link resources)
					return that._totalProjectsByResource;
				},
				getHeaderDataFunction: function(groupingId, key, count){
					var title="";
					
					if(that._projectInfoMap[key]){  //Check if the project is in the map. i.e not archived or inaccessible
						title = that._projectInfoMap[key].name;			 
					}else{
						title = that._messages.unknownProject;
					}
					var values = {
						groupingId: groupingId,
						key: key,
						title: title,
						count: count
					};
					return values;			
				},
				getRowsFunction: function(groupingId, key, sortFunction){
					return that.getLinkTypeRowsFromProjectKey(groupingId, key, sortFunction);
				}
			});
			
			//Group by Date Modified
			this.grid.addGrouping({
				groupingId: GROUP_BY_DATE_MODIFIED,
				hideEmptyGroups: true,
				getKeysFunction: function(groupingId){
					//Get the Last Date Modified from the link	
					var dateRangeKeys=[];				
					dojo.forEach(that._arrayListOfResources, function(item){
						var resourceInfo = that._uriToResourceInfos[that._getTargetURIFromLink(item.link)];	
						if(resourceInfo){
							//Group items in fuzzy date ranges(Today, Yesterday, etc...)
							var dateRange = DateTimeHelper.getDateRange(new Date(resourceInfo.lastModifiedDate));
							if (dojo.indexOf(dateRangeKeys, dateRange) == -1) {
								dateRangeKeys.push(dateRange);						
							}	
						}										
					});
					return dateRangeKeys;
				},
				getHeaderDataFunction: function(groupingId, key, count){				
					var values = {
						groupingId: groupingId,
						key: key,
						title: key,
						count: count
					};
					return values;
				},
				getRowsFunction: function(groupingId, key, sortFunction){
					return that.getLinkTypeRowsFromLastModifiedDateKey(groupingId, key, sortFunction);
				}
			});
		},
		
		_getResourceURIToCompare: function(link) {
			if (com.ibm.rdm.web.query.SavedFilterUtils.isSavedView(link.getSubject())) {
				return link.getSubject();
			} else {
				return (link.isBaseArtifactLink) ? this.boundArtifact : this.resourceURI;
			}
		},
		
		getLinkTypeKeys: function(groupingId){
			var linkTypeKeys = [];
			for (var i = 0; i < this._foundLinkTypes.length; i++) {
				var hasObjectLinks = false;
				var hasSubjectLinks = false;
				var linkType = this._linkTypes[this._foundLinkTypes[i]];
				//if (dojo.indexOf(LinkTypeUtils.SYSTEM_LINKS_EXCLUDE_FROM_VIEW, this._getUniqueUriForLinkType(linkType)) != -1) {
				//	continue;
				//}
				for (var x = 0; x < this._links.length; x++) {
					var link = this._links[x];
					var resourceURIToCompare = this._getResourceURIToCompare(link);
					if (link.getLinkType() == linkType.getResourceURI()) {
						if (link.getObject() == resourceURIToCompare) {
							hasSubjectLinks = true;
						}
						else 
							if (link.getSubject() == resourceURIToCompare) {
								hasObjectLinks = true;
							}
					}
					if (hasObjectLinks && hasSubjectLinks) {
						break;
					}
				}
				var addKey = function(key){
					if (key != null) {
						if (dojo.indexOf(linkTypeKeys, key) == -1) {
							linkTypeKeys.push(key);
						}
					}
				};
				
				if (linkType.isBidirectional()) {
					if (hasObjectLinks || hasSubjectLinks) {
						var key = this._getUniqueUriForLinkType(linkType) + "/bidirectional";
						addKey(key);
					}
				}
				else {
					if (hasObjectLinks) {
						var key = this._getUniqueUriForLinkType(linkType) + "/object";
						addKey(key);
					}
					if (hasSubjectLinks) {
						var key = this._getUniqueUriForLinkType(linkType) + "/subject";
						addKey(key);
					}
				}
				
			}
			return linkTypeKeys;
		},
		
		getLinkTypeNames: function(groupingId, key, count){
			var title = null;
			for (var i = 0; i < this._foundLinkTypes.length; i++) {
				var linkType = this._linkTypes[this._foundLinkTypes[i]];
				if (key == this._getUniqueUriForLinkType(linkType) + "/bidirectional" || key == this._getUniqueUriForLinkType(linkType) + "/subject") {
					title = linkType.getObjectToSubjectLabel();
					break;
				}
				else 
					if (key == this._getUniqueUriForLinkType(linkType) + "/object") {
						title = linkType.getSubjectToObjectLabel();
						break;
					}
			}
			var values = {
				groupingId: groupingId,
				key: key,
				title: title,
				count: count
			};
			return values;
		},
				
		refreshGridGroupHeaders: function(){
		
			var linkTypeKeys = this.getLinkTypeKeys();
			var myGrid = this.grid;
			//Apply this refresh asyncly to get rid of refresh errors.
			setTimeout(function() {
				for (var key in linkTypeKeys) {
					myGrid.refreshRows([{
						key: linkTypeKeys[key],
						rowtype: GROUP_BY_LINK_TYPES
					}]);
				}
			}, 0);
		},
		
		_getLinksForLinkTypeKey: function(key, links){
			// Return the links that match the given link type group key
			var group = [];
			var self = this;
			dojo.forEach(links, function(link){
				var resourceURIToCompare = self._getResourceURIToCompare(link);
				if ((key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[link.getLinkType()]) + "/subject") == 0 || 
					 key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[link.getLinkType()]) + "/bidirectional") == 0) && link.getObject() == resourceURIToCompare) {				
						group.push(link);							
				} else if ((key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[link.getLinkType()]) + "/object") == 0 || 
							key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[link.getLinkType()]) + "/bidirectional") == 0) && link.getSubject() == resourceURIToCompare) {					
					group.push(link);				
				}
			});
			return group;
		},
		
		getLinkTypeRows: function(groupingId, key, sortFunction){
			var rowsInGroup = [];
			var self = this;
			if (sortFunction) {
				sortFunction.call(this, this._arrayListOfResources);
			}
			dojo.forEach(this._arrayListOfResources, function(item){
				var resourceURIToCompare = self._getResourceURIToCompare(item.link);
				if ((key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[item.link.getLinkType()]) + "/subject") == 0 || 
					 key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[item.link.getLinkType()]) + "/bidirectional") == 0) && item.link.getObject() == resourceURIToCompare) {
					rowsInGroup.push({
						key: item.id + "/subject",
						rowtype: RESOURCE_ROW
					});
				}
				else 
					if ((key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[item.link.getLinkType()]) + "/object") == 0 || 
						 key.indexOf(self._getUniqueUriForLinkType(self._linkTypes[item.link.getLinkType()]) + "/bidirectional") == 0) && item.link.getSubject() == resourceURIToCompare) {
						rowsInGroup.push({
							key: item.id + "/object",
							rowtype: RESOURCE_ROW
						});
					}
			});
			this._visibleLinkCount += rowsInGroup.length;
			return rowsInGroup;
		},
		
		_getLinksForProjectKey: function(key, links){
			// Return the links that match the given project group key
			var self = this;
			var group = [];
			dojo.forEach(links, function(link){			
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(link)];
				if (resourceInfo) {
					if (key == resourceInfo.projectURI) {
						group.push(link);
					}
				}			
			});
			return group;
		},
				
		getLinkTypeRowsFromProjectKey: function(groupingId, key, sortFunction){
			var rowsInGroup = [];
			var self = this;
			if (sortFunction) {
				sortFunction.call(this, this._arrayListOfResources);
			}
			dojo.forEach(this._arrayListOfResources, function(item){			
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(item.link)];
				if (resourceInfo) {
					//Check if the current link's project URI corresponds to the project group being selected
					if (key == resourceInfo.projectURI) {
						//Get the target link as it is used as the key for the row.					
						var resourceURIToCompare = self._getResourceURIToCompare(item.link);
						var rowKey = (item.link.getSubject() == resourceURIToCompare) ? item.link.getResourceURI() + "/object" : item.link.getResourceURI() + "/subject";
					
						rowsInGroup.push({
							key: rowKey,
							rowtype: RESOURCE_ROW
						});
					}
				}			
			});
			this._visibleLinkCount += rowsInGroup.length;
			return rowsInGroup;
		},
		
		_getLinksForLastModifiedDateKey: function(key, links){
			// Return the links that match the given last modified group key
			var group = [];
			var self = this;
			dojo.forEach(links, function(link){
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(link)];	
				if (resourceInfo) {			
					//Check if the last modified date for the current resource corresponds to key being selected
					if (key == DateTimeHelper.getDateRange(new Date(resourceInfo.lastModifiedDate))) {
						//Get the target link as it is used as the key for the row.
						group.push(link);
					}
				}
			});
			return group;
		},
							
		getLinkTypeRowsFromLastModifiedDateKey: function(groupingId, key, sortFunction){
			var rowsInGroup = [];
			var self = this;
			if (sortFunction) {
				sortFunction.call(this, this._arrayListOfResources);
			}
			dojo.forEach(this._arrayListOfResources, function(item){
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(item.link)];	
				if (resourceInfo) {			
					//Check if the last modified date for the current resource corresponds to key being selected
					if (key == DateTimeHelper.getDateRange(new Date(resourceInfo.lastModifiedDate))) {
						//Get the target link as it is used as the key for the row.
						var resourceURIToCompare = self._getResourceURIToCompare(item.link);
						var rowKey = (item.link.getSubject() == resourceURIToCompare) ? item.link.getResourceURI() + "/object" : item.link.getResourceURI() + "/subject";
						rowsInGroup.push({
							key: rowKey,
							rowtype: RESOURCE_ROW
						});
					}
				}
			});
			this._visibleLinkCount += rowsInGroup.length;
			return rowsInGroup;
		},
							
		_isLinkOfType: function(linkObject, /*Unique URI for link type */ linkTypekey){
			var self = this;
			var resourceURIToCompare = self._getResourceURIToCompare(linkObject);
			if ((linkTypekey.indexOf(self._getUniqueUriForLinkType(self._linkTypes[linkObject.getLinkType()]) + "/subject") == 0 ||
			linkTypekey.indexOf(self._getUniqueUriForLinkType(self._linkTypes[linkObject.getLinkType()]) + "/bidirectional") == 0) &&
			linkObject.getObject() == resourceURIToCompare) {
				return true;			
			}
			else 
				if ((linkTypekey.indexOf(self._getUniqueUriForLinkType(self._linkTypes[linkObject.getLinkType()]) + "/object") == 0 ||
				linkTypekey.indexOf(self._getUniqueUriForLinkType(self._linkTypes[linkObject.getLinkType()]) + "/bidirectional") == 0) &&
				linkObject.getSubject() == resourceURIToCompare) {
					return true;
				}
			
			return false;
		},
		
		_setupGridModelData: function(){
			var self = this;
			// Define the columns
			var name_col = {
				name: "",
				width: 250,
				isVisible: true,
				isAlwaysVisible: true,
				recordFunction: function(record){
					return self._createResourceLink(record);
				},
				cellClasses: ["result-set-column-cell", "inactive-text"],
				sortFunction: null,
				navModeActionFunction: dojo.hitch(this, function(rowKey, columnId, cellNode){
					this._onItemSelectedByKeyPress(cellNode);
				})
			};
			
			var onSelectionChange = function(selection) {
				self._onSelectionChange(selection);
			};			
			var onMouseOverRow = function(row) {
				self._onMouseOverRow(row);
			};			
			var onMouseOutRow = function(row) {
				self._onMouseOutRow(row);
			};
		
			// Define the row type
			this.grid.configureGrid({
				hideHeader: true,
				onSelectionChange: onSelectionChange,
				onMouseOverRow: onMouseOverRow,
				onMouseOutRow: onMouseOutRow
			});
			
			this.grid.addRowType(RESOURCE_ROW, {
				isSelectable: true,
				hasMenu: true,
				isPrimary: true,
				allowsMultiSelect: true,
				rowDataFunction: function(record){
					return record;
				},
				menuLayout: this.MENU_LAYOUT,
				operations: this.MENU_OPERATIONS,
				specialColumnPositions: {
					before: [com.ibm.rdm.web.grid.GridModel.COL_EXPANDO, com.ibm.rdm.web.grid.GridModel.COL_SELECTOR],
					after: [com.ibm.rdm.web.grid.GridModel.COL_MENU, com.ibm.rdm.web.grid.GridModel.COL_ACTION]
				}
			});
			
			// Add the necessary columns
			if (this.artifactSuspicionModel) {				
				var suspicion_col = {
					name: "",
					width: 16,
					isVisible: true,
					isAlwaysVisible: false,
					isFixedWidth: true,
					recordFunction: function(record){
						return record;
					},
					deferredContentFunction: dojo.hitch(this, function(functionToCall, record, key, containerNode) {
						this.artifactSuspicionModel.getModelObject().then(
							dojo.hitch(this, function(modelObject) {
								var entries = [];
								var suspect = false;
								var sourceUri = this._getInfoForResourceLink(record).resourceUri;
								var suspectEntries = modelObject.suspicionEntries;
								if (suspectEntries) {
									for (var profileUri in suspectEntries) {
										dojo.forEach(modelObject.suspicionProfileURIs, function(suspicionProfileUri) {
											if (suspicionProfileUri === profileUri) {
												entries = entries.concat(suspectEntries[profileUri]);
												return false;
											}
										});
									}
									dojo.forEach(entries, function(suspectEntry) {
										if (suspectEntry.sourceUri === sourceUri ||
												(suspectEntry.sourceUri === this.resourceURI && suspectEntry.targetUri === sourceUri)) {
											if (!this._suspicionEntriesMap[sourceUri]) {
												this._suspicionEntriesMap[sourceUri] = {};
											}
											if (!this._suspicionEntriesMap[sourceUri][suspectEntry.profileUri]) {
												this._suspicionEntriesMap[sourceUri][suspectEntry.profileUri] = [];
											}
											var entryUris = dojo.map(this._suspicionEntriesMap[sourceUri][suspectEntry.profileUri], function(entry) {
												return entry.uri;
											});
											if (dojo.indexOf(entryUris, suspectEntry.uri) == -1) {
												this._suspicionEntriesMap[sourceUri][suspectEntry.profileUri].push(suspectEntry);
											}
											suspect = true;
										}
									}, this);
									
									if (suspect) {
										this._suspicionStatusWidget = new com.ibm.rdm.web.suspicion.SuspicionStatus({
											source: SuspectLinksUtil.EVENT_SOURCE.linksSidebar,
											projectURI: this.viewer.getProjectURI(),
											suspectEntries: this._suspicionEntriesMap,
											targetResourceURI: sourceUri,
											artifactSuspicionModel: this.artifactSuspicionModel
										});
										functionToCall(this._suspicionStatusWidget.domNode);
									}
									else {
										functionToCall("");
									}
								}
								else {
									functionToCall("");
								}
							})
						);
					}),
					cellClasses: ["result-set-column-cell", "inactive-text"],
					sortFunction: null
				};
				this.grid.addColumn(RESOURCE_ROW, "suspicion", suspicion_col);
			}
			this.grid.addColumn(RESOURCE_ROW, "name", name_col);
		},
		
		_onSelectionChange: function (selection) {
			this._gridSelection = selection;
			if (selection.length == 0) {
				// nothing should be selected in the viewer
			} else {
				// get the selected link object
				for (var i = 0; i < selection.length; i++) {
					var row = selection[i];
					var link = this.getLinkFromRecordId(row.uri);
					// see if the link is a source to target link (i.e., an outgoing link) 
					if (link != null) {
						// outgoing link, see if a sourceElement id is specified
						var resourceURIToCompare = this._getResourceURIToCompare(link);
						var isSubject = link.getSubject() == resourceURIToCompare;
						if (isSubject) {
							if (link._subjectLocationId) {
								// an element within the artifact is the source of the link
								this.publish(EventTopics.LINK_SELECTED_TOPIC, [link._subjectLocationId, link._subject, this._fromElementSelect]);
							} else {
								// artifact is the source of the link
								this.publish(EventTopics.LINK_SELECTED_TOPIC, ["", link._subject, this._fromElementSelect]);
							}
						} else {
							if (link._objectLocationId) {
								// an element within the artifact is the target of the link
								this.publish(EventTopics.LINK_SELECTED_TOPIC, [link._objectLocationId, link._object, this._fromElementSelect]);
							} else {
								// incoming artifact link
								this.publish(EventTopics.LINK_SELECTED_TOPIC, ["", link._object, this._fromElementSelect]);
							}
						}
						// 69786
						break;
					}
				}
			}
		},
		
		_onMouseOutRow: function(row) {
			if (row.groupingId) {
				return;
			}
			var link = this.getLinkFromRecordId(row);
			// see if the link is a source to target link (i.e., an outgoing link) 
			if (link != null) {
				var resourceURIToCompare = this._getResourceURIToCompare(link);
				// outgoing link, see if a sourceElement id is specified
				var isSubject = link.getSubject() == resourceURIToCompare;
				if (isSubject) {
					if (link._subjectLocationId) {
						// an element within the artifact is the source of the link
						this.publish(EventTopics.LINK_HOVER_TOPIC, [link._subjectLocationId, false]);
					} else {
						// artifact is the source of the link
						this.publish(EventTopics.LINK_HOVER_TOPIC, ["", false]);
					}
				} else {
					if (link._objectLocationId) {
						// an element within the artifact is the target of the link
						this.publish(EventTopics.LINK_HOVER_TOPIC, [link._objectLocationId, false]);
					} else {
						// incoming link
						this.publish(EventTopics.LINK_HOVER_TOPIC, ["", false]);
					}
				}
			} 
		},

		_onMouseOverRow: function(row) {
			if (row.groupingId) {
				return;
			}
			var link = this.getLinkFromRecordId(row);
			// see if the link is a source to target link (i.e., an outgoing link) 
			if (link != null) {
				var resourceURIToCompare = this._getResourceURIToCompare(link);
				// outgoing link, see if a sourceElement id is specified
				var isSubject = link.getSubject() == resourceURIToCompare;
				if (isSubject) {
					if (link._subjectLocationId) {
						// an element within the artifact is the source of the link
						this.publish(EventTopics.LINK_HOVER_TOPIC, [link._subjectLocationId, true]);
					} else {
						// artifact is the source of the link
						this.publish(EventTopics.LINK_HOVER_TOPIC, ["", true]);
					}
				} else {
					if (link._objectLocationId) {
						// an element within the artifact is the target of the link
						this.publish(EventTopics.LINK_HOVER_TOPIC, [link._objectLocationId, true]);
					} else {
						// incoming artifact link
						this.publish(EventTopics.LINK_HOVER_TOPIC, ["", true]);
					}
				}
			} 
		},
		
		_onItemSelectedByKeyPress: function(cellNode) {
			var linkItemWidget = dijit.byNode(dojo.query("tr", cellNode)[0]);
			if (linkItemWidget && linkItemWidget.handleLinkSelectedByKeyPress){
				linkItemWidget.handleLinkSelectedByKeyPress();
			}
		},
		
		subscribe: function(topic, method) {
			var topic = EventTopics.getTopic(topic, this.resourceURI);
			this._subs.push(
					dojo.subscribe(
						topic, 
						this, 
						method));
		},
		
		publish: function(topic, params) {
			var topic = EventTopics.getTopic(topic, this.resourceURI);
			//console.debug("CommentingWidget: fire event -> " + topic);
			dojo.publish(
				topic, 
				params);
		},
		
		_getLinksFromRowIds: function(rowIds) {
			var deleteLinks = [];
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						deleteLinks.push(item.link);
					}
				});
			}
			return deleteLinks;
		},
		
		_executeRemove: function(rowIds){
			// Prompt the user about deleting links that were created from artifact content.
			var deleteLinks = this._getLinksFromRowIds(rowIds);
			var contentLinks = [];
			for (var i=0; i<deleteLinks.length; i++) {
				var link = deleteLinks[i];
				if ((link._subjectLocationId && link._subjectLocationId != "") || (link._objectLocationId && link._objectLocationId != "")) {
					contentLinks.push(link);
				}
			}
			var self = this;
			var deleteLink = function(){
				self._sendLinksForDeletion(rowIds);
			};
			var message = null;
			var title = null;
			if (contentLinks.length > 0) {
				if (deleteLinks.length == 1) {
					var link = deleteLinks[0];
					title = this._messages.confirmDeleteTitle;
					var resourceURIToCompare = this._getResourceURIToCompare(link);
					// Be helpful in the only one selected case.
					var isSubject = link.getSubject() === resourceURIToCompare;
					if (isSubject && (link._subjectLocationId !== "")) {
						var artifactURI = link.getObject();
						var artifact = this._uriToResourceInfos[artifactURI];
						if (!artifact) {
							// For external web links that reference content, artifact will be undefined
							message = this._messages.confirmDeleteQuestionGenericOneWithEmbedded;
						} else {
							message = dojo.string.substitute(this._messages.confirmDeleteOutgoingLinkQuestion, [artifact.displayName.italics()]);
						}
					} else {
						var artifactURI = link.getSubject();
						var artifact = this._uriToResourceInfos[artifactURI];
						message = dojo.string.substitute(this._messages.confirmDeleteIncomingLinkQuestion, [artifact.displayName.italics()]);
					} 
				} else if (deleteLinks.length === contentLinks.length) {
					title = this._messages.confirmDeleteSomeTitles;
					message = this._messages.confirmContentDeleteQuestionSome;
				} else {
					title = this._messages.confirmDeleteSomeTitles;
					message = this._messages.confirmContentDeleteQuestionSome;
				}
				this.msgForm = new com.ibm.rdm.web.common.MessageForm({
					confirmContent: message,
					dialogTitle: title,
					dialogStyle: com.ibm.rdm.web.common.MessageForm.DIALOG_TYPES.YesNo,
					dialogWidth: "400px",
					onOk: deleteLink
				});
				this.msgForm.show();
			} else {
				if (deleteLinks.length == 1) {
					var link = deleteLinks[0];
					var resourceURIToCompare = this._getResourceURIToCompare(link);
					// Be helpful in the only one selected case.
					var isSubject = link.getSubject() === resourceURIToCompare;
					var artifactURI = null;
					if(isSubject){
						artifactURI = link.getObject();
					}
					else{
						artifactURI = link.getSubject();
					}
					var artifact = this._uriToResourceInfos[artifactURI];
					title = this._messages.confirmDeleteTitle;
					//If artifact does not exist, use a generic message
					if(artifact === undefined){
						message = this._messages.confirmDeleteQuestionGenericOne;
					} 
					//else use the artifact name to personalize the message
					else {
						message = dojo.string.substitute(this._messages.confirmDeleteQuestionOne, [artifact.displayName.italics()]);
					}
				} else {
					title = this._messages.confirmDeleteSomeTitles;
					message = this._messages.confirmDeleteQuestionMany;
				}
				this.msgForm = new com.ibm.rdm.web.common.MessageForm({
					confirmContent: message,
					dialogTitle: title,
					dialogStyle: com.ibm.rdm.web.common.MessageForm.DIALOG_TYPES.YesNo,
					dialogWidth: "250px",
					onOk: deleteLink
				});
				this.msgForm.show();
			}
		},
		
		_executeEditLinkLabel: function(rowIds){
			var self = this;
			var selectedResource = [];
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						selectedResource.push(item);
					}
				});
			}
			
			var linkToEdit = selectedResource[0];
			
			var modifyExternalLink = function(newLink){
			
				var saveDeferred = LinkUtils.saveLink(self.viewer.getProjectURI(), newLink.link);
				saveDeferred.addCallback(self, function(response){
				
					if (newLink.wasTypeChanged) {
						//A change of link Type was made. In this case, refresh the entire grid since a new type grouping may be necessary
						self.handleRefresh();
					}
					else {
						//Only the label or the URL for the external link changed. but not the type. Refresh the row
						var linkToRefreshKey = newLink.link.getResourceURI();
						var resourceURIToCompare = self._getResourceURIToCompare(newLink.link);
						if (newLink.link.getSubject() == resourceURIToCompare) {
							linkToRefreshKey += "/object";
						}
						else {
							linkToRefreshKey += "/subject";
						}
						var refresh = [];
						refresh.push({
							key: linkToRefreshKey,
							rowtype: RESOURCE_ROW
						});
						self.grid.refreshRows(refresh);
					}
				});
				saveDeferred.addErrback(self, function(error){
					//Error updating link
					MessageHandler.showError(7255, [], error);
				});
				
			};
			var modifyResourceLink = function(newLink){
			
				var saveDeferred = LinkUtils.saveLink(self.viewer.getProjectURI(), newLink);
				saveDeferred.addCallback(self, function(response){
					self.handleRefresh();
				});
				saveDeferred.addErrback(self, function(error){
					//Error updating link
					MessageHandler.showError(7255, [], error);
					
				});
				
			};
			
			var linkVar = this._getTargetURIFromLink(linkToEdit.link);
			
			var editLinkForm = linkutils.createEditLinkForm({
				processAreaURI:	this.viewer.getResourceObject().getProcessAreaURI(),
				projectURI: this.viewer.getProjectURI(),
				resourceURI: this.viewer.resourceURI,
				linkExternalURI: this._getTargetURIFromLink(linkToEdit.link),
				resourceInfo: this._viewerResourceURIInfo,
				otherResourceInfo: this._uriToResourceInfos[this._getTargetURIFromLink(linkToEdit.link)],
				linkTypes: this._linkTypes,
				linkToEdit: linkToEdit.link,
				onModifyExternalLink: modifyExternalLink,
				onModifyResourceLink: modifyResourceLink
			});
			
			this._toDestroy.push(editLinkForm);
		},
		
		_sendLinksForDeletion: function(rowIds){
			var self = this;
			// Find the artifact to remove and store them in an array
			this._resourcesMarkedForDeletion = [];
			
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						self._resourcesMarkedForDeletion.push(item);
					}
				});
			}
			
			// Now remove the links from the server
			var deferredArray = [];
			dojo.forEach(this._resourcesMarkedForDeletion, function(item){
				var deferred = null;
				
				if (!linkutils.sameLocation(self._getTargetURIFromLink(item.link))) {
				
					var linkType = self._linkTypes[item.link.getLinkType()];
					if (linkType && linkType.getSameAs && (LinkTypeUtils.isCalmLink(linkType.getSameAs()))) {
						//Link is a CALM link 
						StatusDisplay.showStatusMessage(self._messages.deletingLinks);
						deferred = self._deleteCalmLink(item.link);
						deferred.addBoth(self, function() {
							StatusDisplay.hideStatusMessage();
						});
						//Remove the external link from our list of external links, if it exists.
						var index = dojo.indexOf(self._externalLinks, item.link);
						self._externalLinks.splice(index, 1);					
					}else if(LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs())){
						
						if(item.link.isDiscoveredLink){
							//Check if this is an RM-RM interop link type. If so, use the DerivedLinkUtils
							//class to properly remove the link
							StatusDisplay.showStatusMessage(self._messages.deletingLinks);
							deferred = DerivedLinkUtils.deleteLink(self.viewer.getProjectURI(), item.link).then(dojo.hitch(self, function(){
								StatusDisplay.hideStatusMessage();
								//At this point send a notification to refresh the page
								ChangeNotifier.notifyListeners({
									eventTopic: com.ibm.rdm.web.notifications.EventTopics.DM_LINK_ACTIONS
								});	
								
								//Remove the external link from our list of external links, if it exists.
								var index = dojo.indexOf(self._externalLinks, item.link);
								self._externalLinks.splice(index, 1);									
							}), function (error){
								//Display error deleting link		
								MessageHandler.showError(7256, [], error);	
							});
						}else{
							//The local server owns this link. Delete via _deleteLocalLink
							deferred = self._deleteLocalLink(item.link);
						}
					}
					else {
						deferred = self._deleteLocalLink(item.link);
					}
				}
				else {
					deferred = self._deleteLocalLink(item.link);
				}				
			});			
		},
		
		_deleteLocalLink: function(link){
			//Special case, handle deletion of DM links.
			var linkType = this._linkTypes[link.getLinkType()];
			if(linkType.getSameAs() === LinkTypeUtils.SYSTEM_LINK_TYPE_DERIVES){
				var DMDeleteLinkDeferrred = new dojo.Deferred();
				var DMDeletion = DerivedLinkUtils.deleteLink(this.viewer.getProjectURI(), link);
				StatusDisplay.showStatusMessage(this._messages.deletingLinks);
				DMDeletion.addCallback(this,function(response){
					StatusDisplay.hideStatusMessage();
					//At this point send a notification to refresh the page
					ChangeNotifier.notifyListeners({
					 eventTopic: com.ibm.rdm.web.notifications.EventTopics.DM_LINK_ACTIONS
					});
					DMDeletion.callback(response);
				});
				return DMDeleteLinkDeferrred;
			} else{
				//direction is needed to refresh the grid (to update the cache)
				var direction = null;
				if(linkType.isBidirectional()){
					direction = ArtifactConstants.BIDIRECTIONAL_LINK;
				} else {
					var thisResourceURI = this.viewer.getResourceObject().resourceURI;
					if(link.getObject() === thisResourceURI){
						direction = ArtifactConstants.OBJECT_LINK;
					}else if(link.getSubject() === thisResourceURI){
						direction = ArtifactConstants.SUBJECT_LINK;
					}
				}
				return LinkUtils.deleteLink({link: link, direction: direction});
			}
		},
		
		_deleteCalmLink: function(link){
			return CalmLinkUtils.deleteLink(this.viewer.getProjectURI(), link);
		},
		
		removeText: function(num){
			return num == 1 ? this._messages.deleteLink : dojo.string.substitute(this._messages.deleteLinks, [num]);
		},
		
		isRemoveEnabled: function(rowIds, count){
			// summary:
			//		Method to test for whether or not to enable the remove menu option.  Return false so we will execute 
			//		the getRemoveEnablementDeferred method to check permissions.
			// returns:
			//		false
			return false;
		},
		
		getRemoveEnablementDeferred: function(rowIds, count) {
			// summary:
			//		Method to test for whether or not to enable the remove menu option.  Returns a deferred since enablement
			//		depends on permissions check.  Permission depends on the particular link selected and the check will be
			//		performed when the drop down menu is invoked.
			// returns:
			//		Deferred
			var deferred = new dojo.Deferred();
			var isAllowed = true;
			isAllowed = this._isRemoveEnabled(rowIds, count);
			if (!isAllowed) {
				deferred.callback(false);
			} else if (count > 1) {
				// no permission check if more than one item selected
				deferred.callback(true);
			} else {
				var resource = null;
				for (var key in rowIds) {
					dojo.forEach(this._arrayListOfResources, function(item){
						if (key.indexOf(item.id) == 0) {
							resource = item;
						}
					});
				}
				var link = resource.link;
				var linkType = this._linkTypes[link.getLinkType()];
				
				var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
				var isCalmLink = LinkTypeUtils.isCalmLink(linkType.getSameAs());
				var isRMtoRmLinkType = LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs());
				
				if(isCalmLink || isDMLinkType || isRMtoRmLinkType){
					// no permission check for CLM links
					deferred.callback(true);
				} else {
					// do permission check
					var processAreaURI = this.viewer.getResourceObject().getProcessAreaURI();
					var permissionDeferred = ProcessOperationsCache.isDeleteLinkAllowed(this.projectURI, linkType.getActionId(), processAreaURI);
					// check source resource first since we have all the info we need to do the check
					permissionDeferred.addCallback(this, function(isAllowed) {
						if (!isAllowed) {
							deferred.callback(isAllowed);
						} else {
							// check other end of the link
							var permissionOther = this._checkPermissionForReferencedResource(link, "remove");
							permissionOther.addCallback(this, function(otherAllowed) {
								deferred.callback(otherAllowed);
								return otherAllowed;
							});
						}
						return isAllowed;
					});
				}
			}
			return deferred;
		},
		
		_isRemoveEnabled: function(rowIds, count){
			var isDeleteAllowed = false;
			var projectOperations;
			
			//Check for CLM Link, RM2RM links have aprojectURI, so this needs to be done first
			var resource = null;
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						resource = item;
					}
				});
			}
			var link = resource.link;
			var linkType = this._linkTypes[link.getLinkType()];
			
			var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
			var isCalmLink = LinkTypeUtils.isCalmLink(linkType.getSameAs());
			var isRMtoRmLinkType = LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs());
			
			if(isCalmLink || isDMLinkType || isRMtoRmLinkType){
				return this.isRemoveEnabledForCLM(rowIds, count);
			}
			
			if (this.projectURI) {
				var projectOperations = this._projectOperations;
				isDeleteAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_RESOURCE) && !this.baselineURI &&
				(!this.viewer.isInHistoryMode());
			}
			
			//Do not allow editing any intrinsic links
			var isRemoveEnabledForIntrinsicLinks = this.isRemoveEnabledForIntrinsicLinks(rowIds, count);
			if(isRemoveEnabledForIntrinsicLinks){
				return false;
			}
	
			return isDeleteAllowed;
		},
		
		isRemoveEnabledForIntrinsicLinks: function(rowIds, count){
			var intrinsicLinks = [];
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						var lnk = item.link;
						if (lnk.isBaseArtifactLink) {
							intrinsicLinks.push(lnk);
						}
					}
				});
			}
			return (intrinsicLinks.length > 0);
		},
		
		
		isRemoveEnabledForCLM: function(rowIds, count){
			var self = this;
			var clmLinks = [];
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						var lnk = item.link;
						if (lnk) {
							var linkType = self._linkTypes[lnk.getLinkType()];
							//Check to see if lnk is a CLM link type (DM, RM2RM, CALM)
							if (linkType && linkType.getSameAs && LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
								clmLinks.push(item);
							} else if(linkType && linkType.getSameAs && LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs())) {
								clmLinks.push(item);
							} else if(linkType && linkType.getSameAs && LinkTypeUtils.isDMLinkType(linkType.getSameAs())) {
								clmLinks.push(item);
							}
						}
					}
				});
			}
			if (clmLinks.length != count) {
				// mixed links; not all of them are CLM; deny removal
				return false;
			}
			if (this.projectURI) {
				var projectOperations = this._projectOperations;
				return projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_CLM_LINK) && !this.baselineURI && 
				(!this.viewer.isInHistoryMode());
			}
			return false;
		},
		
		isClearSuspicionEnabled: function(rowIds, count) {
			return false;
		},
		
		isClearSuspicionVisible: function(rowIds, count) {
			var visible = false;
			var link, resourceUri;
			for (var key in rowIds) {
				dojo.some(this._arrayListOfResources, function(item) {
					if (key.indexOf(item.id) == 0) {
						link = item.link;
						resourceUri = this._getTargetURIFromLink(link);
						
						if (this._suspicionEntriesMap[resourceUri]) {
							visible = true;
							return false;
						}
					}
				}, this);
			}
			
			return visible;
		},
		
		getClearSuspicionEnablementDeferred: function(rowIds, count) {
			var clearSuspicionEnabledDfd = new dojo.Deferred();
			
			var isAllowed = true;
			isAllowed = this._isClearSuspicionEnabled(rowIds, count);
			if (!isAllowed) {
				clearSuspicionEnabledDfd.callback(false);
			} else if (count > 1) {
				// no permission check if more than one link selected
				clearSuspicionEnabledDfd.callback(true);
			} else {
				var resource = null;
				for (var key in rowIds) {
					dojo.forEach(this._arrayListOfResources, function(item){
						if (key.indexOf(item.id) == 0) {
							resource = item;
						}
					});
				}
				var link = resource.link;
				var linkType = this._linkTypes[link.getLinkType()];
				if (LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
					// no permission check for CLM links
					clearSuspicionEnabledDfd.callback(true);
				} else {
					// do permission check
					var processAreaURI = this.viewer.getResourceObject().getProcessAreaURI();
					var permissionDeferred = ProcessOperationsCache.isClearSuspectLinkAllowed(this.projectURI, linkType.getActionId(), processAreaURI);
					// check source resource first since we have all the info we need to do the check
					permissionDeferred.addCallback(this, function(isAllowed) {
						clearSuspicionEnabledDfd.callback(isAllowed);
						return isAllowed;
					});
				}
			}
			
			return clearSuspicionEnabledDfd;
		},
		
		_isClearSuspicionEnabled: function(rowIds, count){
			var isEditAllowed = false;
			var projectOperations;
			if (this.projectURI) {
				var projectOperations = this._projectOperations;
				isEditAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.CLEAR_SUSPICION) && !this.baselineURI && 
				(!this.viewer.isInHistoryMode());
			}
			
			return isEditAllowed;
		},
		
		_executeClearSuspicion: function(rowIds) {
			var link, resourceUri, selectedResource;
			var entryUris = [];
			var profileUris = [];
			var suspicionEntries = [];
			for (var key in rowIds) {
				dojo.forEach(this._arrayListOfResources, function(item){
					if (key.indexOf(item.id) == 0) {
						selectedResource = item;
						link = selectedResource.link;
						resourceUri = this._getTargetURIFromLink(link);
						suspicionEntries.push(this._suspicionEntriesMap[resourceUri]);
					}
				}, this);
			}
			
			if (suspicionEntries) {
				var suspicionProfileUris = SuspectLinksUtil.getSuspicionProfileURIsPreference(this.projectURI);
				var clearSuspicionForm = new com.ibm.rdm.web.suspicion.ClearSuspicionForm({
					projectUri: this.projectURI,
					checkSource: true,
					suspicionEntries: suspicionEntries,
					suspicionProfileUris: suspicionProfileUris
				});

				var entryUris = clearSuspicionForm.getEntryUris();
				var msgForm = new com.ibm.rdm.web.common.MessageForm({
					confirmContent: clearSuspicionForm.domNode,
					dialogTitle: this._messages.confirmClearSuspicion,
					dialogWidth: "350px",
					dialogStyle: "okCancel",
					okButtonText: this._messages.clear,
					onDialogOpen: dojo.hitch(this, function() {
						dojo.fadeIn({node: clearSuspicionForm.domNode, duration: 400}).play();
					}),
					onOk: dojo.hitch(this, function() {
						this.artifactSuspicionModel.clearSuspicion(entryUris,
								SuspectLinksUtil.EVENT_SOURCE.linksSidebar, this.projectURI);
					})
				});
				msgForm.show();
				this._toDestroy.push(clearSuspicionForm);
				this._toDestroy.push(msgForm);
			}
		},
		
		_checkPermissionForReferencedResource: function (link, permissionType) {
			// summary:
			//		Method to test for whether or not to a menu action is permissible for the resource referenced in 
			//		the given link.  
			// returns:
			//		Deferred
			var deferred = new dojo.Deferred();
			var object = link.getObject();
			var subject = link.getSubject();
			var toCheck = object;
			// Get the referenced resource URI
			if (this.viewer.getResourceObject().resourceURI === object) {
				toCheck = subject;
			} 

			if (!linkutils.sameLocation(toCheck)) {
				// external url, no permission check necessary
				deferred.callback(true);
			} else {
				var resolvedTarget = ResourceLinksHandler.getArtifactUriFromExternalLink(toCheck);
				if (resolvedTarget) {
					toCheck = resolvedTarget;
				}
				if (!ResourceLinksHandler.isInternalResourceLink(toCheck)) {
					// non-resource url, so in essence an external url
					deferred.callback(true);
				} else {
					// we must do a describe to get the processAreaURI for the referenced resource and then do the
					// permission check
					var descriptionDeferred = ResourceManager.getResourceDescribe({
						resourceURI: toCheck
					});
					descriptionDeferred.addCallback(this, function(descriptionResponse){
						var linkType = this._linkTypes[link.getLinkType()];
						if (descriptionResponse.projectURI !== this.projectURI) {
							if (!linkType.isSystemDefined()) {
								// permissions can only be enforced when cross-project linking if the link type is system defined.
								deferred.callback(true);
							}
						}
						var processAreaURI = descriptionResponse.processArea;
						var permissionDeferred = null;
						if (permissionType === "edit") {
							permissionDeferred = ProcessOperationsCache.isModifyLinkTypeAllowed(this.projectURI, linkType.getActionId(), processAreaURI);
						} else {
							permissionDeferred = ProcessOperationsCache.isDeleteLinkAllowed(this.projectURI, linkType.getActionId(), processAreaURI);
						}		
						permissionDeferred.addCallback(this, function(isAllowed) {
							deferred.callback(isAllowed);
							return isAllowed;
						});
						return descriptionResponse;
					});					
					descriptionDeferred.addErrback(this, function(response) {
						var status = XhrUtils.getResponseCode(response);
						
						if (status === 404) {
							if (permissionType === "remove") {
								// The referenced resource no longer exists, but its link unfortunately does.
								// Allow the remove option to delete the broken link.
								deferred.callback(true);									
							}
							else {
								// Don't allow editting on a resource that no longer exists.
								deferred.callback(false);									
							}							
						}
						else if (status === 403) {
							// The referenced resource is not allowed to be accessed for edits or removals.
							deferred.callback(false);							
						}							
					});			
				}
			}
			return deferred;
		},
		
		getEditEnablementDeferred: function(rowIds, count) {
			// summary:
			//		Method to test for whether or not to enable the edit menu option.  Returns a deferred since enablement
			//		depends on permissions check.  Permission depends on the particular link selected and the check will be
			//		performed when the drop down menu is invoked.
			// returns:
			//		Deferred
			var deferred = new dojo.Deferred();
			var isAllowed = true;
			isAllowed = this._isEditLinkLabelEnabled(rowIds, count);
			if (!isAllowed) {
				deferred.callback(false);
			} else if (count > 1) {
				// no permission check if more than one link selected
				deferred.callback(true);
			} else {
				var resource = null;
				for (var key in rowIds) {
					dojo.forEach(this._arrayListOfResources, function(item){
						if (key.indexOf(item.id) == 0) {
							resource = item;
						}
					});
				}
				var link = resource.link;
				var linkType = this._linkTypes[link.getLinkType()];
				if (LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
					// no permission check for CLM links
					deferred.callback(true);
				} else {
					// do permission check
					var processAreaURI = this.viewer.getResourceObject().getProcessAreaURI();
					var permissionDeferred = ProcessOperationsCache.isModifyLinkTypeAllowed(this.projectURI, linkType.getActionId(), processAreaURI);
					// check source resource first since we have all the info we need to do the check
					permissionDeferred.addCallback(this, function(isAllowed) {
						if (!isAllowed) {
							deferred.callback(isAllowed);
						} else {
							// check other end of the link, only if not an external URL
							var permissionOther = this._checkPermissionForReferencedResource(link, "edit");
							permissionOther.addCallback(this, function(otherAllowed) {
								deferred.callback(otherAllowed);
								return otherAllowed;
							});
						}
						return isAllowed;
					});
				}
			}
			return deferred;
		},
		
		isEditLinkLabelEnabled: function(rowIds, count){
			// summary:
			//		Method to test for whether or not to enable the edit menu option.  Return false so we will execute 
			//		the getEditEnablementDeferred method to check permissions.
			// returns:
			//		false
			return false;
		},

		_isEditLinkLabelEnabled: function(rowIds, count){
			//Edit is allowed one link at a time. Check the count or the number of links selected.
			if(count > 1){
				return false;
			}
			var isEditAllowed = false;
			var projectOperations;
			if (this.projectURI) {
				var projectOperations = this._projectOperations;
				isEditAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_RESOURCE) && !this.baselineURI && 
				(!this.viewer.isInHistoryMode());
			}
			//Check if the user has permissions to edit links.
			if (isEditAllowed) {
			
				var resourceToEdit = [];
				//Check if the resource can be edited
				for (var key in rowIds) {
					dojo.forEach(this._arrayListOfResources, function(item){
						if (key.indexOf(item.id) == 0) {
							resourceToEdit.push(item);
						}
					});
				}
				var link = resourceToEdit[0].link;
				var linkType = this._linkTypes[link.getLinkType()];
				
				if (LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
					//Do not allow to edit CALM links
					return false;
				}
				
				if(LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs()) || LinkTypeUtils.isRMtoDMLinkType(linkType.getSameAs())){
					//Do not allow to edit References or Derives links
					return false;				
				}
				
				//Prevent editing links in core resource
				if(link.isBaseArtifactLink){
					return false;
				}
				
				//Allow other links to be editable
				return true;
				
			}
			else {
				return isEditAllowed;
			}
		},
		
		getLinkFromRecordId: function(record){
			var link = null;
			var rowURI = record.substring(0, record.lastIndexOf("/"));
			for (var i=0; i<this._arrayListOfResources.length; i++) {
				var item = this._arrayListOfResources[i];
				if (item.id == rowURI) {
					link = item.link;
					break;
				}
			}
			return link;
		},
		
		doNavigateSelection: function() {
			// summary:
			//		Flag to indicate whether or not the LinksListWidget supports selection when resource is navigated to.  Introduced to
			//		deal with RelatedResourceLinksListWidget, which does not support this.
			return true;
		},
		
		_isForbidden: function(uriToCheck) {			
			var status = this._uriToResourceStatus[uriToCheck];
			return (status === ResourceManager.FORBIDDEN) ? true : false;	
		},
		
		_createResourceLink: function(record){
			var resourceLinkInfo = this._getInfoForResourceLink(record);
			var uri = resourceLinkInfo.resourceUri;
			var link = resourceLinkInfo.link;
			var linkType = resourceLinkInfo.linkType;
			var resource = this._uriToResourceInfos[uri];
			var moduleHostURI = this.modulefactToModuleHostURIMap[uri];
			var row = null;
			if (resource == null) {
				// External link
				var label = link.getRationale();
				//If label is not defined or empty, use the title of the link
				if (!label || label == "") {
					//Use the title of the link
					label = link.getTitle();
					//If label is still not defined or empty, use the uri
					if (!label || label == "") {
						label = uri;
					}
				}

				row = new com.ibm.rdm.web.links.LinkItemWidget({
					"uri": uri,
					"linkType": linkType,
					"resourceViewer": this.viewer,
					"isViewerInHistoryMode": this.viewer.isInHistoryMode(),
					"label": label,
					"link": link,
					"moduleHostURI": moduleHostURI,
					"savedViewLabel": this._savedViewsNamesMap[link.getSubject()],
					"doNavigateSelection": this.doNavigateSelection(),
					"isForbidden": this._isForbidden(uri)  
				});
				
				this._linkItemWidgets.push(row);
			}
			else {
				var elementDescription = null;
				if (link.getSubjectDescription() != null && link.getSubjectDescription() != "") {
					elementDescription = link.getSubjectDescription();
				} else if (link.getObjectDescription() != null && link.getObjectDescription() != "") {
					elementDescription = link.getObjectDescription();
				}
				row = new com.ibm.rdm.web.links.LinkItemWidget({
					"resource": resource,
					"moduleHostURI": moduleHostURI,
					"linkType": linkType,
					"resourceViewer": this.viewer,
					"isViewerInHistoryMode": this.viewer.isInHistoryMode(),
					"elementDescription": elementDescription,
					"link": link,
					"doNavigateSelection": this.doNavigateSelection()
				});
				
				this._linkItemWidgets.push(row);
			}
			var table = dojo.create("table");
			if (dojo.isIE === 7) {
				// necessary in order for IE7 to properly honor widths specified on table columns
				table.style.tableLayout = "fixed";
			}
			var tbody = dojo.create("tbody");
			table.appendChild(tbody);
			tbody.appendChild(row.domNode);
			var div = dojo.create("div");
			div.appendChild(table);
			return div;
		},
		
		_getInfoForResourceLink: function(record) {
			var uri = null;
			var link = this.getLinkFromRecordId(record);
			var linkType = this._linkTypes[link.getLinkType()];
			var resourceURIToCompare = this._getResourceURIToCompare(link);
			if (linkType.isBidirectional()) {
				uri = link.getObject() == resourceURIToCompare ? link.getSubject() : link.getObject();
			}
			else {
				var isSubject = record.indexOf(link.getResourceURI() + "/subject") == 0;
				uri = isSubject ? link.getSubject() : link.getObject();
			}
			
			return {
				link: link,
				linkType: linkType,
				resourceUri: uri
			};
		},
		
		resize: function(args){
			// summary:
			//		This function is here to receive resize calls that are made when
			//		this widget is used as the sole content of a ContentPane. Please
			//		see ConentPane's 'resize' function for more information.
			if (args && args.w >= 0 && args.h >= 0) {
				var width = args.w;
				var height = args.h;
				if (dojo.isIE) {
					// TODO Is there a better approach for this?
					// Without the following code, the horz scrollbar will always
					// appear when the vert scrollbar is present, even in those
					// [most] cases where it shouldn't ...
					if (this.moduleContentPane) {
						this.moduleContentPane.style.width = Math.max(0, (args.w));
						this.moduleContentPane.style.height = height - 50;
					}
					height = height + 20;
					
					
				}
				
				var contentPaneSize = {
					h: height,
					w: width
				};
				this._contentPane.resize(contentPaneSize);
				this._contentPaneSize = args;
				// groan, this all needs to be fixed properly..
				if (dojo.isIE === 7) {
						args.h -= 30;
				}
				if (this.grid) {
					this.grid.resize(args);
					this.grid.scrollHeaderToMatchContent();
				}
			}
			
		},
				
		_loadLinks: function() {
			var self = this;
			this.setLoadingMessage();
			domutils.createHourGlassCursor();
			
			//[Defect 39729: Links Widget has stale links at times that lead to errors]. Skip the cached link response the first time
			//we invoke the links for a particular resource.
			var preventCachedLinks = (this._linksLoaded) ? false : true;
			
			//Display informational message for RM2RM or DM links in baseline
			this._check4RM2RMOrDMBaselineMsg();
			
			var linkRequestDeferreds = [];
			
			if (this._showBaseLinks()) {
				//Get links for the base resource artifact.. If requested
				linkRequestDeferreds.push(LinkUtils.getLinksForResource({
					sourceAndTargetURI: self.boundArtifact,
					baselineURI: self.baselineURI,
					skipCache: preventCachedLinks
				}));
			}
		  
			//Get links for the current resource
			linkRequestDeferreds.push(LinkUtils.getLinksForResource({
				sourceAndTargetURI: self.resourceURI,
				baselineURI: self.baselineURI,
				skipCache: preventCachedLinks
			}));

			if (this.reviewId) {
				ReviewCache.getReview(this.reviewId).then(
					dojo.hitch(this, function(review){
						var reviewParser = review.getReviewParser();
						// We only want to set the revision date if the review is formal
						// and is in some other state than Draft.  This will result in
						// appropriately filtering out links that have been created AFTER
						// the review's captured start time.
						if (reviewParser.isFormal()
							&& reviewParser.getReviewState() !== reviewTypes.REVIEW_STATE_TYPES.Draft)
						{
							this.setRevisionDate(reviewParser.getStartTimestampDate());
						}
					})
				);
			}
			this._doLinkDeferreds(linkRequestDeferreds, preventCachedLinks);
		},
		
		_doLinkDeferreds: function(linkDeferreds, preventCachedLinks) {
			var doLinkDeferreds = dojo.hitch(this, function(linkDeferreds) {
				//Add the deferreds for object types to a DeferredList	 		
				var linkRequestedDeferredList = new dojo.DeferredList(linkDeferreds);
				linkRequestedDeferredList.addCallback(this, function(allLinksResponse) {
					//This method will call _doLoadLinksPostProcess with a combined list of all the links
					//to be displayed.				
					var links, totalLinks = [];
					var showingBaseArtifactLinks = this._showBaseLinks();
					for (var i = 0, l = allLinksResponse.length; i < l; i++) {
						if (allLinksResponse[i][0]) {
							links = allLinksResponse[i][1].resourceInfos;
							if (allLinksResponse[i][1].isPartialResult) {
								this._check4RM2RMOrDMProjectAssociations();
							}
							for (var j = 0, m = links.length; j < m; j++) {
								// The request for showing base artifact links is
								// the first one in our deferred list
								if (i == 0 && showingBaseArtifactLinks) {
									links[j].isBaseArtifactLink = true;
								}
								totalLinks.push(links[j]);
							}
						}
					}	
					this._doLoadLinksPostProcess(totalLinks);
					return allLinksResponse;				
				});
			});
			
			if (this.viewer.isModuleViewer() && this._showModuleViews()) {
				var queryURI = com.ibm.rdm.web.query.SavedFilterUtils
									.getQueryURI(com.ibm.rdm.web.project.SavedFilter.scope.MODULE_PUBLIC,
													this.baselineURI, this.resourceURI);
				ProjectCache.getSavedFilters(queryURI, this.projectURI, preventCachedLinks).then(
					dojo.hitch(this, function(response) {
						var savedView, materializedViewURI;
						for (var savedViewURI in response) {
							savedView = response[savedViewURI];
							materializedViewURI = com.ibm.rdm.web.query.SavedFilterUtils
													.getMaterializedViewURI(savedView.resourceURI);
							this._savedViewsNamesMap[materializedViewURI] = savedView.title;
							linkDeferreds.push(LinkUtils.getLinksForResource({
								sourceAndTargetURI: materializedViewURI,
								baselineURI: this.baselineURI,
								skipCache: preventCachedLinks
							}));
						}
						doLinkDeferreds(linkDeferreds);
					})
				);
			} else {
				doLinkDeferreds(linkDeferreds);
			}
		},
		
		_showModuleViews: function() {
			// Disabling feature for showing module views as collections for now
			return true;
		},
		
		_doLoadLinksPostProcess: function(links) {
			var self = this;
			
			self._linksLoaded = true;
			self._links = links;
			self._foundLinkTypes = [];
			
			if (links && links.length > 0) {
				var linkTypesDeferred = self._loadLinkTypes(links);
				linkTypesDeferred.addCallback(function(linkTypeInfos){
					for (var i = 0; i < linkTypeInfos.length; i++) {
						var linkType = linkTypeInfos[i];
						self._linkTypes[linkType.getResourceURI()] = linkType;
					}
					
					var linksToSearchFor = 0;
					var queryStr = "resource in [";
					//Remember all the URIs we looked for
					var targetURIs = {};
					for (var i = 0; i < links.length; i++) {
						var link = links[i];
						if (dojo.indexOf(self._foundLinkTypes, link.getLinkType()) == -1) {
							self._foundLinkTypes.push(link.getLinkType());
						}
						var targetURI = self._getTargetURIFromLink(link);
						
						// WI 30724
						// CLM links generate filtering menu items
						// but not the external links 
						if (!linkutils.sameLocation(targetURI)) {
							var linkType = link.getLinkType();
							linkType = self._linkTypes[linkType];
							if (!LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
								self._externalLinks.push(link);
								continue;
							}
						}
						queryStr += queryStr.substr(queryStr.length - 1) == ">" ? ",<" : "<";
						queryStr += targetURI + ">";
						targetURIs[targetURI] = true;
						linksToSearchFor++;
					}
					
					queryStr += "]";
					self._foundLinkTypes.sort();
					
					if (linksToSearchFor == 0) {
						var loadArtifactDeferred = self._loadArtifactTypesForLinks();
						loadArtifactDeferred.addCallback(function(response){
							self._populateFilterButtons();
							self.displayLinks();
						});
					}
					else {
						// Now fetch the resource info of the links
						var loadResourceInfos = ProjectCache.executeMultiFetch({
							query: queryStr,
							disablePagination: true,
							projectSnapshotURI: self.baselineURI
						});
						var processLinks;
						loadResourceInfos.addCallback(function(resourceInfos){
							//For module baselines, some of these links may point to resources outside of the baseline.
							//We must find these, and get them from the live system.
							//Note we only do this here, as the other occurance of this code is for added links - which can't be done in a baseline
							var anotherDeferIssued = false;
							
							for (var i = 0; i < resourceInfos.length; i++) {
								var resourceURI = resourceInfos[i].resourceURI;
								delete targetURIs[resourceURI];
							}
							
							// Any left after deleting the ones we got must not be in the baseline or are either
							// deleted or not allowed to access.
							var targetURIsArray = [];
							for (var target in targetURIs) {
								targetURIsArray.push(target);
							}

							if (self.isInModuleBaseline) {
						
								if (targetURIsArray.length > 0) {
									//We have more resources to get
									anotherDeferIssued = true;	
									var queryStr = "resource in [";
									for (var i = 0; i < targetURIsArray.length; i++) {
										var targetURI = targetURIsArray[i];
										
										queryStr += queryStr.substr(queryStr.length - 1) == ">" ? ",<" : "<";
										queryStr += targetURI + ">";
									}
									queryStr += "]";
												
									var loadResourceInfos2 = ProjectCache.executeMultiFetch({
										query: queryStr,
										disablePagination: true
									});
									loadResourceInfos2.addCallback(function(resourceInfos2){
										//We can just concat them, because we know there are no duplicates
										for (var i = 0; i < resourceInfos2.length; i++) {
											var resourceInfo = resourceInfos2[i];
											// We need to keep track of whether or not the target of the link is in a module baseline.
											// These resources are not in the module baseline, so set the flag to true.
											resourceInfo.isNotInModuleBaseline = true;
											resourceInfo.revisionURI = null;
										}
										resourceInfos = resourceInfos.concat(resourceInfos2);
										processLinks(resourceInfos);
									});
								}
							}
							else {
								// Resources linked to have been either deleted or user does not have the permission to access or
								// it's a CLM link, which should be skipped.  Determine which one it is.
								if (targetURIsArray.length > 0) {																				
																		
									// Generate the array of head requests.
									var headRequestArray = [];
									for (var i = 0; i < targetURIsArray.length; i++) {
										// If it's a CLM link, skip it.
										if (ResourceLinksHandler.isInternalResourceLink(targetURIsArray[i])) {
											headRequestArray.push({
												method: "HEAD",
												uri: targetURIsArray[i]
											});	
										}										
									}																																
									
									if (headRequestArray.length > 0 ) {
										anotherDeferIssued = true;		
										
										// Issue the multiple head requests.
										var headRequestsDeferred = 
											com.ibm.rdm.web.server.client.RDMClient.doMultiRequest(headRequestArray, 50, true);
										headRequestsDeferred.addBoth(self, function(headResponseArray) {																					
											for (var i = 0; i < headResponseArray.length; i++) {
												var headResponse = headResponseArray[i];
												if (headResponse.statusCode === 404) {
													// Not found											
													self._uriToResourceStatus[headResponse.url] = ResourceManager.NOT_FOUND;
												}
												else if (headResponse.statusCode === 403) {
													// Forbidden
													self._uriToResourceStatus[headResponse.url] = ResourceManager.FORBIDDEN;
												}											
											}

											processLinks(resourceInfos);
										});		
									}
								}
							}
							
							if (!anotherDeferIssued) {
								for (var i = 0; i < resourceInfos.length; i++) {
									var resourceInfo = resourceInfos[i];
									resourceInfo.revisionURI = null;
									// module baselines do not need to pass a revisionURI for link creation, so do not set the revisionURI in this
									// case
									if (self.baselineURI && !self.isInModuleBaseline) {
										// module baselines do not use revisionURI
										resourceInfo.revisionURI = ResourceLinksHandler.createRevisionURIFromResourceLocationInBaseline(resourceInfo.resourceURI, resourceInfo.resourceLocation);
									}
								}
								processLinks(resourceInfos);
							}
						});
						
						processLinks = function(resourceInfos) {
							self.modulefactToModuleHostURIMap = [];
							var moduleArtifactURIs = [];
							var moduleURIs = [];
							var moduleArtifactURIsMap = {};
							for (var i = 0; i < resourceInfos.length; i++) {
								var resourceInfo = resourceInfos[i];
								///// HACK !!!!!!!!!!
								if (resourceInfo.boundArtifact && !moduleArtifactURIsMap[resourceInfo.boundArtifact]) {
									moduleArtifactURIs.push(resourceInfo.boundArtifact);
									moduleArtifactURIsMap[resourceInfo.boundArtifact] = resourceInfo.boundArtifact;
								}
								if (resourceInfo.module) {
									moduleURIs.push(resourceInfo.module);
									self.modulefactToModuleHostURIMap[resourceInfo.resourceURI] = resourceInfo.module;
								}
							}
							var loadModuleResourceInfos = ProjectCache.getBatchResourceDescriptionsMultiFetch(moduleArtifactURIs, self.baselineURI, false, true);
							loadModuleResourceInfos.addCallback(this, function(moduleArtifacts){
								com.ibm.rdm.web.server.client.utils.ResourceUtil.getModules(moduleURIs, dojo.hitch(this, function(modulesMap){
									for (var i = 0; i < resourceInfos.length; i++) {
										var boundArtifact = moduleArtifacts[resourceInfos[i].boundArtifact];
										if (boundArtifact) {
											boundArtifact = dojo.clone(boundArtifact);
											boundArtifact.resourceURI = resourceInfos[i].resourceURI;
											boundArtifact.usedInModuleArgs = {
												uri: resourceInfos[i].module,
												title: modulesMap[resourceInfos[i].module].getTitle()
											};
											resourceInfos[i] = boundArtifact;
										}
										
										var resourceInfo2 = resourceInfos[i];
										self._uriToResourceInfos[resourceInfo2.resourceURI] = resourceInfo2;
									}
									
									// Construct artifact type map for links in resource
									var loadArtifactDeferred = self._loadArtifactTypesForLinks(resourceInfos);
									loadArtifactDeferred.addCallback(function(response){
										self._populateFilterButtons();
										self.displayLinks();
									});
								}));
							});
						};
					}
				});
			}
			else {
				self._populateFilterButtons();
				self.displayLinks();
			}
		},
		
		_showBaseLinks: function() {
			return this.viewer.getShowBaseProperties() && !this.viewer.isModuleViewer();
		},
		
		displayLinks: function() {
			var self = this;
			var linksToDisplay = this._getLinksToDisplay();
			// for paging purposes, we need to get all the links in the order in
			// which they will be displayed in the sidebar
			linksToDisplay = this._getLinksToDisplayInOrder(linksToDisplay);
			
			this.grid.addPaging({
				getTotalPagesFunction: function() {
					return Math.ceil((linksToDisplay.length / PAGE_SIZE));
				},
				getCurrentPageFunction : function() {
					return self._pageDisplayed || 1;
				},
				gotoPageFunction: function(pageNum) {
					self._pageDisplayed = pageNum;
					self.setResults(linksToDisplay.slice((pageNum - 1) * PAGE_SIZE, pageNum*PAGE_SIZE));
					self.showArtifacts();
					self.refreshGridGroupHeaders();
					self.updateLinkCountHeader(linksToDisplay.length, pageNum);
				}
			});
					
			this._pageDisplayed = 1;
			this.grid.showPagingWidget();
			
			this.setResults(linksToDisplay.slice((this._pageDisplayed - 1) * PAGE_SIZE, this._pageDisplayed*PAGE_SIZE));
			this.showArtifacts();
			this.refreshGridGroupHeaders();
			this.updateLinkCountHeader(linksToDisplay.length, 1);
			this._selectInitialLink();
		},
		
		_selectInitialLink: function() {
			// if a link to select has been specified, select the link
			// in the sidebar
			if (this.selectLinkURI) {
				// find the link 
				this.expandSidebar();
				var links = this._getLinksToDisplay();
				var selectLink = null;
				if (links) {
					for (var i = 0; i < links.length; i++) {
						var link = links[i];
						if (link.getResourceURI() == this.selectLinkURI) {
							selectLink = link;
							break;
						}
					}
				}
				this.selectLinkURI = null;
				if (selectLink) {
					if (selectLink.isElementLink()) {
						// if we're in the artifact that actually has the link element, wait
						// for the resource to load 
						var resourceURIToCompare = this._getResourceURIToCompare(selectLink);
						if (selectLink.isSubjectElementLink()) {
							if (selectLink.getSubject() == resourceURIToCompare) {
								this._selectInitialLinkItem(selectLink, false, true);
							} else {
								this._selectInitialLinkItem(selectLink, false, false);
							}
						} else {
							if (selectLink.getObject() == resourceURIToCompare) {
								this._selectInitialLinkItem(selectLink, false, true);
							} else {
								this._selectInitialLinkItem(selectLink, false, false);
							}
						}
					} else {
						// we don't have to wait for the artifact to load if it is
						// an artifact level link
						this._selectInitialLinkItem(selectLink, false, false);
					}
				}
			}
		},
		
		_selectGridLinkAfterResourceLoaded: function(rowKey, selectionNotify, /*Integer?*/retryCount) {
			if (retryCount === undefined) {
				retryCount = 0;
			}
			if (this.viewer && this.viewer.hasArtifactBeenLoaded()) {
				if (this._updateTimeout) {
					clearTimeout(this._updateTimeout);
					this._updateTimeout = null;
				}
				this._selectGridLink(rowKey, selectionNotify);
			} else {
				if (this._updateTimeout) {
					clearTimeout(this._updateTimeout);
					this._updateTimeout = null;
				}
				retryCount++;
				// Note: We'll try up to 10 retries to get the content ... we throttle
				// it at 10 in order to avoid recursing forever (which would lead to
				// a symptom of "the browser appears to freeze").
				if (retryCount < 10) {
					this._updateTimeout = window.setTimeout(
						dojo.hitch(
							this,
							function(){
								this._selectGridLinkAfterResourceLoaded(rowKey, selectionNotify, retryCount);
							}
						),
						500
					);
				}
			}
		},
		
		_selectInitialLinkItem: function(selectLink, disableSelectionNotify, selectAfterResourceLoaded) {
			if (typeof(disableSelectionNotify) == "undefined"){
				disableSelectionNotify = false;
			}
			
			var links = this._getLinksToDisplay();
			var index = -1;
			var linksInOrder = null;
			if (links && (links.length > PAGE_SIZE)) {
				// to figure out which page to show for the link to select, we must
				// get all the links in the order in which they will display in the
				// sidebar given the group and sort settings
				var linksInOrder = this._getLinksToDisplayInOrder(links);
				for (var i = 0; i < linksInOrder.length; i++) {
					var link = linksInOrder[i];
					if (link.getResourceURI() == selectLink.getResourceURI()) {
						index = i;
						break;
					}
				}
			}
			if (links) {
				var suffix = null;
				var linkType = this._linkTypes[selectLink.getLinkType()];
				var resourceURIToCompare = this._getResourceURIToCompare(selectLink);
				if (selectLink.getObject() == resourceURIToCompare) {
					suffix = "/subject";
				} else {
					suffix = "/object";
				}
				var rowKey = selectLink.getResourceURI() + suffix;
				if ((index != -1) && linksInOrder && (linksInOrder.length > PAGE_SIZE)) { 
					// we're in page mode, figure out what page the given index
					// is on and go to that page
					var pageNum = Math.ceil((index + 1) / PAGE_SIZE);
					this.grid.pagingWidget._gotoPageFunction(pageNum); 
				}
				if (selectAfterResourceLoaded) {
					this._selectGridLinkAfterResourceLoaded(rowKey, !disableSelectionNotify);
				} else {
					this._selectGridLink(rowKey, !disableSelectionNotify);
				}
			}
		},
		
		_selectGridLink: function(rowKey, selectionNotify) {
			var keys = [];
			keys.push(rowKey);
			this.grid.module.selectAll(false);
			this.grid.module.addToSelection(keys, selectionNotify);
			this.grid.scrollIntoView(rowKey);
		},
		
		_createAddLinkTypeMenu: function(linkTypeRoleTuples) {
			// summary:
			//		Create the items for the Add Link Type menu from the given tuples.
			var self = this;
			
			this._clearMenuEntries(this._createLinksMenu);
			this._linkTypes = {};
			
			// Create the "Select type" label at the top of the menu
			var selectTypeMenuItem = new dijit.MenuItem({
				label: this._messages["selectLinkType"],
				disabled: true,
				onClick: function(){
				}
			});
			selectTypeMenuItem.containerNode.style.fontSize = '8pt';
			if (dojo.isIE) {
				// the disabled state is a little more difficult to see on IE, so increase
				// opacity
				selectTypeMenuItem.containerNode.style.filter = 'alpha(opacity=60)'; 
			}
			this._createLinksMenu.addChild(selectTypeMenuItem);
			this._createLinksMenu.addChild(new dijit.MenuSeparator());
			
			for (var i=0; i<linkTypeRoleTuples.length; i++) {
				var roleTuple = linkTypeRoleTuples[i];
				var itemInTupleCreated = false;
				for (var j=0; j<roleTuple.length; j++) {
					var linkTypeRole = roleTuple[j];
					var linkType = linkTypeRole.getLinkType();
					this._linkTypes[linkType.getResourceURI()] = linkType;		
					if (!linkTypeRole.isPreferenceSpecified() || linkTypeRole.isPreferred()) {
						// if a preference is not specified, this implies that no preferences have been defined for the artifact type,
						// so allow all link type roles to appear on the menu
						var sameAs = linkType.getSameAs();
						if (LinkTypeUtils.isCalmLink(sameAs) || LinkTypeUtils.isDMLinkType(sameAs) || LinkTypeUtils.isRMtoRMLinkType(sameAs)) {
							var isCollection = this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, this.viewer.getProjectURI()) 
								|| this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, this.viewer.getProjectURI());
							var oslcLinkType = LinkTypeUtils.getOslcLinkTypeValueForLinkType(linkType);
							if (isCollection && (oslcLinkType === CalmConstants.ProjectAreaLinkType.trackedBy)) {
								continue;
							} else {
								itemInTupleCreated = true;
								this._createCalmLinkTypeMenuItem(linkTypeRole);
							}
						} else {
							itemInTupleCreated = true;
							this._createLinkTypeMenuItem(linkTypeRole);
						}
					}
				}
				if (itemInTupleCreated) {
					this._createLinksMenu.addChild(new dijit.MenuSeparator());
				}
			}
			// Add the More... menu item to open the LinkDialog when preferred links are specified.
			if (linkTypeRoleTuples[0][0].isPreferenceSpecified()) {
				var moreMenuItem = new dijit.MenuItem({
					label: this._messages["moreLinkTypes"],
					disabled: false,
					onClick: function(){
						self._showCreateLinkDialog(null, false);
					}
				});
				this._createLinksMenu.addChild(moreMenuItem);
			}
		},
		
		_loadLinkTypesAndLinks: function(){
			// Initialize the project permissions variable.
			var projectOperationsDeferred = com.ibm.rdm.web.common.ProjectCache.getProjectPermissions(this.projectURI);
			projectOperationsDeferred.addCallback(this, function(projectOperations){
				this._projectOperations = projectOperations;
			});
			
			var resource = this.viewer.getResourceObject();
			var linkTypeRolesDeferred = LinkTypeUtils.getAvailableCreationLinkTypesRoles(this.projectURI, this.baselineURI, resource.getProcessAreaURI(), resource.getType());
			linkTypeRolesDeferred.addCallback(this, function(linkTypeRoleTuples) {
				this._createAddLinkTypeMenu(linkTypeRoleTuples);
				this._loadLinks();
				return linkTypeRoleTuples;
			});
		},
	
		_createLinkTypeMenuItem: function(linkTypeRole) {
			var self = this;
			var menuItem = new dijit.MenuItem({
				label: linkTypeRole.getLabel(),
				disabled: !linkTypeRole.isAllowed() || this.baselineURI || this.viewer.isInHistoryMode(),
				onClick: function(){
					self._showCreateLinkDialog(linkTypeRole.getLinkType(), linkTypeRole.isSubjectRole());
				}
			});
			this._createLinksMenu.addChild(menuItem);
		},
		
		_createCalmLinkTypeMenuItem: function(linkTypeRole) {
			var self = this;
			var oslcLinkType = LinkTypeUtils.getOslcLinkTypeValueForLinkType(linkTypeRole.getLinkType());
			var linkStub = {
				_resourceUri: oslcLinkType,
				getResourceURI: function(){
					return this._resourceUri;
				}
			};
			var menuItem = new dijit.MenuItem({
				label: linkTypeRole.getLabel(),
				disabled: !linkTypeRole.isAllowed() || this.baselineURI || this.viewer.isInHistoryMode(),
				onClick: function(linkStub){
					return function() {
						self._showCreateLinkDialog(linkStub, linkTypeRole.isSubjectRole());
					};
				}(linkStub) 
			});
			this._createLinksMenu.addChild(menuItem);
		},
			
		_loadLinkTypes: function(links){
			//This gets the link type objects corresponding to the give links			
			var deferred = new dojo.Deferred();
			var linkTypeURIs = [],
				thisTypeURI,
				cachedTypes = [];
			for(var i=0; i < links.length; i++){
				thisTypeURI = links[i].getLinkType();
				if (!this._linkTypes[thisTypeURI]){
					linkTypeURIs.push(thisTypeURI);
				}
				else{
					cachedTypes.push(this._linkTypes[thisTypeURI]);
				}
			}
			// NOTE:  Due to the way cross-project linking works (i.e., links that are created across RRC projects), the
			// following query cannot be scoped to a project since some of the linkTypeURIs may be from other projects.
			var linkTypesDeferred = LinkTypeUtils.getLinkTypes(linkTypeURIs);
			linkTypesDeferred.addCallback(this, function(linkTypeInfos){
				var linkTypesJoined = linkTypeInfos.concat(cachedTypes);
				deferred.callback(linkTypesJoined);
				return linkTypesJoined;
			});			
			return deferred;
		},
		
		_check4RM2RMOrDMBaselineMsg: function(){
				
			//Add informational message to sidebar view for baselines when there 
			//is an associated RM or DM project
			//The message will be added below the toolbar

			// Get registered OSLC link types and check if we have a DM or a RM2RM association				
			if((this.baselineURI || this.viewer.isInReviewMode() || this.viewer.isInHistoryMode())  && !this._RM2RMOrDMBaselineMsgShown){
				var assocDeferred = LinkTypeUtils.getRM2RMAndDMProjectAssociations(this.viewer.getProjectURI());
				assocDeferred.addCallback(this, function(results){								
					//Display warning message if we are in a baseline and an association to 
					// DM or RM2RM has been found.
					if(results.length > 0){
						//Display warning message, if not already showing.										
						this._rm2rmBaselineMsgHandler.showMessage({ 
							type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
							summary:this._messages.inaccurateLinks,
							closable: true
						});
						this._RM2RMOrDMBaselineMsgShown=true;						
					}else{
						this._rm2rmBaselineMsgHandler.hideMessage();
						this._RM2RMOrDMBaselineMsgShown=false;
					}		
				});
			}			
		},
		
		_check4RM2RMOrDMProjectAssociations: function(){
			LinkUtils.getDisconnectedAssociatedProjects(this.viewer.getProjectURI()).then(dojo.hitch(this, function(detailsString){
				this._messageBox.showMessage({ 
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary:this._messages.incompleteLinkResults,
					closable: true,
					details: [{message: detailsString}]
				});
			}));
		},

		_rebuildProjectsByResourceList: function(){
			var self = this;
			//Reset total project by resource Array
			this._totalProjectsByResource = [];
			
			//Check if the deleted resource is found in the links
			dojo.forEach(self._arrayListOfResources, function(item){
				var targetURI = self._getTargetURIFromLink(item.link);
				var resourceInfo = self._uriToResourceInfos[targetURI];
				if (resourceInfo) {
					//Make sure the project is not already in the totalProject array
					if (dojo.indexOf(self._totalProjectsByResource, resourceInfo.projectURI) == -1) {
						self._totalProjectsByResource.push(resourceInfo.projectURI);
					}
				}
				else {
					//Check if resource is external               
					if (!linkutils.sameLocation(targetURI)) {
						var linkType = item.link.getLinkType();
						linkType = self._linkTypes[linkType];
						if (!LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
							if (dojo.indexOf(self._totalProjectsByResource, self.viewer.getProjectURI()) == -1) {
								self._totalProjectsByResource.push(self.viewer.getProjectURI());
							}
						}
					}
				}
			});
			
		},
		
		_loadArtifactTypesForLinks: function(resourceInfos){
		
			var fakeDeferred = new dojo.Deferred();
			
			//Collect the project URIs from the resource Infos
			if (resourceInfos) {
				//Determine if there are any external links in the project. External links can only be
				//filtered by link type and project (which is always current project)
				if (this._externalLinks.length > 0) {
					//Add the current project to the links of projects.
					//Make sure the project is not already in the totalProject array
					var project = this.viewer.getProjectURI();
					if (dojo.indexOf(this._totalProjectsByResource, project) == -1) {
						this._totalProjectsByResource.push(project);
					}
				}
				
				//Collect the project URIs from the resource Infos   
				
				for (var i = 0; i < resourceInfos.length; i++) {
					var resourceInfo = resourceInfos[i];
					if (resourceInfo) {
						var project = resourceInfos[i].projectURI;
						//Make sure the project is not already in the totalProject array
						if (dojo.indexOf(this._totalProjectsByResource, project) == -1) {
							this._totalProjectsByResource.push(project);
						}
					}
				}
				
				if (resourceInfos.length == 0) {
					fakeDeferred.callback({});
				}
				
			}
			else {
				this._rebuildProjectsByResourceList();
				fakeDeferred.callback({});
			}
			
			//Now that we have all the projects for which the links belong to. Create
			//a map of resourceInfo.type to Artifact Type
			
			var objectTypesDeferreds = [];
			for (var i = 0; i < this._totalProjectsByResource.length; i++) {
				objectTypesDeferreds.push(com.ibm.rdm.web.model.types.TypesManager.getObjectTypesForProject({
					projectURI: this._totalProjectsByResource[i],
					projectSnapshotURI: this.baselineURI
				}));
			}
			//Add the deferreds for object types to a DeferredList	 		
			var objectTypesDeferredList = new dojo.DeferredList(objectTypesDeferreds);
			
			objectTypesDeferredList.addCallback(this, function(objectTypes){
				for (var i = 0; i < objectTypes.length; i++) {
					// If a response was good, the first
					// element of an element in responseArray
					// will be true.
					var isValidObjectType = objectTypes[i][0];
					if (isValidObjectType == true) {
						var objectTypeArray = objectTypes[i][1];
						for (var j = 0; j < objectTypeArray.length; j++) {
							this._resourceInfoTypeToObjectType[objectTypeArray[j].getResourceURI()] = objectTypeArray[j];
						}
					}
				}
				fakeDeferred.callback(this._resourceInfoTypeToObjectType);
				return objectTypes;
			});
			objectTypesDeferredList.addErrback(this, function(response){
				//Errors occured in deferredlist
				fakeDeferred.callback({});
			});
			
			return fakeDeferred;
		},
		
		_showCreateLinkDialog: function(linkType, isSubject){
			
			var self = this;
			var filteredLinks = [];
			
			var resourceInfo = this.viewer.resource;
			resourceInfo.name = this.viewer.getArtifactTitle();
			resourceInfo.identifier = resourceInfo.getID();	
			if (this.viewer._moduleSidebar && this.viewer._moduleSidebar.moduleTitle) {
				resourceInfo.usedInModuleArgs = {uri:this.viewer.resource.getModuleURI(), title: this.viewer._moduleSidebar.moduleTitle};
			}
			var isCollection = this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, this.viewer.getProjectURI())
				|| this.viewer.contentType == MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, this.viewer.getProjectURI());
			var linkDialog = new com.ibm.rdm.web.links.LinkDialog({
				projectURI: this.viewer.getProjectURI(),
				processAreaURI: this.viewer.getResourceObject().getProcessAreaURI(),
				artifactTypeURI: this.viewer.getResourceObject().getType(),
				resourceURI: this.resourceURI,
				resourceName: this.viewer.getArtifactTitle(),
				resourceInfos: [resourceInfo],
				linkTypeURI: linkType ? linkType.getResourceURI() : null,
				resourceIsObject: !isSubject,
				callback: function(links){
					//Links Dialog sends back any links that were created. This widget is listening to 
					//com.ibm.rdm.web.notifications.EventTopics.LINK_CREATED events, do no do anything here.
				},
				filterArtifacts: filteredLinks,
				isCollection: isCollection
			});
			linkDialog.show();
		},
		
		setRevisionDate: function(revisionDate){
			this.revisionDate = revisionDate;
		},
		
		setBaseline: function(baselineURI) {
			this.baselineURI = baselineURI;
		},
		
		setIsInModuleBaseline: function(isInModuleBaseline) {
			this.isInModuleBaseline = isInModuleBaseline;
		},
		
		getLinksInRevision: function(links){
			var filteredLinks = [];
			if (links && links.length > 0) {
				for (var i = 0; i < links.length; i++) {
					var includeLink = true;
					// Filter links based on the specified revisionDate.  If the link modified
					// date is > the revision date, do not include the link
					if (this.revisionDate && !this.baselineURI) {
						var linkDate = new Date(links[i].getLastModifiedDate());
						var compare = dojo.date.compare(linkDate, this.revisionDate);
						includeLink = compare != 1;
					}
					if (includeLink) {
						filteredLinks.push(links[i]);
						if (dojo.indexOf(this._foundLinkTypes, links[i].getLinkType()) == -1) {
							this._foundLinkTypes.push(links[i].getLinkType());
						}
					}
				}
			}
			else {
				return [];
			}
			
			return filteredLinks;
		},
		
		_onLinkCreate: function(eventParams) {
			if (!this._linksLoaded) {
				return;
			}
			//Summary: reacts to the creation of a link. 
			var links = eventParams && eventParams.notifierParams && eventParams.notifierParams.links;
			
			
			//Check if the current resource is either the object or the subject of the link. If so, then
			//we should process the link in the link sidebar section.
			var totalLinks = [];
			for(var i=0; i < links.length ; i++){	
				var link = links[i];
				var resourceURIToCompare = null;
				if (this.boundArtifact) {
					// we are in a modulefact viewer
					if (link.isBaseArtifactLink === undefined) {
						// for newly created links this variable won't be set, so dynamically set it here, this widget depends on the
						// links having this value set
						link.isBaseArtifactLink = (link.getSubject() === this.boundArtifact) || (link.getObject() === this.boundArtifact);
					} 
					if ((link.getSubject() === this.resourceURI) || (link.getObject() === this.resourceURI)) {
						// Add the link if it is referencing the modulefact
						totalLinks.push(links[i]);
					} else if (this.viewer.getShowBaseProperties()) {
						// Add the link to the bound artifact if we are showing base properties
						if ((link.getSubject() === this.boundArtifact) || (link.getObject() === this.boundArtifact)) {
							totalLinks.push(links[i]);
						}
					}			
				} else {	
					if ((link.getSubject() == this.resourceURI) || (link.getObject() == this.resourceURI) ||
							com.ibm.rdm.web.query.SavedFilterUtils.isSavedView(link.getSubject())) {
						totalLinks.push(links[i]);
					}			
				}
			}				
			if (totalLinks && totalLinks.length > 0) {
				 this.addLinks(totalLinks);		
			}
		},
		
		_onLinkPreferencesChanged: function(eventParams) {
			// summary:
			//		If the project link preferences have changed for this artifact's type, the Add Link Type menu should be updated
			//		to reflect the changes.
			var resource = this.viewer.getResourceObject();
			if (eventParams.notifierParams.artifactTypeURI !== resource.getType()) {
				return;
			}
			var resource = this.viewer.getResourceObject();
			var linkTypeRolesDeferred = LinkTypeUtils.getAvailableCreationLinkTypesRoles(this.projectURI, this.baselineURI, resource.getProcessAreaURI(), resource.getType());
			linkTypeRolesDeferred.addCallback(this, function(linkTypeRoleTuples) {
				this._createAddLinkTypeMenu(linkTypeRoleTuples);
				return linkTypeRoleTuples;
			});
		},
		
		_onLinkTypesChanged: function(eventParams) {
			// summary:
			//		If the project link types have changed, the Add Link Type menu should be updated
			//		to reflect the changes.
			var resource = this.viewer.getResourceObject();
			var linkTypeRolesDeferred = LinkTypeUtils.getAvailableCreationLinkTypesRoles(this.projectURI, this.baselineURI, resource.getProcessAreaURI(), resource.getType());
			linkTypeRolesDeferred.addCallback(this, function(linkTypeRoleTuples) {
				this._createAddLinkTypeMenu(linkTypeRoleTuples);
				return linkTypeRoleTuples;
			});
		},
				
		_onLinkDelete: function(eventParams) {
			// summary:
			//		Reacts to the creation of a link.
			//		Go through the rows in this grid and remove the row that contains the deleted link.
			if (!this._linksLoaded) {
				return;
			}			
			var resourceToDelete = [];
			var self =this;
		
			var linkUrl = eventParams && eventParams.notifierParams && eventParams.notifierParams.resource;		
				
			//Remove the link from our list of resources
			var keyIndex=0;
			var index=-1;
			
			for(var key in this._arrayListOfResources){				
				if(this._arrayListOfResources[key].id == linkUrl){
					index = keyIndex;
				}			
				keyIndex = keyIndex + 1;
			}
			
			if(index != -1){
				this._arrayListOfResources.splice(index, 1);
			}
					
			//Remove the link from the list of links
			keyIndex=0;
			index = -1;
			for(var key in this._links){				
				if(this._links[key].getResourceURI() == linkUrl){
					index = keyIndex;
				}			
				keyIndex = keyIndex + 1;
			}
			if (index != -1) {
				this._links.splice(index, 1);
			}
			var loadArtifactDeferred = this._loadArtifactTypesForLinks();
			loadArtifactDeferred.addCallback(function(response){
				self._populateFilterButtons();
				self.displayLinks();
			});
		},
		
		_onDMLinkCreatedOrDeleted: function(eventParams){
			//var linkUrl = eventParams && eventParams.notifierParams && eventParams.notifierParams.resource;
			//Refresh the links section after adding or deleting a DM link
			this.handleRefresh();
		},

		_onShowBasePropertiesNotify: function(){
			// summary:
			//		Reacts to the action for showing base artifact links.
			if (this.viewer.isModuleViewer()) {
				return;
			}
			this.handleRefresh();
		},
		
		addLinks: function(links){
			if (!this._hasBeenActivated) {
				return;
			}
			this._links = this._links.concat(links);
			for (var x = 0; x < links.length; x++) {
				var linkType = this._linkTypes[links[x].getLinkType()];
				var found = false;
				for (var i = 0; i < this._foundLinkTypes.length; i++) {
					var linkT = this._foundLinkTypes[i];
					if (linkT == linkType.getResourceURI()) {
						found = true;
						break;
					}
				}
				if (!found) {
					this._foundLinkTypes.push(linkType.getResourceURI());
					this._foundLinkTypes.sort();
				}
			}
			
			var totalAdded = 0;
			var externalAdded = false;
			var queryStr = "resource in [";
			for (var i = 0; i < links.length; i++) {
				var link = links[i];
				var resourceURI = this._getTargetURIFromLink(link);
				
				if (!linkutils.sameLocation(resourceURI)) {
					var linkType = link.getLinkType();
					linkType = this._linkTypes[linkType];
					if (!LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
						this._externalLinks.push(link);
						externalAdded = true;
						continue;
					}
				}
				queryStr += queryStr.substr(queryStr.length - 1) == ">" ? ",<" : "<";
				queryStr += resourceURI + ">";
				totalAdded++;
			}
			queryStr += "]";
			
			// Now fetch the resource info of the links
			if (totalAdded == 0) {
				//Check if any external links were added
				if (externalAdded) {
					//Determine if there are any external links in the project. External links can only be
					//filtered by link type and project (which is always current project)
					if (this._externalLinks.length > 0) {
						//Add the current project to the links of projects.
						//Make sure the project is not already in the totalProject array
						var project = this.viewer.getProjectURI();
						if (dojo.indexOf(this._totalProjectsByResource, project) == -1) {
							this._totalProjectsByResource.push(project);
						}
					}
					this._populateFilterButtons();
					this.displayLinks();
				}
			}
			else {
				var self = this;
				var loadResourceInfos = ProjectCache.executeMultiFetch({
					query: queryStr,
					disablePagination: true
				});
				loadResourceInfos.addCallback(function(resourceInfos){
					// No need to deal with baselines here - this is only dealing with adding links, which doesn't happen in a baseline
					// or history.
					if(!self.modulefactToModuleHostURIMap){
						 self.modulefactToModuleHostURIMap = [];
					}
					var moduleArtifactURIs = [];
					var moduleURIs = [];
					var moduleArtifactURIsMap = {};
					for (var i = 0; i < resourceInfos.length; i++) {
						var resourceInfo = resourceInfos[i];
						///// HACK !!!!!!!!!!
						if (resourceInfo.boundArtifact && !moduleArtifactURIsMap[resourceInfo.boundArtifact]) {
							moduleArtifactURIs.push(resourceInfo.boundArtifact);
							moduleArtifactURIsMap[resourceInfo.boundArtifact] = resourceInfo.boundArtifact;
						}
						if (resourceInfo.module) {
							moduleURIs.push(resourceInfo.module);
							self.modulefactToModuleHostURIMap[resourceInfo.resourceURI] = resourceInfo.module;
						}
					}
					var loadModuleResourceInfos = ProjectCache.getBatchResourceDescriptionsMultiFetch(moduleArtifactURIs, self.baselineURI, false, true);
					loadModuleResourceInfos.addCallback(this, function(moduleArtifacts){
						com.ibm.rdm.web.server.client.utils.ResourceUtil.getModules(moduleURIs, dojo.hitch(this, function(modulesMap){
							for (var i = 0; i < resourceInfos.length; i++) {
								var boundArtifact = moduleArtifacts[resourceInfos[i].boundArtifact];
								if(boundArtifact) {
									boundArtifact = dojo.clone(boundArtifact);
									boundArtifact.resourceURI = resourceInfos[i].resourceURI;
									boundArtifact.usedInModuleArgs = {uri:resourceInfos[i].module, title: modulesMap[resourceInfos[i].module].getTitle()};
									resourceInfos[i] = boundArtifact;
								}
								
								var resourceInfo2 = resourceInfos[i];
								self._uriToResourceInfos[resourceInfo2.resourceURI] = resourceInfo2;
							}
							
							// Construct artifact type map for links in resource
							var loadArtifactDeferred = self._loadArtifactTypesForLinks(resourceInfos);
							loadArtifactDeferred.addCallback(function(response){
								self._populateFilterButtons();
								self.displayLinks();
								self._selectInitialLinkItem(links[0], true);
							});
						}));
					});
				});
			}
		},
		
		// This will typically return the sameAs property of a link type
		// unless it is a user-defined linke type, in which case it will
		// be the resourceURI
		_getUniqueUriForLinkType: function(linkType){
			return linkType.isSystemDefined() ? linkType.getSameAs() : linkType.getResourceURI();
		},
		
		_clearMenuEntries: function(menuWidget){
			//Delete any menu items prior to adding new ones
			menuWidget.destroyDescendants();
			if (menuWidget.hasChildren()) {
				var children = menuWidget.getChildren();
				for (var i = 0; i < children.length; i++) {
					menuWidget.removeChild(children[i]);
				}
			}
		},
		
		_createLinkTypesMenu: function(){
		
			var self = this;
			this._clearMenuEntries(this._linkTypesMenu);
			var comparator = function(type1, type2){
				// show all option does not have a value, we want it
				// to sort first
				if (type1.value == null) {
					return -1;
				}
				if (type2.value == null) {
					return 1;
				}
				return ((type1.name.toLowerCase() < type2.name.toLowerCase()) ? -1 : 1);
			};
			
			var linkTypeKeys = this.getLinkTypeKeys();
			var linkTypeNames = [];
			var count = 0;
			
			for (var key in linkTypeKeys) {
				linkTypeNames.push(this.getLinkTypeNames(GROUP_BY_LINK_TYPES, linkTypeKeys[key], count));
				count++;
			}
			
			var menuEntries = [];
			for (var i = 0; i < linkTypeNames.length; i++) {
				var resourceType = {
					name: linkTypeNames[i].title,
					value: linkTypeNames[i].key,
					filterCategory: LinkFilterTypes.LINK_TYPES
				};
				menuEntries.push(resourceType);
			}
			this._createShowAllCheckbox(self._linkTypesMenu, LinkFilterTypes.LINK_TYPES);
			menuEntries.sort(comparator);
			for (var i = 0; i < menuEntries.length; i++) {
				self._createFilterCheckboxMenuItem(self._linkTypesMenu, menuEntries[i], LinkFilterTypes.LINK_TYPES);
			}
		},
		
		_createArtifactTypeMenu: function(){
			var self = this;
			this._clearMenuEntries(this._artifactTypesMenu);
			var comparator = function(type1, type2){
				// show all option does not have a value, we want it
				// to sort first
				if (type1.value == null) {
					return -1;
				}
				if (type2.value == null) {
					return 1;
				}
				return ((type1.name.toLowerCase() < type2.name.toLowerCase()) ? -1 : 1);
			};
			
			//Display the Artifact Types based on the current links
			var uniqueArtifactTypes = {};
			var menuEntries = [];
			var addExternalURLEntry = false;
			
			//Display the Artifact Types based on the current links
			for (var i = 0; i < self._links.length; i++) {
				var link = self._links[i];
				var resourceURI = self._uriToResourceInfos[self._getTargetURIFromLink(link)];
				//External links do not map to a resourceURI, so make sure to check if the 
				//resourceURI is defined for this link
				if (resourceURI) {
					var adjustedFilterTypeName;
					if (resourceURI.projectURI != self.viewer.getProjectURI()) {
						var projectName="";
						if(self._projectInfoMap[resourceURI.projectURI]){
							projectName = " (" + self._projectInfoMap[resourceURI.projectURI].name + ")";
						}else{
							projectName = " (" + self._messages.unknownProject+ ")";
						}				
						adjustedFilterTypeName = self._resourceInfoTypeToObjectType[resourceURI.ofType].getTitle() + projectName;
					}
					else if (self._resourceInfoTypeToObjectType[resourceURI.ofType]) {
						adjustedFilterTypeName = self._resourceInfoTypeToObjectType[resourceURI.ofType].getTitle();
					}
					else {
						adjustedFilterTypeName = EXTERNAL_TYPE_NAME_COMPARE;
					}
					
					var resourceType = {
						name: adjustedFilterTypeName,
						value: resourceURI.ofType,
						filterCategory: LinkFilterTypes.ARTIFACT_TYPE
					};
					//Gather all unique artifact types
					uniqueArtifactTypes[resourceURI.ofType] = resourceType;
				} else {
					addExternalURLEntry = true;
				}
			}
			
			for (var artifactTypeEntry in uniqueArtifactTypes) {
				menuEntries.push(uniqueArtifactTypes[artifactTypeEntry]);
			}
			if (addExternalURLEntry) {
				var entry = {
					name: this._messages.externalURL,
					value: EXTERNAL_URL_TYPE,
					filterCategory: LinkFilterTypes.ARTIFACT_TYPE
				};
				menuEntries.push(entry);
			}
			menuEntries.sort(comparator);
				
			this._createShowAllCheckbox(self._artifactTypesMenu, LinkFilterTypes.ARTIFACT_TYPE);
			for (var i = 0; i < menuEntries.length; i++) {
				self._createFilterCheckboxMenuItem(self._artifactTypesMenu, menuEntries[i], LinkFilterTypes.ARTIFACT_TYPE);
			}
		},
		
		_createProjectAndArtifactTypeMenu: function(){
		
			//Artifact Types are obtained from the linked resources projects. We first get the projects and their details and then 
			//Populate the artifact types menu.
			
			var self = this;
			this._clearMenuEntries(this._projectTypesMenu);
			this._projectInfoMap = {};
			var comparator = function(type1, type2){
				// show all option does not have a value, we want it
				// to sort first
				if (type1.value == null) {
					return -1;
				}
				if (type2.value == null) {
					return 1;
				}
				return ((type1.name.toLowerCase() < type2.name.toLowerCase()) ? -1 : 1);
			};
			
			this._createShowAllCheckbox(self._projectTypesMenu, LinkFilterTypes.PROJECT_TYPE);
			if (this._totalProjectsByResource.length == 0) {
				//Invoke the ArticatTypeMenu creation
				this._createArtifactTypeMenu();
				return;
			}
			//Get details for all projects the resourceInfos belong to.  
			var projectDeferreds = [];
			for (var i = 0; i < this._totalProjectsByResource.length; i++) {
				projectDeferreds.push(ProjectCache.getProject(this._totalProjectsByResource[i], true));
			}
			//Add the deferreds for object types to a DeferredList	 		
			var projectDeferredList = new dojo.DeferredList(projectDeferreds);
			
			projectDeferredList.addCallback(this, function(projectInfos){
				for (var i = 0; i < projectInfos.length; i++) {
					// If a response was good, the first
					// element of an element in responseArray
					// will be true.
					var isValidProjectInfoResponse = projectInfos[i][0];
					if (isValidProjectInfoResponse == true) {
						var projectInfo = projectInfos[i][1];
						this._projectInfoMap[projectInfo.url] = projectInfo;
					}
				}
				//Display the Artifact Types based on the current links
				var menuEntries = [];
				for (var projectDetail in this._projectInfoMap) {
					var resourceType = {
						name: self._projectInfoMap[projectDetail].name,
						value: projectDetail,
						filterCategory: LinkFilterTypes.PROJECT_TYPE
					};
					menuEntries.push(resourceType);
				}
				
				menuEntries.sort(comparator);
				for (var i = 0; i < menuEntries.length; i++) {
					this._createFilterCheckboxMenuItem(this._projectTypesMenu, menuEntries[i], LinkFilterTypes.PROJECT_TYPE);
				}
				
				this._createArtifactTypeMenu();
				
				// Add "Other" to the project filter type.
				this._createOtherCheckboxForProjectFilterType(self._projectTypesMenu, LinkFilterTypes.PROJECT_TYPE);
				
			});
			
		},
		
		_createShowAllCheckbox: function(subMenu, filterType){
			// viewers are cached, so create a unique id
			var resourceURI = this.viewer.resourceURI;
			var index = resourceURI.indexOf("rrc");
			var id = resourceURI.substring(index);
			var showAllMenuItem = new com.ibm.rdm.web.links.FilterCheckedMenuItem({
				type: filterType,
				label: this._messages.showAll,
				checked: true,
				disabled: true
			});
			this._toDestroy.push(showAllMenuItem);
			
			switch (filterType) {
				case LinkFilterTypes.LINK_TYPES:
					this._showAllLinkTypeWidget = showAllMenuItem;
					break;
				case LinkFilterTypes.ARTIFACT_TYPE:
					this._showAllArtifactTypeWidget = showAllMenuItem;
					break;
				case LinkFilterTypes.PROJECT_TYPE:
					this._showAllProjectTypeWidget = showAllMenuItem;
					break;
			}
			
			showAllMenuItem.contentType = null;
			subMenu.addChild(showAllMenuItem);
			this.connect(showAllMenuItem, "onChange", function(event){
				this._onShowAllChanged(showAllMenuItem);
			});
			var separatorMenuItem = new dijit.MenuSeparator();
			separatorMenuItem.contentType = null;
			subMenu.addChild(separatorMenuItem);
		},
		
		_onShowAllChanged: function(menuItem){
			var menu = null;
			switch (menuItem.type) {
				case LinkFilterTypes.LINK_TYPES:
					menu = this._linkTypesMenu;
					break;
				case LinkFilterTypes.ARTIFACT_TYPE:
					menu = this._artifactTypesMenu;
					break;
				case LinkFilterTypes.PROJECT_TYPE:
					menu = this._projectTypesMenu;
					break;
			}
			this._showAllItems(menuItem, menu);
		},
		
		_createOtherCheckboxForProjectFilterType: function(subMenu, filterType){
			if (filterType == LinkFilterTypes.PROJECT_TYPE) {
				var menuItem = new com.ibm.rdm.web.links.FilterCheckedMenuItem({
					label: this._messages.other,
					checked: true,
					iconURL: ""
				});
				this._toDestroy.push(menuItem);
				subMenu.addChild(menuItem);
				menuItem.value = EXTERNAL_URL_TYPE;
				menuItem.filterCategory = filterType;
				
				var isChecked =  (this._getSelectedFilterFromMenu(menuItem))?true:false;
				menuItem.set("checked", isChecked);
				//Now disable the appropriate "Show all" check box if this menu option is checked
				if(isChecked) {
					var showAllWidget = this._showAllProjectTypeWidget;
					showAllWidget.set("checked", false);
					showAllWidget.set("disabled", false);
				}
				this.connect(menuItem, "onChange", function(event){
					this._onFilterCheckBoxChanged(filterType, menuItem);
				});
			}
		},
		
		_createFilterCheckboxMenuItem: function(subMenu, resource, filterType){
			//var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceType.contentType);
			var menuItem = new com.ibm.rdm.web.links.FilterCheckedMenuItem({
				label: resource.name,
				checked: true,
				iconURL: ""
			});
			this._toDestroy.push(menuItem);
			subMenu.addChild(menuItem);
			menuItem.value = resource.value;
			menuItem.filterCategory = resource.filterCategory;
			
			var isChecked =  (this._getSelectedFilterFromMenu(menuItem))?true:false;
			menuItem.set("checked", isChecked);
			//Now disable the appropriate "Show all" check box if this menu option is checked
			if(isChecked) {
				var showAllWidget;
				switch (filterType) {
					case LinkFilterTypes.LINK_TYPES:
						showAllWidget = this._showAllLinkTypeWidget;
						break;
					case LinkFilterTypes.ARTIFACT_TYPE:
						showAllWidget = this._showAllArtifactTypeWidget;
						break;
					case LinkFilterTypes.PROJECT_TYPE:
						showAllWidget = this._showAllProjectTypeWidget;
						break;
				}
				showAllWidget.set("checked", false);
				showAllWidget.set("disabled", false);
			}
			this.connect(menuItem, "onChange", function(event){
				this._onFilterCheckBoxChanged(filterType, menuItem);
			});
		},
		
		_onFilterCheckBoxChanged: function(filterType, menuItem){
			var widget = null;
			var menu = null;
			//Keep track of the current selected filter
			this._storeSelectedFilterFromMenu(menuItem);
			switch (filterType) {
				case LinkFilterTypes.LINK_TYPES:
					widget = this._showAllLinkTypeWidget;
					menu = this._linkTypesMenu;
					break;
				case LinkFilterTypes.ARTIFACT_TYPE:
					widget = this._showAllArtifactTypeWidget;
					menu = this._artifactTypesMenu;
					break;
					
				case LinkFilterTypes.PROJECT_TYPE:
					widget = this._showAllProjectTypeWidget;
					menu = this._projectTypesMenu;
					break;
			}
			if (menuItem.checked) {
				widget.set("disabled", false);
				widget.set("checked", false);
			} else {
				if (this._areAllItemsUnchecked(menu)) {
					widget.set("disabled", true);
					widget.set("checked", true);
				}
			}
			this.displayLinks();
		},
		
		_storeSelectedFilterFromMenu: function(menuItem){		
			if(menuItem.checked){			
				//Add it to the list.
				this._selectedFilterFromMenu[menuItem.value] = {
					"value" : menuItem.value,
					"filterCategory" : menuItem.filterCategory
				};
			}else{
				//Remove if from the list
				delete this._selectedFilterFromMenu[menuItem.value];				
			}		
		},
		
		_getSelectedFilterFromMenu: function(menuItem){
			return this._selectedFilterFromMenu[menuItem.value];
		},
		
		_showAllItems: function(showAllMenuItem, menu){
			// unchecked is not handled, since there is no way for the user to
			// uncheck the showAll item
			var self = this;
			if (showAllMenuItem.checked) {
				showAllMenuItem.set("disabled", true);
				dojo.forEach(menu.getChildren(), function(menuItem){
					// skip show all and separator items
					if (menuItem.value != null) {
						if (menuItem.get("checked")){
							menuItem.set("checked", false);
							self._onFilterCheckBoxChanged(showAllMenuItem.type, menuItem);
						}
					}
				});
			} 
			self.displayLinks();
		},
		
		_areAllItemsUnchecked: function(menu){
			var allUnchecked = true;
			var menuItems = menu.getChildren();
			for (var i=0; i<menuItems.length; i++) {
				if (menuItems[i].checked) {
					allUnchecked = false;
					break;
				}
			}
			return allUnchecked;
		},
		
		_getLinksToDisplayInOrder: function(links) {
			// figure out the order in which the given links will be displayed 
			// given the current sort and group settings
			var self = this;
			var groupDictionary = new dojox.collections.Dictionary();
			var allLinks = [];
			var sortFunction = null;
			switch (this._resultsSortBy) {
				case com.ibm.rdm.web.links.LinksListWidget.sorts.NAME:
					sortFunction = this._sortLinksByName;
					break;
				case com.ibm.rdm.web.links.LinksListWidget.sorts.TYPE:
					sortFunction = this._sortLinksByArtifactType;
					break;
				case com.ibm.rdm.web.links.LinksListWidget.sorts.DATE:
					sortFunction = this._sortLinksByDate;
					break;
			}
			var keys = null;
			// put the links into the groups
			if (this._groupOrder == GROUP_BY_LINK_TYPES) {
				keys = this.getLinkTypeKeys(GROUP_BY_LINK_TYPES);	
				for (var i=0; i<keys.length; i++) {
					var items = this._getLinksForLinkTypeKey(keys[i], links);
					groupDictionary.add(keys[i], items);
				}
			} else if (this._groupOrder == GROUP_BY_PROJECT) {
				keys = this._totalProjectsByResource;
				for (var i=0; i<keys.length; i++) {
					var items = this._getLinksForProjectKey(keys[i], links);
					groupDictionary.add(keys[i], items);
				}
			} else if (this._groupOrder == GROUP_BY_DATE_MODIFIED) {
				var dateRangeKeys=[];				
				dojo.forEach(links, function(link){
					var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(link)];	
					if(resourceInfo){
						//Group items in fuzzy date ranges(Today, Yesterday, etc...)
						var dateRange = DateTimeHelper.getDateRange(new Date(resourceInfo.lastModifiedDate));
						if (dojo.indexOf(dateRangeKeys, dateRange) == -1) {
							dateRangeKeys.push(dateRange);						
						}	
					}										
				});
				keys = dateRangeKeys;
				for (var i=0; i<keys.length; i++) {
					var items = this._getLinksForLastModifiedDateKey(keys[i], links);
					groupDictionary.add(keys[i], items);
				}
			}
			// sort the links in each group and build a list of all the links
			// as they will appear in the sidebar
			for (var i=0; i<keys.length; i++) {
				var groupLinks = groupDictionary.item(keys[i]);
				var toSortItems = [];
				dojo.forEach(groupLinks, function(item){
					toSortItems.push({
						id: item.getResourceURI(),
						link: item
					});
				});
				sortFunction.call(this, toSortItems);
				dojo.forEach(toSortItems, function(item){
					allLinks.push(item.link);
				});
			}
			return allLinks;
		},
		
		_getLinksToDisplay: function(){
			var self = this;
			var linksInResource = this.getLinksInRevision(this._links);
			var filteredArtifacts = this._getAppliedFilters();
			
			var linksToDisplay = [];
			
			var canAdd = function(linkObject){
			
				//TODO: handle CALM link
				var resourceInfo = self._uriToResourceInfos[self._getTargetURIFromLink(linkObject)];
				for (var j = 0; j < filteredArtifacts.length; j++) {
					if (filteredArtifacts[j].filterCategory == LinkFilterTypes.LINK_TYPES) {
						//Determine if the current link matches the filter link type
						if (self._isLinkOfType(linkObject, filteredArtifacts[j].value)) {
							//Do not display
							return false;
						}
					}
					if (filteredArtifacts[j].filterCategory == LinkFilterTypes.ARTIFACT_TYPE) {
						if (resourceInfo) {
							if (resourceInfo.ofType == filteredArtifacts[j].value) {
								//Filter out the link if the type has been checked
								return false;
							}
						} else {
							// external URL
							if (filteredArtifacts[j].value == EXTERNAL_URL_TYPE) {
								return false;
							}
						}
					}
					
					if (filteredArtifacts[j].filterCategory == LinkFilterTypes.PROJECT_TYPE) {
						if (resourceInfo) {
							if (resourceInfo.projectURI == filteredArtifacts[j].value) {
								//Filter out the link if the type has been checked
								return false;
							}
						}
						else {
							// external link
							if (filteredArtifacts[j].value == EXTERNAL_URL_TYPE) {
								return false;
							}
						}
						
					}
				}
				return true;
			};
			
			if (filteredArtifacts.length > 0) {
				for (var i = 0; i < linksInResource.length; i++) {
					var linkObject = linksInResource[i];
					if (linkObject) {
						if (canAdd(linkObject)) {
							linksToDisplay.push(linkObject);
						}
					}
					else {
						//If a resource info does not exist for this link, then assume external link type
						var linkType = this._linkTypes[linkObject.getLinkType()];
						if (LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
							//TODO: handle CALM link
						}
						else {
							//External link type    				
							linksToDisplay.push(linkObject);
						}
					}
				}
			}
			else {
				linksToDisplay = linksInResource;
			}
			
			return linksToDisplay;
		},
		
		_getAppliedFilters: function(){
		
			var self = this;
			var filterItem = [];
			// Gather active types
			var showAllLinkTypes = this._showAllLinkTypeWidget.checked;
			dojo.forEach(this._linkTypesMenu.getChildren(), function(menuItem){
				if (menuItem.value != null && !showAllLinkTypes && menuItem.checked == false) {
					filterItem.push({
						value: menuItem.value,
						filterCategory: menuItem.filterCategory
					});
				}
			});
			if (this._artifactTypesMenu) {
				var showAllArtifactTypes = this._showAllArtifactTypeWidget.checked;
				dojo.forEach(this._artifactTypesMenu.getChildren(), function(menuItem){
					if (menuItem.value != null && !showAllArtifactTypes && menuItem.checked == false) {
						filterItem.push({
							value: menuItem.value,
							filterCategory: menuItem.filterCategory
						});
					}
				});
			}
			var showAllProjectTypes = this._showAllProjectTypeWidget.checked;
			dojo.forEach(this._projectTypesMenu.getChildren(), function(menuItem){
				if (menuItem.value != null && !showAllProjectTypes && menuItem.checked == false) {
					filterItem.push({
						value: menuItem.value,
						filterCategory: menuItem.filterCategory
					});
				}
			});
			
			return filterItem;
			
		},
		
		expandSidebar: function(){
			this._requestActivate();
		},
		
		handleSortByName: function(){
			this._resultsSortBy = com.ibm.rdm.web.links.LinksListWidget.sorts.NAME;
			this.showArtifacts();
			
		},
		
		handleSortByType: function(){
			this._resultsSortBy = com.ibm.rdm.web.links.LinksListWidget.sorts.TYPE;
			this.showArtifacts();
		},
		
		handleSortByDate: function(){
			this._resultsSortBy = com.ibm.rdm.web.links.LinksListWidget.sorts.DATE;
			this.showArtifacts();
		},
		
		handleGroupByType: function(){
			this._groupOrder = GROUP_BY_LINK_TYPES;
			//Remove selection from other choices, select Group by Type or Relationship
			dojo.removeClass(this._groupByProjectNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._groupByDateNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._groupByRelationShipNode.iconNode, "commentingToolbarDropdownBullet");
			this.showArtifacts();
			
		},
		
		handleGroupByProject: function(){
			this._groupOrder = GROUP_BY_PROJECT;
			//Remove selection from other choices, select Group by Project		
			dojo.removeClass(this._groupByDateNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._groupByRelationShipNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._groupByProjectNode.iconNode, "commentingToolbarDropdownBullet");
			this.showArtifacts();
		},
		
		handleGroupByDate: function(){
			this._groupOrder = GROUP_BY_DATE_MODIFIED;
			//Remove selection from other choices, select Group by date
			dojo.removeClass(this._groupByProjectNode.iconNode, "commentingToolbarDropdownBullet");			
			dojo.removeClass(this._groupByRelationShipNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._groupByDateNode.iconNode, "commentingToolbarDropdownBullet");
			this.showArtifacts();
		},
		
		dateSort: function(links){
			links.sort(com.ibm.rdm.web.links.LinksListWidget.dateComparator);
			links.reverse();
			dojo.removeClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
		},
		
		nameSort: function(links){
			links.sort(com.ibm.rdm.web.links.LinksListWidget.nameComparator);
			dojo.removeClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.removeClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
			dojo.addClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
		},
		
//		typeSort: function(links){
//			var self = this;
//			var commonMessages = this._commonMessages;
//			var typeComparator = function(linkObj1, linkObj2){
//				var contentType1 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(linkObj1.contentType, this.projectURI);
//				var contentType2 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(linkObj2.contentType, this.projectURI);
//				return (contentType1 == contentType2) ? com.ibm.rdm.web.links.LinksListWidget.nameComparator(linkObj1, linkObj2) : ((contentType1 < contentType2) ? -1 : 1);
//			};
//			links.sort(typeComparator);
//			dojo.removeClass(this._sortByNameNode.iconNode, "commentingToolbarDropdownBullet");
//			dojo.removeClass(this._sortByDateNode.iconNode, "commentingToolbarDropdownBullet");
//			dojo.addClass(this._sortByTypeNode.iconNode, "commentingToolbarDropdownBullet");
//		},
		
		getLinkItemWidgets: function(){
			return this._linkItemWidgets;
		},
		
		setProjectURI: function(projURI){
			this.projectURI = projURI;
		},
		
		updateLinkCountHeader: function(totalLength, pageNum){
			var numLinksInResource = this.getLinksInRevision(this._links).length;
			var message = null;
			if (totalLength > this._visibleLinkCount){
				var startIdx = (PAGE_SIZE * (pageNum-1) + 1);
				var endIdx = (startIdx + (this._visibleLinkCount -1));
				if (totalLength != numLinksInResource) {
					if (this.viewer.isModuleViewer()) {
						message = dojo.string.substitute(this._messages.moduleShowingNofXofM, [startIdx, endIdx, totalLength, numLinksInResource]);
					} else {
						message = dojo.string.substitute(this._messages.showingNofXofM, [startIdx, endIdx, totalLength, numLinksInResource]);
					}
				} else {
					if (this.viewer.isModuleViewer()) {
						message = dojo.string.substitute(this._messages.moduleShowingNofX, [startIdx, endIdx, totalLength]);
					} else {
						message = dojo.string.substitute(this._messages.showingNofX, [startIdx, endIdx, totalLength]);
					}
				}
			} else{
				if (this._visibleLinkCount != numLinksInResource) {
					if (this.viewer.isModuleViewer()) {
						message = dojo.string.substitute(this._messages.moduleShowingNofM, [this._visibleLinkCount, numLinksInResource]);
					} else {
						message = dojo.string.substitute(this._messages.showingNofM, [this._visibleLinkCount, numLinksInResource]);
					}
				} else {
					if (this.viewer.isModuleViewer()) {
						message = dojo.string.substitute(this._messages.moduleShowingN, [this._visibleLinkCount]);
					} else {
						message = dojo.string.substitute(this._messages.showingN, [this._visibleLinkCount]);
					}
				}
			}
			
			this._setTitle(message);
		},
		
		setLoadingMessage: function(){
			if (this._setTitle) {
				var message = null;
				if (this.viewer.isModuleViewer()) {
					message = this._messages.moduleLoading;
				} else {
					message = this._messages.loading;
				}
				this._setTitle(message);
			}
		}
		
	});
	
	com.ibm.rdm.web.links.LinksListWidget.sorts = {
		NAME: 0,
		TYPE: 1,
		DATE: 2
	};
	
	com.ibm.rdm.web.links.LinksListWidget.nameComparator = function(linkObj1, linkObj2){
	
		var resourceName1 = linkObj1.displayName;
		var resourceName2 = linkObj2.displayName;
		return ((resourceName1 == resourceName2) ? ((linkObj1.source) ? com.ibm.rdm.web.links.LinksListWidget.dateComparator(linkObj1, linkObj2) : 0) : ((resourceName1.toLowerCase() < resourceName2.toLowerCase()) ? -1 : 1));
	};
	
	com.ibm.rdm.web.links.LinksListWidget.dateComparator = function(linkObj1, linkObj2){
		var nameComparator = function(linkObj1, linkObj2){
			var resourceName1 = linkObj1.displayName;
			var resourceName2 = linkObj2.displayName;
			return ((resourceName1 == resourceName2) ? 0 : ((resourceName1.toLowerCase() > resourceName2.toLowerCase()) ? -1 : 1));
		};
		var resourceDate1 = linkObj1.lastModifiedDate;
		var resourceDate2 = linkObj2.lastModifiedDate;
		var compare = dojo.date.compare(resourceDate1, resourceDate2);
		if (compare == 0) {
			return nameComparator(linkObj1, linkObj2);
		}
		else {
			return compare;
		}
	};
	
})();
