/****************************************************************************** 
 * Licensed Materials - Property of IBM
 *  Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.util.Utility");

dojo.require("dojo.date");
dojo.require("jazz.app.i18n");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");;
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStates");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionElementResultStates");
dojo.require("com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStateClient");
dojo.require("jazz.app.context");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.defects.web.client.internal.CmIntegrationClient");
dojo.require("com.ibm.rqm.defects.web.client.internal.ResourceLinkService");
dojo.require("com.ibm.rqm.defects.web.helper.AutomationItemPicker");
dojo.require("com.ibm.rqm.defects.web.helper.AutomationItemInstancePicker");
dojo.require("com.ibm.rqm.defects.web.util.ItemPickerUtil");
dojo.require("com.ibm.rqm.defects.web.client.DeploymentProviderManager");


(function() 
{
	var ImageCache 					= com.ibm.rqm.execution.web.ui.internal.view.common.ImageCache;
	var Constants  					= com.ibm.rqm.planning.web.client.internal.Constants;
	var ExecutionResultStates 		= com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStates;
	var ExecutionElementResultStates = com.ibm.rqm.execution.web.client.internal.execution.ExecutionElementResultStates;
	var ASQUtility					= com.ibm.asq.common.web.ui.internal.util.Utility;
	var CommonImageCache			= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var ACTION_PREFIX   			= "#action=com.ibm.rqm.planning.home.actionDispatcher";
	var getCurrentProjectArea			= jazz.app.context.get;
    var ServiceResponseHandler      = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var ExecutionResultStateClient  = com.ibm.rqm.execution.web.client.internal.execution.ExecutionResultStateClient;
    var ItemPickerUtil 				= com.ibm.rqm.defects.web.util.ItemPickerUtil;
	var AutomationItemInstancePicker = com.ibm.rqm.defects.web.helper.AutomationItemInstancePicker;
	var AutomationItemPicker = com.ibm.rqm.defects.web.helper.AutomationItemPicker;
	var DeploymentProviderManager	= com.ibm.rqm.defects.web.client.DeploymentProviderManager;
    var DefectsUIUtil 				= com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var ResourceLinkService			= com.ibm.rqm.defects.web.client.internal.ResourceLinkService;
    var CmIntegrationClient 		= com.ibm.rqm.defects.web.client.internal.CmIntegrationClient;
    var ProgressDialog              = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
	var LABEL_NOT_RUN=Constants.getMessage("labelNotRun");
			
	com.ibm.rqm.execution.web.ui.internal.util.Utility = 
	{
			_stepsHaveVerdict: true,
			executionPreferences: {},
			
			PLANNING_PERSPECTIVE: 1,
			CONSTRUCTION_PERSPECTIVE: 2,
			EXECUTION_PERSPECTIVE: 3,
			
		_planningPrefix: ACTION_PREFIX + "&subAction=",
		_constructionPrefix: ACTION_PREFIX + "&subAction=",
		_executionPrefix: ACTION_PREFIX + "&subAction=",
		
		ATTRIBUTENAMES: ["SoftwareInstall", "cpu", "operatingsystem"],
		
		/******************************************************************************************
		 * Generic URI functions
		 * 
		 *****************************************************************************************/
		_getPlanningActionUri: function(action, kwArgs)
		{
			var str = this._planningPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getConstructionActionUri: function(action, kwArgs)
		{
			var str = this._constructionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getExecutionActionUri: function(action, kwArgs)
		{
			var str = this._executionPrefix + action;
			return(this._getActionUri(str, kwArgs));
		},
		
		_getActionUri: function(str, kwArgs) 
		{
		    for (var key in kwArgs) 
			{
	        	str += "&" + key + "=" + kwArgs[key];
	        }
			return str;
		},

		_buildUri: function(method, args, perspective)
		{
			var result;
			
			switch(perspective)
			{
				case this.PLANNING_PERSPECTIVE:
					result = this._getPlanningActionUri(method, args);
					break;
				case this.CONSTRUCTION_PERSPECTIVE:
					result = this._getConstructionActionUri(method, args);
					break;
				case this.EXECUTION_PERSPECTIVE:
					result = this._getExecutionActionUri(method, args);
					break;
			}
			
			return(result);
		},
		
		/******************************************************************************************
		 * Action URI methods
		 * 
		 * @param {Object} id
		 *****************************************************************************************/
		getGenericSearchUri: function(str)	{
			return(this._getPlanningActionUri("genericSearch", {searchStr: str}));
		},

		getNewQueryUri: function()	{
			return(this._getPlanningActionUri("newQuery"));
		},

		getViewWorkItemURI: function(id){
			return this._getPlanningActionUri("viewWorkItem", {id: id});
		},
		
		getNewTestCaseURI: function(name) 
		{
			return this._getPlanningActionUri("newTestCase", {name: name});
		},
			
		getTestCaseURI: function(testId, /*optional*/ revision, perspective)
        {
            var args = {};
        
            if (testId != null)
            {
                if (revision != null) {
                    args.uid = testId;
                    args.revision = revision;                    
                } else {
                    args.id = testId;
                }
            }
			        
            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

			return this._getConstructionActionUri("viewTestCase", args);
		},
				
		getTestCasesURI: function(queryName){
			var args = {};
			if (queryName != null)	{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestCases", args);
		},
		
		getNewTestScriptURI: function(name, perspective){
			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}

			return(this._buildUri("newTestScript", {name: name}, perspective));
		},
			
		getTestScriptURI: function(scriptId, perspective, /*optional*/ revision, /*optional*/ snapshotName){
			var args = {};
            if (scriptId != null){
                if (revision != null) {
                    args.uid = scriptId;
                    args.revision = revision;
                    if (snapshotName!=null) {
                        args.snapshotName = encodeURIComponent(snapshotName);
                    }
                } else {
                    args.id = scriptId;
                }
            }
			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}
			return(this._buildUri("viewTestScript", args, perspective));
		},
				
		getTestScriptsURI: function(queryName, perspective)	{
			var args = {};

			if (queryName != null) {
				args = {name: queryName};
			}

			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}
			return(this._buildUri("viewTestScripts", args, perspective));
		},
		
		getNewTestPlanURI: function(name){
			return this._getPlanningActionUri("newTestPlan", {name: name});
		},
			
		getTestPlanURI: function(kwargs){
			var args = {};
			if ( kwargs.itemId != null ){
				args = {itemId: kwargs.itemId};
			}
			else if (kwargs.id != null)	{
				args = {id: kwargs.id};
			}
			else if (kwargs.name != null){
				args = {name: kwargs.name};
			}
			return this._getPlanningActionUri("viewTestPlan", args);
		},
				
		getTestPlansURI: function(queryName){
			var args = {};
			if (queryName != null) 	{
				args = {name: queryName};
			}
			return this._getPlanningActionUri("viewTestPlans", args);
		},
		
		getNewEwiURI: function(name, testCaseId) {
			var testId = testCaseId != null ? testCaseId : null; 
			return this._getPlanningActionUri("newEwi", {name: name, testCaseId: testId});
		},
		
		getTCERListingPageURI: function(args){
		    var uriArgs = { updateAction: "update-saved-query-selection" };
		    dojo.mixin(uriArgs, args);
			return(this._buildUri("viewExecutedItems", uriArgs, this.EXECUTION_PERSPECTIVE));
		},
		
		getTSERListingPageURI: function(args){
            var uriArgs = { updateAction: "update-saved-query-selection" };
            dojo.mixin(uriArgs, args);
			return(this._buildUri("viewSuiteExecutionRecords", uriArgs, this.EXECUTION_PERSPECTIVE));
		},
			
		getEwiURI: function(id, perspective,isItemId, archived) {
			var args = {};
			if (id != null) {
				var name = isItemId ? 'itemId' : 'id';
				args[name] = id;
			}

			if (perspective == null){
				perspective = this.CONSTRUCTION_PERSPECTIVE;
			}
			return(this._buildUri("viewEwi", args, perspective));
		},
		
		getSERURI: function(id, perspective, isItemId, archived) {
			var args = {};
			if (id != null) {
                var name = isItemId ? 'itemId' : 'id';
				args[name] = id;
			}

			if (perspective == null){
				perspective = this.CONSTRUCTION_PERSPECTIVE;
			}
			return(this._buildUri("viewSER", args, perspective));
		},
				
		getEwisURI: function(queryName, perspective){
			var args = {};
			if (queryName != null) {
				args.name = queryName;
			}
			if (perspective == null){
				perspective = this.PLANNING_PERSPECTIVE;
			}
			return(this._buildUri("viewEwis", args, perspective));
		},
		
		getSizingRecordURI: function(sizingRecordId) {
			var args = {};
			if (sizingRecordId != null)	{
				args = {uid: sizingRecordId};
			}
			return this._getPlanningActionUri("viewTestSizing", args);
		},
		
		startExecution: function(ewi, scriptId, resultItemId, requestId, ergId,updateVariables, channelId)	{
			var args = {};
			
			if (ewi != null){
				args = {
					ewi: ewi,
					runScript: scriptId
				};
				if(resultItemId != null)
					args._resultId = resultItemId;
			}
			if(requestId)
				args.requestId = requestId;
			if(ergId)
				args.ergId = ergId;	
			if(updateVariables == true)
				args.updateVariables = updateVariables;
			if(channelId)
				args.channelId = channelId
			return this._getExecutionActionUri("startExecution", args);
		},
		
		runSuite: function(kwArgs){
			var args = {};
			if (kwArgs.groupId != null)
				args.groupId = kwArgs.groupId;
			if (kwArgs.itemId != null)
				args.itemId = kwArgs.itemId;
			return this._getExecutionActionUri("runSuite", args);
		},
	
		executedItemsURI: function (queryName){
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}
			return this._getExecutionActionUri("viewExecutedItems", args);
		},
	
		getEWIResultsURI: function(ewiItemId){
			var args = {};			
			if (ewiItemId != null) {
				args.ewiItemId = ewiItemId;
			}			
			return this._getExecutionActionUri("viewEWIResults", args);
		},
		
		getSERResultsURI: function(serItemId){
			var args = {};			
			if (serItemId != null) {
				args.serItemId = serItemId;
			}			
			return this._getExecutionActionUri("viewSERResults", args);
		},
		
		showResultsURI: function(queryName){
			var args = {};			
			if (queryName != null) {
				args.name = queryName;
			}			
			return this._getExecutionActionUri("viewResults", args);
		},
		
		getResultURI: function(args){
		// args.editable = true means it is scriptless execution
			if(args.editable){
				return this._getExecutionActionUri("startScriptlessExecution", args );
			}else{
				return this._getExecutionActionUri("viewResult", args );
			}
		},
		
		showSuiteResult: function(kwArgs){
			var args = {};			
			if (kwArgs.id != null)
				args.id = kwArgs.id;
			if(kwArgs.itemId != null)
				args.itemId = kwArgs.itemId;
			return this._getExecutionActionUri("showSuiteResult", args);
		},
		
		getTestSuiteURI: function(suiteId, /*optional*/ revision, perspective)
        {
            var args = {};
        
            if (suiteId != null)
            {
                if (revision != null) {
                    args.uid = suiteId;
                    args.revision = revision;                    
                } else {
                    args.id = suiteId;
                }
            }
			        
            if (perspective == null)
            {
                perspective = this.CONSTRUCTION_PERSPECTIVE;
            }

            return(this._buildUri("viewTestSuite", args, perspective));
        },

		/******************************************************************************************
		 * Generate Unique URL
		 * 
		 * @param {Object} url
		 *****************************************************************************************/
		reuniquifyUrl: function(url){
			return(url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId());
		},
		
		getUniqueId: function(){
			return Date.parse(new Date().toString());
		},

		/******************************************************************************************
		 * Set the window location to an action with keywords
	 	 *****************************************************************************************/
		setLocationToActionWithKeywords: function(action, kwArgs){
			//TODO: fix this for all perspectives
			this._setLocationHash(this.getPlanningActionURI(action, kwArgs));
		},
	
		/******************************************************************************************
		 * Set the window location hash.
		 * This is pushed to a method so all changes to the URI can be traced if needed 
		 *****************************************************************************************/
		_setLocationHash: function(locationHash){
			window.location.hash = locationHash;
		},

		/******************************************************************************************
		 * QSort functions
		 * 
		 * @param {Object} list
		 * @param {Object} a
		 * @param {Object} b
		 *****************************************************************************************/
		swap: function(list, a, b){
			var tmp = list[a];
			list[a] = list[b];
			list[b] = tmp;
		},
		
		partition: function(array, begin, end, pivot){
			var piv = array[pivot];
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix) {
				if(array[ix] <= piv){
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort: function(list, begin, end){
			if(end-1>begin){
				var pivot = begin+Math.floor(Math.random()*(end-begin));
				pivot = this.partition(list, begin, end, pivot);
				this.qsort(list, begin, pivot);
				this.qsort(list, pivot+1, end);
			}		
		},
		
		partition_obj: function(array, index, begin, end, pivot){
			var piv = array[pivot].elements[index].data;
			if (typeof piv == "string"){
				piv = piv.toLowerCase();
			}
	
			this.swap(array, pivot, end-1);
			
			var store=begin;
	
			for(var ix = begin; ix < end-1; ++ix){
				data = array[ix].elements[index].data;
				if (typeof data == "string"){
					data = data.toLowerCase();
				}
				
				if(data <= piv)	{
					this.swap(array, store, ix);
					++store;
				}
			}
			
			this.swap(array, end-1, store);
		
			return store;
		},
		
		qsort_obj: function(list, index, begin, end){
			if(end-1>begin)	{
				var pivot = begin+Math.floor(Math.random()*(end-begin));
	
				pivot = this.partition_obj(list, index, begin, end, pivot);
	
				this.qsort_obj(list, index, begin, pivot);
				this.qsort_obj(list, index, pivot+1, end);
			}		
		},
		
		/******************************************************************************************
		 * Get Icon/Image functions
		 *  
		 * @param {Object} path
		 *****************************************************************************************/
		getArtifactIconURL: function(artifactType){
			switch(artifactType) {
				case Constants.TYPE_PLAN:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_PLAN_SIZING:
					return ImageCache.TEST_PLAN;
				case Constants.TYPE_GROUP_SIZING:
					return ImageCache.TEST_GROUP;
				case Constants.TYPE_CASE_SIZING:
					return ImageCache.TEST_CASE;
				case Constants.TYPE_SCRIPT_SIZING:
					return ImageCache.TEST_SCRIPT;
				case Constants.TYPE_EWILIST:
					return ImageCache.EWI;	
				default: 
					return "";
					
			};		
		},
		
		/******************************************************************************************
		 * Returns the Constant representing the artifactType
		 *  
		 * @param {Object} artifact - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getArtifactType: function(artifact) {
			var fullType = artifact._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch(type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.TYPE_SCRIPT;
				case Constants.CLASSNAME_EWI:
					return Constants.TYPE_EWI;	
				case Constants.CLASSNAME_PLAN_SIZING:
					return Constants.TYPE_PLAN_SIZING;
				case Constants.CLASSNAME_GROUP_SIZING:
					return Constants.TYPE_GROUP_SIZING;
				case Constants.CLASSNAME_CASE_SIZING:
					return Constants.TYPE_CASE_SIZING;
				case Constants.CLASSNAME_SCRIPT_SIZING:
					return Constants.TYPE_SCRIPT_SIZING;
				default:
					return null;											
			}			
		},		
		
		/******************************************************************************************
		 * Returns the Constant representing the item's model type
		 *  
		 * @param {Object} item - the QM artifact DTO object instance 
		 *****************************************************************************************/		
		getModelType: function(item) {
			var fullType = item._eQualifiedClassName;
			var type = fullType.substr(fullType.lastIndexOf(':') + 1);
			
			switch (type) {
				case Constants.CLASSNAME_PLAN:
					return Constants.MODEL_TYPE_PLAN;
				case Constants.CLASSNAME_GROUP:
					return Constants.MODEL_TYPE_GROUP;
				case Constants.CLASSNAME_CASE:
					return Constants.MODEL_TYPE_CASE;
				case Constants.CLASSNAME_SCRIPT:
					return Constants.MODEL_TYPE_SCRIPT;
				default:
					return null;
			}											
		},
		
		
		/******************************************************************************************
		 * Get time as a string
		 *****************************************************************************************/
		//@Deprecated
		//Use com.ibm.asq.common.web.ui.internal.util.Utility.getTimeString instead
		getTimeString: function(/*accepts long*/time) {
			return ASQUtility.getTimeString(time);	
		},
		
		/******************************************************************************************
		 * Get Date object from ISO string.  Need to wrap call to dojo.date.stamp.fromISOString.
		 *****************************************************************************************/	
		 //@Deprecated
		 //Use com.ibm.asq.common.web.ui.internal.util.Utility.dateFromISOString instead
		dateFromISOString: function(timestamp) {
			return ASQUtility.dateFromISOString(timestamp);
		},		
	
		/******************************************************************************************
		 * Function: formatDate
		 * @param {Object} modified
		 * @param {Object} isIso
		 *****************************************************************************************/
		//@Deprecated
		 //Use com.ibm.asq.common.web.ui.internal.util.Utility.formatDate instead
		formatDate: function(modified, isIso){
           return ASQUtility.formatDate(modified, isIso);
		},
	
		getImageForState: function(/* id or object */ result){
			var img = ImageCache.NOT_RUN_ICON;
			if (dojo.isString(result)) {
				return ExecutionResultStates.ICONS[result];
			} else if (result && result.id) {
				return ExecutionResultStates.ICONS[result.id];
			}
			return img;
		},
		
        getImageForResult: function(/* id or object */ result){
            var img = ImageCache.INCON_ICON;
            if (dojo.isString(result)) {
                return ExecutionElementResultStates.ICONS[result];
            } else if (result && result.id) {
                return ExecutionElementResultStates.ICONS[result.id];
            }
            return img;
        },		
	
	/******************************************************************************************
       * Function: formatDateAsLong
       * @param {Object} modifiedAsLong
       * @param {Object} isIso
       *****************************************************************************************/
	  //@Deprecated
	   //Use com.ibm.asq.common.web.ui.internal.util.Utility.formatDateFromLong instead
      formatDateFromLong: function(modifiedAsLong, isIso){
		return ASQUtility.formatDateFromLong(modifiedAsLong, isIso);
      },	
     
      
	  getStatusText: function(/* ExecutionResultDTO */ result) {
         if (result && result.state) {
			return result.state.name;
         }
         return LABEL_NOT_RUN;
      },		
		
		isPaused: function(/* ExecutionResultDTO */ result) {
         if (result && (result.state.id == ExecutionResultStates.IN_PROGRESS || result.state.id == ExecutionResultStates.PAUSED)){ 
            return true;         
         }
         return false;			
		},
		
		getResultsTooltip: function(args){
			if(args){
				var str = [];
				if(args.result.state && args.result.state.id == ExecutionResultStates.IN_PROGRESS){
					str.push(args.result.state.name+"<br/>");
				}
				else
					str.push(Constants.getMessage("labelLastResult")+": <b>"+args.result.state.name+"</b><br/>");
				str.push(Constants.getMessage("labelLastExecTime",this._convertLongToDate(parseInt(args.result.modified))));
				return str.join("");
			}else{
                var str = Constants.getMessage("labelNoResultsFound");
				return str;
			}	
		},
		
		getResultsTooltipForTSER: function(args){
			if(args){
				var str = [];
				str.push(Constants.getMessage("labelLastResult")+": <b>"+args.result+"</b><br/>");				
				return str.join("");
			}else{
                var str = Constants.getMessage("labelNoResultsFound");
				return str;
			}	
		},
		
		getTestInformationToolTip: function(args){
			var str = [];
			if(args){
				str.push(Constants.getMessage("labelType")+": "+args.type+" ");				
				str.push(Constants.getMessage("labelTestPlan")+": "+args.testPlan+" ");				
				str.push(Constants.getMessage("labelIteration")+": "+args.testPhase); 		// CML: should be "test milestone"				
				
			}else{
				str.push(Constants.getMessage("labelNoInfoFound"));
			}	
			return str.join("");
		},

		getAdapterInfoToolTip: function(adapter, availablity){
			var str = [];
			if(adapter){				
				var status = availablity ? availablity : (adapter.busy ? Constants.getMessage("labelUnavailable") : Constants.getMessage("labelAvailable"));
				
				str.push(Constants.getMessage("labelMachine")+": "+adapter.hostName+" ");
				str.push(Constants.getMessage("labelStatus")+": "+ status +" ");
				str.push(Constants.getMessage("labelDescription")+": "+adapter.description);
			}else{						
				str.push(Constants.getMessage("labelNoInfoFound"));
			}	
			return str.join("");
		},
		
		getAdapterHealthSrcALT: function(adapter){
			var adapterStatus = {
				image: ImageCache.NOIMAGE,
				alternateText: ""
			};
			var status = adapter.health;
			if(status == "RED"){
				adapterStatus.image = CommonImageCache.ENGINE_NOTAVAILABLE;
				adapterStatus.alternateText = Constants.getMessage("labelUnAvailable");
			}else if(status == "YELLOW"){
				adapterStatus.image = CommonImageCache.ENGINE_NOTRESPONDED;
				adapterStatus.alternateText = Constants.getMessage("labelNotResponded");
			}else if(status == "GREEN"){
				adapterStatus.image = CommonImageCache.ENGINE_AVAILABLE;
				adapterStatus.alternateText = Constants.getMessage("labelAvailable");
			}
			return adapterStatus;
		},

		getRequestStateToolTip: function(state){
			var str = [];
			if(state && state.id){
				var status = this.getStateInfo(state.id, true);
				str.push(Constants.getMessage("labelstatus")+": "+ state.name +" ");
				str.push(Constants.getMessage("labelDescription")+": "+status);
			}else{
				str.push(Constants.getMessage("labelNoInfoFound"));
			}
			return str.join("");
		},
		
		getStateInfo: function(stateId, getMsg, getIcon){
			var msg = Constants.getMessage("labelNoInfoFound");
			var icon = ImageCache.NOIMAGE;
			switch (stateId){
				case "com.ibm.rqm.executionframework.common.requeststate.nottaken":
					msg = Constants.getMessage("msgReqQueued");
					icon = ImageCache.REQ_QUEUED;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.taken":
					msg = Constants.getMessage("msgReqRunning");
					icon = ImageCache.REQ_RUNNING;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.cancelled":
					msg = Constants.getMessage("msgReqCancelled");
					icon = ImageCache.REQ_CANCELLED;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.complete":
					msg = Constants.getMessage("msgReqCompleted");
					icon = ImageCache.REQ_COMPLETED;
					break;
				case "com.ibm.rqm.executionframework.common.requeststate.paused":
					msg = Constants.getMessage("msgReqPaused");
					icon = ImageCache.REQ_PAUSED;
					break;
			}
			if(getMsg){
				return msg;
			}else if(getIcon){
				return icon;
			}
			return null;
		},
		
		_convertLongToDate: function(/*accepts long*/longDate,/*boolean*/returnAsDate){
			var date = new Date();
			//subtract time zone offset; January is month #0; year base is perhaps 1900
			date.setTime(longDate);
			if(!returnAsDate){
				return jazz.app.i18n.date.locale.format(date, {formatLength: "medium"});
			}else{
				var dateFormat = {selector: "date", formatLength: "medium"};
				return jazz.app.i18n.date.locale.format(date, dateFormat);
			}
		},
		
		_getYearOffset: function (){
			dateNow = new Date();
			if (dateNow.getYear() < 1900)
				return 1900;
			else
				return 0;
		},
		
		_convertDateToLong: function (/*accepts Date*/date) {
			output = date.getTime();
			return output;
		},

		// returns the local-specific time zone offset in milliseconds, ready to be
		// added to the long values
		_getTimeZoneOffset: function()
		{
			dateNow = new Date();
			tzo = dateNow.getTimezoneOffset();	//minutes!
			return (tzo * 60 * 1000);
		},

		getTimeDifferenceInHrMinSec: function(duration){
			if(duration < 0)
				duration = 0;
			if(duration == 0)
				return Constants.getMessage("labelNumberSec", duration); 
			var time = duration / 1000;
			var hours = Math.floor(parseFloat((time / 3600), 10));
			var minutes = Math.floor(parseFloat((time % 3600)/60), 10);
			var seconds = Math.ceil(parseFloat((time % 60)), 10);
			var testDuration = "";
			if (hours != 0) {
				testDuration += Constants.getMessage("labelNumberHr", hours);
			}
			if (minutes != 0) {
				testDuration += Constants.getMessage("labelNumberMin", minutes);
			}

			if (seconds != 0) {
				testDuration += Constants.getMessage("labelNumberSec", seconds);
			}
			return testDuration;
		},
        /**
         * Don't know why we need a slightly modified copy of utility.js here from the planning.web plugin.
         * This is a work around copy as well from utility.js in the planning.web plugin.
         */
        getMessage: function(keyword, tokens) {
            var MessageBundleQM = "RQM";
            var args = [MessageBundleQM];
			var length = arguments.length;
            for( var i = 0; i < length; i++){ 
                args.push(arguments[i]);
            }
            return MessageUtil.getMessage.apply(MessageUtil, args);
        },
		
		_restoreTags: function(value) {
            if(!value) {
                return "";
            }
			var rawText= value;
			rawText= rawText.replace(/\&gt;/gi,">");
			rawText= rawText.replace(/\&lt;/gi,"<");
			rawText= rawText.replace(/\&amp;/gi,"&");
			rawText= rawText.replace(/\&tilde;/gi,"~");
			rawText= rawText.replace(/\&quot;/gi,"\"");
			rawText= rawText.replace(/\&circ;/gi,"^");
			rawText= rawText.replace(/\&nbsp;/gi," ");
			rawText= rawText.replace(/<br\/>/gi,"\n");
			rawText= rawText.replace(/\&#09;/g, "\t");
			return rawText;
		},
		
		getStepsHaveVerdict: function(){
			return this._stepsHaveVerdict;
		},

		setStepsHaveVerdict: function(stepsHaveVerdict){
			this._stepsHaveVerdict = stepsHaveVerdict;
		},
		
		getExecutionPreferences: function(){
			var util = com.ibm.rqm.execution.web.ui.internal.util.Utility;
			return util.executionPreferences;
		},
		
		initExecutionPreference: function(callback){
			var util = com.ibm.rqm.execution.web.ui.internal.util.Utility;
			var handler = {
				success: function(executionPref){
					util.executionPreferences = executionPref && executionPref.value ? dojo.fromJson(executionPref.value) : {};
					if(callback)callback();
				},
				failure: function(){
					util.executionPreferences = {};
					if(callback)callback();
					console.error("Unable to load execution pref");
				}
			};
			var option = new com.ibm.asq.common.web.client.internal.option();
			var handler = new ServiceResponseHandler(handler, "success", "failure");
			option.getProjectPreferences(handler, getCurrentProjectArea().itemId, Constants.EXECUTION_SYSTEM_CONSTRAINTS);
		},

		buildEwiHistory: function(ewi) {
			var history = [];
			 //sort the ewi list first .. WI 3236
			if(ewi.results){
				ewi.results.sort(function(result1,result2){
				 	var time1 = result1.endTime?result1.endTime:result1.startTime;
					var time2 = result2.endTime?result2.endTime:result2.startTime;
					return time2-time1;  //if time2 is greater than time1 return positive 
				 });
			 }
			 var len = ewi.results ? ewi.results.length : 0;
	         for (var i = 0; i < len; i++) {   //ExecutionResultDTO
				var ewiResult = ewi.results[i];
	            var start = ewiResult.startTime;
	            var end = ewiResult.endTime;
				var scriptVersionableItemId =  (ewiResult.script && ewiResult.script.versionableItemId) ? ewiResult.script.versionableItemId : null;
	            var result = {
	               date: this._convertLongToDate(start, true),
	               time: this.getTimeString(start),
				   longTime: end ? end : start,               
	               duration: end ? this.getTimeDifferenceInHrMinSec(end - start) : Constants.getMessage("labelNA"),
				   state:ewiResult.state.name,
	               scriptName: ewiResult.script? ewiResult.script.name : "",
				   scriptTypeName: ewiResult.script? (ewiResult.script.scriptType ? ewiResult.script.scriptType.name : ""): "",
				   scriptId: ewiResult.script? ewiResult.script.id : "",
				   scriptVersionableItemId: scriptVersionableItemId,
				   ownerName: ewiResult.ownerName,
				   id:ewiResult.id
	            };				
	            //handle linkage TODO add logic to determine if incomplete
	            result.icon = ImageCache.EWI_LOG;
				var args ={};
				if(scriptVersionableItemId == null){
					args.ewiId = ewi.id;
				}
				args.resultItemId = ewiResult.itemId;
	            result.link = this.getResultURI(args);
				
				if(ewiResult.state.id === ExecutionResultStates.IN_PROGRESS){
					//All executions resumeable only from Execution console
					result.icon = ImageCache.EWI;
					if(ewiResult.scriptVersionableItemId)
			            result.link = this.startExecution(ewi.itemId, ewiResult.scriptVersionableItemId, ewiResult.itemId, ewiResult.requestId);
					else
						result.link = this.getResultURI({ewiId: ewi.id, resultItemId: ewiResult.itemId});
				}
				history.push(result);

			}
			return history;			
		},
		fixupExecutionStates: function(executionStates) {
            if (dojo.isArray(executionStates) && (executionStates.length > 0)) {
                // sort the states by name
                function stateCompare(stateA, stateB){
                    var a = stateA.name;
                    var b = stateB.name;
                    return a > b ? 1 : a < b ? -1 : 0;
                }
                executionStates.push({
                    name: Constants.getMessage("labelNotRun"),
                    id: "com.ibm.rqm.execution.common.state.notrun"
                });
                executionStates = executionStates.sort(stateCompare);
            }
            return executionStates;
		},
        getExecutionStates: function(callback, errCallback, includeHidden, projectAreaItemId){
            // initialize the execution state filter including the "not run" state
        	var self = this;
            var serviceObj = {
                success: function(executionStates){
                	callback(self.fixupExecutionStates(executionStates));
                },
                
                failure: function(e){
                    errCallback(e);
                }
            };
            var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
			var args = {
                'includeHidden': includeHidden				
			};
			if (projectAreaItemId) {
                args.projectArea = projectAreaItemId;
            }
            ExecutionResultStateClient.getExecutionResultStates(responseHandler, args);
        },
        
        addSelectedAutomationItem: function(currentServiceProvider, onSelected){
			var self = this;
            var selectionDialog = this.getAutoPlanSelectDialog(currentServiceProvider);
            if (selectionDialog) {
                var selectionDialogUri = selectionDialog.uri;
                // select an item from the selection dialog
                var params = {
                    dialogTitle: DefectsUIUtil.getMessage("labelAutomationItemPicker"),
                    secondaryTitle: DefectsUIUtil.getMessage("labelAutomationItemPickerTitle"),
                    pickerURL: selectionDialogUri,
                    height: selectionDialog.height,
                    width: selectionDialog.width,
                    onOk: function(selectedItems){
                        var resources = self.parsePickerResults(currentServiceProvider, selectedItems);
                        onSelected(resources);
                    },
                    onClose: function(){
                    }
                };
                params.OSLCCoreVersion = currentServiceProvider.getOSLCCoreVersion();
                new AutomationItemPicker(params);
            }
            else {
                // service provider does not provide a picker
				var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({'message': DefectsUIUtil.getMessage("msgProvidersDoesNotProvideDialog")});
        		alertDialog.open();
            }
		},
        
        addSelectedAutomationItemInst: function(currentServiceProvider, autoPlan, onSelected){
			var self = this;
            var selectionDialog = dojo.clone(this.getAutoResultListDialog(currentServiceProvider));
            var createDialog = dojo.clone(this.getAutoRequestCreatDialog(currentServiceProvider));
            if (selectionDialog && createDialog) {
               	
                // perform prefilling for delegation UIs
                postData = currentServiceProvider.getAutomationResultSelectionPostData(autoPlan.uri || autoPlan.url, autoPlan.summary);
                currentServiceProvider.getPreFilledAutomationResourceUri(selectionDialog.uri, postData, 
					dojo.hitch(this, function(uri){
						selectionDialog.uri = uri;
						self._doDelegationUIProfilling(currentServiceProvider, autoPlan, selectionDialog, createDialog, onSelected);
					}), dojo.hitch(this, function(error){
						selectionDialog.uri += ("?autoPlan="+encodeURI(autoPlan.uri || autoPlan.url));
						self._doDelegationUIProfilling(currentServiceProvider, autoPlan, selectionDialog, createDialog, onSelected);
					})
				);
                
                
            }
            else {
                // service provider does not provide a picker
				var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({'message': com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil.getMessage("msgProvidersDoesNotProvideDialog")});
        		alertDialog.open();
            }
		},
		
		_doDelegationUIProfilling: function(currentServiceProvider, autoPlan, selectionDialog, createDialog, onSelected){
			var self = this;
			var postData = currentServiceProvider.getAutomationRequestCreationPostData(autoPlan.uri || autoPlan.url, autoPlan.summary);
            currentServiceProvider.getPreFilledAutomationResourceUri(createDialog.uri, postData, 
				dojo.hitch(this, function(uri){
					createDialog.uri = uri;
					self._invokeRemoteDialogPicker(currentServiceProvider, autoPlan, selectionDialog, createDialog, onSelected);
				}), dojo.hitch(this, function(error){
					createDialog.uri += ("?autoPlan="+encodeURI(autoPlan.uri || autoPlan.url));
					self._invokeRemoteDialogPicker(currentServiceProvider, autoPlan, selectionDialog, createDialog, onSelected);
				})
			);
		},
		
		_invokeRemoteDialogPicker: function(currentServiceProvider, autoPlan, selectionDialog, createDialog, onSelected){
            var self = this;
			// select an automation result from the selection dialog
            var params = {
                dialogTitle: com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil.getMessage("labelAutomationItemInstPicker"),
                secondaryTitle: com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil.getMessage("labelAutomationItemInstPickerTitle"),
                itemDefinitionName: autoPlan.summary,
                itemDefinitionURL: autoPlan.uri || autoPlan.url,
                selectExistPickerURL: selectionDialog.uri,
                newPickerURL: createDialog.uri,
                selHeight: selectionDialog.height,
                selWidth: selectionDialog.width,
                creHeight: createDialog.height,
                creWidth: createDialog.width,
                onOk: function(selectedItems, option){
                    var resources = self.parsePickerResults(currentServiceProvider, selectedItems);
                    onSelected(resources, option);
                },
                onClose: function(){           
                }
            };
            params.OSLCCoreVersion = currentServiceProvider.getOSLCCoreVersion();
            new AutomationItemInstancePicker(params);
		},
		
		_getEWIResourceUri: function(ewi){
			if(!ewi)
				return null;
				
	        var projectAreaId = jazz.app.context.get().itemId;
	        var resourceUri = dojo.getObject(Constants.PUBLIC_URL) + "oslc_qm/contexts/" + projectAreaId + this.getEWIResourcePrefix() + ewi.itemId;
	        return resourceUri;
	    },
	    
	    getEWIResourcePrefix: function(){
	        return '/resources/com.ibm.rqm.execution.TestcaseExecutionRecord/';
	    },
	    
	    getServiceProviderManager:function(){
			return DeploymentProviderManager.getInstance();
		},
		getAutoPlanSelectDialog: function(provider){
	        return provider.services.autoPlanSelectDialog;
	    },
	    
	    getAutoResultListDialog: function(provider){
	        return provider.services.autoResultListDialog;
	    },
	    
	    getAutoRequestCreatDialog: function(provider){
	        return provider.services.autoRequestCreateDialog;
	    },
	    
	    parsePickerResults: function(provider, selectedItems){
	        var oslcCoreVersion = provider.getOSLCCoreVersion();
	        var autoItems = [];
	        if (oslcCoreVersion == "2.0") {
	            autoItems = com.ibm.rqm.defects.web.util.ItemPickerUtil.parseV2(selectedItems);
	        }
	        
	        return autoItems;
	    },
	    
	    isEnableAutomationItemColumnBySystemProperty: function(callback){
	    	ResourceLinkService.getOSLCAutomationConsumerEnabled(function(status, autoConsumerEnabled){
			 	if (status.isOK()) {
			 		if(!autoConsumerEnabled || autoConsumerEnabled.toLowerCase() != "true"){
				 		callback(false);	 		
			 		}
			 		else
			 			callback(true);
				}
				else {
					var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({
		                'message': status.getMessage()
		            });
		            alertDialog.open();
					callback(false);
				}
			});
	    },
	    
	    /***
		 * Link Defects to Execution result
		 */
		linkDefectsToExecutionResults: function(resultSummaryDTOs, ewi, workitems, callback){
			if (!resultSummaryDTOs || (resultSummaryDTOs.length <= 0)) {
				console.debug("ExecutionUtility: Given result dto array is empty or null");
				return;
			}
			ProgressDialog.doRunWithProgress(Constants.getMessage("labelAssociateDefect"), dojo.hitch(this, function(monitor){
				monitor.beginTask(Constants.getMessage("labelAssociateDefect"));
				var index = 0;
				var self = this;
				var linkDefectToResult = function() {
					if(index == resultSummaryDTOs.length) {
						monitor.done();
						if (callback)
							callback();
						return;
					}
					var r = resultSummaryDTOs[index];
					index++;
					self._postLinkToExecutionResults(workitems,	r.itemId, r.name, (ewi != null ? ewi : r.ewi),	linkDefectToResult);
				};
				// Link defect to first result
				linkDefectToResult();						
			}));
		},
				
		_postLinkToExecutionResults: function(workItems, resultItemId, resultName, ewi, callback){
			if (workItems.length > 0) {
				//Post local link to TER
				var i = 0;
				var workItem = null;
				var currentProvider = null;
				var postLinkToExecutionResult = dojo.hitch(this, function(){
					if (i == workItems.length) {
						callback();
						return;
					}else{
						workItem = workItems[i];
						currentProvider = workItem.provider;
						var linkParams = {
							itemId: resultItemId, 
							cmUri: workItem.url,
							summary: workItem.summary,
							linkType: CmIntegrationClient.RESULT_RELATEDTO_DEFECT,
							createRelatedLinks: true
						};
						if(workItem.providerProjectKey){
							linkParams.providerProjectKey = workItem.providerProjectKey;
						}
						CmIntegrationClient.postNewLink(linkParams, dojo.hitch(this, function(status){
							i++;
							currentProvider.createBackLinks4AssociatingDefectsWithTestResult(
								resultItemId, 
								resultName, 
								null, 
								ewi.testCase.versionableItemId, 
								ewi.testCase.name, 
								(ewi.testPlan? ewi.testPlan.versionableItemId : null), 
								(ewi.testPlan ? ewi.testPlan.name : null), 		
								workItem.url, 
								workItem.summary, 
								null, 
								dojo.hitch(this, function(status){
									console.debug("defect back links created");
									postLinkToExecutionResult();
								}),	
								dojo.hitch(this, function(status, response){
									console.debug("failed to create execution result link");
									postLinkToExecutionResult();
									this._displayAlertDialog(Constants.getMessage("labelError"), status.getMessage() || response);
								})
							);
		                }));
					}
				});
				postLinkToExecutionResult();
			}
		}

	};
}
)();
