/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");
dojo.require("dijit.Menu");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit.MenuItem");

dojo.require("jazz.ui.internal.MenuItemGroup");
dojo.require("jazz.ui.menu.NodeMenuItem");
dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.util.html");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.MenuPopup");
dojo.require("jazz.util.moduleResource");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "MenuPopupMessages");

var messages = dojo.i18n.getLocalization("jazz.ui", "MenuPopupMessages"), _currentOpen = null, _currentOver = null, _deferred = null;

(function(){
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var _textContent = dojo.isIE ? "innerText" : "textContent";
    var FiltersQueriesClient = com.ibm.asq.common.web.client.internal.FiltersQueriesClient;
    var WrapHints = jazz.ui.MegaMenu.WrapHints;
    var LOADING = com.ibm.rqm.planning.web.client.internal.Constants.getMessage("labelLoading");
    var MY_QUERIES_GROUP_ID = "MYQUERIESGROUP";
    var SHARED_QUERIES_GROUP_ID = "SHAREDQUERIESGROUP";
    var VIEWED_GROUP_ID = "VIEWEDGROUP";
    var UNSAVED_GROUP_ID = "UNSAVEDGROUP";
    var QUERIES_GROUP_ID = "QUERIESGROUP";
    var MAXVIEWEDCOUNT = 10;
    var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
    var CommonUtil = com.ibm.asq.common.web.ui.internal.util.Utility;
    var PlanningUtil = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.ComponentMenu", jazz.app._ComponentMenu, {
        selectedMenuItem: null,
        
        postCreate: function(){
            this.inherited(arguments);
            this.menuItemUpdateTasks = [];
        },

        getSelected: function() {
            return this.selectedMenuItem;
        },
        
        _setSelectedAttr: function(menuItem) {
            this.selectedMenuItem = menuItem;
        },

        onExecute: function(){
            this.inherited(arguments);
            if (!this._contentArea) {
                this._contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
            }

            // update the selected menubar item with the one of the menuItemEntry
            this.menuItemEntry.updateMenubarItem();

            // notify the content area on the menuItemEntry executed to reset the breadcrumb trail
            this._contentArea.onMenuActionExecute(this.menuItemEntry);
        },

        // Manipulate the menu items while the menu is opened can cause display issues at the
        // same time with the selection and hover changed.  It also gets exception in IE that
        // the removed menu item is still being referenced if it is destroyed explicitly to
        // avoid memory leak.  This method will add the specified task to the menuItemUpdateTasks
        // array and defer the running until the menu is closed.
        // 
        addMenuItemUpdateTask: function(task) {
            if (this.menuOpened) {
                // defer task until menu is closed
                this.menuItemUpdateTasks.push(task);
            } else {
                task();
            }
        },

        onOpen: function() {
            this.inherited(arguments);
            this.menuOpened = true;
        },

        onClose: function() {
            this.inherited(arguments);
            this.menuOpened = false;
            if (this.menuItemUpdateTasks.length > 0) {
                // run deferred menu items update tasks
                dojo.forEach(this.menuItemUpdateTasks, function(task){
                    try {
                        // make sure any exception are caught so that we don't leave with a menu stuck opened
                        task();
                    } catch (ex) {
                        console.error(ex);
                    }
                });
                this.menuItemUpdateTasks = [];
            }
        }
    });

    // Pop-up menu item for sub-menu in the main menu
    //
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.PopupMenuItem", dijit.PopupMenuItem, {
    
        postCreate: function(){
            this.inherited(arguments);
            var self = this;
            dojo.addClass(this.domNode, "com-ibm-team-rqm-PopupMenuItem");
            dojo.addClass(this.iconNode, "icon-img");
            this.popup = new com.ibm.rqm.planning.web.ui.internal.view.common.ComponentMenu({
                // make sure parent menu is closed when sub-menu is closed after executed the action
                onExecute: dojo.hitch(this, function(){
                    if (self.parentMenu) {
                        self.parentMenu.menu.close();
                        self.parentMenu.popupOpened = null;
                    }
                })
            });
            this.popup.connect(this.popup._menu, "onMouseOver", dojo.hitch(this, function(evt){
                self.onSubMenu = true;
            }));
            this.popup.connect(this.popup._menu, "onMouseOut", dojo.hitch(this, function(evt){
                var toNode = dojo.isIE ? evt.toElement : evt.relatedTarget;
                if (toNode && dojo.isDescendant(toNode, self.popup.domNode)) {
                    // mouse out to another dom element in sub-menu, do nothing
                    return;
                } else {
                    // not on the sub-menu, parent can close but delay a bit in case the mouse
                    // just accidentally moved a bit too fast too far off the sub-menu
                    self.onSubMenu = false;
                    if (!self.hoverHandler) {
                        // skip if there is already a handler running
                        self.hoverHandler = setTimeout(function(){
                            if (!self.onSubMenu) {
                                // mouse is still not over the sub-menu, parent can close
                                self.parentMenu.popupOpened = null;
                            }
                            delete self.hoverHandler;
                        }, 500); // give half a second buffer to move back to the sub-menu
                    }
                }
            }));
            
            this.updateArrow();
        },
        
        _onHover: function(evt){
            // do more only if we have a sub-menu of menu items
            if (this.hasPopup) {
                this.inherited(arguments);
                // simulate a click to open the sub-menu
                this.hoverClicked = true;
                this._onClick(evt);
                if (this.parentMenu) {
                    // set the flag of the parent not to dismiss when sub-menu is opened
                    this.parentMenu.popupOpened = this.popup;
                }
            } else {
                // only update the hover style if there is no sub-menu, this.inherited() from
                // Chrome will show an empty sub-menu
                dojo.addClass(this.domNode, 'dijitMenuItemHover');
                // without this.inherited(), need to have the parent menu focus this menu item
                // to update the style
                this.getParent().focusChild(this);
            }
        },
        
        _onUnhover: function(evt){
            // do more only if we have a sub-menu of menu items
            if (this.hasPopup) {
                this.inherited(arguments);
                evt = evt || window.event;
                if (evt) {
                    var toNode = dojo.isIE ? evt.toElement : evt.relatedTarget;
                    if (toNode && dojo.isDescendant(toNode, this.popup.domNode)) {
                        // unhover to the sub-menu, parent should not dismiss
                        this.parentMenu.popupOpened = this.popup;
                        return;
                    }
                }
                if (this.parentMenu) {
                    // sub-menu is close on unhover, reset the flag of the parent
                    this.parentMenu.popupOpened = null;
                }
            } else {
                // only update the hover style if there is no sub-menu to match up with the
                // work around for Chrome on hover
                dojo.removeClass(this.domNode, 'dijitMenuItemHover');
            }
        },
        
        _onClick: function(evt){
            if (this.hoverClicked) {
                // click simulated from hover, do the regular click
                // to bring up the sub-menu
                delete this.hoverClicked;
                this.inherited(arguments);
            } else {
                // needs to clear the hovering state. The parent's onItemClick (called by parent
                // class's _onClick) would eventually clear the hovering state of this item.
                // Since we don't call the parent's onItemClick, we need to explicitly clear this.
                com.ibm.rqm.planning.web.ui.internal.view.common.PopupMenuItem.superclass._onUnhover.apply(this, arguments);
                
                // real click from the user, run the command
                this.onClick();
                // close sub-menu if it is up and parent menu too
                this.popup.close();
                if (this.parentMenu) {
                    this.parentMenu.menu.close();
                    this.parentMenu.popupOpened = null;
                }
                dojo.stopEvent(evt);
            }
        },
        
        addMenuItem: function(dropDownArg){
            if (this.dropDownArgs) {
                this.dropDownArgs.push(dropDownArg);
            } else {
                this.dropDownArgs = [dropDownArg];
            }
            var self = this;
            if (dropDownArg.label == "-") {
                this.addSeparator();
            } else {
                var menuItem = this.popup.addItemByArgs(dropDownArg);
                if (self.parentMenu) {
                    menuItem._onUnhover = function(evt){
                        // perform the regular highlight
                        dojo.removeClass(this.domNode, 'dijitMenuItemHover');
                        this.getParent().onItemUnhover(this);
                    };
                    menuItem._onHover = function(evt){
                        // perform the regular highlight
                        dojo.addClass(this.domNode, 'dijitMenuItemHover');
                        this.getParent().onItemHover(this);
                    };
                }
            }
            this.updateArrow();
        },
        
        addSeparator: function(){
            var self = this;
            this.popup.add(new dijit.MenuSeparator());
        },
        
        removeAllDropdownMenuItems: function(){
            var dropDownArgs = this.dropDownArgs;
            var itemsCount = dropDownArgs ? dropDownArgs.length : 0;
            this.popup.clear();
            this.updateArrow();
        },
        
        updateArrow: function(){
            var menuIitems = this.popup.getItems();
            var itemsCount = menuIitems ? menuIitems.length : 0;
            this.hasPopup = itemsCount > 0;
            if (this.hasPopup) {
                dojo.style(this.arrowWrapper, "display", "");
                dojo.style(this.popup.domNode, "visibility", "visible");
                if (dojo.isIE == 7) {
                    dojo.style(this.arrowWrapper.parentNode, "display", "block"); // right arrow <td>
                    dojo.style(this.containerNode, "width", "100%"); // label <td>
                    dojo.style(this.containerNode, "padding-left", "0px");
                }
            } else {
                dojo.style(this.arrowWrapper, "display", "none");
                dojo.style(this.popup.domNode, "visibility", "hidden");
            }
        }
        
    });
    
    // Simple 'in-place' menu action that creates a link inside 
    // a menu. The link can have a label before and/or after it.
    // 
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.InplaceLinkAction", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="menu-inplace-action login-item" dojoAttachPoint="domNode">' +
                        '  <span class="first-label" style="display:none;" dojoAttachPoint="firstLabelNode"></span>' +
                        '  <a href=${href} dojoAttachPoint="linkNode" dojoAttachEvent="onclick:_handleClicked">' +
                        '    <span dojoAttachPoint="linkLabelNode">' +
                        '    </span>' +
                        '  </a>' +
                        '  <span class="secondary-label" dojoAttachPoint="labelNode">' +
                        '  </span>' +
                        '</div>',
        name: null,
        label: null,
        firstLabel: null,
        secondaryLabel: null,
        href: "javascript:{}",
        
        // if firstLabel is specified, user can specify a new line before 
        // the link to separate the label and the link.
        newLineBeforeLink: false,
        
        // set to true to remove the action link when it's clicked
        // and show a busy message.
        showBusyWhenClicked: false,
        
        postCreate: function(){
            this.linkLabelNode[_textContent] = this.label;
            if (this.secondaryLabel) {
                this.labelNode[_textContent] = this.secondaryLabel;
            }
            if (this.firstLabel) {
                dojo.style(this.firstLabelNode, "display", "");
                this.firstLabelNode[_textContent] = this.firstLabel;
            }
            if (this.newLineBeforeLink) {
                dojo.place(document.createElement("br"), this.linkNode, "before");
                dojo.place(document.createElement("br"), this.linkNode, "before");
            }
            if (this.name) {
                this.domNode.name = this.name;
            }
        },
        setErrorRetry: function(errorTxt){
            dojo.style(this.linkNode, "display", "");
            dojo.style(this.firstLabelNode, "display", "");
            dojo.style(this.labelNode, "display", "none");
            this.firstLabelNode[_textContent] = errorTxt;
        },
        onClick: function(){
        },
        _handleClicked: function(e){
            if (this.showBusyWhenClicked) {
                dojo.style(this.linkNode, "display", "none");
                dojo.style(this.firstLabelNode, "display", "none");
                dojo.style(this.labelNode, "display", "");
                this.labelNode[_textContent] = LOADING;
            }
            this.onClick();
        }
    });
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.VerticalMenuPopup", [jazz.ui.MenuPopup], {
    
        imageAreaTemplate:'<div>' +
                              '<div class="hover-center" dojoAttachPoint="domNode, _hoverAreaCenter">' +
                                 '<div class="action-bar-icon" dojoAttachPoint="_iconArea">' +
                                    '<div dojoAttachPoint="_iconImg">' +
                                       '<div dojoAttachPoint="_caretNode" style="float:right;margin-right:5px;margin-top:-5px"></div>' +
                                    '</div>' +
                                    '<div class="action-bar-entry-icon-text" dojoAttachPoint="_iconText"></div>' +
                                 '</div>' +
                              '</div>' +
                           '</div>',
        
        postCreate: function(){
            // don't let the parent class manage the caret
            this.injectCaret = false;
            this.inherited(arguments);
            dojo.addClass(this.domNode, "vertical-action-bar-entry");
            this.imageArea = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.imageAreaTemplate);
            
            if (this.iconURL) {
                this.imageArea._iconImg.className = this.iconURL;
            }

            // if this is CJK locale add in our cjk locale specific CSS
            if (com.ibm.asq.common.web.ui.internal.util.Utility.isCJKLocale()) {
                dojo.addClass(this.imageArea._iconText, "action-bar-entry-cjk");
            }
            
            if (this.iconText) {
                var iconWrappedText = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText(this.iconText,
                    "action-bar-entry-icon-text", 68, // hard code the max width for now. The dom node has not been added to the tree. 
                    false, "<br/>");
                this.imageArea._iconText.innerHTML = iconWrappedText;
                this.imageArea._iconImg.alt = this.iconText;
            }
            this.domNode.appendChild(this.imageArea.domNode);
            
            this._caret = dojo.create("img", {
                "class": "caret",
                src: jazz.util.moduleResource(this.caretSrc)
            }, this.imageArea._caretNode);
            
            this.injectCaret = true;
            this.around = this._caret;
            dojo.style(this._caret, "visibility", "hidden");
            this.connect(this.domNode, "onmouseenter", this._onMouseEnter);
            this.connect(this.domNode, "onmouseleave", this._onMouseLeave);
        },
        
        setDirty: function(dirty){
            if (!this._caretWrapper) {
                this._caretWrapper = dojo.create("span", {
                    "class": "vertical-dirty-wrapper",
                    title: messages.unsaved
                }, this.imageArea._caretNode);
                dojo.place(this._caret, this._caretWrapper);
			} 
            dojo.style(this._caretWrapper, "visibility", dirty ? "visible" : "hidden");
        },
        
        open: function(){
            this._setHoverImage(true);
            this.inherited(arguments);
            if (this._currentMenu) {
                // get the parent mega menu of the pop-up which is the top-most DIV that has style for shadow
                var parentMegaMenu = this._currentMenu.domNode.parentNode;
                if (parentMegaMenu && !dojo.hasClass(parentMegaMenu, "qm-vertical-popup-menu")) {
                    // adjust the position to line up to the action icon
                    dojo.addClass(parentMegaMenu, "qm-vertical-popup-menu");
                }
            }
        },
        
        close: function(focus){
            this._setHoverImage(false);
            this.inherited(arguments);
        },
        
        _handleKeypress: function(e){
            var k = dojo.keys;
            switch (e.charOrCode) {
                case k.SPACE:
                case k.RIGHT_ARROW:
                    if (!this._menuOpen) {
                        this.open();
                        e.preventDefault();
                    }
                    break;
                case k.ESCAPE:
                case k.LEFT_ARROW:
                    if (this._menuOpen) {
                        this.close();
                    }
                    break;
                case k.UP_ARROW:
                    if (this._menuOpen) 
                        this._currentMenu.focus("end");
					else if (this.prevHandler && dojo.isFunction(this.prevHandler)) {
                            this.prevHandler();
                            dojo.stopEvent(e);
                        }
                    break;
                case k.DOWN_ARROW:
                    if (this._menuOpen) 
                        this._currentMenu.focus("start");
					else if (this.nextHandler && dojo.isFunction(this.nextHandler)) {
                            this.nextHandler();
                            dojo.stopEvent(e);
                        }
                    break;
            }
        },
        
        _onMouseEnter: function(event){
            this._setHoverImage(true);
        },
        
        _onMouseLeave: function(event){
            if (!this._menuOpen) {
                this._setHoverImage(false);
            }
        },
        
        _setHoverImage: function(isOver){
            if (isOver) {
                dojo.addClass(this.imageArea._hoverAreaCenter, "hovered-center");
            } else {
                dojo.removeClass(this.imageArea._hoverAreaCenter, "hovered-center");
            }
        }
    });
    
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.ActionBarEntry", [dijit._Widget], {
        dropdownGroupsCount: 0,
        skipUnsavedAndViewedGroups: false,  // default to show unsaved and recently viewed menu items group
        skipRecentQueriesGroups: false,  // default to show recent queries menu items group
        blankIcon: ImageCache._getImageUri("transparent16.gif"),
        viewedGroupTitleKey: "labelRecentlyViewedGroupTitle",
        unsavedGroupTitleKey: "labelUnsavedGroupTitle",
        queriesGroupTitleKey: "labelRecentQueriesGroupTitle",
        
        // Menu items are declared in the constructor and created during initialize by default.
        // Subclass can skip and manage when to create by setting this to false.
        createMenuItemsOnInit: true,
        
        constructor: function(args){
            dojo.mixin(this, args);
            this.alreadyLoaded = [];
            this.menuEntries = [];
            this.viewedMenuEntries = [];   // recently viewed artifact menu entries
            this.queriesMenuEntries = [];  // recently used saved queries menu entries
            this.unsavedMenuEntries = [];  // unsaved artifact menu entries
            this.uriTitle = {};
            this.groups = {};
            this.dropdowns = {};
            this.dropdownGroups = {};
            this.ariaGroups = {}; // Groups that need to add prefix to the menu items to clarify the menu item action.
                                  // The menu items are just the artifact type name with while the menu items group title is the action.
                                  // Same artifact type can be in Browse and Create menu items group and the screen reader can't
                                  // distinguish what the menu items are for.

            // The menu is subscribed to tabs update events to update the menu groups for unsaved artifacts,
            // recent viewed page and recent saved queries.  Each menu defines the subActions that it is
            // interested in in the following three types.
            this.doNewArtifact = {};   // subActions that create new artifact in unsaved menu panel
            this.doRecentViewed = {};  // subActions that open a page in recently viewed menu panel
            this.doRecentQueries = {}; // subActions for saved queries in recent queries panel
            
            this.createActionEntriesAdded = {};  // keep track of the action entries that create new artifacts
            this.createActionEntryIndex = 0;     // and their position in the create menu items group
            this.staticMenuItems = {}; // static menu items such as those for browse list views, they are not
                                       // part of the view history but have the current selected indicator
            
            this.canHide = true;        // by default the actionBar entry can be hidden
            this.groupsInfo = {};       // name info of the static menu item groups by the id
            this.allMenuEntries = [];   // all the static menu entries under this actionBar entry in this format:
                                        //  {
                                        //     hidden          <-- do not display if true
                                        //     staticType      <-- true if menu entry is static i.e. no history
                                        //     permission      <-- do not display if false
                                        //     label
                                        //     defaultLabel
                                        //     iconURL
                                        //     action
                                        //     actionArgs
                                        //     groupName
                                        //     groupId
                                        //     queryType
                                        //  }
            
            // The entire menu is configured to be hidden if args.entireMenuHidden is true.  Just skip adding menu items to the UI
            // but keep all the logic and action dispatching i.e. 
            //
        },
        
        postCreate: function(args){
        },
        
        /**
         * convenient method to add an entry to the allMenuEntries
         */
        _addOneMenuEntry: function(hidden, staticType, permission, label, defaultLabel, iconURL, action, actionArgs, groupId, queryType) {
            var entry = {
                hidden: hidden,
                staticType: staticType,
                permission: permission,
                label: label,
                defaultLabel: defaultLabel,
                iconURL: iconURL,
                action: action,
                actionArgs: actionArgs,
                groupId: groupId,
                queryType: queryType
            };
            this.allMenuEntries.push(entry);
        },
        
        /**
         * convenient method to add a menu item group using default display string from the id and add an entry to the groupsInfo
         */
        _addOneMenuItemsGroup: function(groupId) {
            var group = groupId;
            if (groupId == "labelBrowse") {
                group = this.insertBrowseMenuItemsGroups("labelBrowse", null, null, Constants.getMessage("labelBrowse"));
                this.groupsInfo[group] = Constants.getMessage("labelBrowse");
            } else if (groupId == "labelCreate") {
                group = this.insertMenuItemsGroup("labelCreate", null, null, Constants.getMessage("labelCreate"));
                this.createGroupId = group;
                this.groupsInfo[group] = Constants.getMessage("labelCreate");
            } else {
                group = this.insertMenuItemsGroup(groupId, null, null, Constants.getMessage(groupId));
                this.groupsInfo[group] = Constants.getMessage(groupId);
            }
            return group;
        },
        
        /**
         * Get all the static menu entries 
         */
        getAllMenuEntries: function() {
            return this.allMenuEntries;
        },
        
        /**
         * Get all info, not the instances, of the static menu item groups
         */
        getAllGroupsInfo: function() {
            return this.groupsInfo;
        },
        
        /**
         * Check if the menu item of the specified subAction should be hidden
         */
        isHidden: function(subAction) {
            var hidden = false;
            if (this.hiddenMenuItemsConfig) {
                hidden = this.hiddenMenuItemsConfig[subAction] ? true : false;
            }
            return hidden;
        },
        
        initialize: function(args){
            var actionMenuDTO = args.actionMenuDTO;
            if (actionMenuDTO.icon) 
                this._iconURL = actionMenuDTO.icon;
            if (actionMenuDTO.iconHover) 
                this._iconHoverURL = actionMenuDTO.iconHover;
            this.title = Constants.getMessage(actionMenuDTO.name);
            this._contentArea = args.contentArea;
            this._popupMenuArea = args.popupMenuArea;
            this._parent = args.parent;
            this.recentlyViewedViewHistoryType = "com.ibm.rqm.web.ui.ActionBarEntry." + actionMenuDTO.name + ".RecentlyViewed";
            this.recentQueriesViewHistoryType = "com.ibm.rqm.web.ui.ActionBarEntry." + actionMenuDTO.name + ".RecentQueries";
            
            if (args.actionDispatcherName) {
                this.actionDispatcherName = args.actionDispatcherName;
            }
            
            // A action dispatcher name is provided. Invocations of menu entries
            // will be dispatched thru the handler.
            if (this.actionDispatcherName) {
                var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
                var action = actionRegistry.findAction(this.actionDispatcherName);
                
                if (action) {
                    this.actionDispatcher = action._contextObject;
                }
            }
			
			// add subAction specific ignored params
			this.actionDispatcher.addSubActionIgnoredParams("viewTestScript", [ "stepUUID" ]);
            
            // We need to handle the action dispatching to open the page but we don't need to create
            // the menu if the entire menu is configured to be hidden.
            if (!this.entireMenuHidden) {
                // If asMenuBarEntry is true the action bar will be horizontal in the banner
                // nav bar as a menu bar.  Otherwise the action bar entry will vertical at
                // the left of the page.  The action bar entry will be click to drop
                // down (horizontal) or pull out (vertical) the component menu.
                //
                this._asMenuBarEntry = args.asMenuBarEntry;
                
                var verticalHoverMenuItemArg = {
                    iconURL: this._iconURL,
                    iconText: this.title,
                    prevHandler: dojo.hitch(this, "selectPrevActionEntry"),
                    nextHandler: dojo.hitch(this, "selectNextActionEntry")
                };
                
                var hoverMenuItem = this.realize(this._asMenuBarEntry ? null : verticalHoverMenuItemArg);
                this.topDomElement = hoverMenuItem;
            }

            // initialize artifacts/tabs tracking of tabs changes for menus update unless they are skipped
            if (!this.skipUnsavedAndViewedGroups || !this.skipRecentQueriesGroups) {
				// Subscribe to global events
                dojo.subscribe(GlobalEvents.UpdateTabUri, this, "onTabPageUriUpdated");
                dojo.subscribe(GlobalEvents.UpdateTabLabel, this, "onTabPageLabelUpdated");
                dojo.subscribe(GlobalEvents.ArtifactsDeleted, this, "onTabDeleted");

				// Connect to actions from the tab controller that required contentArea to be defined.
                // The "contentArea" attribute is specified in the args from the vertical action bar.  It is not defined
                // with the horizontal nav bar that created this instance.  However, there is a chance that HomePageView
                // may already be loaded i.e. the method initTrackTabChange() in HomePageController is already invoked
                // so the dojo.connect later on will never happen.  Double check if the contentArea is already set
                // by the HomePageView.
                if (!this._contentArea) {
                    this._contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
                }
                
                if (this._contentArea) {
                    this.initTrackArtifactTab();
                } else {
                    // The HomePageView may not be loaded if it is using the horizontal nav bar.  Connect to
                    // the HomePageController which the HomePageView will invoke the initTrackTabChange()
                    // method after the contentArea is created.
                    var controller = dojo.getObject("com.ibm.rqm.web.homePageController");
                    this.deferredInitTrackHandle = this.connect(controller, "initTrackTabChange", dojo.hitch(this, "initTrackArtifactTab"));
                }
            }
            
            // create the menu item entries declared in the constructor
            //
            if (this.createMenuItemsOnInit) {
                this.createMenuItems();
            }
        },
        
        /**
         * Create the menu items in allMenuEntries declared in the constructor
         */
        createMenuItems: function() {
            for (var i = 0; i < this.allMenuEntries.length; i++) {
                var menuItemEntry = this.allMenuEntries[i];
                if (menuItemEntry.permission) {
                    // has valid permission
                    var actionArgs = menuItemEntry.actionArgs;
                    if (menuItemEntry.hidden || this.entireMenuHidden) {
                        // add hidden flag to actionArgs for menu item that is configured to be hidden or the entire menu is configured to be hidden
                        actionArgs.hidden = true;
                    }
                    if (menuItemEntry.staticType) {
                        if (menuItemEntry.staticType == "NON-DISP") {
                            // non-dispatchable clone entry - duplicate entry from other menu such as Browse Test Case from Construction in Execution
                            this.addStaticNonDispatchableMenuEntry(menuItemEntry.label, menuItemEntry.defaultLabel, menuItemEntry.action, menuItemEntry.iconURL, actionArgs, menuItemEntry.groupId, menuItemEntry.queryType);
                        } else {
                            // dispatchable static entry - no history tracking
                            this.addStaticMenuEntry(menuItemEntry.label, menuItemEntry.defaultLabel, menuItemEntry.action, menuItemEntry.iconURL, actionArgs, menuItemEntry.groupId, menuItemEntry.queryType);
                        }
                    } else {
                        this._addMenuEntry(menuItemEntry.label, menuItemEntry.defaultLabel, menuItemEntry.action, menuItemEntry.iconURL, actionArgs, menuItemEntry.groupId, menuItemEntry.queryType);
                    }
                }
            }
        },

        _deferredRestoreViewHistory: function() {
            // restore recently viewed history
            if (!this.skipUnsavedAndViewedGroups) {
                if (this.groups[VIEWED_GROUP_ID]) {
                    this.groups[VIEWED_GROUP_ID].restoringHistory = true;
                    // set provider to trigger showing "loading..." as the menu item before posting the viewhistory server request
                    this.groups[VIEWED_GROUP_ID].set('itemProvider', dojo.hitch(this, "_postRestoreRecentlyViewedHistory"));
                } else {
                    this._postRestoreRecentlyViewedHistory(true);
                }
            }
            // restore recent queries history
            if (!this.skipRecentQueriesGroups) {
                if (this.groups[QUERIES_GROUP_ID]) {
                    this.groups[QUERIES_GROUP_ID].restoringHistory = true;
                    // set provider to trigger showing "loading..." as the menu item before posting the viewhistory server request
                    this.groups[QUERIES_GROUP_ID].set('itemProvider', dojo.hitch(this, "_postRestoreRecentQueriesHistory"));
                } else {
                    this._postRestoreRecentQueriesHistory(true);
                }
            }
        },
        
        _postRestoreRecentlyViewedHistory: function(skipDeferred) {
            var deferred = null;
            if (!skipDeferred) {
                // use deferred object for the menu item group to clear the "loading..." on callback
                deferred = new dojo.Deferred();
                this.groups[VIEWED_GROUP_ID].restoringHistory = true;
            }
            CommonUtil.getViewHistory(this.recentlyViewedViewHistoryType, MAXVIEWEDCOUNT, dojo.hitch(this, "_restoreRecentlyViewedHistory", deferred));
            return deferred;
        },

        _postRestoreRecentQueriesHistory: function(skipDeferred) {
            var deferred = null;
            if (!skipDeferred) {
                // use deferred object for the menu item group to clear the "loading..." on callback
                deferred = new dojo.Deferred();
                this.groups[QUERIES_GROUP_ID].restoringHistory = true;
            }
            CommonUtil.getViewHistory(this.recentQueriesViewHistoryType, MAXVIEWEDCOUNT, dojo.hitch(this, "_restoreRecentQueriesHistory", deferred));
            return deferred;
        },
        
        // callback from the viewHistory service
        _restoreRecentlyViewedHistory: function(deferred, history) {
            if (deferred) {
                // we track the menu items individually, update the menu item group with no items to clear the "loading..."
                if (this.groups[VIEWED_GROUP_ID].restoringHistory)
                    delete this.groups[VIEWED_GROUP_ID].restoringHistory;
                deferred.callback([]);
            }
            this._restoreViewHistory(history, this.groups[VIEWED_GROUP_ID], dojo.hitch(this, "addRecentlyViewedEntry"));
            if (this.getSelected()) {
                // The history is restored when the menu is first opened.  There can be a current menu item selected
                // if the page is loaded from a specific uri of an artifact directly.  The menu item group is in the
                // loading state so the ui of selected won't be set until now.
                this.menu.updateSelected();
            }
        },
        
        // callback from the viewHistory service
        _restoreRecentQueriesHistory: function(deferred, history) {
            if (deferred) {
                // we track the menu items individually, update the menu item group with no items to clear the "loading..."
                if (this.groups[QUERIES_GROUP_ID].restoringHistory)
                    delete this.groups[QUERIES_GROUP_ID].restoringHistory;
                deferred.callback([]);
            }
            this._restoreViewHistory(history, this.groups[QUERIES_GROUP_ID], dojo.hitch(this, "addQueriesEntry"));
        },

        _restoreViewHistory: function(history, group, addEntryFunc) {
            var count = history.count || 0;
            if (count > 0) {
                try {
                    this.restoringViewHistory = true;
                    for (var i = count; i > 0; i--) {
                        var item = history.items[i - 1];
                        if (item && item.label && item.uri) {                            
                            // uri is fully qualified, get the href - uri QM uses in the action dispatcher
                            var uri = CommonUtil.getViewHistoryUri(item.uri);
                            addEntryFunc(item.label, uri, item.iconUri);
                        }
                    }
                } catch (ex) {
                    // just want to make sure any exception are caught so that we can clear this.restoringViewHistory
                }
                delete this.restoringViewHistory;
            } else {
                // no history, hide the group
                group.set('hide', true);
                group.set('wrapHint', WrapHints.NO_WRAP);
            }
        },
        
        saveViewHistory: function(type, label, uri, iconUri, data) {
            if (this.entireMenuHidden) {
                // skip saving history if the entire menu is hidden
                // the page is brought up from other link not from the menu item in this menu
                return;
            }
            
            // Adding menu entry will implicitly update the history, skip if the entry is
            // added from restoring the history.
            if (label && uri && !this.restoringViewHistory) {
                var uriArgs = CommonUtil.getArgsFromUri(uri);
                // static menu item does not participate in the history as well
                var staticMenuItem = this._isStaticMenuItemEntry(uriArgs);
                if (!staticMenuItem) {
                    // filter out subAction specific ignored params in the uri
                    if (this.actionDispatcher) {
                        var ignoredParams = this.actionDispatcher.getSubActionIgnoredParams(uriArgs.subAction);
                        if (ignoredParams) {
                            uri = CommonUtil.removeUriParams(uri, ignoredParams);
                        }
                    }

                    // add to history only if it is not static
                    CommonUtil.addViewHistory(type, label, uri, iconUri, data);
                }
            }
        },

        initTrackArtifactTab: function() {
            if (!this._contentArea) {
                // if deferred init, the content area is set by the HomePageView
                this._contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
            }
            if (!this.trackArtifactTabInitialized && this._contentArea) {
                // connect to the tab controller of the contentArea for setting current and close tabs
                this.connect(this._contentArea.getTabController(), "onSelectChild", dojo.hitch(this, "onTabSelected"));
                this.connect(this._contentArea.getTabController(), "onRemoveChild", dojo.hitch(this, "onTabRemoved"));
                this.trackArtifactTabInitialized = true;

                if (this.deferredInitTrackHandle) {
                    // remove the handle if this is a deferred init
                    this.disconnect(this.deferredInitTrackHandle);
                    delete this.deferredInitTrackHandle;
                }
            }
        },
        
        getSelected: function() {
            return this.currentEntry;
        },
        
        _setSelectedAttr: function(menuItemEntry) {
            if (this.currentEntry) {
                delete this.currentEntry.isCurrent;
            }
            this.currentEntry = menuItemEntry;
            if (this.currentEntry) {
                this.currentEntry.isCurrent = true;
            }
            if (this.menu) {
                this.menu.set('selected', this.currentEntry ? this.currentEntry.menuItem : null);
            }
        },

        // given the uri, move the recently view entry up to the top if it already in the list
        //
        _upExistingEntry: function(uri) {
            var entry = null;
            for (var i = 0; i < this.viewedMenuEntries.length; i++) {
                if (this.viewedMenuEntries[i].key == uri) {
                    entry = this.viewedMenuEntries[i];
                    if (i > 0) {
                        // move the entry to the top, make it more recent
                        this.viewedMenuEntries.splice(i, 1);
                        this.viewedMenuEntries.unshift(entry);
                        // move the menu item to the top if the menu is already created
                        var group = this.groups[VIEWED_GROUP_ID];
                        if (group && entry.menuItem) {
                            this._moveEntryToTop(group, entry);
                        }
                        this.saveViewHistory(this.recentlyViewedViewHistoryType, entry.label, entry.key, entry.icon);
                    }
                    break;
                }
            }
            return entry;
        },
        
        // Subclass can override this method to explicitly skip processing particular page
        // based on the uri on page selected or added, and on label or uri changed.  The
        // default is using the default definition in the common utility on page selected. 
        //
        _skipProcessMenuEntry: function(uri) {
            return CommonUtil.skipProcessSelectedPage(uri);
        },
        
        // For static menu item entries such as those for list views, we skip adding them to the history.
        // This method returns the menu item if the all the uriArgs match those for the static entry.
        //
        _isStaticMenuItemEntry: function(uriArgs) {
            var staticArgs = uriArgs.subAction ? this.staticMenuItems[uriArgs.subAction] : null;
            if (staticArgs) {
                for (var key in uriArgs) {
                    // compare all args in the entry
                    if (key == "action") {
                        // action is the same dispatcher action, subAction is what we use
                        continue;
                    }
                    if (uriArgs[key] != staticArgs[key]) {
                        // there is a difference 
                        if (key == "subAction" && (uriArgs[key].indexOf("nonDisp-") == 0) && (uriArgs[key].slice(8) == staticArgs[key])) {
                            // it is fine if the difference is the subAction of the equivalent non dispatchable subAction
                            continue;
                        } else {
                            // there is a difference
                            return false;
                        }
                    }
                }
                // all uriArgs are the same
                return staticArgs;
            }
            // subAction is not defined or not static
            return false;
        },
        
        onTabSelected: function(page) {
            var uri = page.unfilteredUri || page.uri;
            if (!uri || this._skipProcessMenuEntry(uri)) {
                // can't do anything if there is no uri, should not happen anyway
                // or this uri is explicitly should not be processed
                return;
            }
            
            var uriArgs = CommonUtil.getArgsFromUri(uri);
            var subAction = uriArgs.subAction;
            
            var currentEntry = null;
            var isStaticMenuItem = this._isStaticMenuItemEntry(uriArgs);
            // check if it is static or one of the subAction the menu cares
            if (isStaticMenuItem) {
                // static menu item will set to current only, no history
                currentEntry = isStaticMenuItem;
            } else if (subAction && this.doRecentViewed[subAction]) {
                // update the selected menubar item highlight for this page if necessary
                this._updatePageMenubarItem(page);
                // move the entry to the top, make it more recent if it is already there
                currentEntry = this._upExistingEntry(uri);
                
                if (!currentEntry) {
                    // the viewed entry may be modified and in the unsaved artifacts panel
                    for (var i = 0; i < this.unsavedMenuEntries.length; i++) {
                        if (this.unsavedMenuEntries[i].key == uri) {
                            currentEntry = this.unsavedMenuEntries[i];
                            break;
                        }
                    }
                }
                if (!currentEntry) {
                    // This is a page that the menu interested in and selected but there is no recently viewed entry.
                    // It can happened if the menu support view all artifacts but not create or view individual artifact.
                    // The onTabAdded() method add the recently viewed entry in the menu of the originated menu.  Need to
                    // check if this is a clone and fake an page added to add the recent query entry here.
                    if (this.doRecentViewed["nonDisp-" + subAction]) {
                        this.onTabAdded(page);
                        return;
                    }
                }
                if (!currentEntry && subAction == "viewUserHome") {
                    // The home page is added on startup in the HomePageView and selected implicitly even the uri is
                    // for other artifact.  In that case we don't want to add an recently viewed entry. i.e. the
                    // subaction in current uri is not "viewUserHome".  However, explicit user click on the home page
                    // tab to select will get here before the uri is updated to "viewUserHome", which appears as the
                    // same condition as on startup.  Add a little delay to allow the uri change before checking.
                    setTimeout(dojo.hitch(this, function() {
                        var currentUri = CommonUtil.getDecodedLocationHash();
                        var currentUriArgs = CommonUtil.getArgsFromUri(currentUri);
                        if (currentUriArgs.subAction == "viewUserHome") {
                            this.onTabAdded(page);
                        }
                    }), 100);
                }
            } else if (subAction && this.doNewArtifact[subAction]) {
                // update the selected menubar item highlight for this page if necessary
                this._updatePageMenubarItem(page);
                
                for (var i = 0; i < this.unsavedMenuEntries.length; i++) {
                    if (this.unsavedMenuEntries[i].key == uri) {
                        currentEntry = this.unsavedMenuEntries[i];
                        // there is no limit with the unsaved entries so keep the entries in the order
                        // that they are created not by recently viewed
                        break;
                    }
                }
            } else if (subAction && this.doRecentQueries[subAction]) {
                // update the selected menubar item highlight for this page if necessary
                this._updatePageMenubarItem(page);

                // Saved queries must have a "updateAction" param in the uri unless this is from the UI clicking
                // on the opened tab which have all params filtered out in the uri.  In that case select the
                // last saved query entry viewed.
                var getLastVIewed = (uriArgs.updateAction == undefined);
                // The same page is used for saved queries with different filters params.  The uri of the page is filtered
                // with the subAction in the hash only.  Need to get the uri from the location hash with all the params.
                var unfilteredUri = page.unfilteredUri || CommonUtil.getDecodedLocationHash();
                var unfilteredUriArgs = CommonUtil.getArgsFromUri(unfilteredUri);
                if (subAction == unfilteredUriArgs.subAction) {
                    // The location hash may not be set by the saved queries, such as user clicks on the tab to bring it to the
                    // front.  We only use the location hash if it the same subAction i.e. only adding the missing filter params.
                    getLastVIewed = (unfilteredUriArgs.updateAction == undefined);
                }
                for (var i = 0; i < this.queriesMenuEntries.length; i++) {
                    if ((this.queriesMenuEntries[i].key == unfilteredUri) || (getLastVIewed && this.queriesMenuEntries[i].key.indexOf(uri) != -1)){
                        currentEntry = this.queriesMenuEntries[i];
                        if (i > 0) {
                            // move the entry to the top, make it more recent
                            this.queriesMenuEntries.splice(i, 1);
                            this.queriesMenuEntries.unshift(currentEntry);
                            // move the menu item to the top if the menu is already created
                            var group = this.groups[QUERIES_GROUP_ID];
                            if (group && currentEntry.menuItem) {
                                this._moveEntryToTop(group, currentEntry);
                            }
                            this.saveViewHistory(this.recentQueriesViewHistoryType, currentEntry.label, currentEntry.key, currentEntry.icon);
                        }
                        break;
                    }
                }
                if (!currentEntry) {
                    // This is a page that the menu interested in and selected but there is no recent query entry.
                    // It can happened if the menu has cloned menu entry.  The onTabAdded() method add the recent
                    // query entry in the menu of the original menu.  Need to check if this is a clone and fake
                    // to add the recent query entry here.
                    if (this.doRecentQueries["nonDisp-" + subAction]) {
                        this.onTabAdded(page);
                        return;
                    }
                }
            }
            this.set('selected', currentEntry);
        },
        
        onTabRemoved: function(page) {
            // a tab is closed
            var uri = page.uri;
            if (!uri) {
                // can't do anything if there is no uri, should not happen anyway
                return;
            }
            var uriArgs = CommonUtil.getArgsFromUri(uri);
            var subAction = uriArgs.subAction;
            
            // we only care when a dirty tab is closed
            if (subAction && this.doNewArtifact[subAction]) {
                // close a new artifact
                this.removeUnsavedEntry(uri);
            } else if (subAction && this.doRecentViewed[subAction]) {
                // remove entry from unsaved panel
                var entry = this.removeUnsavedEntry(uri);
                if (entry) {
                    // There is unsaved entry i.e. closed a dirty editor of an existing recently viewed artifact.
                    // Restore recently viewed entry.
                    var defaultInfo = this.doRecentViewed[subAction];
                    var title = entry.label;
                    if (title) {
                        if (title.indexOf("* ") == 0) {
                            // remove dirty indicator added from setEntryDirty()
                            title = title.substring(2);
                        }
                    }
                    else {
                        title = this.uriTitle[uri] ? this.uriTitle[uri] : defaultInfo ? defaultInfo.label : Constants.getMessage("labelArtifact");
                    }
                    var icon = entry.icon ? entry.icon : defaultInfo ? defaultInfo.icon : this.blankIcon;
                    this.addRecentlyViewedEntry(title, uri, icon);
                }
            }
        },

        onTabAdded: function(page) {
            // our safety net for just in case somehow we didn't get triggered to connect
            // to the tabs controller for tracking, initialize the tracking here
            if (!this.trackArtifactTabInitialized && this.deferredInitTrackHandle) {
                this.initTrackArtifactTab();
            }

            var uri = page.uri;
            if (!uri || this._skipProcessMenuEntry(uri)) {
                // can't do anything if there is no uri, should not happen anyway
                // or this uri is explicitly should not be processed
                return;
            }
            var uriArgs = CommonUtil.getArgsFromUri(uri);
            var subAction = uriArgs.subAction;
            
            // update the selected menubar item highlight for this page if necessary
            if (!this.doRecentViewed["nonDisp-" + subAction]) {
                // Skip nonDispatchable clone subAction such as viewTestCase in Execution, let
                // the main subAction under Construction have the selection.
                this._updatePageMenubarItem(page);
            }
            
            var argsTitle = uriArgs.label ? decodeURIComponent(uriArgs.label) : (uriArgs.name ? decodeURIComponent(uriArgs.name) : "");
            var pageTitle = argsTitle || page.name;
            if (!argsTitle && page.title && page.title != Constants.getMessage("labelLoading")) {
                pageTitle = page.title;
            }
            if (subAction && this.doNewArtifact[subAction]) {
                // a new artifact is added
                var defaultInfo = this.doNewArtifact[subAction];
                // Use default label unless one is specified in args.  Do not use the page/tab title which is
                // typically "Create <artifact>" while the menu item labels do not use action verb "Create"
                var title = argsTitle ? argsTitle : defaultInfo ? defaultInfo.label : Constants.getMessage("labelArtifact");
                var icon = defaultInfo ? defaultInfo.icon : this.blankIcon;
                this.addUnsavedEntry(title, uri, icon);
                // indicate that the new tab is for a new artifact that won't be auto closed in tab-less mode
                page.initiallyNew = true;
            } else if (subAction && this.doRecentViewed[subAction]) {
                // move the entry to the top, make it more recent if it is already there
                if (!this._upExistingEntry(uri)) {
                    // otherwise add entry for recently viewed
                    var defaultInfo = this.doRecentViewed[subAction];
                    var title = pageTitle ? pageTitle : defaultInfo ? defaultInfo.label : Constants.getMessage("labelArtifact");
                    var icon = defaultInfo ? defaultInfo.icon : this.blankIcon;
                    this.addRecentlyViewedEntry(title, uri, icon);
                }
            } else if (subAction && this.doRecentQueries[subAction]) {
                    // The location hash may not be set by the saved queries, such as user clicks on the tab to bring it to the
                    // front.  We only use the location hash if it the same subAction i.e. only adding the missing filter params.
                var unfilteredUri = CommonUtil.getDecodedLocationHash();
                var unfilteredUriArgs = CommonUtil.getArgsFromUri(unfilteredUri);
                if (subAction == unfilteredUriArgs.subAction) {
                    // add the missing filter params
                    unfilteredUriArgs.uri = unfilteredUri;
                    this._updateRecentQueriesMenu(unfilteredUriArgs);
                } else {
                    uriArgs.uri = uri;
                    this._updateRecentQueriesMenu(uriArgs);
                }
            }
        },
        
        onTabDeleted: function(artifactType, artifacts, uri) {
            // an artifact is deleted, remove the corresponding menu entry
            if (uri) {
                // we only care if the specified uri matches to the menu entry
                var uriArgs = CommonUtil.getArgsFromUri(uri);
                var subAction = uriArgs.subAction;
                if (subAction && this.doNewArtifact[subAction]) {
                    this.removeUnsavedEntry(uri);
                } else if (subAction && this.doRecentViewed[subAction]) {
                    this.removeRecentlyViewedEntry(uri);
                } else if (subAction && this.doRecentQueries[subAction]) {
                    this.removeQueriesEntry(uri);
                }
            }
        },

        onTabPageLabelUpdated: function(uri, newLabel, iconClass, dirty, skipCompactLabel) {
            if (this._skipProcessMenuEntry(uri)) {
                // this uri is explicitly should not be processed
                return;
            }
            // tab label is updated
            var uriArgs = CommonUtil.getArgsFromUri(uri);
            var subAction = uriArgs.subAction;
            
            newLabel = CommonUtil.encodeHTMLTags(newLabel);
                        
            if (subAction && this.doRecentViewed[subAction]) {
                if (dirty) {
                    // recently viewed artifact is modified with dirty flag set
                    this.setEntryDirty(uri, newLabel, iconClass);
                } else {
                    // recently viewed dirty artifact is saved or canceled changes
                    if (newLabel) {
                        // save the label for this uri
                        this.uriTitle[uri] = newLabel;
                    }
                    // remove dirty entry
                    this.removeUnsavedEntry(uri);
                    // create new recently viewed entry
                    var defaultInfo = this.doRecentViewed[subAction];
                    var title = newLabel ? newLabel : this.uriTitle[uri] ? this.uriTitle[uri] : defaultInfo ? defaultInfo.label : Constants.getMessage("labelArtifact");
                    var icon = defaultInfo ? defaultInfo.icon : this.blankIcon;
                    this.addRecentlyViewedEntry(title, uri, icon);
                }
            } else if (subAction && this.doNewArtifact[subAction]) {
                // label for new artifact is updated when saved, keep track of the title and let the uri changed handler
                // handle the menu entries update
                var defaultInfo = this.doNewArtifact[subAction];
                var title = newLabel ? newLabel : defaultInfo ? defaultInfo.label : Constants.getMessage("labelArtifact");
                this.uriTitle[uri] = title;
                if (defaultInfo.updateUnsavedLabelOnPageLabelUpdated) {
                    for (var i = 0; i < this.unsavedMenuEntries.length; i++) {
                        var menuEntries = this.unsavedMenuEntries[i];
                        if (menuEntries.key == uri) {
                            menuEntries.label = title;
                            if (menuEntries.menuItem) {
                                menuEntries.menuItem.set('label', title);
                                break;
                            }
                        }
                    }
                }
            }
        },

        onTabPageUriUpdated: function(oldUri, newUri) {
            if (this._skipProcessMenuEntry(oldUri)) {
                // this uri is explicitly should not be processed
                return;
            }
            // tab uri is updated
            var oldUriArgs = CommonUtil.getArgsFromUri(oldUri);
            var newUriArgs = CommonUtil.getArgsFromUri(newUri);
            var subAction = oldUriArgs.subAction;
            if (subAction && this.doRecentQueries[subAction]) {
                // saved queries use the same subAction and tab with different params in the uri that are changed
                // remove existing recent query entry if one already exists
                this.removeQueriesEntry(newUri);
                // add a new recent query entry to the top of the menu
                newUriArgs.uri = newUri;
                this._updateRecentQueriesMenu(newUriArgs);
            } else if (subAction && this.doNewArtifact[subAction]) {
                if (newUriArgs.subAction == subAction) {
                    // same subAction but uri changed, execution does that to add the result id on new execution
                    var group = this.groups[UNSAVED_GROUP_ID];
                    for (var i = 0; i < this.unsavedMenuEntries.length; i++) {
                        var menuEntries = this.unsavedMenuEntries[i];
                        if (menuEntries.key == oldUri) {
                            menuEntries.key = menuEntries.href = menuEntries.callbackData = newUri;
                            if (menuEntries.menuItem && this.menu) {
                                // menu item is already created so need to update its href but...
                                // can't get to the inner anchor dom to update the href so recreate the entry
                                this.menu.addMenuItemUpdateTask(function(){
                                    var menuItem = group.remove(menuEntries.menuItem);
                                    var menuItemArgs = {
                                        label: menuItem.label,
                                        icon: menuItem.icon,
                                        href: newUri
                                    }
                                    menuEntries.menuItem = null;
                                    var updatedMenuItem = group.addItemByArgs(menuItemArgs, i);
                                    dojo.addClass(updatedMenuItem.containerNode, "j-dirty");
                                    menuEntries.menuItem = updatedMenuItem;
                                    menuItem.destroy();
                                });
                            }
                            break;
                        }
                    }
                } else {
                    // different subAction when new artifact is saved, remove unsaved menu entry
                    var entry = this.removeUnsavedEntry(oldUri);
                    // get title from onTabPageLabelUpdated() before uri changed
                    var label = this._getManagedTitle(newUriArgs.label, oldUri, newUri);
                    // create an recently viewed entry
                    var defaultInfo = this.doRecentQueries[newUriArgs.subAction];
                    var title = label ? label : defaultInfo ? defaultInfo.label : Constants.getMessage("labelArtifact");
                    var icon = entry.icon ? entry.icon : defaultInfo ? defaultInfo.icon : this.blankIcon;
                    this.addRecentlyViewedEntry(title, newUri, icon);
                    // select the current tab again to trigger onTabSelected() for other menu
                    // that is interested in the artifact for "cloned" recently viewed
                    this._contentArea.showPageByUri(newUri);
                }
            }
        },

        // internal method to update the menubar item selection only if the page does not
        // have the associated menubar item info i.e. this is delageted from the ContentArea
        _updatePageMenubarItem: function(page) {
            if (!page.fromMenuItemEntry) {
                // ContentArea delegated to set the menuItemEntry here based on the subAction
                page.fromMenuItemEntry = this;
                this.updateMenubarItem();
            }
        },

        // Set the menubar item of this actionBarEntry as currently selected with highlight
        // css.  The Foundation manages the highlight based on the action of the page, but
        // sice we only have a single page and manage our own tabs we need to manually update
        // the menubar item selection too.
        //
        updateMenubarItem: function() {
            var navbar = dojo.getObject("com.ibm.rqm.web.globalNavBar");
            if (navbar.selectedMenubarItem) {
                // clear currently selected menubar item and remove css
                dojo.removeClass(navbar.selectedMenubarItem.domNode, "selected");
            }
            // set new selected menubar item and add css
            var menubarItem = this.hoverMenu;
            navbar.selectedMenubarItem = menubarItem;
            if (menubarItem) {
                // this.hoverMenu might be null if the entire menu is configured to be hidden
                dojo.addClass(navbar.selectedMenubarItem.domNode, "selected");
            }
        },

        initializeMenu: function(){
            // override to initialize menu
        },
        
        uninitialize: function(){
            this.topDomElement = null;
            dojo.forEach(this.menuEntries.concat(this.viewedMenuEntries, this.unsavedMenuEntries, this.queriesMenuEntries), function(entry) {
                entry.menuItem = null;
            });
            for (var titleKey in this.uriTitle) {
                delete(this.uriTitle[titleKey]);
            }
            for (var dropdownKey in this.dropdownGroups) {
                delete this.dropdownGroups[dropdownKey].menuEntries;
                delete this.dropdownGroups[dropdownKey].menuItems;
            }
            for (var groupKey in this.groups) {
                delete(this.groups[groupKey]);
            }
        },
        
        _loadClassAsync: function(className, callback){
            CommonUtil.loadClassAsync(className, callback);
        },
        
        // Utility method to parse the service URL string of the provider to return the host name. External provider
        // action entries can use this method to extract the full domain hostname from the service URL. 
        //
        _getProviderHost: function(projectLink){
            var providerUrl = projectLink.servicesUrl;
            var hostStart = providerUrl.indexOf("//");
            var hostEnd = providerUrl.indexOf("/", hostStart + 2);
            var providerHost = providerUrl.substring(hostStart + 2, hostEnd);
            return providerHost;
        },
        
        // Create artifact actions use the datetime as a temporary unique key for the editor in the uri.
        // The uri is the href of the <a> link of the dom node in the menu item.  A new menu item with
        // new datetime unique key in the href is needed so that invoking the menu item action will have
        // a different editor with different uri.  This method is invoked after the create artifact action
        // is invoked.  See _addMenuEntry().
        // Note that position in the params is the index position of the entry in the menu item group.  The
        // order of the entries in the items array of the menu item group is not the order shown in the display.
        updateCreateArtifactMenuItemEntry: function(actionArgs, position) {
            var createGroup = this.createGroupId ? this.groups[this.createGroupId] : null;
            if (createGroup) {
                // only work on the create artifact menu items group entries
                var menuItems = createGroup.items || [];
                var menuItemsCount = menuItems.length;
                for (var i = 0; i < menuItemsCount; i++) {
                    var menuItem = menuItems[i];
                    var href = menuItem.href;
                    if (href) {
                        // get the subAction from the href
                        var args = CommonUtil.getArgsFromUri(href);
                        if ((args.subAction == actionArgs.subAction) && args.tq) {
                            // update the href only if there is a "tq" arg
                            var oldKey = "tq=" + args.tq;
                            var newKey = "tq=" + Date.parse(new Date().toString());
                            var menuItemArgs = {
                                label: menuItem.label,
                                icon: menuItem.icon,
                                href: href.replace(oldKey, newKey)
                            }
                            if (this.menu) {
                                this.menu.addMenuItemUpdateTask(function() {
                                    var entry = createGroup.remove(menuItem);
                                    createGroup.addItemByArgs(menuItemArgs, position);
                                    entry.destroy();
                                });
                            }
                            return;
                        }
                    }
                }
            }
        },
        
        _routeAction: function(actionArgs){
            var uri = PlanningUtil.getRouteActionUri(actionArgs);
            window.location.hash = uri;
        },
        
        removeAllDropdownMenuItems: function(inDropdown){
            var dropdownGroup = this.dropdownGroups[inDropdown];
            if (dropdownGroup) {
                dropdownGroup.menuItems = [];
                dropdownGroup.menuEntries = [];
            }
            // update to refresh all menu items groups
            this.updateSavedQueries();
        },
        
        _addFilterQueryMenuItem: function(title, method, actionArgs, inDropdown){
            var subAction = actionArgs.subAction;
            if (this.actionDispatcher && inDropdown && subAction && (subAction.indexOf("nonDisp-") != 0)) {
                // inDropdown is the saved query type, register the subAction of the query type
                // in the action dispatcher except we skip non-dispatchable subActions
                this.actionDispatcher.registerSavedQueryAction(inDropdown, actionArgs.subAction);
            }
            
            // no need to update the UI when the entire menu is hidden
            if (!this.entireMenuHidden) {
                var queries = FiltersQueriesClient.getInMenuQueries(inDropdown);
                for (var i = 0; i < queries.length; i++) {
                    var query = queries[i];
                    if (query.scope == FiltersQueriesClient.publicScopeId) {
                        this._addOneFilterQueryMenuItem(query, method, actionArgs, inDropdown, /* isPublic */ true);
                    }
                    else {
                        this._addOneFilterQueryMenuItem(query, method, actionArgs, inDropdown, /* isPublic */ false);
                    }
                }
                this.updateSavedQueries();
            }
        },
        
        _addOneFilterQueryMenuItem: function(query, method, actionArgs, inDropdown, isPublic){
            var newArgs = dojo.mixin({}, actionArgs);
            var title = query.title;
            var subAction = actionArgs.subAction;
            newArgs.subAction = subAction;
            newArgs.updateAction = "update-saved-query-selection";
            newArgs.sqScope = dojo.clone(query.scope);
            newArgs.sqItemId = dojo.clone(query.itemId);
            newArgs.sqTitle = encodeURIComponent(dojo.clone(query.title));
            newArgs.isPublic = isPublic;
            newArgs.inDropdown = inDropdown;
            if (query.folder) {
                newArgs.sqFolder = encodeURIComponent(dojo.clone(query.folder));
            }
            var icon = FiltersQueriesClient.getQueryTypeIconURL(inDropdown);
            this._addMenuEntry(title, "", method, icon, newArgs, null, inDropdown);
        },

        // Saved queries need to include query specific params in the uri besides the subAction.
        // The order of the params in the hash is ordered in _addOneFilterQueryMenuItem() above.
        // This method append the saved query params to produces the same uri of the saved query
        // as if it is invoked from the ActionDispatcher.
        _appendFilterQueryUriHash: function(hash, args) {
                if (args.updateAction) {
                    hash += "&updateAction=" + args.updateAction;
                }
                if (args.sqScope) {
                    hash += "&sqScope=" + args.sqScope;
                }
                if (args.sqItemId) {
                    hash += "&sqItemId=" + args.sqItemId;
                }
                if (args.sqTitle) {
                    hash += "&sqTitle=" + encodeURIComponent(args.sqTitle);
                }
                if (args.isPublic) {
                    hash += "&isPublic=" + args.isPublic;
                }
                if (args.inDropdown) {
                    hash += "&inDropdown=" + args.inDropdown;
                }
                if (args.sqFolder) {
                    hash += "&sqFolder=" + encodeURIComponent(args.sqFolder);
                }
            return hash;
        },

        _updateRecentQueriesMenu: function(args) {
            var uri = "";
            if (args.uri) {
                uri = args.uri;
            } else {
                // uri is not defined in the args, use location hash
                uri = CommonUtil.getDecodedLocationHash();
            }
            // add new updated entry at the top
            var defaultInfo = this.doRecentQueries[args.subAction];
            var title = args.sqTitle ? decodeURIComponent(args.sqTitle) : args.filterDisplayLabel ? decodeURIComponent(args.filterDisplayLabel) : defaultInfo ? defaultInfo.label : Constants.getMessage("labelAllView");
            var icon = args.inDropdown ? FiltersQueriesClient.getQueryTypeIconURL(args.inDropdown) : defaultInfo ? defaultInfo.icon : FiltersQueriesClient.getQueryTypeIconURL(null);

            // The view all page may not have "updateAction" in args if tab is selected from user click on the tab.
            // Need to append it to the uri or the same view all page will have another entry.
            var uriArgs = CommonUtil.getArgsFromUri(uri);
            var count = 0;
            for (var key in uriArgs) {
                count++;
            }
            if ((count == 2) && uriArgs['action'] && uriArgs['subAction']) {
                // this saved query is view all page with only action and subAction
                uri += "&updateAction=clear-filter-selection";
            }
            this.addQueriesEntry(title, uri, icon);
        },
        
        _trackDropdownMenuEntry: function(itemName, method, iconURL, actionArgs, inGroup, inDropdown, nonDispatchable){
            // Cache the view artifact queries info to recreate menu items.  Same menu can be appears in different fly-out,
            // such as view test cases, that is convenient but can not be dispatchable with already registered action.
            var inDropdownKey = nonDispatchable ? "nonDispatchable-" + inDropdown : inDropdown;
            var dropdownEntry = dojo.getObject(inDropdownKey);
            if (!dropdownEntry) {
                dojo.setObject(inDropdownKey, {
                    title: itemName,
                    actionbarEntry: this,
                    method: method,
                    actionArgs: actionArgs
                });
            }
        },
        
        updateSavedQueries: function() {
            var privateGroup = this.groups[MY_QUERIES_GROUP_ID];
            var publicGroup = this.groups[SHARED_QUERIES_GROUP_ID];
            
            if (!privateGroup) {
                // nothing to do for menu without any saved queries i.e. saved queries menu items groups are not created
                return;
            }
            if (this.menu) {
                // clear the saved queries menu items groups if menu is already created
                privateGroup.set('items', null);
                publicGroup.set('items', null);
            }
            
            var privateQueriesCount = 0;
            var publicQueriesCount = 0;
            
            var maxCount = PlanningUtil.getPromotedSavedQueriesMaxCount();
            
            for (var dropdownKey in this.dropdownGroups) {
                var dropdownGroup = this.dropdownGroups[dropdownKey];
                var menuEntries = dropdownGroup.menuEntries;
                dropdownGroup.menuItems = [];
                for (var i = 0; i < menuEntries.length; i++) {
                    var menuEntry = menuEntries[i];
                    if (menuEntry.isPublic) {
                        publicQueriesCount++;
                    } else {
                        privateQueriesCount++;
                    }
                    var group = menuEntry.isPublic ? publicGroup : privateGroup;
                    var entry = group.addItemByArgs(menuEntry);
                    dropdownGroup.menuItems.push(entry);
                }
            }
            
            if (privateQueriesCount > maxCount) {
                privateGroup.set('groupTitle', Constants.getMessage("labelBrowseMyQueriesMaxOver", privateQueriesCount, maxCount));
            } else if (privateQueriesCount == maxCount) {
                privateGroup.set('groupTitle', Constants.getMessage("labelBrowseMyQueriesMax", privateQueriesCount));
            } else {
                privateGroup.set('groupTitle', Constants.getMessage("labelBrowseMyQueries", privateQueriesCount));
            }
            if (publicQueriesCount > maxCount) {
                publicGroup.set('groupTitle', Constants.getMessage("labelBrowseSharedQueriesMaxOver", publicQueriesCount, maxCount));
            } else if (publicQueriesCount == maxCount) {
                publicGroup.set('groupTitle', Constants.getMessage("labelBrowseSharedQueriesMax", publicQueriesCount));
            } else {
                publicGroup.set('groupTitle', Constants.getMessage("labelBrowseSharedQueries", publicQueriesCount));
            }

            // update the display of the menu item groups panels
            this._updateGroupsDisplay();
        },
        
        isPromotedSavedQueriesMax: function(queryType, isPublic) {
            if (this.skipRecentQueriesGroups) {
                // this menu doesn't support saved queries
                return false;
            }
            var dropdownGroup = this.dropdownGroups[queryType];
            if (dropdownGroup == undefined) {
                // this menu doesn't handle this type of query
                return false;
            }
            var maxCount = PlanningUtil.getPromotedSavedQueriesMaxCount();
            var group;
            if (isPublic) {
                group = this.groups[SHARED_QUERIES_GROUP_ID];
            } else {
                group = this.groups[MY_QUERIES_GROUP_ID];
            }
            if (group) {
                return group && group.items && group.items.length >= maxCount;
            } else {
                // menu is not created yet, check the menuEntries
                var menuEntries = dropdownGroup.menuEntries;
                var privateQueriesCount = 0;
                var publicQueriesCount = 0;
                for (var i = 0; i < menuEntries.length; i++) {
                    var menuEntry = menuEntries[i];
                    if (menuEntry.isPublic) {
                        publicQueriesCount++;
                    } else {
                        privateQueriesCount++;
                    }
                }
                if (isPublic) {
                    return publicQueriesCount >= maxCount;
                } else {
                    return privateQueriesCount >= maxCount;
                }
            }
        },
        
        _composeMenuItemArgs: function(name, callback, iconURL, callbackData, inGroup, inDropdown, href) {
            var args = {
                label: CommonUtil.encodeHTMLTags(name),
                icon: iconURL,
                inGroup: inGroup,
                inDropdown: inDropdown
            };
            // callback is either a function or link href (string)
            if (dojo.isString(callback)) {
                args.href = callback;
            }
            else {
                args.callbackMethod = callback;
                args.callbackData = callbackData;
                args.onClick = function(){
                    this.callbackMethod(this.callbackData);
                };
            }
            if (href) {
                args.href = href;
            }
            return args;
        },
        
        _addEntry: function(name, callback, iconURL, callbackData, inGroup, inDropdown, href){
            if (this.entireMenuHidden || (callbackData && callbackData.hidden)) {
                // skip adding the menu item to the UI if it is configured to be hidden
                return;
            }
            var args = this._composeMenuItemArgs(name, callback, iconURL, callbackData, inGroup, inDropdown, href);
            if (inGroup || !inDropdown) {
                this.menuEntries.push(args);
                if (this.menu) {
                    this._addToMenu(this.menu, args, inGroup, inDropdown);
                }
            }
            
            if (inDropdown) {
                var dropdownGroup = this.dropdownGroups[inDropdown];
                if (dropdownGroup == undefined) {
                    // the very first inDropdown entry is the "View all" entry in the Browse menu items group created above
                    dropdownGroup = this.dropdownGroups[inDropdown] = {
                        index: this.dropdownGroupsCount++,
                        menuEntries: [],
                        menuItems: []
                    };
                } else {
                    args.isPublic = callbackData.isPublic;
                    dropdownGroup.menuEntries.push(args);
                }
            }
        },
        
        // Wrapper method to _addMenuEntry to set the entry as a static menu item entry.  Static menu item entry
        // would not appear in the history but still get the current indicator when the corresponding page is displayed.
        //
        addStaticMenuEntry: function(itemName, defaultLabel, method, iconURL, actionArgs, inGroup, inDropdown, href) {
            var subAction = actionArgs ? actionArgs.subAction : null;
            if (subAction) {
                this.staticMenuItems[subAction] = actionArgs;
                this._addMenuEntry(itemName, defaultLabel, method, iconURL, actionArgs, inGroup, inDropdown, href);
            }
        },
        
        // In order to have the right click menu to open the menu item as link, href must be used in the jazz menu item.
        // The action dispatcher uses href as uri to dispatch the registered action so the href is generated.  To not to
        // use href and use the onClick to invoke the action, specify null for the href to skip.
        _addMenuEntry: function(itemName, defaultLabel, method, iconURL, actionArgs, inGroup, inDropdown, href){
            var bindName = (actionArgs && actionArgs.subAction) ? actionArgs.subAction : null;
            // Only saved queries are inDropdown.  The others will depending on the group ID to determine
            // which panel the menu item would contribute to.  A null value for defaultLabel is to skip
            // participating in the recent menu panel completely.
            if (defaultLabel !== null) {
                var itemType = inDropdown ? this.doRecentQueries : (inGroup && inGroup == this.createGroupId) ? this.doNewArtifact : this.doRecentViewed;
                // don't add static menu items to the history except for saved queries (inDropdown) because the saved queries
                // use the same subAction as the "Browse"/"View All" entry
                if (bindName && (inDropdown || (!this.staticMenuItems[bindName] && !this._isStaticMenuItemEntry(actionArgs)))) {
                    // saved queries use the same subAction so only register once that is the first "View All" entry
                    if ((!this.skipUnsavedAndViewedGroups || !this.skipRecentQueriesGroups) && (!inDropdown || (itemType[bindName] == undefined))) {
                        itemType[bindName] = {
                            'icon': iconURL ? iconURL : this.blankIcon,
                            'label': defaultLabel ? defaultLabel : itemName
                        };
                    }
                }
            }
            if (!bindName) {
                bindName = itemName;
            }
            if (inDropdown) {
                // track the dropdown entry to allow dynamic changing of the menu items
                this._trackDropdownMenuEntry(itemName, method, iconURL, actionArgs, inGroup, inDropdown);
            }
            
            // route the handling of the menu invocation to the action handler
            // if one is defined.
            if (this.actionDispatcher && dojo.isFunction(method)) {
                if (!href && (href !== null)) {
                    href = PlanningUtil.getRouteActionUri(actionArgs);
                    
                    if (inGroup && (inGroup == this.createGroupId) && (actionArgs.tq != undefined)) {
                        if (this.createActionEntriesAdded[actionArgs.subAction] == undefined) {
                            // set the position of a new action entry to be added to the menu
                            this.createActionEntriesAdded[actionArgs.subAction] = this.createActionEntryIndex++;
                        }
                        // This menu entry creates new artifact and use datetime as the unique key for the uri.
                        // Bind the method and associated actionName/itemName with additional update in the action dispatcher.
                        this.actionDispatcher.registerDispatchAction(bindName, dojo.hitch(this, function(){
                            // After the action is invoked and "used" the datetime unique key, the menu item should be updated to have
                            // a new unique key so that creating another artifact will have a different editor with different uri.
                            method();
                            this.updateCreateArtifactMenuItemEntry(actionArgs, this.createActionEntriesAdded[actionArgs.subAction]);
                        }));
                    } else {
                        // bind the method and associated actionName/itemName in the action dispatcher.
                        this.actionDispatcher.registerDispatchAction(bindName, method);
                    }
                }
                if (href === null) {
                    // not using href, use onClick to route the action
                    this._addEntry(itemName, dojo.hitch(this, "_routeAction"), iconURL, actionArgs, inGroup, inDropdown, href);
                } else {
                    this._addEntry(itemName, "", iconURL, actionArgs, inGroup, inDropdown, href);
                }
            }
            else if (!actionArgs || !actionArgs.hidden) {
                // no actionArgs is used here so check if it should be hidden/skip adding entry to the menu
                this._addEntry(itemName, method, iconURL, null, inGroup, inDropdown, href);
            }
        },
        
        // Wrapper method to _addNonDispatchableMenuEntry to set the entry as a static menu item entry.  Static menu item entry
        // would not appear in the history but still get the current indicator when the corresponding page is displayed.
        //
        addStaticNonDispatchableMenuEntry: function(itemName, defaultLabel, method, iconURL, actionArgs, inGroup, inDropdown) {
            var subAction = actionArgs ? actionArgs.subAction : null;
            if (subAction && !this.staticMenuItems[subAction]) {
                this.staticMenuItems[subAction] = actionArgs;
                this._addNonDispatchableMenuEntry(itemName, defaultLabel, method, iconURL, actionArgs, inGroup, inDropdown);
            }
        },
        _addNonDispatchableMenuEntry: function(itemName, defaultLabel, method, iconURL, actionArgs, inGroup, inDropdown){
            // Only saved queries are inDropdown.  The others will depending on the group ID to determine
            // which panel the menu item would contribute to.  A null value for defaultLabel is to skip
            // participating in the recent menu panel completely.
            if (defaultLabel !== null) {
                var itemType = inDropdown ? this.doRecentQueries : (inGroup && inGroup == this.createGroupId) ? this.doNewArtifact : this.doRecentViewed;
                var bindName = (actionArgs && actionArgs.subAction) ? actionArgs.subAction : null;

                if (bindName && (inDropdown || (!this.staticMenuItems[bindName] && !this._isStaticMenuItemEntry(actionArgs)))) {
                    if ((!this.skipUnsavedAndViewedGroups || !this.skipRecentQueriesGroups) && (!inDropdown || (itemType[bindName] == undefined))) {
                        itemType[bindName] = {
                            'icon': iconURL ? iconURL : this.blankIcon,
                            'label': defaultLabel ? defaultLabel : itemName
                        };
                        // add a copy for non-dispatchable entry
                        itemType["nonDisp-" + bindName] = {
                            'icon': iconURL ? iconURL : this.blankIcon,
                            'label': defaultLabel ? defaultLabel : itemName
                        };
                    }
                }
            }
            if (inDropdown) {
                // Track the dropdown entry to allow dynamic changing of the menu items.  Non-dispatchable action
                // is used for clone, such as view test suites, which the main action is in another fly-out menu.
                this._trackDropdownMenuEntry(itemName, method, iconURL, actionArgs, inGroup, inDropdown, true);
            }
            
            // Add menu entry without going through the action handler.  This is used
            // for action that redirect the page to a new URL and should not have the
            // dispatcher create a URL for a page to do the redirecting.
            //
            this._addEntry(itemName, method, iconURL, actionArgs, inGroup, inDropdown);
        },
        
        // wrapper to register a dispatch action and add the subAction to contribute to the recent viewed menu panel
        registerRecentViewedAction: function(name, method, defaultLabel, iconURL){
            if (this.actionDispatcher) {
                this.actionDispatcher.registerDispatchAction(name, method);
            }
            this.doRecentViewed[name] = {
                'icon': iconURL ? iconURL : this.blankIcon,
                'label': defaultLabel ? defaultLabel : name
            };
        },
        
        // wrapper to register a new artifact dispatch action and add the subAction to contribute to the unsaved menu panel
        registerNewUnsavedAction: function(name, method, defaultLabel, iconURL){
            if (this.actionDispatcher) {
                this.actionDispatcher.registerDispatchAction(name, method);
            }
            var defaultInfo = {
                'icon': iconURL ? iconURL : this.blankIcon
            }
            // The defaultLabel can be a string or an associated array of label and additional attributes.
            // For starting an execution, the menu item label needs to be updated from a global event
            // while other new artifact don't need any update.
            // Other methods that accept default label as param should be updated as such if there is a
            // need to specify additional default label attributes.
            if (defaultLabel && typeof defaultLabel == "string") {
                defaultInfo['label'] = defaultLabel;
            } else if (defaultLabel.label) {
                dojo.mixin(defaultInfo, defaultLabel);
            } else {
                defaultInfo['label'] = name;
            }
            this.doNewArtifact[name] = defaultInfo;
        },
        
        // check if the specified subAction is one of the new artifact subActions of this menu
        isNewArtifactSubAction: function(subAction) {
            return this.doNewArtifact[subAction] ? true : false;
        },
        
        _addMenuSeparator: function(inDropdown){
            if (inDropdown) {
                // add separator to the drop down menu if it is specified
                var dropdown = this.dropdowns[inDropdown];
                if (dropdown) {
                    dropdown.menu.addSeparator();
                }
                else {
                    this._addEntry("-", null, false, null, null, inDropdown);
                }
            }
            else {
                this.menuEntries.push(null);
            }
        },
        
        // Insert a menu item group with the specified groupID.  If displayMessageKey is not specified,
        // the groupID will be used as the message key for localized display label.
        insertMenuItemsGroup: function(groupID, /* optional */ displayMessageKey, /* optional */ groupArgs, /* optional */ menuItemPrefix){
            if (this.groups[groupID]) {
                // group is already created
                return groupID;
            }
            if (menuItemPrefix) {
                // set the prefix label for this menu item group that need it
                this.ariaGroups[groupID] = menuItemPrefix;
            }
            
            var displayName = Constants.getMessage(displayMessageKey || groupID);
            var entryArgs = {
                type: "group",
                name: groupID,
                title: displayName
            };
            
            // default group to auto wrap and hidden until entry is added
            if (!groupArgs) {
                groupArgs = {
                    'wrapHint': WrapHints.AUTO_WRAP,
                    'hide': true
                };
            } else {
                if (groupArgs['wrapHint'] == undefined) {
                    groupArgs['wrapHint'] = WrapHints.AUTO_WRAP;
                }
                if (groupArgs['hide'] == undefined) {
                    groupArgs['hide'] = true;
                }
            }
            entryArgs['groupArgs'] = groupArgs;

            this.menuEntries.push(entryArgs);
            
            // add the title to the menu if it is already created
            if (this.menu) {
                var menuGrpArgs = {
                    menu: this.menu,
                    groupTitle: displayName
                };
                dojo.mixin(menuGrpArgs, groupArgs);
                var group = new jazz.ui.internal.MenuItemGroup(menuGrpArgs);
                
                // save the group for later reference
                this.groups[groupID] = group;
            }
            return groupID;
        },
        
        // Insert a Browse menu items group
        insertBrowseMenuItemsGroups: function(browseGroupID, /* optional */ displayMessageKey, /* optional */ groupArgs, /* optional */ menuItemPrefix){
            // browse group for static menu items for consumer to populate
            this.insertMenuItemsGroup(browseGroupID, displayMessageKey, groupArgs, menuItemPrefix);
            return browseGroupID;
        },
        
        _moveEntryToTop: function(group, entry) {
            this.menu.addMenuItemUpdateTask(function() {
                var menuItem = group.remove(entry.menuItem);
                entry.menuItem = group.addItemByArgs(menuItem, "start");
            });
        },
                
        // Add a new entry or move existing entry to the front of the given menuEntries array of the group.
        // If maxCount is specified, entries beyond the max will be removed.
        //
        _addTopMenuEntry: function(group, menuEntries, uniqueKey, entryArgs, maxCount) {
            // skip if entry is a static menu item
			var uriArgs = CommonUtil.getArgsFromUri(uniqueKey);  // uniqueKey is the uri
            var isStaticMenuItem = this._isStaticMenuItemEntry(uriArgs);
            if (isStaticMenuItem) {
                if (!this.restoringViewHistory) {
                    // simply set the menu item to current
                    this.set('selected', isStaticMenuItem);
                }
                return;
            }
            
            // delete entry if already exists
            var existingEntry = this._removeMenuEntry(group, menuEntries, uniqueKey);
            
            // remove last entry if adding new entry will exceed the max
            while (menuEntries.length >= maxCount) {
                var extraEntry = menuEntries.pop();
                if (group && extraEntry.menuItem && this.menu) {
                    this.menu.addMenuItemUpdateTask(function() {
                        var menuItem = group.remove(extraEntry.menuItem);
                        menuItem.destroy();
                        extraEntry.menuItem = null;
                     });
                }
            }
            if (existingEntry) {
                if (entryArgs.label)
                    existingEntry.label = entryArgs.label;
                if (entryArgs.icon)
                    existingEntry.icon = entryArgs.icon;
                entryArgs = existingEntry;
            } else {
                // add new entry to the beginning
                entryArgs.key = uniqueKey;
            }
            // the unique key is the uri of the entry
            entryArgs.href = uniqueKey;
            
            // save the latest label for the uri
            this.uriTitle[uniqueKey] = entryArgs.label;
            menuEntries.unshift(entryArgs);
            if (group) {           
                menuEntries[0].menuItem = group.addItemByArgs(entryArgs, "start"); // add to the top of the group
                if (group == this.groups[UNSAVED_GROUP_ID]) {
                    dojo.addClass(menuEntries[0].menuItem.containerNode, "j-dirty"); // set color for unsaved entry
                }
            }
            if (!this.restoringViewHistory || menuEntries[0].isCurrent) {
                // set entry from last action to current unless this entry is added from restoring viewHistory
                // or the added top entry is set as the current which is the case when the browser is loaded with
                // specific uri of an artifact
                this.set('selected', menuEntries[0]);
            }
        },

        // Remove and return an entry with the unique key in the menuEntries array of the group.
        //
        _removeMenuEntry: function(group, menuEntries, uniqueKey) {
            var entry = null;
            for (var i = menuEntries.length - 1; i >= 0 ; i--) {
                // just in case there is error from the page that causing multiple entries created
                // for the same uri, loop through the entire list which has default limit of 10
                if (menuEntries[i].key == uniqueKey) {
                    // delete entry if already exists
                    if (group && menuEntries[i].menuItem && this.menu) {
                        var menuItem = menuEntries[i].menuItem;
                        this.menu.addMenuItemUpdateTask(function() {
                            var item = group.remove(menuItem);
                            item.destroy();
                        });
                    }
                    entry = menuEntries[i];
                    menuEntries[i].menuItem = null;
                    menuEntries.splice(i, 1);
                }
            }
            return entry;
        },
        
        // Update the group title to show the number of entries
        //
        _updateGroupTitle: function(group, titleKey, maxCount) {
            if (group && this.menu) {
                // menu item maybe pending remove until menu is close unless we are restoring view history
                if (this.restoringViewHistory) {
                    this._deferredUpdateGroupTitle(group, titleKey, maxCount);
                } else {
                    this.menu.addMenuItemUpdateTask(dojo.hitch(this, function(){
                        this._deferredUpdateGroupTitle(group, titleKey, maxCount);
                    }));
                }
            } else if (this.hoverMenu) {
                // menu is not created yet but still need to update the menu dirty indicator, but only if
                // the entire menu is not hidden - this.hoverMenu is valid
                this.hoverMenu.setDirty(this.unsavedMenuEntries.length > 0);
            }
        },
        
        _deferredUpdateGroupTitle: function(group, titleKey, maxCount) {
            var items = group.items;
            var count = items ? items.length : 0;
            if (maxCount && count == maxCount) {
                group.set('groupTitle', Constants.getMessage(titleKey + "Max", maxCount));
            } else {
                group.set('groupTitle', Constants.getMessage(titleKey, count));
            }
            
            if (group == this.groups[UNSAVED_GROUP_ID] && this.hoverMenu) {
                // update the menu dirty indicator if it is the unsaved group, but only if the entire menu
                // is not hidden - this.hoverMenu is valid
                this.hoverMenu.setDirty(count > 0);
            }
            this._updateGroupsDisplay();
        },
        
        // group without any entry is hidden and only the first visible group will start a new panel
        _updateGroupsDisplay: function() {
            if (this.menu) {
                var groups = [];
                if (this.groups[MY_QUERIES_GROUP_ID]) {
                    groups.push(this.groups[MY_QUERIES_GROUP_ID]);
                }
                if (this.groups[SHARED_QUERIES_GROUP_ID]) {
                    groups.push(this.groups[SHARED_QUERIES_GROUP_ID]);
                }
            
                if (!this.skipUnsavedAndViewedGroups) {
                    groups.push(this.groups[VIEWED_GROUP_ID]);
                    groups.push(this.groups[UNSAVED_GROUP_ID]);
                }
                if (!this.skipRecentQueriesGroups) {
                    groups.push(this.groups[QUERIES_GROUP_ID]);
                }
                var hasVisible = false;
                for (var i = 0; i < groups.length; i++) {
                    var group = groups[i];
                    var items = group.items;
                    var visible = group.restoringHistory ? true: items ? items.length > 0 : false; // show group loading... when restoring viewHistory
                    group.set('hide', !visible);
                    if (visible && !hasVisible) {
                        group.set('wrapHint', WrapHints.FORCE_WRAP);
                        hasVisible = true;
                    }
                    else {
                        group.set('wrapHint', visible ? WrapHints.AUTO_WRAP : WrapHints.NO_WRAP);
                    }
                }
            }
        },

        // Add an entry in the currently viewed group
        //
        _addRecentlyViewedMenuEntry: function(name, uniqueKey, callback, iconURL, callbackData) {
            var group = this.groups[VIEWED_GROUP_ID];
            var args = this._composeMenuItemArgs(name, callback, iconURL, callbackData, VIEWED_GROUP_ID);
            this._addTopMenuEntry(group, this.viewedMenuEntries, uniqueKey, args, MAXVIEWEDCOUNT);
            this._updateGroupTitle(group, this.viewedGroupTitleKey, MAXVIEWEDCOUNT);
			
			// avoid updating the interrim loading entry when an empty loading page is first added
			// when the page's content fills in, the entry is updated again with actual artifact name.
			if (name != LOADING) {
				this.saveViewHistory(this.recentlyViewedViewHistoryType, name, uniqueKey, iconURL);
			}
        },

        // Add an entry in the unsaved group
        //
        _addUnsavedMenuEntry: function(name, uniqueKey, callback, iconURL, callbackData) {
            var group = this.groups[UNSAVED_GROUP_ID];
            var args = this._composeMenuItemArgs(name, callback, iconURL, callbackData, UNSAVED_GROUP_ID);
            this._addTopMenuEntry(group, this.unsavedMenuEntries, uniqueKey, args);
            this._updateGroupTitle(group, this.unsavedGroupTitleKey);
        },
        
        // Add an entry in the queries group
        //
        _addQueriesMenuEntry: function(name, uniqueKey, callback, iconURL, callbackData) {
            var group = this.groups[QUERIES_GROUP_ID];
            var args = this._composeMenuItemArgs(name, callback, iconURL, callbackData, QUERIES_GROUP_ID);
            this._addTopMenuEntry(group, this.queriesMenuEntries, uniqueKey, args, MAXVIEWEDCOUNT);
            this._updateGroupTitle(group, this.queriesGroupTitleKey, MAXVIEWEDCOUNT);
            this.saveViewHistory(this.recentQueriesViewHistoryType, name, uniqueKey, iconURL);
        },

        // Convenient method to add an entry in the currently viewed group that will set the uri of page
        // If the uri is not specified, the current uri of the page will be used in the entry.
        //
        addRecentlyViewedEntry: function(title, uri, icon) {
            var uri = uri || CommonUtil.getDecodedLocationHash();
            this._addRecentlyViewedMenuEntry(title, uri, function(uri) {
                window.location.hash = uri;
            }, icon, uri);
        },
        
        // Convenient method to add an entry in the currently viewed group that will set the uri of page
        // If the uri is not specified, the current uri of the page will be used in the entry.
        //
        addUnsavedEntry: function(title, uri, icon) {
            var uri = uri || CommonUtil.getDecodedLocationHash();
            this._addUnsavedMenuEntry(title, uri, function(uri) {
                window.location.hash = uri;
            }, icon, uri);
        },
        
        // Convenient method to add an entry in the queries group that will set the uri of page
        // If the uri is not specified, the current uri of the page will be used in the entry.
        //
        addQueriesEntry: function(title, uri, icon) {
            var uri = uri || CommonUtil.getDecodedLocationHash();
            this._addQueriesMenuEntry(title, uri, function(uri) {
                window.location.hash = uri;
            }, icon, uri);
        },
        
        // Convenient method to remove the menu entry with the unique key in the queries group.
        //
        removeQueriesEntry: function(uniqueKey) {
            var group = this.groups[QUERIES_GROUP_ID];
            var entry = this._removeMenuEntry(group, this.queriesMenuEntries, uniqueKey);
            this._updateGroupTitle(group, this.queriesGroupTitleKey);
            return entry;
        },
        
        // Convenient method to remove the menu entry with the unique key in the unsaved group.
        //
        removeUnsavedEntry: function(uniqueKey) {
            var group = this.groups[UNSAVED_GROUP_ID];
            var entry = this._removeMenuEntry(group, this.unsavedMenuEntries, uniqueKey);
            this._updateGroupTitle(group, this.unsavedGroupTitleKey);
            return entry;
        },
        
        // Convenient method to remove the menu entry with the unique key in the recently viewed group.
        //
        removeRecentlyViewedEntry: function(uniqueKey) {
            var group = this.groups[VIEWED_GROUP_ID];
            var entry = this._removeMenuEntry(group, this.viewedMenuEntries, uniqueKey);
            this._updateGroupTitle(group, this.viewedGroupTitleKey, MAXVIEWEDCOUNT);
            return entry;
        },

        // Convenient method to set an recently viewed entry dirty and move to the unsaved group.
        //
        setEntryDirty: function(uniqueKey, label, iconClass) {
            var unsavedGroup = this.groups[UNSAVED_GROUP_ID];
            // remove and get the menu entry from the recently viewed group
            var entry = this.removeRecentlyViewedEntry(uniqueKey);
            if (entry) {
                var cleanLabel = entry.label;
                // update the label to reflect dirty and add to unsaved group
               if (entry.label.indexOf("* ") != 0) {
                   entry.label = "* " + entry.label;
               }
                entry.inGroup = UNSAVED_GROUP_ID;
                this._addTopMenuEntry(unsavedGroup, this.unsavedMenuEntries, uniqueKey, entry);
	            this._updateGroupTitle(unsavedGroup, this.unsavedGroupTitleKey);
                // update the latest label for the uri, undo _addTopMenuEntry() that added the "* " prefix
                this.uriTitle[uniqueKey] = cleanLabel;
            } else if (uniqueKey == PlanningUtil.getHomeTabUri()) {
                // Edge case that the home page added from HomePageView is not yet tracked by this
                // module, i.e. there is no recently viewed entry for it yet.  Setting it dirty
                // won't have an entry to delete.  Just created an unsaved enry for it here.
                this.addUnsavedEntry(label, uniqueKey, iconClass);
            }
        },
        
        // The title and uri of the tabs (menu entries here) are changed in different global events.  We
        // are invoked from the actionDispatcher and uri update does not contain the title in the args.
        // Keep track of the titles with the corresponding uri.
        //
        // Get the corresponding title of the uri and manage the update for uri changed
        // 
        _getManagedTitle: function(title, oldUri, newUri) {
            if (title) {
                // use new title with new uri and remove old entry
                 this.uriTitle[newUri] = title;
                 if (this.uriTitle[oldUri]) {
                    delete this.uriTitle[oldUri];
                 }
                 return title;
            }
            var label = "";
            if (oldUri) {
                // no title, use the one associated with the old uri
                var label = this.uriTitle[oldUri];
                if (label) {
                    // update the label to associate with the new uri
                    this.uriTitle[newUri] = label;
                    delete this.uriTitle[oldUri];
                }
            }
            return label;
        },
        
        // Summary:
        //  Add a plain label for the menu. It could be a loading message, etc. and can be removed.
        //
        _addMenuLabel: function(label){
            this.menuEntries.push({
                type: "label",
                name: label
            });
        },
        
        // Summary:
        //  Add a loading message into the menu. The name is provided so that it can be removed later.
        //
        _addMenuLoading: function(name){
            this.menuEntries.push({
                type: "loading",
                name: name
            });
            // add to menu if it is already created
            if (this.menu) {
                var item = this.menu.addLoading();
                this.removeableItems[name] = item;
            }
        },
        
        // Add an action in the menu that can be invoked without popping down
        // the menu. The action appears as a link. Expected actionArgs:
        //   title
        //   secondaryLabel
        //   onClick
        //
        _addMenuInPlaceAction: function(actionId, actionArgs, inGroup){
            var actionEntry = {
                type: "inplaceAction",
                name: actionId,
                inGroup: inGroup,
                args: actionArgs
            };
            this.menuEntries.push(actionEntry);
            
            // add the action to the menu if it is already created
            if (this.menu) {
                this._createInplaceAction(this.menu, actionEntry, inGroup);
            }
        },
        
        // Get the inplace action widget by its id name.
        _getMenuInPlaceAction: function(actionId){
            return (this.inplaceActions) ? this.inplaceActions[actionId] : null;
        },
        
        // Removes an item from the menu. The only removeable items are labels & inline actions.
        removeItem: function(itemName){
            var item = (this.removeableItems) ? this.removeableItems[itemName] : null;
            if (item && this.menu) {
                this.menu.remove(item);
            }
            var entries = this.menuEntries;
            var len = entries.length;
            for (var i = 0; i < len; i++) {
                var entry = entries[i];
                if ((entry.type == "inplaceAction" || entry.type == "label") && entry.name == itemName) {
                    entries.splice(i, 1);
                    break;
                }
            }
        },
        
        realize: function(verticalArgs){
            var navbar = dojo.getObject("com.ibm.rqm.web.globalNavBar");
            if (!navbar) {
                throw new Error("global navbar is not set");
            }
            
            // create the button on the navbar, but we'll create the
            // hover menu ourselves. We want the hover menu to reuse a single
            // menu that was created for it. The navbar's createItem creates
            // a hover menu that doesn't reuse menu.
            // 
            // We need this capability to change the menu's structure on the fly.
            //
            var menuButton = navbar._createItem({
                label: verticalArgs ? " " : this.title,
                align: "left"
            });
            var args = {
                group: "jazz-app-Navbar",
                menuProvider: dojo.hitch(this, "_createDropdownMenu"),
                menuClass: "jazz-app-Navbar-menu",
                injectCaret: true, // isVertical ? false : true, 
                cacheMenu: true
            };
            if (verticalArgs) {
                dojo.mixin(args, verticalArgs);
                this.hoverMenu = new com.ibm.rqm.planning.web.ui.internal.view.common.VerticalMenuPopup(args, menuButton);
            }
            else {
                this.hoverMenu = new jazz.ui.MenuPopup(args, menuButton);
                menuButton.parentActionBarEntry = this;
            }

            var navbarItems = dojo.getObject("com.ibm.rqm.web.globalNavBarItems");
            if (dojo.isArray(navbarItems)) {
                navbarItems.push(menuButton);
            }
            else {
                dojo.setObject("com.ibm.rqm.web.globalNavBarItems", [menuButton]);
            }
            return menuButton;
        },
        
        // convenient function to insert an item into the menu. 
        _addToMenu: function(menu, args, inGroup, inDropdown){
            var parent = menu;
            var menuItem = null;
            if (inGroup) {
                var group = this.groups[inGroup];
                if (group) {
                    parent = group;
                }
                
                if (this.ariaGroups[inGroup]) {
                    // menu item belongs to a group that need prefix in menu item label
                    var prefix = this.ariaGroups[inGroup];
                    // add a off-screen span so that the screen reader can read it but not in the display
                    args.label = '<span style="position:absolute;top:-10000px">' + prefix + '</span>' + args.label;
                }
                menuItem = parent.addItemByArgs(args);
                // set color for unsaved entry
                if (inGroup == UNSAVED_GROUP_ID) {
                    dojo.addClass(menuItem.containerNode, "j-dirty");
                }
                
                // display this group once we add an entry
                if (group.get('hide')) {
                    group.set('hide', false);
                }
            }
            else if (inDropdown) {
                var dropdown = this.dropdowns[inDropdown];
                var dropdownMenu = null;
                if (dropdown) {
                    // dropdown already exists
                    dropdownMenu = dropdown.menu;
                    menuItem = dropdownMenu.addMenuItem(args);
                }
                else {
                    // create new dropdown
                    args.parentMenu = this;
                    var menuPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.PopupMenuItem(args);
                    this.dropdowns[inDropdown] = {
                        menu: menuPopup
                    };
                    menuItem = parent.add(menuPopup);
                }
            }
            else {
                menuItem = parent.addItemByArgs(args);
            }
            // check if this should be a static menu item
            var menuItemHref = args.href;
            var uriArgs = {};
            if (menuItemHref) {
                uriArgs = CommonUtil.getArgsFromUri(menuItemHref);
            } else {
                // non dispatchable menu items use callback
                uriArgs = args.callbackData ? args.callbackData : {};
            }
            var isStaticMenuItem = this._isStaticMenuItemEntry(uriArgs);
            if (isStaticMenuItem) {
                // for static menu item, add the reference to the widget for ui update
                isStaticMenuItem.menuItem = menuItem;
                if (isStaticMenuItem.isCurrent) {
                    // the menu item may be set to be current before the menu is created from loading
                    // the url of the view artifacts list view directly
                    menu.set('selected', menuItem);
                }
            }
            return menuItem;
        },
        
        _createInplaceAction: function(menu, entry, inGroup){
            var args = entry.args;
            var name = entry.name;
            var actionArgs = {
                label: args.title,
                // disable the new line in embedded inline action in horizontal menu
                // the underlying menu already has sufficient spacing between items.
                newLineBeforeLink: false,
                showBusyWhenClicked: args.showBusyWhenClicked,
                firstLabel: args.firstLabel,
                secondaryLabel: args.secondaryLabel
            };
            if (args.onClick) {
                actionArgs.onClick = args.onClick;
            }
            if (args.href) {
                actionArgs.href = args.href;
            }
            var inplaceAction = new com.ibm.rqm.planning.web.ui.internal.view.common.InplaceLinkAction(actionArgs);
            this.inplaceActions[name] = inplaceAction;
            var inserted = false;
            
            // insert the action into the group, if it's
            // a part of the group. 
            if (inGroup) {
                var group = this.groups[inGroup];
                if (group) {
                    // unless specified in the args to append the new action, the
                    // new action will replace the only single action in the group
                    var actionType = args.appendAction ? "items" : "noItems";
                    group.set(actionType, inplaceAction.domNode);
                    inserted = true;
                }
            }
            // not in a group or can't find the group, insert it
            // into the menu directly.
            if (!inserted) {
                // create an wrapper menu item to hold our action link
                // entry. This is required, the menu expects a contain-able
                // child.
                var item = new jazz.ui.menu.NodeMenuItem({
                    className: "menu-text"
                });
                dojo.place(inplaceAction.domNode, item.containerNode, "only");
                menu.add(item);
            }
            if (!args.appendAction) {
                this.removeableItems[name] = item;
            }
        },
        
        _createDropdownMenu: function(){
            var self = this;
            this.initializeMenu();
            
            // saved queries browse groups managed internally before the recently viewed menu item groups
            this.insertMenuItemsGroup(MY_QUERIES_GROUP_ID, "labelBrowseMyQueries", {
                'hide': true,
                'wrapHint': WrapHints.NO_WRAP
            });
            this.insertMenuItemsGroup(SHARED_QUERIES_GROUP_ID, "labelBrowseSharedQueries", {
                'hide': true,
                'wrapHint': WrapHints.NO_WRAP
            });

            if (!this.skipUnsavedAndViewedGroups) {
                // These groups will be hidden when it is empty.  To cover the case of refreshing the browser with
                // the action that will create an entry in the unsaved groups, default the count to 1 and show the group
                // when creating the menu.  Recently viewed and recent queries group will be loading from viewHistory.
                this.insertMenuItemsGroup(VIEWED_GROUP_ID, Constants.getMessage(this.viewedGroupTitleKey, 0), {'wrapHint':WrapHints.NO_WRAP,'hide':true});
                this.insertMenuItemsGroup(UNSAVED_GROUP_ID, Constants.getMessage(this.unsavedGroupTitleKey, 1), {'wrapHint':WrapHints.NO_WRAP,'hide':true});
            }
            if (!this.skipRecentQueriesGroups) {
                this.insertMenuItemsGroup(QUERIES_GROUP_ID, Constants.getMessage(this.queriesGroupTitleKey, 0), {'wrapHint':WrapHints.NO_WRAP,'hide':true});
            }

            var menu = new com.ibm.rqm.planning.web.ui.internal.view.common.ComponentMenu({
                canDismiss: dojo.hitch(this, function(){
                    // for menu item of sub-menu, we use popupOpened to track if the sub-menu is opened 
                    return !self.popupOpened;
                }),
                menuItemEntry: this
            });
            var entries = this.menuEntries.concat(this.viewedMenuEntries, this.unsavedMenuEntries, this.queriesMenuEntries);
            var entriesCount = entries.length;
            this.removeableItems = {};
            this.inplaceActions = {};
            
            for (var i = 0; i < entriesCount; i++) {
                var entry = entries[i];
                if (entry) {
                    var inGroup = entry.inGroup || null;
                    
                    // NOTE: menus entries are expected to be added in the order
                    // of which it appears. If any items that needs to be under 
                    // a group, they need to be added below the title.
                    // 
                    if (entry.type && entry.type == "group") {
                        var args = {
                            menu: menu,
                            groupTitle: entry.title
                        };
                        if (entry.groupArgs) {
                            dojo.mixin(args, entry.groupArgs);
                        }
                        var group = new jazz.ui.internal.MenuItemGroup(args);
                        // save the group for later reference
                        this.groups[entry.name] = group;
                    }
                    else if (entry.type && entry.type == "label") {
                        var labelItem = menu.addTitle(entry.name);
                        this.removeableItems[entry.name] = labelItem;
                    }
                    else if (entry.type && entry.type == "loading") {
                        var labelItem = menu.addLoading();
                        this.removeableItems[entry.name] = labelItem;
                    }
                    else if (entry.type && entry.type == "inplaceAction") {
                        this._createInplaceAction(menu, entry, inGroup);
                    }
                    else {
                        entry.menuItem = this._addToMenu(menu, entry, inGroup, entry.inDropdown);
                        if (entry.isCurrent) {
                            menu.set('selected', entry.menuItem);
                        }
                    }
                }
                else {
                    menu.addSeparator();
                }
            }
            
            this.menu = menu;

            // we deferred populating the recently viewed and recent queries viewHistory until the menu
            // is created/first opened here
            this._deferredRestoreViewHistory();

            this.updateSavedQueries();
            // update the display of the recent panels
            this._updateGroupsDisplay();
            return menu;
        },
        
        _addPage: function(args){
            if (!this._contentArea) {
                // For horizontal menu bar, the menu tries are created by the application in
                // the common banner navBar which the HomePageView may not be loaded yet to
                // the contentArea set.  The HomePageView set this upon loading.
                this._contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
            }
            var page = this._contentArea.addPage(args);
            // trigger tab added for menus update unless they are skipped
            if (!this.skipUnsavedAndViewedGroups || !this.skipRecentQueriesGroups) {
                this.onTabAdded(page);
            }
            return page;
        },
        
        selectPrevActionEntry: function(){
            this._parent.selectPrevActionEntry(this);
        },
        
        selectNextActionEntry: function(){
            this._parent.selectNextActionEntry(this);
        }
    });
})();
