/****************************************************************************** 
 * Licensed Materials - Property of IBM
 *  Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactCopyWizardPage");

dojo.require("dijit._Templated");
dojo.require("com.ibm.asq.common.web.ui.internal.wizard.WizardPage");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.selector.ProjectAreaSelector");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("dijit.form.CheckBox");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.MessageBox");

(function(){
var CommonConstants 		= com.ibm.asq.common.web.client.internal.Constants;
var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;
var AlertDialog 			= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
var ProjectAreaSelector = com.ibm.asq.common.web.ui.internal.widgets.selector.ProjectAreaSelector;
var getCurrentProjectArea = jazz.app.context.get;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactCopyWizardPage", 
			 [com.ibm.asq.common.web.ui.internal.wizard.WizardPage, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/wizard/templates/ArtifactCopyWizardPage.html"),
	values:[],
	items:[],
	pageId:null,
	allowUnique:false,
	lastSelectedValues:[],	
	faultyTargetValues:[],
	
	constructor: function(args) {
		this.items = args.items || [];
		this.lastSelectedValues = [];
        this.faultyTargetValues = [];
        this.mappingTypeLabel = args.mappingTypeLabel;
	},
	postCreate: function() {
		this.targetSelects = [];
		this._isPageComplete = false;
		this.initialized = false;
	},
	
	setWizardDialog: function(wizardDialog) {
		this._wizardDialog= wizardDialog;		
	},
	
	getContentNode: function() {
		if (!this.initialized) {
			this._isPageComplete = true;
			this._displayContents(this.items);
			this.initialized = true;
		}
		return this._rootNode;
	},
	
	_displayContents:function(items){
		var doc = document;
		var tbody = this._detailsTable;
		var len = items.length;
		for (var i = 0; i < len; i++) {
		    var hint = "";
		    if (items[i].hint) {
		        hint = items[i].hint.itemId || items[i].hint.name;
		    }
			this._createRow(doc,tbody,items[i].sourceItem,items[i].alternatives || [],hint);
		}
		
		if (this.allowUnique) {
		    for (var id in this.lastSelectedValues) {
		        var value = this.lastSelectedValues[id];
				var targetSelects  = this.targetSelects;
		        var len = targetSelects.length;
		        for (var i = 0; i < len; i++) {
		            var targetSelect = targetSelects[i];
		            if (targetSelect.id == id) {
		                continue;
		            }
					var len2 = targetSelect.options.length;
		            for (var j = 0; j < len2; j++) {
		                var option = targetSelect.options[j];
		                if (option.value == value) {
		                    option.disabled = true;
		                }
		            }
		        }
		    }
		}	
		
		var values = [];
		var showAlert = false;
		var len = this.targetSelects.length;
		for (var i = 0; i < len; i++) {
			var options = this.targetSelects[i].options;
			var available = false;
	        var len2 = options.length;
			for (var j = 0; j < len2; j++) {
				var value = options[j].value;
				if (value && value != "" && !values[value]) {
					values[value] = this.allowUnique;
					available = true;
					break;
				}
			}
			if (!available) {
				showAlert = true;
				break;
			}
		}	
		if(showAlert){			
			var targetProjectArea = this._wizardDialog._wizard.targetPA;
			var mappingType = this.mappingTypeLabel;
			var message = mappingType ? 
				Constants.getMessage("msgCorrelateFailCustom", mappingType, targetProjectArea) : 
				Constants.getMessage("msgCorrelateFail", targetProjectArea);
            this._displayAlertDialog(message);
            this._isPageComplete = false;
        }
	},
	_createRow:function(doc,tbody,source,targets,hint){
		var tr = this._getEmptyRow(doc);
		var td1 = this._getEmptyCell(doc);
		var sourceLabel = doc.createElement("label");
		sourceLabel.innerHTML = source.name;
		sourceLabel.setAttribute("for", this.id+source.name);
		td1.appendChild(sourceLabel);
		td1.style.color = "rgb(0,0,0)";
		tr.appendChild(td1);
		var td2 = this._getEmptyCell(doc);
		var targetSelect = doc.createElement("select");
		targetSelect.id = this.id+source.name;
		var len = targets.length;
		targetSelect.options[0] = new Option("", "");
		targetSelect.sourceNameString = source.name;
		for(var i=0;i<len;i++){
			var value = targets[i].itemId;
			if (!value) {
				value = targets[i].name;
			}
			targetSelect.options[i + 1] = new Option(targets[i].name,value);
			if(hint && hint == value){
				targetSelect.options[i + 1].selected = true;
				this.lastSelectedValues[targetSelect.id] = value;
			}
		}
            
		this.targetSelects.push(targetSelect);
		td2.appendChild(targetSelect);
		tr.appendChild(td2);
		tbody.appendChild(tr);
		
		if (!hint) {
            this.setPageComplete(false);
            this.faultyTargetValues.push(targetSelect.id);
            targetSelect.parentNode.previousSibling.style.color="rgb(255,0,0)";
        }
		
        this.connect(targetSelect, "onchange", this._onChange);
	},
	_onChange:function(e){
	    var targetSelect = e.target;
	    var selectedValue = e.target.value;
		setTimeout(dojo.hitch(this, this._handleOnChange,targetSelect,selectedValue),0);
		if(e.stopPropagation) //FF
			e.stopPropagation();
		e.cancelBubble = true;//IE
		return;
	},
	_handleOnChange:function(targetSelect,selectedValue){
		this._wizardDialog.hideHeaderMessage();
		var faultyTargetValues = this.faultyTargetValues;
	    targetSelect.parentNode.previousSibling.style.color="rgb(0,0,0)";
	    if (selectedValue == "") {
	    	var message = CommonConstants.getMessage("msgInvalidAttribute", targetSelect.sourceNameString?targetSelect.sourceNameString:"");
	    	this._wizardDialog.displayHeaderMessage(message, jazz.ui.MessageBox.STATUS_TYPES.ERROR); 
	        targetSelect.parentNode.previousSibling.style.color="rgb(255,0,0)";
	        faultyTargetValues.push(targetSelect.id);
	    } else {
			var len = faultyTargetValues.length;
	        for (var i=0;i<len;i++) {
	            if (faultyTargetValues[i] == targetSelect.id) {
	                faultyTargetValues.splice(i, 1);
					i-=1;
					len-=1;
	            }
	        }
	    }
	        
        if (this.allowUnique) {
            var lastValue = this.lastSelectedValues[targetSelect.id];
			var targetSelects = this.targetSelects;
			var len2 = targetSelects.length;
            for (var i = 0; i < len2 ; i++) {
                if (targetSelects[i].id == targetSelect.id) {
                    continue;
                }
                var len3 = targetSelects[i].options.length;
                for (var j = 0; j < len3; j++) {
                    var option = targetSelects[i].options[j];
                    if (option.value != "" && option.value == selectedValue) {
                        option.disabled = true;
                    }
                    if (lastValue && lastValue != "" && option.value == lastValue) {
                        option.disabled = false;
                    }
                }
            }
            this.lastSelectedValues[targetSelect.id] = selectedValue;
	    }
	    
	    if (this.faultyTargetValues.length > 0) {
	        this.setPageComplete(false);
	    } else {
	        this.setPageComplete(true);
	    }
	},
	_getEmptyRow: function(doc) {
		var row = doc.createElement("tr");
		return row;
	},
		
	_getEmptyCell: function(doc,span, align){
		var cell = doc.createElement("td");
		cell.width = "50%";
		return cell;
	},
	aboutToHide:function(){
		var items = this.items;
		var len = items.length;
		var sourceValues = [];
		var targetValues = [];
		var optTargetValues = [];
		var targetSelects = this.targetSelects;
		for (var i = 0; i < len; i++) {
		    var source = items[i].sourceItem.itemId;
		    if (!source) {
		        source = items[i].sourceItem.name;
		    }
			sourceValues.push(source);
			targetValues.push(targetSelects[i].value);
			optTargetValues.push("");
		}
		this.values = {sourceItems:sourceValues,targetItems:targetValues,optTargetData:optTargetValues};
	},
	getValues:function(){
		return this.values;
	},
	_displayAlertDialog: function(message){
        var args = {
        	primaryTitle: Constants.getMessage("labelCorrelationTitle"),
			message: message
		};
		var alertDialog = new AlertDialog( args );
        alertDialog.open();
},
	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactProjectAreaSelectorWizardPage", 
		 [com.ibm.asq.common.web.ui.internal.wizard.WizardPage, dijit._Templated], {
	
	
	_appArgs: {},
	
	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/wizard/templates/ArtifactCopyInitialWizardPage.html"),
	
	referencedCopyLabelValue: Constants.getMessage("labelDuplicateWithReferences"),
	deepCopyLabelValue: Constants.getMessage("labelDeepduplicate"),
	basicDuplicateLabelValue: Constants.getMessage("labelBasicDuplicate"),
	projectAreaLabelValue: CommonConstants.getMessage("labelSelectProjectArea"), 
	widgetsInTemplate: true,
	
	constructor: function(args) {
		this.title = args.title;	
		this.itemTypeName = args.itemTypeName;
	},
	
	postCreate: function(){
		var args = {
				onValueChanged : dojo.hitch(this, "onValueChanged"),
				checkValueChangedOnInterval: false,
				enableValueChangedOnKeyboard: true,
				labelledBy: this.id + "_paSelectorLabel"
		};									
		this._referencedCopyLabel.setAttribute("for",this.referencedCopyFieldCheck.id);		
		this.paSelector = new ProjectAreaSelector(args);
		this._paSelectorContainer.appendChild(this.paSelector.domNode);
		this.displayDeepCopyField();
	},
	
	getContentNode: function() {
		return this._rootNode;
	},

	setWizardDialog: function(wizardDialog) {
		this._wizardDialog= wizardDialog;
		this.wizardContainer.setWizardDialog(wizardDialog);
	},

	setPageComplete: function(complete) {
		this._isPageComplete= complete;
		if (this._wizardDialog != null) {
			this._wizardDialog.updateButtons();
		}
	},
	
	displayDeepCopyField: function(){
		if(this.itemTypeName && this.itemTypeName == Constants.CASE_TYPE){
   			dojo.style(this._deepCopyRow, "display", "");
   			dojo.style(this._simpleCopyRow, "display", "");
   			dojo.style(this.deepCopyField.domNode,"display","");
   			dojo.style(this.referencedCopyFieldRadio.domNode,"display","");
   			dojo.style(this.referencedCopyFieldCheck.domNode,"display","none");
   			this._referencedCopyLabel.setAttribute("for",this.referencedCopyFieldRadio.id);
   			this._simpleCopyLabel.setAttribute("for",this.simpleField.id);
   			this._deepCopyLabel.setAttribute("for", this.deepCopyField.id);
   		} else {
   			dojo.style(this._deepCopyRow, "display", "none");
   			dojo.style(this._simpleCopyRow, "display", "none");
   		}
	},	
	
	onValueChanged: function(value){		
		this.referencedCopyFieldCheck.set("checked",false);
		this.deepCopyField.set("checked", false);
		var isValid = this.paSelector.isValid();		
		if(value != getCurrentProjectArea().name || !isValid ){
			dojo.style(this._copyWithReferencesRow,"display", "none");		
			dojo.style(this._deepCopyRow,"display", "none");
			dojo.style(this._simpleCopyRow,"display", "none");
			if(isValid){
				dojo.style(this.deepCopyField.domNode,"display","none");
				dojo.style(this._deepCopyRow,"display", "");				
				this.deepCopyField.set("checked", true);
				this.deepCopyField.set("enabled", false);
			}
		} else {
			this.displayDeepCopyField();
			dojo.style(this._copyWithReferencesRow,"display", "");
			this.referencedCopyFieldCheck.set("checked",true);
			this.referencedCopyFieldRadio.set("checked",true);
		}
		if(!isValid){
			this._isPageComplete = false;
			//no need to propagate the event to the wizard container
			this._wizardDialog.updateButtons();
		} else { 
			this._isPageComplete = true;
			this.paSelector.getSelection(dojo.hitch(this.wizardContainer, "teamAreaSelectionChanged"));
		}
	},
	
	runWithProgress: function(progressMonitoredCode) {
		if (this._wizardDialog != null) {
			this._wizardDialog.runWithProgress(progressMonitoredCode);
		} else {
			this._wizardDialog.runWithProgress(new NullProgressMonitor());
		}
	},
	
	setReadOnly: function(){
		this.paSelector.setDisabled(true);
	},	
	
	_simpleCopyChanged:function(){
		return this.simpleField.checked;
	},
	
	isDeepCopy: function(){
		if(this.itemTypeName && this.itemTypeName == Constants.CASE_TYPE){
			return this.deepCopyField.checked;
		} 
		return null;
	},
	
	isReferencedCopy:function(){
		if(this.itemTypeName && this.itemTypeName == Constants.CASE_TYPE){
			return !(this.simpleField.checked);
		} else {
			return this.referencedCopyFieldCheck.checked;
		}
	},
	isValid: function(){
		return this.paSelector.isValid();
	}

});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.wizard.WaitPage", 
		 [com.ibm.asq.common.web.ui.internal.wizard.WizardPage, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/wizard/templates/ArtifactCopyWizardPage.html"),	
	
	constructor: function(args) {
		this.title = args.title;		
	},
	
	postCreate: function(){			
		var image =  document.createElement("img");
		image.src = com.ibm.asq.common.web.ui.internal.view.common.ImageCache.PROGRESS;		
		this._extraContent.appendChild(image);		
		var label = document.createElement("label");
		label.innerHTML = Constants.getMessage("labelLoading");
		this._extraContent.appendChild(label);		
	},
	
	setPageReadyToFinish: function(message, hasErrors){		
		dojox.xml.parser.removeChildren(this._extraContent);		
		var label = document.createElement("label");		
		label.innerHTML = message;
		if(hasErrors){
			var image =  document.createElement("img");
			image.src = com.ibm.asq.common.web.ui.internal.view.common.ImageCache.ERROR_ICON;		
			this._extraContent.appendChild(image);						
		}
		this._extraContent.appendChild(label);		
	},

	getContentNode: function() {
		return this._rootNode;
	},
	
	setWizardDialog: function(wizardDialog) {
		this._wizardDialog= wizardDialog;	
	}
});

com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactCopyWizardPage.create= function(params) {
	return new com.ibm.rqm.planning.web.ui.internal.wizard.ArtifactCopyWizardPage(params);
};
})();
