<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:styleinfo="http://jazz.net/xmlns/alm/qm/styleinfo/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:proxy="com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestCaseProxy"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    exclude-result-prefixes="xsl xalan rqm dc alm styleinfo prep exslt HttpClientExtension proxy utils log">
 
    <!-- included stylesheets -->
    <xsl:import  href="cache://stylesheets/prep/abstract-prep-handler.xsl"/>
    <xsl:include href="cache://stylesheets/_util/to-executionscript.xsl"/>
    
    <!-- params -->
    <xsl:param name="artifact"/>
    <xsl:param name="repoService"/>
    <xsl:param name="catTypeRestService"/>
    
    <!-- proxy definition -->
    <xalan:component prefix="proxy">
        <xalan:script lang="javaclass" src="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestCaseProxy"/>
    </xalan:component>
    
    <xsl:variable name="item" select="proxy:new($artifact, $repoService, $catTypeRestService)"/>
    
    <!-- templates -->
    <xsl:template match="rqm:testcase">
        <xsl:copy>
            <xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:description"/>
            <xsl:apply-templates select="rqm:creationDate"/>
            <xsl:apply-templates select="alm:updated"/>
            <xsl:apply-templates select="alm:state"/>
            <xsl:apply-templates select="dc:creator"/>
            <xsl:apply-templates select="alm:owner"/>
            <xsl:apply-templates select="rqm:priority"/>
            <xsl:apply-templates select="rqm:copiedFrom"/>
            <xsl:call-template name="_insert-copiedTo"/>
            
            <xsl:apply-templates select="proxy:getSections($item)/rqm:section">
                <xsl:with-param name="testcase" select="current()"/>
            </xsl:apply-templates>            
        </xsl:copy>
    </xsl:template>
    
    <!-- copiedFrom -->
    <xsl:template match="rqm:copiedFrom">
        <xsl:apply-templates mode="copiedFrom" select="HttpClientExtension:get($restClient, @href, 'fields=/feed/entry/content/testcase/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedFrom" match="rqm:testcase">
        <prep:copiedFrom>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedFrom>
    </xsl:template>
    
    <!-- copiedTo -->
    <xsl:template name="_insert-copiedTo">
        <xsl:variable name="copiedToFeed">
            <xsl:variable name="tsFilter">copiedFrom/@href="<xsl:value-of select="utils:encodeUri(dc:identifier)"/>"</xsl:variable>
            
            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../testcase'"/>
                <xsl:with-param name="query" select="concat('fields=/feed/entry/content/testcase[', $tsFilter, ']/(identifier|title)')"/>
            </xsl:call-template>    
        </xsl:variable>
        
        <xsl:apply-templates mode="copiedTo" select="exslt:node-set($copiedToFeed)//rqm:testcase"/>
    </xsl:template>
    
    <xsl:template mode="copiedTo" match="rqm:testcase">
        <prep:copiedTo>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedTo>
    </xsl:template>
    
    <!-- Summary -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.testCaseSummary']">
        <xsl:param name="testcase"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="proxy:getCategories($item)/rqm:category"/>
            <xsl:apply-templates select="$testcase/rqm:customAttributes"/>
            <xsl:apply-templates select="$testcase/rqm:estimate"/>
            <xsl:apply-templates select="$testcase/rqm:weight"/>
            <xsl:apply-templates select="$testcase/rqm:trigger"/>
            <xsl:apply-templates select="$testcase/rqm:activity"/>
            <xsl:copy-of select="utils:rest($testcase/rqm:stylesheet/@href)/styleinfo:preferences/testcaseActivityTrigger"/>            
        </xsl:copy>
    </xsl:template>
    
    <!-- Test Case Design -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Formal Review -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.editor.section.review']">
        <xsl:param name="testcase"/>
        <xsl:variable name="approvals" select="$testcase/rqm:approvals"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $approvals">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$approvals"/>
            </xsl:copy>    
        </xsl:if>
    </xsl:template>
    
    <!-- Development Items -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.defects.web.RTCWorkItem']">
        <xsl:param name="testcase"/>
        <xsl:variable name="developmentPlans" select="$testcase/rqm:workitem"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $developmentPlans">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$developmentPlans">
                    <prep:developmentPlans>
                        <xsl:apply-templates select="$developmentPlans"/>    
                    </prep:developmentPlans>    
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Requirement Links -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.defects.web.RmLinks']">
        <xsl:param name="testcase"/>
        <xsl:variable name="requirementLinks" select="$testcase/rqm:requirement"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $requirementLinks">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$requirementLinks">
                    <prep:requirementLinks>
                        <xsl:apply-templates select="$requirementLinks"/>    
                    </prep:requirementLinks>    
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Risk Assessment -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.risk.editor.section.caseAssessment']">
        <xsl:param name="testcase"/>
        <xsl:variable name="risk-text" select="$testcase/*[@extensionDisplayName='RQM-KEY-TC-RISK-TITLE']"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $risk-text or rqm:risk">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$risk-text"/>
                <xsl:copy-of select="$testcase/rqm:risk"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Pre-Condition -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Post-Condition -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Expected Results -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Scripts -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.testCaseScripts']">
        <xsl:param name="testcase"/>
        <xsl:param name="testscripts" select="$testcase/rqm:testscript"></xsl:param>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testscripts">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list xsl:exclude-result-prefixes="prep">
                    <xsl:attribute name="key">RQM-TS-TABLEVIWER</xsl:attribute>
                    <xsl:apply-templates select="$testscripts"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Case Execution Records -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.execution.editor.section.ewis']">
        <xsl:param name="testcase"/>
        <xsl:variable name="ewis" select="proxy:getTestExecutionRecords($item)"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $ewis">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list xsl:exclude-result-prefixes="prep dc alm rqm">
                    <xsl:attribute name="key">RQM-TER-TABLEVIWER</xsl:attribute>
                    <xsl:apply-templates select="$ewis"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Attachments -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.attachments']">
        <xsl:param name="testcase"/>
        <xsl:variable name="attachments" select="$testcase/rqm:attachment"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $attachments">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <prep:attachments>
                    <xsl:for-each select="$attachments">
                        <xsl:copy-of select="proxy:attachment($item, @href)"/>
                    </xsl:for-each>    
                </prep:attachments>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Execution Variables -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.testcaseExecVars']">
        <xsl:param name="testcase"/>
        <xsl:variable name="variables" select="$testcase/rqm:variables"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $variables">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$variables"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Normative and Informative Documents -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planNormativeInformative']">
        <!-- TODO support this section -->
    </xsl:template>
    
    <!-- Notes -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Requirements -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.requirements.editor.section.caseRequirements']">
        <!-- TODO support this section -->
    </xsl:template>
    
    <!-- Test Case Scope -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Description -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Preparation -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Custom Grid Sections -->
    <xsl:template match="rqm:section[@type='GRID']">
        <xsl:param name="testcase"/>
        <xsl:variable name="section-name" select="@name"/>
        <xsl:variable name="section-content" select="$testcase/*[@extensionDisplayName=$section-name]"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:copy-of select="proxy:gridJSONtoXHTML($item, $section-content)"/>    
        </xsl:copy>
    </xsl:template>
    
    <!-- Custom Sections -->
    <!-- This template is only invoked if no other template matched a rqm:section.  Any section that is not 
         explicitly handled above will simply be copied into the result document. -->
    <xsl:template match="rqm:section">
        <xsl:param name="testcase"/>
        <xsl:variable name="section-name" select="@name"/>
        <xsl:variable name="section-content" select="$testcase/*[@extensionDisplayName=$section-name]/*"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $section-content">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$section-content"/>
            </xsl:copy>    
        </xsl:if>
    </xsl:template>
 	
 	<!-- eSignatures -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.esign.audit']">
        <xsl:param name="testcase"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$testcase/rqm:esignatures"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>