<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    exclude-result-prefixes="xsl exslt HttpClientExtension log">
    
    <!-- templates -->
    <xsl:template match="table-list/rqm:testsuite">
        <xsl:param name="columns"/>
        <xsl:param name="exec-status"/>
        <xsl:variable name="uuid" select="@uuid"/>
        <xsl:variable name="tpHref" select="../@parent"/>

        <xsl:variable name="fields">
            <xsl:variable name="tmp-fields">
                <xsl:text>identifier|</xsl:text>
                <xsl:for-each select="$columns">
                    <xsl:apply-templates mode="suite-param" select="prep:name"/>
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:value-of select="substring($tmp-fields, 0, string-length($tmp-fields))"/>
        </xsl:variable>
        
        <xsl:if test="log:isDebugEnabled()">
            <xsl:value-of select="log:debug(concat('testsuite $fields=', $fields))"/></xsl:if>
        
        <xsl:apply-templates mode="suite-copy" select="HttpClientExtension:get($restClient, @href, concat('fields=feed/entry/content/testsuite/(', $fields, ')'))">
            <xsl:with-param name="exec-status" select="$exec-status/prep:testsuite[@uuid=$uuid]"/>
            <xsl:with-param name="tpHref" select="$tpHref"/>
        </xsl:apply-templates>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name">
        <!-- intentionally left blank -->
        <!-- ignores all columns that are not otherwise matched by a template -->
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='id']">
        <xsl:text>webId|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='priority']">
        <xsl:text>priority|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='name']">
        <xsl:text>title|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='modified']">
        <xsl:text>updated|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='state']">
        <xsl:text>state|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='locked']">
        <xsl:text>locked|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='owner']">
        <xsl:text>owner|</xsl:text>
    </xsl:template>

    <xsl:template mode="suite-param" match="prep:column/prep:name[.='originator']">
        <xsl:text>creator|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='estimate']">
        <xsl:text>estimate|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='weight']">
        <xsl:text>weight|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='calculatedRisk' or .='communityRisk']">
        <xsl:text>risk|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='tracereqCollLink']">
        <xsl:text>requirementset|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='tracedevPlanLink']">
        <xsl:text>developmentplan|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[starts-with(., 'isCategoryColumn')]">
        <xsl:text>category|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[starts-with(., 'isCustomColumn')]">
        <xsl:text>customAttributes|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='copiedFrom']">
        <xsl:text>copiedFrom|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-param" match="prep:column/prep:name[.='copiedRoot']">
        <xsl:text>copiedRoot|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="suite-copy" match="@*|node()">
        <xsl:param name="columns"/>
        <xsl:param name="tpHref"/>
        
        <xsl:copy>
            <xsl:apply-templates mode="tser" select="@*|node()">
                <xsl:with-param name="columns" select="$columns"/>
                <xsl:with-param name="tpHref" select="$tpHref"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="suite-copy" match="rqm:testsuite">
        <xsl:param name="tpHref"/>
        <xsl:param name="exec-status"/>

        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
            <xsl:copy-of select="$exec-status/*"/>
            
            <xsl:if test="$tpHref">
                <prep:tserCount>
                    <xsl:variable name="tserFeed">
                        <xsl:variable name="suiteParam">
                            <xsl:text>testsuite[@href='</xsl:text>
                            <xsl:value-of select="utils:encodeUri(dc:identifier)"/>
                            <xsl:text>']</xsl:text>
                        </xsl:variable>
                        
                        <xsl:variable name="tpParam">
                            <xsl:text>testplan[@href='</xsl:text>
                            <xsl:value-of select="utils:encodeUri($tpHref)"/>
                            <xsl:text>']</xsl:text>
                        </xsl:variable>
                        
                        <xsl:call-template name="restGET">
                            <xsl:with-param name="path"  select="'../suiteexecutionrecord'"/>
                            <xsl:with-param name="query" select="concat('fields=feed/entry/content/suiteexecutionrecord/(testplan|testsuite|', $suiteParam, '|', $tpParam, ')')"/>
                        </xsl:call-template>    
                    </xsl:variable>
                    
                    <!-- count them -->
                    <xsl:value-of select="count(exslt:node-set($tserFeed)//rqm:suiteexecutionrecord)"/>
                </prep:tserCount>
            </xsl:if>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="rqm:testsuite/rqm:copiedFrom">
	     <!-- we only need the copied from test suite's url and webId -->
	     <xsl:apply-templates mode="copiedFrom" select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testsuite/(identifier|webId)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedFrom" match="rqm:testsuite">
        <prep:copiedFrom>
        	<xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
        </prep:copiedFrom>
    </xsl:template>
    
    <xsl:template match="rqm:testsuite/rqm:copiedRoot">
	     <!-- we only need the copied root test suite's url and webId -->
	     <xsl:apply-templates mode="copiedRoot" select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testsuite/(identifier|webId)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedRoot" match="rqm:testsuite">
        <prep:copiedRoot>
        	<xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
        </prep:copiedRoot>
    </xsl:template>
    
</xsl:stylesheet>