/******************************************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.QueryProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.cache.internal.CacheUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var CacheUtil = com.ibm.team.workitem.web.cache.internal.CacheUtil;

var QueryProxy= dojo.declare("com.ibm.team.workitem.web.cache.internal.QueryProxy", [com.ibm.team.workitem.web.cache.internal.ItemProxy], {
	/*
	 * Construct the proxy passing the object
	 */
	isQuery: true,
	constructor: function(parms) {
		/* QueryDescriptor */ this.object= parms.query;
		this.storedObject= dojo.clone(parms.query);
		if(parms.tq) {
			this._newQueryTimestamp = parms.tq;
		}
		// TODO(QueryDescriptor): New concept for storing noticing query name changes
		if (this.object._oldName) this.storedObject.setName(this.object._oldName);
		this.storedObject.attributes = {};
        this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
	},
	getNewQueryTimestamp: function() {
		return this._newQueryTimestamp;
	},
	getItemId: function() {
		return this.object.getCacheItemId(); // The cache id is more likely to return a true itemid
	},
	getMenuLabel: function() {
		return CacheUtil.getQueryMenuLabel(this);
	},
	getMenuIconUri: function() {
		// Even though only the relative part of the iconUri is used, the view history requires a valid fully qualified uri
		return window.location.protocol + "//" + window.location.host  + this.getIconUrl().toUri();
	},
	getLinkLabel: function() {
		if (this.object && this.object.getName() && this.object.getName().length > 0) {
			if (!this.isNew() && this.object.getName() == this.messages.labelDefaultQueryName) {
				return this.messages.labelNoName;
			}
			var result= this.object.getName();
			if (this.object.hasParameterVariables()) {
				result+= QueryProxy.QUERY_LABEL_ELLIPSIS;
			}
			return result;
		}
		if (this.isNew()) {
			return this.messages.labelNewQuery;
		}
		return this.messages.labelNoName;
	},
	getSummaryLabel: function() {
		var result= this.messages.labelNewQuery;
		if (this.object) {
			if (this.object.getDescription() && this.object.getDescription().length > 0) {
				result= this.object.getName() + ": " + this.object.getDescription();
			} else {
				result= this.object.getName();
			}
			if (this.object.hasParameterVariables()) {
				result+= QueryProxy.QUERY_LABEL_ELLIPSIS;
			}
		}
		return result;
	}, 
	setIsBeingEdited: function(isBeingEdited) {
		this._isBeingEdited = isBeingEdited;
	},
	getIsBeingEdited: function() {
		return this._isBeingEdited == true;
	},
	getIconUrl: function() {
		return com.ibm.team.workitem.web.ui.internal.view.common.ImagePool.WORKITEM_QUERY;
	},
	getUrl: function() {		
		if (this._queryUrl) {		
			if (this.isNew() 
				|| this._queryUrl.indexOf("action=com.ibm.team.workitem.search") != -1
				|| this._queryUrl.indexOf("action=com.ibm.team.workitem.similarityQuery") != -1) return this._queryUrl;
		}
		if (this.isNew() && this.object.attributes.hasNewQueryUrl || this.getItemId() == null) {
			return WorkItemUtil.getNewQueryUri();
		}	
		if (this.isChanged() || this._isBeingEdited) {
			return WorkItemUtil.getEditQueryUri(this.getItemId());
		}
		return WorkItemUtil.getQueryUri(this.object);
	},
	/* 
	 * Answer true if the value has changed
	 */
	isChanged: function() {
		return this.getValue({path:"attributes.isChanged"});
	},
	/* 
	 * Answer true if the value is new
	 */
	isNew: function() {
		// TODO(QueryDescriptor): New concept for query.isDirty(): Replace _isSaved with !isDirty()
		if (this.object && this.object._isSaved) {
			return false;
		}
		return (this.object && this.object.isNew)
				|| (this.object && this.object.getItemId() === "") 
				|| (this._queryUrl && this._queryUrl.indexOf("action=com.ibm.team.workitem.newQuery") !== -1);
	},
	/* 
	 * Ordering is supported for search, template, find duplicate queries, or queries that are not new or edited
	 */
	supportsOrdering: function() {
		if (! this.object) {
			return false;
		}
		if (this.isSearchQuery() || this.isTemplateQuery() || this.isSimilarityQuery()) {
			return true;
		}
		return !(this.getIsBeingEdited() || this.isNew());
	},
	/* 
	 * Answer true if the query is a similarity query (so don't show it in the componenent menu)
	 */
	isSimilarityQuery: function() {
		if (!this.object) {
			return false;
		}
		return this.object._isSimilarityQuery; // set in QueryResults
	},
	/* 
	 * Answer true if the query is a search query (so don't show it in the componenent menu)
	 */
	isSearchQuery: function() {
		if (!this.object) {
			return false;
		}
		return this.object._isSearchQuery;  // set in QueryResults
	},
	/* 
	 * Answer true if the query is a template query (so don't show it in the componenent menu)
	 */
	isTemplateQuery: function() {
		if (!this.object) {
			return false;
		}
		return this.object._isTemplateQuery;  // set in QueryResults
	},	
	resetToStored: function() {
	    var wasNew = this.object.isNew
		this.object = this.storedObject;
	    // TODO(QueryDescriptor): New concept for query.isNew()
		this.object._isNew= wasNew;
	},
	__sential: true
});

QueryProxy.QUERY_LABEL_ELLIPSIS= "..."; //$NON-NLS-1$

})();
