#if defined(MVS)                                                                
#pragma nosequence nomargins                                                    
#if !defined(MVSCICS)                                                           
#pragma strings(readonly)                                                       
#pragma linkage(QMMAIN, FETCHABLE)                                              
#endif                                                                          
#endif                                                                          
#ifdef SCCSID_MAKEFILE                                                          
static char *sccsid_makefile = SCCSID_MAKEFILE;                                 
#endif                                                                          
static char *sccsid = "@(#)MQlib.c 1.22 MQlib 10/16/01 SSB ITD";                
/****************************************************************************** 
**                                                                              
**      State Street Bank and Trust                                             
**                                                                              
******************************************************************************* 
**                                                                              
**      File: MQlib.c                                                           
**                                                                              
**      SCCS Information                                                        
**          version: 1.22                                                       
**          date changed: 10/16/01 13:31:32                                     
**          date retrieved: 10/17/01                                            
**                                                                              
**      Description:                                                            
**    This file provides a set of functions to access MQSeries Queues.          
**                                                                              
**                                                                              
******************************************************************************* 
**                                                                              
**   R E V I S I O N   L O G                                                    
**                                                                              
**   Date    Author        Modification                                         
**   --------    ---------------    ------------------------------------        
**   06/10/03    G. Weinberger    Implement Archive Message                     
**   01/03/03    G. Weinberger    Add XA errors to MQr2string.                  
**                Indicate transaction started when MQBEGIN                     
**                returns XA error.                                             
**   11/20/02    G. Weinberger    Add XA transaction handling.                  
**   10/16/01   B. Sylvester    Since we're under time constraints to           
**                              get the bug fix below I forced Expiry           
**                              to -1 (unlimited retention) if default          
**                              retention from qmInitDescriptor is              
**                              7 days(old MQlib.h).  This means the            
**                              apps don't need to recompile their              
**                              code for the bug fix to work.  We'll            
**                              need to address this later when we get          
**                              the bug fix from IBM.                           
**                   !!!NOTE!!! Until then we can't use 7 days                  
**                              as the initial value of Expiry.                 
**   10/05/01   B. Sylvester    We no longer have a limit of 1 week             
**                              for retention (msg descriptor).  The            
**                              default was changed to unlimited in MQlib.h     
**                              because of an MQ bug that deletes all           
**                              messages with expiry that are a multiple        
**                              of 4 Gs since 1970.                             
**   09/17/99    G. Weinberger    MVS only changes: Reset Hobj on close.        
**                Handle PAGESET_FULL reason code.                              
**   10/21/98    G. Weinberger    Don't set MQGMO_SYNCPOINT option for browse   
**   09/04/98    G. Weinberger    Modify disconnect message.                    
**   09/03/98    G. Weinberger    Turn off safestore. Inquire for max msglen    
**                when opening queue. Various bug fixes.                        
**   05/28/98    G. Weinberger    Initialize rv in safestore_message()          
**   05/01/98    G. Weinberger    various modifications to safestore.           
**                QM_BINARY now option for get.                                 
**                add new DEBUG_LEVEL DBUG_ERRMSG.                              
**   03/26/98   Ketan Patel     Added safestore code for put and get messages   
**   02/18/98    G. Weinberger    Remove all rererence to util functions for    
**                MVSCICS                                                       
**   02/10/98    G. Weinberger    Convert text message. qmInitDescriptor now a  
**                macro.                                                        
**   01/07/98    G. Weinberger    new method for debugging                      
**   11/20/97    G. Weinberger    port for MVS CICS                             
**   10/29/97    G. Weinberger    add MVS single entry point QMMAIN             
**   10/03/97    G. Weinberger    change one debug to DBUG_MEDIUM               
**   09/26/97    G. Weinberger    make qmMsgidString() thread-safe              
**   09/17/97    G. Weinberger    rewrite for new api                           
**   07/10/97    G. Weinberger    allow qmOpenQueue for input and output.       
**                make msg_id input parm for qmPutMsg.                          
**                make msg_id input parm for qmGetMsg.                          
**                make msg_correlation_id output parm                           
**                for qmGetMsg.                                                 
**   05/16/97    G. Weinberger      created                                     
**                                                                              
******************************************************************************/ 
#include <stdio.h>                                                              
#include <stdlib.h>                                                             
#include <string.h>                                                             
#include <stdarg.h>                                                             
#include <ctype.h>                                                              
                                                                                
#if defined(MVS)                                                                
#include <cics.h>                                                               
#include <manifest.h>                                                           
#endif                                                                          
                                                                                
/* #include <cmqc.h> */    /* include for MQI */                                
                                                                                
#include "util.h"                                                               
#include "MQlib.h"                                                              
#include "archmsg.h"                                                            
                                                                                
/* defines for put_type */                                                      
#define PUT_NORMAL    0                                                         
#define PUT1        1                                                           
#define REPLYTO        2                                                        
                                                                                
/* defines for descriptor_type */                                               
#define PUT        0                                                            
#define GET        1                                                            
#define REPLY        2                                                          
#define ARCHMSG        3                                                        
                                                                                
/* special debug level */                                                       
#define DBUG_ERRMSG    -1                                                       
                                                                                
/* macro for debug file pointer */                                              
#define DBGFP(h) (((h) == NULL_CONNECTION_HANDLE) ? \                           
          ((FILE *)NULL) : ((h)->debug_fp))                                     
                                                                                
/* macros for determining result of get */                                      
#define BROWSE_OPTS (QM_BROWSE_FIRST|QM_BROWSE_NEXT)                            
#define MSG_BROWSED(d) ((d->options)&BROWSE_OPTS)                               
#define MSG_RETRIEVED_FULL(r) ((r==QM_SUCCESS)||(r==QM_CONVERSION_ERROR))       
#define MSG_RETRIEVED(r) ((MSG_RETRIEVED_FULL(r))||(r==QM_MSG_TOO_BIG))         
#define MSG_TOO_BIG_DEQUEUED(r,d) ((r==QM_MSG_TOO_BIG)&&\                       
                   ((d->options)&QM_ACCEPT_TRUNCATED_MSG))                      
#define MSG_DEQUEUED(r,d) ((!MSG_BROWSED(d))&&\                                 
               ((MSG_RETRIEVED_FULL(r))||\                                      
                (MSG_TOO_BIG_DEQUEUED(r,d))))                                   
                                                                                
const static QM_MSGID    qm_msgid_none = {MQMI_NONE_ARRAY};                     
                                                                                
static int debugging = 0;                                                       
                                                                                
/* static void make_connections_invalid( CONNECTION_HANDLE ); */                
static void init_connection_list( CONNECTION_HANDLE );                          
static void copy_MQMD( int, MQMD *, char * );                                   
static int check_msg_descriptor( CONNECTION_HANDLE, MSG_DESCRIPTOR *,           
                char *,int );                                                   
static int check_queue_handle( CONNECTION_HANDLE, QUEUE_HANDLE );               
static int check_connection( CONNECTION_HANDLE );                               
static int check_queue_handle_id( QUEUE_HANDLE );                               
static int open_safestore_queue( CONNECTION_HANDLE );                           
static int check_sstor_buf( char * );                                           
#if !defined(MVS)                                                               
static int start_luw( CONNECTION_HANDLE );                                      
#endif                                                                          
static int commit_luw( CONNECTION_HANDLE );                                     
static int backout_luw( CONNECTION_HANDLE );                                    
static int open_queue( CONNECTION_HANDLE, unsigned short int, char *, int );    
static int get_max_msglen( CONNECTION_HANDLE, unsigned short );                 
static int put_msg( CONNECTION_HANDLE, int, unsigned short, char *,             
           MSG_DESCRIPTOR *, char *, long );                                    
static int get_msg( CONNECTION_HANDLE, unsigned short,                          
           MSG_DESCRIPTOR *, char *, long );                                    
static int close_queue( CONNECTION_HANDLE handle, unsigned short int );         
static int connect_to_qm( CONNECTION_HANDLE, char * );                          
static int disconnect( CONNECTION_HANDLE );                                     
static int handle_MQ_error( CONNECTION_HANDLE, char * );                        
static const char *MQr2string( int );                                           
static void pdebug( int debug_level, CONNECTION_HANDLE, char *, char *, ... );  
static int close_safestore_queue( CONNECTION_HANDLE );                          
static int safestore_message( CONNECTION_HANDLE, MSG_DESCRIPTOR *,              
                 MQMD *, char *, long, char * );                                
static void build_header( MSG_DESCRIPTOR *, MQMD *, CONNECTION_HANDLE,          
             long, char *, char *);                                             
static char *convertByteFld( const QM_BYTEFLD data, BYTE_STRNG pstring,         
                 int fldsize );                                                 
static void copy_field(char *archive_msg, int  *bufIndex, char *field,          
               int max_field_len);                                              
static char *get_store( int );                                                  
static void free_store( char * );                                               
static void get_system_id( CONNECTION_HANDLE );                                 
                                                                                
#define DEBUGGING     "MQ_DEBUGGING"                                            
                                                                                
#if defined(MVS)                                                                
                                                                                
#define DBUG_DDNAME     "MQLOG"                                                 
#define ENV_DDNAME     "MQENV"                                                  
#define ENV_FILE_LRECL     80                                                   
#define DEBUG_LEVEL     "MQ_DEBUG_LEVEL"                                        
                                                                                
static int getMVSenv( void );                                                   
                                                                                
/*=========================================================================     
**                                                                              
**    QMMAIN( )                                                                 
**                                                                              
**    This MVS only function is the fetchable entry point for MQlib             
**    on MVS, It calls the MQlib functions depending on the values in           
**    the QM_COMM structure.                                                    
**                                                                              
**    Input:                                                                    
**        QM_COMM                                                               
**    Output:                                                                   
**        QM_COMM.return_code                                                   
**        Output of called function                                             
**                                                                              
**============================================================================  
*/                                                                              
#if defined(MVSCICS)                                                            
DFHEIBLK *dfheptr;                                                              
void main( int argc, char *argv)                                                
#else                                                                           
void QMMAIN( QM_COMM *qm_comm )                                                 
#endif                                                                          
{                                                                               
#if defined(MVSCICS)                                                            
  QM_COMM     *qm_comm;                                                         
#endif                                                                          
  int         (**elist)() = NULL;                                               
  FILE         *open_debug_fp = (FILE *)NULL;                                   
  char         *env_value = NULL;                                               
  int         debug_level = 0;                                                  
  FILE         *debug_fp = (FILE *)NULL;                                        
  int        i;                                                                 
  int         rv = QM_SUCCESS;                                                  
  char        *func_name = "QMMAIN";                                            
                                                                                
  pdebug( DBUG_ENTRY_EXIT, qm_comm->conn_handle, func_name,                     
    "Entry, function = %d", qm_comm->function );                                
                                                                                
#if defined(MVSCICS)                                                            
  EXEC CICS ADDRESS EIB(dfheptr);                                               
  EXEC CICS ADDRESS COMMAREA(qm_comm);                                          
#endif                                                                          
                                                                                
  switch( qm_comm->function )                                                   
    {                                                                           
#if !defined(MVSCICS)                                                           
    case QMELIST:                                                               
                                                                                
      /*******************************************************************      
       **    malloc storage for entry point list                                
       ******************************************************************/      
                                                                                
      elist = malloc( (ELIST_SIZE*sizeof(int (*)())) );                         
      if ( elist == NULL )                                                      
    {                                                                           
      pdebug( DBUG_ERRMSG, qm_comm->conn_handle, func_name,                     
        "Unable to malloc elist" );                                             
      qm_comm->name2 = (char *)NULL;                                            
      qm_comm->return_code = QM_INSUFFICIENT_MEMORY;                            
      break;                                                                    
    }                                                                           
                                                                                
      /*******************************************************************      
       **    add all the entry points to the list                               
       ******************************************************************/      
                                                                                
      elist??(QMCONNECT??) =                                                    
    (int (*)())fetchep( (void (*)())&qmConnect );                               
      elist??(QMDISCONNECT??) =                                                 
    (int (*)())fetchep( (void (*)())&qmDisconnect );                            
      elist??(QMBEGINTRANSACTION??) =                                           
    (int (*)())fetchep((void (*)()) &qmBeginTransaction );                      
      elist??(QMCOMMITTRANSACTION??)  =                                         
    (int (*)())fetchep((void (*)()) &qmCommitTransaction );                     
      elist??(QMBACKOUTTRANSACTION??) =                                         
    (int (*)())fetchep( (void (*)())&qmBackoutTransaction );                    
      elist??(QMOPENQUEUE??) =                                                  
    (int (*)())fetchep( (void (*)())&qmOpenQueue );                             
      elist??(QMCLOSEQUEUE??) =                                                 
    (int (*)())fetchep( (void (*)())&qmCloseQueue );                            
      elist??(QMPUTMSG??) =                                                     
    (int (*)())fetchep( (void (*)())&qmPutMsg );                                
      elist??(QMPUTONEMSG??) =                                                  
    (int (*)())fetchep( (void (*)())&qmPutOneMsg );                             
      elist??(QMGETMSG??) =                                                     
    (int (*)())fetchep( (void (*)())&qmGetMsg );                                
      elist??(QMREPLYTOMSG??) =                                                 
    (int (*)())fetchep( (void (*)())&qmReplyToMsg );                            
      elist??(QMALLOCBUF??) =                                                   
    (int (*)())fetchep( (void (*)())&qmAllocBuf );                              
      elist??(QMFREEBUF??) =                                                    
    (int (*)())fetchep( (void (*)())&qmFreeBuf );                               
      elist??(QMMSGIDCPY??) =                                                   
    (int (*)())fetchep( &qmMsgidCpy );                                          
      elist??(QMMSGIDNONE??) =                                                  
    (int (*)())fetchep( &qmMsgidNone );                                         
      elist??(QMMSGIDSTRING??) =                                                
    (int (*)())fetchep( (void (*)())&qmMsgidString );                           
      elist??(QMARCHIVEMSG??) =                                                 
    (int (*)())fetchep( (void (*)())&qmArchiveMsg );                            
                                                                                
      /*******************************************************************      
       **    get file pointer for open debug file                               
       ******************************************************************/      
                                                                                
      open_debug_fp = (FILE *)(qm_comm->queue_handle);                          
                                                                                
      /*******************************************************************      
       **    return pointer to elist in QM_COMM structure                       
       ******************************************************************/      
                                                                                
      qm_comm->name2 = (char *)elist;                                           
                                                                                
#endif         /* !defined(MVSCICS) */                                          
                                                                                
    case QMCONNECT:                                                             
      if ( qm_comm->conn_handle == NULL_CONNECTION_HANDLE )                     
    {                                                                           
      /*******************************************************************      
       **    the first time through get env and open debug log file             
       ******************************************************************/      
      rv = getMVSenv( );                                                        
#if !defined(MVSCICS)                                                           
      if ( rv == QM_SUCCESS )                                                   
        {                                                                       
          /************************************************************         
           **    get DEBUGGING environment value                                
           ***********************************************************/         
                                                                                
          env_value = getenv( DEBUGGING );                                      
          if ( env_value != NULL )                                              
        if ( ( strcmp( env_value, "ON" ) == 0 ) ||                              
            ( strcmp( env_value, "YES" ) == 0 ) )                               
          debugging = 1;                                                        
                                                                                
          /**************************************************************       
           **    set debug_level from env variable                              
           *************************************************************/       
                                                                                
          env_value = getenv( DEBUG_LEVEL );                                    
          if ( env_value != NULL )                                              
        debug_level = (int) strtol(env_value, (char **) NULL, 0);               
          reset_debug_level( debug_level );                                     
                                                                                
          /**************************************************************       
           **    if debug file not open, open it now                            
           *************************************************************/       
                                                                                
          if ( !iscics( ) &&  debugging )                                       
        if ( open_debug_fp == (FILE *)NULL )                                    
          {                                                                     
            debug_fp = open_debug_file( DBUG_DDNAME );                          
            if ( debug_fp == (FILE *)NULL )                                     
              {                                                                 
            pdebug( DBUG_ERRMSG, qm_comm->conn_handle, func_name,               
                  "QMMAIN: Unable to open file %s\n",                           
                  DBUG_DDNAME );                                                
            rv = QM_FAIL;                                                       
              }                                                                 
          }                                                                     
        else                                                                    
          debug_fp = open_debug_fp;                                             
        }                                                                       
#endif    /* !defined(MVSCICS) */                                               
    }    /* endif ( qm_comm->conn_handle == NULL ) */                           
      else                                                                      
    debug_fp = (qm_comm->conn_handle)->debug_fp;                                
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    {                                                                           
      qm_comm->return_code = qmConnect( qm_comm->name,                          
                       qm_comm->options,                                        
                       debug_fp,                                                
                       &(qm_comm->conn_handle) );                               
                                                                                
      if ( qm_comm->conn_handle == NULL_CONNECTION_HANDLE )                     
        if ( !iscics( ) && debugging )                                          
          if ( open_debug_fp == (FILE *)NULL )                                  
        fclose( debug_fp );                                                     
    }                                                                           
      else                                                                      
    qm_comm->return_code = rv;                                                  
      break;                                                                    
    case QMDISCONNECT:                                                          
      qm_comm->return_code = qmDisconnect( qm_comm->conn_handle );              
      break;                                                                    
    case QMBEGINTRANSACTION:                                                    
      if ( iscics( ) )                                                          
    {                                                                           
      pdebug( DBUG_ERRMSG, qm_comm->conn_handle, func_name,                     
        "QMBEGINTRANSACTION invalid for CICS" );                                
      qm_comm->return_code = QM_INVALID;                                        
    }                                                                           
      else                                                                      
    qm_comm->return_code = qmBeginTransaction( qm_comm->conn_handle );          
      break;                                                                    
    case QMCOMMITTRANSACTION:                                                   
      if ( iscics( ) )                                                          
    {                                                                           
      pdebug( DBUG_ERRMSG, qm_comm->conn_handle, func_name,                     
        "QMCOMMITTRANSACTION invalid for CICS" );                               
      qm_comm->return_code = QM_INVALID;                                        
    }                                                                           
      else                                                                      
    qm_comm->return_code = qmCommitTransaction( qm_comm->conn_handle );         
      break;                                                                    
    case QMBACKOUTTRANSACTION:                                                  
      if ( iscics( ) )                                                          
    {                                                                           
      pdebug( DBUG_ERRMSG, qm_comm->conn_handle, func_name,                     
        "QMBASCKOUTTRANSACTION invalid for CICS" );                             
      qm_comm->return_code = QM_INVALID;                                        
    }                                                                           
      else                                                                      
    qm_comm->return_code = qmBackoutTransaction( qm_comm->conn_handle );        
      break;                                                                    
    case QMOPENQUEUE:                                                           
      qm_comm->return_code = qmOpenQueue( qm_comm->conn_handle,                 
                       qm_comm->name,                                           
                       qm_comm->options,                                        
                       &(qm_comm->queue_handle) );                              
       break;                                                                   
    case QMCLOSEQUEUE:                                                          
      qm_comm->return_code = qmCloseQueue( qm_comm->conn_handle,                
                      qm_comm->queue_handle );                                  
      break;                                                                    
    case QMPUTMSG:                                                              
      qm_comm->return_code = qmPutMsg( qm_comm->conn_handle,                    
                      qm_comm->queue_handle,                                    
                      qm_comm->descriptor,                                      
                      qm_comm->msg_buffer,                                      
                      qm_comm->buf_len );                                       
      break;                                                                    
    case QMPUTONEMSG:                                                           
      qm_comm->return_code = qmPutOneMsg( qm_comm->conn_handle,                 
                     qm_comm->name,                                             
                     qm_comm->descriptor,                                       
                     qm_comm->msg_buffer,                                       
                     qm_comm->buf_len );                                        
      break;                                                                    
    case QMGETMSG:                                                              
      qm_comm->return_code = qmGetMsg( qm_comm->conn_handle,                    
                      qm_comm->queue_handle,                                    
                      qm_comm->descriptor,                                      
                      qm_comm->msg_buffer,                                      
                      qm_comm->buf_len );                                       
      break;                                                                    
    case QMREPLYTOMSG:                                                          
      qm_comm->return_code = qmReplyToMsg( qm_comm->conn_handle,                
                       qm_comm->descriptor,                                     
                      qm_comm->msg_buffer,                                      
                      qm_comm->buf_len );                                       
      break;                                                                    
    case QMALLOCBUF:                                                            
      qm_comm->msg_buffer = qmAllocBuf( qm_comm->buf_len );                     
      if ( qm_comm->msg_buffer == (char *)NULL )                                
    qm_comm->return_code = QM_INSUFFICIENT_MEMORY;                              
      else                                                                      
    qm_comm->return_code = QM_SUCCESS;                                          
      break;                                                                    
    case QMFREEBUF:                                                             
      qm_comm->return_code = qmFreeBuf( qm_comm->msg_buffer );                  
      break;                                                                    
    case QMMSGIDCPY:                                                            
      qmMsgidCpy( (QM_MSGID *)(qm_comm->name), qm_comm->name2 );                
      qm_comm->return_code = QM_SUCCESS;                                        
      break;                                                                    
    case QMMSGIDNONE:                                                           
      qmMsgidNone( (QM_MSGID *)(qm_comm->name) );                               
      qm_comm->return_code = QM_SUCCESS;                                        
      break;                                                                    
    case QMMSGIDSTRING:                                                         
      (void)qmMsgidString( qm_comm->name, qm_comm->name2 );                     
      qm_comm->return_code = QM_SUCCESS;                                        
      break;                                                                    
    case FUNCMQCCODE:                                                           
      qm_comm->return_code = MQCCODE(qm_comm->conn_handle);                     
      break;                                                                    
    case FUNCMQREASON:                                                          
      qm_comm->return_code = MQREASON(qm_comm->conn_handle);                    
      break;                                                                    
    case FUNCQMERRMSG:                                                          
      strncpy( qm_comm->name, QMERRMSG(qm_comm->conn_handle), QM_ERRMSG_LEN );  
      for( i = QM_ERRMSG_LEN-1;                                                 
      ( i >= 0 ) && ( (qm_comm->name)??(i??) == '\0' );                         
      i-- )                                                                     
    (qm_comm->name)??(i??) = ' ';                                               
      qm_comm->return_code = QM_SUCCESS;                                        
      break;                                                                    
    case FUNCQMMAXMSGLEN:                                                       
      qm_comm->return_code = qmMaxMsglen( qm_comm->conn_handle,                 
                      qm_comm->queue_handle );                                  
      break;                                                                    
    case QMARCHIVEMSG:                                                          
      qm_comm->return_code = qmArchiveMsg( qm_comm->conn_handle,                
                       qm_comm->descriptor,                                     
                       qm_comm->msg_buffer,                                     
                       qm_comm->buf_len );                                      
      break;                                                                    
    default:                                                                    
      pdebug( DBUG_ERRMSG, qm_comm->conn_handle, func_name,                     
          "Invalid function identifier %d",                                     
          qm_comm->function );                                                  
      qm_comm->return_code = QM_INVALID;                                        
      break;                                                                    
    }                                                                           
                                                                                
  pdebug( DBUG_ENTRY_EXIT, qm_comm->conn_handle, func_name,                     
    "Exit, return_code = %d", qm_comm->return_code );                           
}                                                                               
                                                                                
                                                                                
/*=========================================================================     
**                                                                              
**    getMVSenv( )                                                              
**                                                                              
**    This internal MVS only function reads the MQENV file and sets             
**    Environment values.                                                       
**    Each line in the MQENV should be of the form:                             
**        NAME=VALUE                                                            
**    where the environment variable NAME is set to VALUE.                      
**                                                                              
**    Input:                                                                    
**        File with ddname MQENV                                                
**    Output:                                                                   
**        Set environment variables                                             
**                                                                              
**==========================================================================    
*/                                                                              
static int getMVSenv( )                                                         
{                                                                               
  char         ddname??(12??);                                                  
  FILE         *env_fp;                                                         
  char         env_line??(ENV_FILE_LRECL+1??);                                  
  size_t     amt;                                                               
  char         *p1, *p2;                                                        
  int         stat, rv = QM_SUCCESS;                                            
  char        *func_name = "getMVSenv";                                         
                                                                                
#if !defined(MVSCICS)                                                           
  /*******************************************************************          
   **    open the environment data set                                          
   ******************************************************************/          
                                                                                
  sprintf( ddname, "%s%s", "DD:", ENV_DDNAME );                                 
  env_fp = fopen( ddname, "rb, type=record" );                                  
  if ( env_fp == (FILE *)NULL )                                                 
    {                                                                           
      pdebug( DBUG_ERRMSG, NULL_CONNECTION_HANDLE, func_name,                   
        "Unable to open environment file %s", ENV_DDNAME );                     
      rv = QM_FAIL;                                                             
    }                                                                           
  else                                                                          
    while( 1 )                                                                  
      {                                                                         
    /*******************************************************************        
     **    read each record and set the env strings                             
     ******************************************************************/        
                                                                                
    amt = fread( env_line, 1, ENV_FILE_LRECL, env_fp );                         
    if ( amt <= 0 )                                                             
      break;                                                                    
    env_line??(ENV_FILE_LRECL??) = '\0';                                        
    p1 = strtok( env_line, "=" );                                               
    if ( p1 == NULL )                                                           
      continue;                                                                 
    p2 = strtok( NULL, " " );                                                   
    stat = setenv( p1, p2, 1 );                                                 
    if ( stat != 0 )                                                            
      pdebug( DBUG_NORMAL, NULL_CONNECTION_HANDLE, func_name,                   
        "setenv failed for %s", p1 );                                           
                                                                                
      }                                                                         
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
                                                                                
#endif     /* defined(MVS) */                                                   
                                                                                
                                                                                
/*==========================================================================    
**                                                                              
**    qmConnect( )                                                              
**                                                                              
**    This function calls MQCONN to connect to an MQSeries Queue Manager.       
**                                                                              
**    Input:                                                                    
**        Queue Manager name                                                    
**        Options                                                               
**        Debug file pointer                                                    
**    Output:                                                                   
**        Return Code                                                           
**        CONNECTION_HANDLE                                                     
**                                                                              
**==========================================================================    
*/                                                                              
int qmConnect(                                                                  
          char *qm_name,                                                        
          QM_OPTION options,                                                    
          FILE *debug_fp,                                                       
          CONNECTION_HANDLE *conn_handle                                        
          )                                                                     
{                                                                               
  MQLONG    CompCode;                                                           
  MQLONG    Reason;                                                             
  CONNECTION_HANDLE local_handle = (CONNECTION_HANDLE)NULL;                     
  int        stat, rv = QM_SUCCESS;                                             
  int        i, list_size;                                                      
  char        *env_value = NULL;                                                
  char        *func_name = "qmConnect";                                         
                                                                                
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, *conn_handle, func_name,                             
    "qmConnect: Entry, qm_name = %.*s, options = %08x",                         
    QM_QUEUE_LEN, qm_name, options );                                           
#endif                                                                          
  if ( *conn_handle == NULL_CONNECTION_HANDLE )                                 
    {                                                                           
      /*******************************************************************      
       **    get DEBUGGING envirionment value                                   
       **    for MVS, this was already done in QMMAIN                           
       ******************************************************************/      
                                                                                
#if !defined(MVS)                                                               
      env_value = getenv( DEBUGGING );                                          
      if ( env_value != NULL )                                                  
    if ( ( strcmp( env_value, "ON" ) == 0 ) ||                                  
        ( strcmp( env_value, "YES" ) == 0 ) )                                   
      debugging = 1;                                                            
#endif                                                                          
                                                                                
      /*******************************************************************      
       **    malloc new QM_CONNECTION                                           
       ******************************************************************/      
                                                                                
      pdebug( DBUG_FINE, *conn_handle, func_name,                               
         "Allocating new QM_CONNECTION structure" );                            
                                                                                
      local_handle = (CONNECTION_HANDLE)get_store( sizeof(QM_CONNECTION) );     
      if ( local_handle == (CONNECTION_HANDLE)NULL )                            
    {                                                                           
      pdebug( DBUG_ERRMSG, local_handle, func_name,                             
         "allocation of storage for QM_CONNECTION failed" );                    
      rv = QM_INSUFFICIENT_MEMORY;                                              
    }                                                                           
      else                                                                      
    {                                                                           
      /**************************************************************           
       **    initialize new QM_CONNECTION                                       
       *************************************************************/           
                                                                                
      strncpy( local_handle->struct_id, QM_CONNECTION_ID, QM_ID_LEN );          
      local_handle->version = QM_CONNECTION_VERSION;                            
      local_handle->state = QM_UNCONNECTED;                                     
      local_handle->luw_flag = 0;                                               
      local_handle->connection_seqno = 0;                                       
      local_handle->debug_fp = debug_fp;                                        
      memset( local_handle->queue_manager_name, 0, QM_QUEUE_LEN );              
      local_handle->max_queues = 0;                                             
#if defined(MVS)                                                                
      local_handle->mqmain = NULL;                                              
#endif                                                                          
      local_handle->connection_list = (QUEUE_CONNECTION *)NULL;                 
      local_handle->CompCode = MQCC_OK;                                         
      local_handle->Reason = MQRC_NONE;                                         
      memset( local_handle->errmsg, 0, QM_ERRMSG_LEN );                         
      memset( local_handle->system_id, 0, QM_SYSID_LEN );                       
      memset( local_handle->reserve, 0, QM_RESERVE_LEN );                       
    }                                                                           
    }                                                                           
  else                                                                          
    {                                                                           
      /*******************************************************************      
       **    check existing CONNECTION_HANDLE                                   
       ******************************************************************/      
#if 0                                                                           
      pdebug( DBUG_FINE, *conn_handle, func_name,                               
        "Reusing QM_CONNECTION structure" );                                    
#endif                                                                          
      local_handle = *conn_handle;                                              
      if ( strncmp( local_handle->struct_id,                                    
           QM_CONNECTION_ID,                                                    
           QM_ID_LEN ) != 0 )                                                   
    {                                                                           
      pdebug( DBUG_ERRMSG, NULL_CONNECTION_HANDLE, func_name,                   
        "connection handle has invalid structure id" );                         
      rv = QM_INVALID;                                                          
    }                                                                           
      else if ( local_handle->version != QM_CONNECTION_VERSION )                
    {                                                                           
      pdebug( DBUG_ERRMSG, NULL_CONNECTION_HANDLE, func_name,                   
        "connection handle has invalid version" );                              
      rv = QM_INVALID;                                                          
    }                                                                           
      else if ( CONNECTED( local_handle ) )                                     
    {                                                                           
      pdebug( DBUG_ERRMSG, local_handle, func_name,                             
        "Already connected to Queue Manager" );                                 
      rv = QM_ALREADY_CONNECTED;                                                
    }                                                                           
      else                                                                      
    {                                                                           
      local_handle->state = QM_UNCONNECTED;                                     
      local_handle->luw_flag = 0;                                               
      local_handle->debug_fp = debug_fp;                                        
    }                                                                           
    }                                                                           
                                                                                
  /*******************************************************************          
   **    malloc new queue_connection_list                                       
   ******************************************************************/          
                                                                                
  if ( ( rv == QM_SUCCESS ) &&                                                  
      ( local_handle->connection_list == (QUEUE_CONNECTION *)NULL ) )           
    {                                                                           
      list_size = (MAX_QUEUES+1)*sizeof(QUEUE_CONNECTION);                      
      pdebug( DBUG_FINE, local_handle, func_name,                               
        "about to allocate connection list, size = %d",                         
        list_size );                                                            
                                                                                
      local_handle->connection_list =                                           
    (QUEUE_CONNECTION *)get_store( list_size );                                 
      if ( local_handle->connection_list == (QUEUE_CONNECTION *)NULL )          
    {                                                                           
      pdebug( DBUG_ERRMSG, local_handle, func_name,                             
        "storage allocation for connection list, size %d, failed",              
        list_size );                                                            
      rv = QM_INSUFFICIENT_MEMORY;                                              
    }                                                                           
      else                                                                      
    {                                                                           
      local_handle->max_queues = MAX_QUEUES;                                    
    }                                                                           
    }                                                                           
                                                                                
  /*******************************************************************          
   **    get system id                                                          
   ******************************************************************/          
                                                                                
  if ( ( rv == QM_SUCCESS ) && ( (local_handle->system_id)??(0??) == 0 ) )      
    {                                                                           
      pdebug( DBUG_FINE, local_handle, func_name,                               
         "Getting system id" );                                                 
      get_system_id( local_handle );                                            
    }                                                                           
                                                                                
  /*******************************************************************          
   **    Connect to Queue Manager                                               
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      init_connection_list( local_handle );                                     
      pdebug( DBUG_FINE, local_handle, func_name,                               
        "Connecting to Queue Manager" );                                        
                                                                                
      rv = connect_to_qm( local_handle, qm_name );                              
      if ( rv != QM_SUCCESS )                                                   
    {                                                                           
      if ( rv == QM_CONNECTION_BROKEN )                                         
        rv = QM_NOT_AVAILABLE;                                                  
      local_handle->state = QM_UNCONNECTED;                                     
    }                                                                           
    }                                                                           
  *conn_handle = local_handle;                                                  
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, local_handle, func_name,                             
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    qmDisconnect( )                                                           
**                                                                              
**    This function calls MQDISC to disconnect from the currently connected     
**    Queue Manager. The QUEUE_HANDLE for all open queues are made invalid.     
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
int qmDisconnect( CONNECTION_HANDLE conn_handle )                               
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmDisconnect";                                      
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    disconnect                                                             
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      /* if safestore requested, close the queue */                             
      if ( (conn_handle->connection_list)??(0??).Hobj != MQHO_UNUSABLE_HOBJ )   
        {                                                                       
       pdebug( DBUG_NORMAL, conn_handle, func_name,                             
             "closing safestore queue" );                                       
          rv = close_safestore_queue( conn_handle );                            
          if ( rv != QM_SUCCESS )                                               
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "close of safestore queue failed" );                                
        }                                                                       
        }                                                                       
                                                                                
      rv = disconnect( conn_handle );                                           
      if ( ( rv == QM_SUCCESS ) ||                                              
      ( rv == QM_BACKED_OUT ) ||                                                
      ( rv == QM_CONNECTION_BROKEN ) )                                          
    {                                                                           
      if ( rv == QM_SUCCESS )                                                   
        pdebug( DBUG_MEDIUM, conn_handle, func_name,                            
            "Successfully disconnected from Queue Manager" );                   
      else                                                                      
        pdebug( DBUG_MEDIUM, conn_handle, func_name,                            
            "Now disconnected from Queue Manager" );                            
      conn_handle->state = QM_UNCONNECTED;                                      
      conn_handle->luw_flag = 0;                                                
    }                                                                           
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    qmBeginTransaction( )                                                     
**                                                                              
**    This function starts a unit of work. luw_flag is set.                     
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
int qmBeginTransaction( CONNECTION_HANDLE conn_handle )                         
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmBeginTransaction";                                
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    make sure unit of work not already started                             
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    if ( LUW_STARTED( conn_handle ) )                                           
      {                                                                         
    pdebug( DBUG_ERRMSG, conn_handle, func_name,                                
          "Unit of Work already started" );                                     
    rv = QM_ALREADY_STARTED;                                                    
      }                                                                         
                                                                                
  /*******************************************************************          
   **    set luw_flag                                                           
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
      conn_handle->luw_flag = 1;                                                
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    qmBeginXATransaction( )                                                   
**                                                                              
**    This function starts a unit of work. MQBEGIN is called.                   
**    luw_flag is set.                                                          
**                                                                              
**    Note: As of mqseries 5.2 this is not supported in MVS.                    
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
#if !defined(MVS)                                                               
int qmBeginXATransaction( CONNECTION_HANDLE conn_handle )                       
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmBeginXATransaction";                              
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    make sure unit of work not already started                             
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    if ( LUW_STARTED( conn_handle ) )                                           
      {                                                                         
    pdebug( DBUG_ERRMSG, conn_handle, func_name,                                
          "Unit of Work already started" );                                     
    rv = QM_ALREADY_STARTED;                                                    
      }                                                                         
                                                                                
  /*******************************************************************          
   **    Start the transaction                                                  
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      rv = start_luw( conn_handle );                                            
      if ( rv == QM_SUCCESS )                                                   
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "Transaction Successfully Started" );                                 
      conn_handle->luw_flag = 1;                                                
    }                                                                           
      else if ( ( rv == QM_XA_NOT_CONFIGURED ) ||                               
        ( rv == QM_XA_NOT_AVAILABLE ) )                                         
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
          "Transaction started without XA participation" );                     
      conn_handle->luw_flag = 1;                                                
    }                                                                           
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
#endif    /* !defined(MVS) */                                                   
                                                                                
/*=========================================================================     
**                                                                              
**    qmCommitTransaction( )                                                    
**                                                                              
**    This function calls MQCMIT to commit a unit of work.                      
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
int qmCommitTransaction( CONNECTION_HANDLE conn_handle )                        
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmCommitTransaction";                               
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    make sure unit of work started                                         
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    if ( !LUW_STARTED( conn_handle ) )                                          
      {                                                                         
    pdebug( DBUG_ERRMSG, conn_handle, func_name,                                
          "Unit of Work not started" );                                         
    rv = QM_NOT_STARTED;                                                        
      }                                                                         
                                                                                
  /****************************************************************             
   **    Commit the transaction                                                 
   ***************************************************************/             
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      rv = commit_luw( conn_handle );                                           
      if ( rv == QM_SUCCESS )                                                   
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
          "Transaction Successfully Committed" );                               
      if ( ( rv == QM_SUCCESS) || ( rv == QM_BACKED_OUT ) )                     
    conn_handle->luw_flag = 0;                                                  
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    qmBackoutTransaction( )                                                   
**                                                                              
**    This function calls MQBACK to back out a unit of work.                    
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
int qmBackoutTransaction( CONNECTION_HANDLE conn_handle )                       
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmBackoutTransaction";                              
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    make sure unit of work started                                         
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    if ( !LUW_STARTED( conn_handle ) )                                          
      {                                                                         
    pdebug( DBUG_ERRMSG, conn_handle, func_name,                                
          "Unit of Work not started" );                                         
    rv = QM_NOT_STARTED;                                                        
      }                                                                         
                                                                                
  /****************************************************************             
   **    Back the transaction out                                               
   ***************************************************************/             
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      rv = backout_luw( conn_handle );                                          
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Transaction Successfully Backed out" );                                
      conn_handle->luw_flag = 0;                                                
    }                                                                           
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    qmOpenQueue( )                                                            
**                                                                              
**    This function calls MQOPEN to open a queue. If it succeeds, the           
**    queue handle is added to the connection list.                             
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue name                                                            
**        access type                                                           
**    Output:                                                                   
**        Return Code                                                           
**        QUEUE_HANDLE                                                          
**                                                                              
**==========================================================================    
*/                                                                              
int qmOpenQueue(                                                                
        CONNECTION_HANDLE conn_handle,                                          
        char *queue_name,                                                       
        int access_type,                                                        
        QUEUE_HANDLE *queue_handle                                              
        )                                                                       
{                                                                               
  int        found;                                                             
  unsigned short index;                                                         
  union queue_handle_union qhu;                                                 
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmOpenQueue";                                       
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry, queue_name = %.*s, access_type = %08x",                             
    QM_QUEUE_LEN, queue_name, access_type );                                    
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    find unused slot in connection list                                    
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      for ( found = 0, index = 1; index <= conn_handle->max_queues; index++ )   
    if ( QUEUE_HANDLE_UNUSABLE( conn_handle, index ) )                          
      {                                                                         
        found = 1;                                                              
        break;                                                                  
      }                                                                         
      if ( !found )                                                             
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
        "maximum queues already open" );                                        
      rv = QM_FAIL;                                                             
    }                                                                           
    }                                                                           
                                                                                
  /*******************************************************************          
   **    open the queue                                                         
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "queue_index = %d", index );                                            
      rv = open_queue( conn_handle, index, queue_name, access_type );           
      if ( rv == QM_SUCCESS )                                                   
    {                                                                           
      pdebug( DBUG_MEDIUM, conn_handle, func_name,                              
        "Queue Successfully Opened" );                                          
      qhu.qh_struct.connection_seqno = conn_handle->connection_seqno;           
      qhu.qh_struct.queue_index = index;                                        
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "queue_handle = %08lx", qhu.qh_long );                                  
      *queue_handle = qhu.qh_long;                                              
    }                                                                           
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    qmCloseQueue( )                                                           
**                                                                              
**    This function call MQCLOSE to close a queue. If it succeeds, the          
**    queue handle is removed from the connection list.                         
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        QUEUE_HANDLE                                                          
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**===========================================================================   
*/                                                                              
int qmCloseQueue( CONNECTION_HANDLE conn_handle, QUEUE_HANDLE queue_handle )    
{                                                                               
  union queue_handle_union qhu;                                                 
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmCloseQueue";                                      
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check QUEUE_HANDLE                                                     
   *********************************************************************/       
                                                                                
  rv = check_queue_handle( conn_handle, queue_handle );                         
                                                                                
  /*******************************************************************          
   **    close the queue                                                        
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      qhu.qh_long = queue_handle;                                               
      rv = close_queue( conn_handle, qhu.qh_struct.queue_index );               
      if ( rv == QM_SUCCESS )                                                   
     pdebug( DBUG_MEDIUM, conn_handle, func_name,                               
           "Successfully closed Queue" );                                       
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    qmPutMsg( )                                                               
**                                                                              
**    The function calls MQPUT to put a message on a queue. The user may        
**    specify the msgid or the msg_correlation_id to use for the message.       
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        QUEUE_HANDLE                                                          
**        MSG_DESCRIPTOR                                                        
**        message                                                               
**        message length                                                        
**    Output:                                                                   
**        Return Code                                                           
**        Updated MSG_DESCRIPTOR                                                
**                                                                              
**===========================================================================   
*/                                                                              
int qmPutMsg(                                                                   
         CONNECTION_HANDLE    conn_handle,                                      
         QUEUE_HANDLE    queue_handle,                                          
         MSG_DESCRIPTOR    *descriptor,                                         
         char         *msg,                                                     
         int         msg_len                                                    
         )                                                                      
{                                                                               
  union queue_handle_union qhu;                                                 
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmPutMsg";                                          
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check QUEUE_HANDLE                                                     
   *********************************************************************/       
                                                                                
  rv = check_queue_handle( conn_handle, queue_handle );                         
                                                                                
  /*******************************************************************          
   **    check MSG_DESCRIPTOR                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    rv = check_msg_descriptor( conn_handle, descriptor, msg, PUT );             
                                                                                
  /*******************************************************************          
   **    put message on queue                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      qhu.qh_long = queue_handle;                                               
      rv = put_msg( conn_handle,                                                
           PUT_NORMAL,                                                          
           qhu.qh_struct.queue_index,                                           
           (char *)NULL,                                                        
           descriptor,                                                          
           msg,                                                                 
           msg_len );                                                           
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
          "Message successfully put on Queue" );                                
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    qmPutOneMsg( )                                                            
**                                                                              
**    The function calls MQPUT1 to put a message on a queue. The user may       
**    specify the msgid or the msg_correlation_id to use for the message.       
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue_name                                                            
**        MSG_DESCRIPTOR                                                        
**        message                                                               
**        message length                                                        
**    Output:                                                                   
**        Return Code                                                           
**        Updated MSG_DESCRIPTOR                                                
**                                                                              
**===========================================================================   
*/                                                                              
int qmPutOneMsg(                                                                
         CONNECTION_HANDLE    conn_handle,                                      
         char        *queue_name,                                               
         MSG_DESCRIPTOR    *descriptor,                                         
         char         *msg,                                                     
         int         msg_len                                                    
         )                                                                      
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmPutOneMsg";                                       
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry, queue_name = %.*s",                                                 
    QM_QUEUE_LEN, queue_name );                                                 
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    check MSG_DESCRIPTOR                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    rv = check_msg_descriptor( conn_handle, descriptor, msg, PUT );             
                                                                                
  /*******************************************************************          
   **    put message on queue                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      rv = put_msg( conn_handle,                                                
           PUT1,                                                                
           (unsigned short)0,                                                   
           queue_name,                                                          
           descriptor,                                                          
           msg,                                                                 
           msg_len );                                                           
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
          "Message successfully put on Queue" );                                
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    qmGetMsg( )                                                               
**                                                                              
**    This function calls MQGET to get a message from a queue. The user         
**    may specify the msg_id or the msg_correlation_id for the message          
**    to be retrieved.                                                          
**    The user may also request that a unit of work be started if a             
**    message is retrieved, and that truncated messages be accepted. A          
**    timeout may be specified.                                                 
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        QUEUE_HANDLE                                                          
**        MSG_DESCRIPTOR                                                        
**        buffer to receive message                                             
**        buffer length                                                         
**    Output:                                                                   
**        Return Code                                                           
**        message (in buffer)                                                   
**        updated MSG_DESCRIPTOR                                                
**                                                                              
**===========================================================================   
*/                                                                              
int qmGetMsg(                                                                   
         CONNECTION_HANDLE    conn_handle,                                      
         QUEUE_HANDLE    queue_handle,                                          
         MSG_DESCRIPTOR    *descriptor,                                         
         char        *buffer,                                                   
         int        buf_len                                                     
         )                                                                      
{                                                                               
  union queue_handle_union qhu;                                                 
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "qmGetMsg";                                          
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check QUEUE_HANDLE                                                     
   *********************************************************************/       
                                                                                
  rv = check_queue_handle( conn_handle, queue_handle );                         
                                                                                
  /*******************************************************************          
   **    check MSG_DESCRIPTOR                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    rv = check_msg_descriptor( conn_handle, descriptor, buffer, GET );          
                                                                                
  /*******************************************************************          
   **    get message from queue                                                 
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      qhu.qh_long = queue_handle;                                               
      rv = get_msg( conn_handle,                                                
           qhu.qh_struct.queue_index,                                           
           descriptor,                                                          
           buffer,                                                              
           buf_len );                                                           
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
          "Message successfully retrieved from Queue" );                        
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    qmReplyToMsg( )                                                           
**                                                                              
**    This function sends a reply to the reply_to_queue on a REQUEST            
**    message.                                                                  
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        MSG_DESCRIPTOR as returned from qmGetMsg                              
**        message                                                               
**        message length                                                        
**    Output:                                                                   
**        Return Code                                                           
**        Updated MSG_DESCRIPTOR                                                
**                                                                              
**===========================================================================   
*/                                                                              
int qmReplyToMsg(                                                               
         CONNECTION_HANDLE    conn_handle,                                      
         MSG_DESCRIPTOR        *descriptor,                                     
         char            *msg,                                                  
         long            msg_len                                                
         )                                                                      
{                                                                               
  int         rv;                                                               
  char        *func_name = "qmReplyToMsg";                                      
#if 0                                                                           
    pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                            
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    check MSG_DESCRIPTOR                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    rv = check_msg_descriptor( conn_handle, descriptor, msg, REPLY );           
                                                                                
  /*******************************************************************          
   **    put message on queue                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      rv = put_msg( conn_handle,                                                
           REPLYTO,                                                             
           (unsigned short)0,                                                   
           (char *)NULL,                                                        
           descriptor,                                                          
           msg,                                                                 
           msg_len );                                                           
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
          "Message successfully put on Queue" );                                
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**     qmArchiveMsg( )                                                          
**                                                                              
**    This function copies a message to the archive queue.                      
**                                                                              
**    It should be called after a successful call to qmGetMsg(), qmPutMsg(),    
**    qmPutOneMsg() or qmReplyToMsg(). The MSG_DESCRIPTOR should not be         
**    modified( except for safestore_info) between the call to the              
**    above functions and this function.                                        
**                                                                              
**    It is usually preferable to call the above functions with the             
**    QM_SAFESTORE option.                                                      
**                                                                              
**     This function is meant to be used after a call to qmGetMsg().            
**    It allows the user to decide whether to archive a retrieved message       
**    and to set the safestore_info based on the message content.               
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        MSG_DESCRIPTOR        For a successfully put or retrieved             
**                    message.                                                  
**        msg            The message to be archived. The message                
**                    must be in a buffer allocated with                        
**                    qmAllocBuf().                                             
**        msg_len            The length of the message to archived.             
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**===========================================================================   
*/                                                                              
int qmArchiveMsg(                                                               
         CONNECTION_HANDLE    conn_handle,                                      
         MSG_DESCRIPTOR        *descriptor,                                     
         char            *msg,                                                  
         long            msg_len                                                
         )                                                                      
{                                                                               
  int         rv;                                                               
  char        *func_name = "qmArchiveMsg";                                      
#if 0                                                                           
    pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                            
    "Entry" );                                                                  
#endif                                                                          
  /**********************************************************************       
   **    check CONNECTION_HANDLE                                                
   *********************************************************************/       
                                                                                
  rv = check_connection( conn_handle );                                         
                                                                                
  /*******************************************************************          
   **    check MSG_DESCRIPTOR                                                   
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    rv = check_msg_descriptor( conn_handle, descriptor, msg, ARCHMSG );         
                                                                                
  /*******************************************************************          
   **    Archive the message                                                    
   *******************************************************************/         
                                                                                
  if ( rv == QM_SUCCESS ) {                                                     
    rv = safestore_message( conn_handle,                                        
                descriptor,                                                     
                &(descriptor->msg_md),                                          
                msg,                                                            
                msg_len,                                                        
                descriptor->msg_queue );                                        
                                                                                
    if ( rv == QM_SUCCESS )                                                     
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "Message successfully Archived" );                                    
  }                                                                             
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
                                                                                
/*===========================================================================   
**                                                                              
**    qmMsgidString( )                                                          
**                                                                              
**    This function returns a QM_MSGID as a printable hexadecimal string.       
**                                                                              
**    Input:                                                                    
**        QM_MSGID                                                              
**    Output:                                                                   
**        Printable string of form X'...'                                       
**                                                                              
**===========================================================================   
*/                                                                              
char *qmMsgidString( const QM_MSGID msg_id, MSGID_STRING pstring )              
{                                                                               
  int i;                                                                        
  char *pptr;                                                                   
  pptr = pstring;                                                               
  strcpy( pptr, "X'" );                                                         
  for( pptr+=2, i = 0; i < MQ_MSG_ID_LENGTH; i++, pptr+=2 )                     
    sprintf( pptr, "%02X", msg_id??(i??) );                                     
  strcpy( pptr, "'" );                                                          
  return( pstring );                                                            
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    qmAllocBuf( )                                                             
**                                                                              
**    This function allocates a buffer that the user can use to send or         
**    receive a message to be safestored. The buffer has space for the          
**    user's message and the safestore header.                                  
**                                                                              
**    Input:                                                                    
**        size of user's message                                                
**    Output:                                                                   
**        Pointer to start of the user's message in the buffer.                 
**                                                                              
**==========================================================================    
*/                                                                              
char *qmAllocBuf( int size )                                                    
{                                                                               
  char     *buf;                                                                
  int    header_size;                                                           
  int    total_size;                                                            
                                                                                
  /* total_size = size + SAFESTORE_HDR_LEN; */                                  
  header_size = get_arch_header_size( );                                        
  total_size = size + SSTOR_ID_LEN + header_size;                               
                                                                                
  buf = get_store( total_size );                                                
  if ( buf != (char *)NULL )                                                    
    {                                                                           
      memcpy( buf, SSTOR_ID, SSTOR_ID_LEN );                                    
      buf += ( SSTOR_ID_LEN + header_size );                                    
    }                                                                           
                                                                                
  return( buf );                                                                
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    qmFreeBuf( )                                                              
**                                                                              
**    This function freees a buffer that was allocated by qmAllocBuf( ).        
**    It checks the buffer id to verify that the buffer is valid.               
**                                                                              
**    Input:                                                                    
**        Pointer to buffer                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
int qmFreeBuf( char *buffer )                                                   
{                                                                               
  int header_size;                                                              
  int rv;                                                                       
                                                                                
  rv = check_sstor_buf( buffer );                                               
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    header_size = get_arch_header_size( );                                      
    free_store( buffer - ( header_size + SSTOR_ID_LEN ) );                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
                                                                                
/*===========================================================================   
**                                                                              
**    convertByteFld( )                                                         
**                                                                              
**    This function converts a binary field to hex.                             
**                                                                              
**    Input:                                                                    
**        QM_BYTEFLD                                                            
**    Output:                                                                   
**        byte field converted to hex                                           
**                                                                              
**===========================================================================   
*/                                                                              
static char *convertByteFld( const QM_BYTEFLD data, BYTE_STRNG pstring,         
                 int fldsize )                                                  
{                                                                               
  int i;                                                                        
  char *pptr;                                                                   
  pptr = pstring;                                                               
  for( pptr, i = 0; i < fldsize; i++, pptr+=2 )                                 
    sprintf( pptr, "%02X", data??(i??) );                                       
  return( pstring );                                                            
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    qmMsgidCpy( )                                                             
**                                                                              
**    This function copies a QM_MSGID to a target area.                         
**                                                                              
**    Input:                                                                    
**        QM_MSGID                                                              
**    Output:                                                                   
**        QM_MSGID                                                              
**                                                                              
**==========================================================================    
*/                                                                              
void qmMsgidCpy( QM_MSGID *target, const QM_MSGID source )                      
{                                                                               
  memcpy( (char *)target, (char *)source, MQ_MSG_ID_LENGTH );                   
                                                                                
  return;                                                                       
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    qmMsgidNone( )                                                            
**                                                                              
**    This function sets a QM_MSGID to a special NULL value.                    
**                                                                              
**    Input:                                                                    
**        QM_MSGID                                                              
**    Output:                                                                   
**        The same QM_MSGID nulled out                                          
**                                                                              
**==========================================================================    
*/                                                                              
void qmMsgidNone( QM_MSGID *msgid )                                             
{                                                                               
  qmMsgidCpy( msgid, qm_msgid_none );                                           
  return;                                                                       
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    init_connection_list( )                                                   
**                                                                              
**    This internal function initializes all the members                        
**    of the connection_list.                                                   
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Updated connection_list                                               
**                                                                              
**===========================================================================   
*/                                                                              
static void init_connection_list( CONNECTION_HANDLE conn_handle )               
{                                                                               
  int         i;                                                                
  char        *func_name = "init_connection_list";                              
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "init_connection_list, Entry" );                                            
                                                                                
  for( i = 0; i <= (int)(conn_handle->max_queues); i++ )                        
    {                                                                           
      strncpy( (conn_handle->connection_list)??(i??).struct_id,                 
          QUEUE_CONNECTION_ID,                                                  
          QM_ID_LEN);                                                           
      (conn_handle->connection_list)??(i??).version = QUEUE_CONNECTION_VERSION; 
      (conn_handle->connection_list)??(i??).Hobj = MQHO_UNUSABLE_HOBJ;          
      memset( (conn_handle->connection_list)??(i??).queue_name,                 
         0,                                                                     
         QM_QUEUE_LEN );                                                        
      memset( (conn_handle->connection_list)??(i??).reserve,                    
         0,                                                                     
         QM_RESERVE_LEN );                                                      
    }                                                                           
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "init_connection_list, Exit" );                                             
                                                                                
  return;                                                                       
}                                                                               
                                                                                
/*========================================================================      
**                                                                              
**    connect_to_qm( )                                                          
**                                                                              
**    This internal function call MQCONN() to connect to the queue              
**    manager. It then calls MQINQ to get the real queue manager name.          
**    This is done because the the caller may not have specified the            
**    real name if using the default queue manager or in MVSCICS.               
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue manager name                                                    
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**========================================================================      
*/                                                                              
static int connect_to_qm( CONNECTION_HANDLE conn_handle, char *qm_name )        
{                                                                               
  MQOD        qm_od = {MQOD_DEFAULT};                                           
  MQLONG    O_Options;                                                          
  MQLONG    C_Options;                                                          
  MQHOBJ    Hobj;                                                               
  MQLONG        scount = 1;                                                     
  MQLONG    selectors??(1??);                                                   
  MQLONG    intcount = 0;                                                       
  MQLONG    charlen = MQ_Q_MGR_NAME_LENGTH;                                     
  MQCHAR    charattr??(MQ_Q_MGR_NAME_LENGTH??);                                 
  int        rv, rvc;                                                           
  char        *func_name = "connect_to_qm";                                     
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
        "Entry" );                                                              
                                                                                
  MQCONN( qm_name,                                                              
     &(conn_handle->Hcon),                                                      
     &(conn_handle->CompCode),                                                  
     &(conn_handle->Reason) );                                                  
                                                                                
  rv = handle_MQ_error( conn_handle, "MQCONN" );                                
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
         "Successfully connected to Queue Manager" );                           
                                                                                
      (conn_handle->connection_seqno)++;                                        
      pdebug( DBUG_FINE, conn_handle, func_name,                                
         "Connection_seqno = %d",                                               
         conn_handle->connection_seqno );                                       
      conn_handle->state = QM_CONNECTED;                                        
                                                                                
      /*******************************************************************      
       **    open queue manager for inquire                                     
       ******************************************************************/      
                                                                                
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "opening queue manager for inquire" );                                  
      qm_od.ObjectType = MQOT_Q_MGR;                                            
      O_Options = MQOO_FAIL_IF_QUIESCING | MQOO_INQUIRE;                        
      MQOPEN( conn_handle->Hcon,                                                
         &qm_od,                                                                
         O_Options,                                                             
         &Hobj,                                                                 
         &(conn_handle->CompCode),                                              
         &(conn_handle->Reason) );                                              
      rv = handle_MQ_error( conn_handle, "MQOPEN" );                            
      if ( rv != QM_SUCCESS )                                                   
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
        "open queue manager failed" );                                          
    }                                                                           
      else                                                                      
    {                                                                           
      /*******************************************************************      
       **    inquire for real queue manager name                                
       ******************************************************************/      
      pdebug( DBUG_FINE, conn_handle, func_name,                                
         "Issue MQINQ to get queue manager name" );                             
      selectors??(0??) = MQCA_Q_MGR_NAME;                                       
      MQINQ( conn_handle->Hcon,                                                 
        Hobj,                                                                   
        scount,                                                                 
        selectors,                                                              
        intcount,                                                               
        (MQLONG *)NULL,                                                         
        charlen,                                                                
        charattr,                                                               
        &(conn_handle->CompCode),                                               
        &(conn_handle->Reason) );                                               
      rv = handle_MQ_error( conn_handle, "MQINQ" );                             
      if ( rv != QM_SUCCESS )                                                   
        pdebug( DBUG_NORMAL, conn_handle, func_name,                            
          "inquire queue manager name failed" );                                
      else                                                                      
        {                                                                       
          pdebug( DBUG_FINE, conn_handle, func_name,                            
             "real queue manager name: %.*s",                                   
             MQ_Q_MGR_NAME_LENGTH, charattr );                                  
          strncpy( conn_handle->queue_manager_name,                             
              charattr,                                                         
              QM_QUEUE_LEN );                                                   
        }                                                                       
      /*******************************************************************      
       **    close queue manager                                                
       ******************************************************************/      
      pdebug( DBUG_FINE, conn_handle, func_name,                                
         "closing queue manager" );                                             
      C_Options = MQCO_NONE;                                                    
      MQCLOSE( conn_handle->Hcon,                                               
          &Hobj,                                                                
          C_Options,                                                            
          &(conn_handle->CompCode),                                             
          &(conn_handle->Reason) );                                             
      rvc = handle_MQ_error( conn_handle, "MQCLOSE" );                          
      if ( rvc != QM_SUCCESS )                                                  
        {                                                                       
          pdebug( DBUG_NORMAL, conn_handle, func_name,                          
             "close queue manager failed" );                                    
          if ( rvc == QM_CONNECTION_BROKEN )                                    
        rv = rvc;                                                               
        }                                                                       
    }                                                                           
      /*******************************************************************      
       **    if inquire failed, use the supplied queue manager name             
       ******************************************************************/      
                                                                                
      if ( ( rv != QM_SUCCESS ) && ( rv != QM_CONNECTION_BROKEN ) )             
    {                                                                           
      strncpy( conn_handle->queue_manager_name,                                 
          qm_name,                                                              
          QM_QUEUE_LEN );                                                       
      rv = QM_SUCCESS;                                                          
    }                                                                           
    }                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
     "Exit, rv = %d", rv );                                                     
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    check_msg_descriptor( )                                                   
**                                                                              
**    This internal function checks a MSG_DESCRIPTOR for validity               
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        MSG_DESCRIPTOR                                                        
**        message buffer                                                        
**        descriptor_type ( PUT, GET or REPLY )                                 
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
static int check_msg_descriptor(                                                
                CONNECTION_HANDLE conn_handle,                                  
                MSG_DESCRIPTOR *descriptor,                                     
                char *msg_buf,                                                  
                int descriptor_type                                             
                )                                                               
{                                                                               
  QM_OPTION browse_opts = QM_BROWSE_FIRST | QM_BROWSE_NEXT;                     
  MSGID_STRING     pstring;                                                     
  QM_OPTION    options;                                                         
  int         rv = QM_SUCCESS;                                                  
  char        *func_name = "check_msg_descriptor";                              
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  if ( descriptor == (MSG_DESCRIPTOR *)NULL )                                   
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
          "NULL MSG_DESCRIPTOR" );                                              
      rv = QM_INVALID;                                                          
    }                                                                           
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "Parameters:" );                                                      
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    struct_id           = %.*s",                                     
          QM_ID_LEN, descriptor->struct_id );                                   
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    version             = %ld", descriptor->version );               
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    msg_type            = %ld", descriptor->msg_type );              
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    priority            = %ld", descriptor->priority );              
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    options             = %08lx", descriptor->options );             
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    timeout             = %ld", descriptor->timeout );               
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    retention           = %ld", descriptor->retention );             
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    safestore_info      = %.*s",                                     
          QM_ERRMSG_LEN, descriptor->safestore_info );                          
#if !defined(MVSCICS)                                                           
      if ( is_debug_level( DBUG_FINE ) )                                        
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    msg_id              = %s",                                       
          qmMsgidString( descriptor->msg_id, pstring ) );                       
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "    msg_correlation_id  = %s",                                       
          qmMsgidString( descriptor->msg_correlation_id, pstring ) );           
    }                                                                           
#endif                                                                          
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "   reply_to_queue       = %.*s",                                     
          QM_QUEUE_LEN, descriptor->reply_to_queue );                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "   reply_to_qmgr        = %.*s",                                     
          QM_QUEUE_LEN, descriptor->reply_to_qmgr );                            
                                                                                
      options = descriptor->options;                                            
                                                                                
      /*******************************************************************      
      **    check struct_id and version                                         
      ******************************************************************/       
                                                                                
      if ( strncmp( descriptor->struct_id,                                      
            QM_DESCRIPTOR_ID,                                                   
            QM_ID_LEN ) != 0 )                                                  
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
          "msg_descriptor has invalid structure id" );                          
      rv = QM_INVALID;                                                          
    }                                                                           
      else if ( ( descriptor->version != QM_DESCRIPTOR_VERSION_1 ) &&           
        ( descriptor->version != QM_DESCRIPTOR_VERSION_2 ) )                    
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
          "msg_descriptor has invalid version" );                               
      rv = QM_INVALID;                                                          
    }                                                                           
    }                                                                           
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      /*******************************************************************      
       **    check options for all types                                        
       ******************************************************************/      
                                                                                
      if ( ( descriptor_type != ARCHMSG ) && ( options & QM_SAFESTORE ) )       
    {                                                                           
      if ( options & QM_NO_SYNCPOINT )                                          
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
             "SAFESTORE and NO_SYNCPOINT are incompatible options" );           
          rv = QM_INVALID;                                                      
        }                                                                       
      else if ( ( rv = check_sstor_buf( msg_buf ) ) != QM_SUCCESS ) {           
        pdebug( DBUG_ERRMSG, conn_handle, func_name,                            
           "Message Buffer invalid for Safestore" );                            
        rv = QM_INVALID;                                                        
      }                                                                         
    }                                                                           
    }                                                                           
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      /*******************************************************************      
       **    check some options for PUT                                         
       ******************************************************************/      
                                                                                
      if ( descriptor_type == PUT )                                             
    {                                                                           
      switch( descriptor->msg_type )                                            
        {                                                                       
        case QM_REQUEST:                                                        
          if ( ( (descriptor->reply_to_queue)??(0??) == '\0' ) ||               
          ( (descriptor->reply_to_queue)??(0??) == ' ' ) )                      
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "reply_to_queue required for msg_type QM_REQUEST" );                
          rv = QM_INVALID;                                                      
        }                                                                       
        case QM_DATAGRAM:                                                       
        case QM_REPLY:                                                          
          break;                                                                
        default:                                                                
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "Unsupported msg_type" );                                           
          rv = QM_INVALID;                                                      
        }                                                                       
    }                                                                           
                                                                                
      /*******************************************************************      
       **    check some options for REPLY                                       
       ******************************************************************/      
                                                                                
      else if ( descriptor_type == REPLY )                                      
    {                                                                           
      if (  ( (descriptor->reply_to_queue)??(0??) == '\0' ) ||                  
          ( descriptor->reply_to_queue??(0??) == ' ' ) )                        
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "reply_to_queue required for qmReplyToMsg" );                       
          rv = QM_INVALID;                                                      
        }                                                                       
    }                                                                           
                                                                                
      /*******************************************************************      
       **    check some options for GET                                         
       ******************************************************************/      
                                                                                
      else if ( descriptor_type == GET )                                        
    {                                                                           
      if ( ( options & QM_BEGIN_TRANSACTION ) &&                                
          LUW_STARTED( conn_handle ) )                                          
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "Unit of Work already started" );                                   
          rv = QM_ALREADY_STARTED;                                              
        }                                                                       
      else if ( ( options & QM_BEGIN_TRANSACTION ) &&                           
           ( options & QM_NO_SYNCPOINT ) )                                      
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "BEGIN_TRANSACTION and NO_SYNCPOINT are "                           
            "incompatible options" );                                           
          rv = QM_INVALID;                                                      
        }                                                                       
      else if ( ( options & QM_BROWSE_FIRST ) &&                                
           ( options & QM_BROWSE_NEXT ) )                                       
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "incompatible BROWSE options" );                                    
          rv = QM_INVALID;                                                      
        }                                                                       
      else if ( ( options & QM_BEGIN_TRANSACTION ) &&                           
           ( options & browse_opts ) )                                          
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "BROWSE option incompatible with QM_BEGIN_TRANSCTION" );            
          rv = QM_INVALID;                                                      
        }                                                                       
      else if ( ( options & QM_MSG_UNDER_CURSOR ) &&                            
           ( options & browse_opts ) )                                          
        {                                                                       
          pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
            "BROWSE option incompatible with QM_MSG_UNDER_CURSOR" );            
          rv = QM_INVALID;                                                      
        }                                                                       
                                                                                
    }                                                                           
                                                                                
      /*******************************************************************      
       **    check some options for ARCHMSG                                     
       ******************************************************************/      
                                                                                
      else if ( descriptor_type == ARCHMSG )                                    
    {                                                                           
      if ( descriptor->version == QM_DESCRIPTOR_VERSION_1 ) {                   
        pdebug( DBUG_ERRMSG, conn_handle, func_name,                            
            "Version 1 descriptor invalid for ArchiveMsg" );                    
        rv = QM_INVALID;                                                        
      }                                                                         
      else if ( options & QM_NO_SYNCPOINT ) {                                   
        pdebug( DBUG_ERRMSG, conn_handle, func_name,                            
            "NO_SYNCPOINT option invalid for ArchiveMsg" );                     
        rv = QM_INVALID;                                                        
      }                                                                         
      else if ( ( rv = check_sstor_buf( msg_buf ) ) != QM_SUCCESS ) {           
        pdebug( DBUG_ERRMSG, conn_handle, func_name,                            
           "Message Buffer invalid for ArchiveMsg" );                           
        rv = QM_INVALID;                                                        
      }                                                                         
    }                                                                           
    }                                                                           
                                                                                
  /*******************************************************************          
   **    check some options common to PUT and REPLY                             
   ******************************************************************/          
                                                                                
  if ( ( rv == QM_SUCCESS ) &&                                                  
      ( ( descriptor_type == PUT ) || ( descriptor_type == REPLY ) ) )          
    {                                                                           
      if ( ( options & QM_PERSISTENT ) &&                                       
      ( options & QM_NOT_PERSISTENT ) )                                         
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
        "QM_PERSISTENT incompatible with QM_NOT_PERSISTENT" );                  
      rv = QM_INVALID;                                                          
    }                                                                           
     /* We allow unlimited retention as of 10/3/01                              
     *else if ( ( descriptor->retention <= 0 ) ||                               
     *  ( descriptor->retention > QM_MAX_RETENTION ) )                          
     *  {                                                                       
     *    pdebug( DBUG_ERRMSG, conn_handle, func_name,                          
     *    "Invalid retention period: %d",                                       
     *        descriptor->retention );                                          
     *    rv = QM_INVALID;                                                      
     *  }                                                                       
     */                                                                         
    }                                                                           
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
/*==========================================================================    
**                                                                              
**    check_queue_handle( )                                                     
**                                                                              
**    This internal function checks a QUEUE_HANDLE for validity                 
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        QUEUE_HANDLE                                                          
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**==========================================================================    
*/                                                                              
static int check_queue_handle(                                                  
                  CONNECTION_HANDLE conn_handle,                                
                  QUEUE_HANDLE queue_handle                                     
                  )                                                             
{                                                                               
  union queue_handle_union qhu;                                                 
  int         rv;                                                               
  char        *func_name = "check_queue_handle";                                
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry, queue_handle = %08lx", queue_handle );                              
#endif                                                                          
  rv = check_connection( conn_handle );                                         
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      qhu.qh_long = queue_handle;                                               
      if ( qhu.qh_struct.connection_seqno != conn_handle->connection_seqno )    
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
        "stale QUEUE_HANDLE" );                                                 
      rv = QM_INVALID;                                                          
    }                                                                           
      else if ( ( qhu.qh_struct.queue_index < 1 ) ||                            
           ( qhu.qh_struct.queue_index > conn_handle->max_queues ) )            
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
        "queue_index out of range" );                                           
      rv = QM_INVALID;                                                          
    }                                                                           
      else if ( QUEUE_HANDLE_UNUSABLE( conn_handle,                             
                      qhu.qh_struct.queue_index ) )                             
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
        "QUEUE_HANDLE is unusable" );                                           
      rv = QM_INVALID;                                                          
    }                                                                           
    }                                                                           
                                                                                
  return( rv );                                                                 
                                                                                
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    check_connection( )                                                       
**                                                                              
**    This internal function checks the conection handle for validity           
**    and sees if we're connected.                                              
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**===========================================================================   
*/                                                                              
static int check_connection( CONNECTION_HANDLE conn_handle )                    
{                                                                               
  int         rv = QM_SUCCESS;                                                  
  char        *func_name = "check_connection";                                  
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
#endif                                                                          
  if ( conn_handle == NULL_CONNECTION_HANDLE )                                  
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
        "NULL CONNECTION_HANDLE" );                                             
      rv = QM_INVALID;                                                          
    }                                                                           
  else if ( strncmp( conn_handle->struct_id,                                    
            QM_CONNECTION_ID,                                                   
            QM_ID_LEN ) != 0 )                                                  
    {                                                                           
      pdebug( DBUG_NORMAL, NULL_CONNECTION_HANDLE, func_name,                   
        "connection handle has invalid structure id" );                         
      rv = QM_INVALID;                                                          
    }                                                                           
  else if ( conn_handle->version != QM_CONNECTION_VERSION )                     
    {                                                                           
      pdebug( DBUG_NORMAL, NULL_CONNECTION_HANDLE, func_name,                   
        "connection handle has invalid version" );                              
      rv = QM_INVALID;                                                          
    }                                                                           
  else                                                                          
    {                                                                           
      conn_handle->CompCode = MQCC_OK;                                          
      conn_handle->Reason = MQRC_NONE;                                          
      memset( conn_handle->errmsg, 0, QM_ERRMSG_LEN );                          
      if ( !CONNECTED( conn_handle ) )                                          
    {                                                                           
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
         "not connected to Queue Manager" );                                    
      rv = QM_NOT_CONNECTED;                                                    
    }                                                                           
    }                                                                           
#if 0                                                                           
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
#endif                                                                          
  return( rv );                                                                 
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    open_safestore_queue( conn_handle )                                       
**                                                                              
**    This internal function opens the safestore queue                          
**                                                                              
**    input:    CONNECTION_HANDLE                                               
**    Output: Return Code                                                       
**                                                                              
**===========================================================================   
*/                                                                              
static int open_safestore_queue( CONNECTION_HANDLE conn_handle )                
{                                                                               
  char *archive_queue = get_arch_queue_name( );                                 
  int  rv;                                                                      
  unsigned short index = 0;                                                     
  char        *func_name = "open_safestore_queue";                              
                                                                                
  pdebug( DBUG_NORMAL, conn_handle, func_name,                                  
    "opening the queue %s", archive_queue );                                    
                                                                                
  /* position 0 in the connection_list is reserved for the safestore queue */   
  rv = open_queue( conn_handle, index, archive_queue, QM_OUTPUT );              
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
    "Successfully opened the queue %s", archive_queue );                        
    }                                                                           
  else                                                                          
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
    "Error opening the queue %s", archive_queue );                              
    }                                                                           
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    close_safestore_queue( conn_handle )                                      
**                                                                              
**    This internal function closes the safestore queue                         
**                                                                              
**    input:    CONNECTION_HANDLE                                               
**    Output: Return Code                                                       
**                                                                              
**===========================================================================   
*/                                                                              
static int close_safestore_queue( CONNECTION_HANDLE conn_handle )               
{                                                                               
  int  rv;                                                                      
  char        *func_name = "close_safestore_queue";                             
  char        *archive_queue = get_arch_queue_name( );                          
                                                                                
  pdebug( DBUG_NORMAL, conn_handle, func_name,                                  
    "closing the queue %s", archive_queue );                                    
                                                                                
  /* position 0 in the connection_list is reserved for the safestore queue */   
  rv = close_queue( conn_handle, 0 );                                           
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
    "Successfully closed the queue %s", archive_queue );                        
    }                                                                           
  else                                                                          
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
    "Error closing the queue %d", archive_queue );                              
    }                                                                           
                                                                                
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    check_sstor_buf( )                                                        
**                                                                              
**    This internal function checks the beginning of a safestore buffer         
**    for a valid eye catcher.                                                  
**                                                                              
**    Input:                                                                    
**        Pointer to buffer                                                     
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**===========================================================================   
*/                                                                              
static int check_sstor_buf( char *buffer )                                      
{                                                                               
  char     *buf_head;                                                           
  int header_size = get_arch_header_size( );                                    
                                                                                
  buf_head = buffer - ( SSTOR_ID_LEN + header_size );                           
                                                                                
  if ( memcmp( buf_head, SSTOR_ID, SSTOR_ID_LEN ) == 0 )                        
    return( QM_SUCCESS );                                                       
  else                                                                          
    return( QM_INVALID );                                                       
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    start_luw( )                                                              
**                                                                              
**    This internal function starts an XA logical unit of work.                 
**                                                                              
**    Input:    CONNECTION_HANDLE                                               
**    Output:    Return Code                                                    
**                                                                              
**===========================================================================   
*/                                                                              
#if !defined(MVS)                                                               
static int start_luw( CONNECTION_HANDLE conn_handle )                           
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "start_luw";                                         
  MQBO        mqbo = {MQBO_DEFAULT};                                            
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  MQBEGIN( conn_handle->Hcon,                                                   
       &mqbo,                                                                   
       &(conn_handle->CompCode),                                                
       &(conn_handle->Reason) );                                                
                                                                                
  rv = handle_MQ_error( conn_handle, "MQBEGIN" );                               
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
  return( rv );                                                                 
                                                                                
}                                                                               
#endif    /* !defined(MVS) */                                                   
                                                                                
/*===========================================================================   
**                                                                              
**    commit_luw( )                                                             
**                                                                              
**    This internal function commits a logical unit of work.                    
**                                                                              
**    Input:  CONNECTION_HANDLE                                                 
**    Output:    Return Code                                                    
**                                                                              
**===========================================================================   
*/                                                                              
static int commit_luw( CONNECTION_HANDLE conn_handle )                          
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "commit_luw";                                        
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  MQCMIT( conn_handle->Hcon,                                                    
     &(conn_handle->CompCode),                                                  
     &(conn_handle->Reason) );                                                  
                                                                                
  rv = handle_MQ_error( conn_handle, "MQCMIT" );                                
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
  return( rv );                                                                 
                                                                                
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    backout_luw( )                                                            
**                                                                              
**    This internal function backs out a logical unit of work.                  
**                                                                              
**    Input:    CONNECTION_HANDLE                                               
**    Output:    Return Code                                                    
**                                                                              
**===========================================================================   
*/                                                                              
static int backout_luw( CONNECTION_HANDLE conn_handle )                         
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "backout_luw";                                       
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  MQBACK( conn_handle->Hcon,                                                    
     &(conn_handle->CompCode),                                                  
     &(conn_handle->Reason) );                                                  
                                                                                
  rv = handle_MQ_error( conn_handle, "MQBACK" );                                
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
  return( rv );                                                                 
                                                                                
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    open_queue( )                                                             
**                                                                              
**    This internal function calls MQOPEN to open a queue.                      
**    If it succeeds, the                                                       
**    QUEUE_CONNECTION is added to the connection list.                         
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue_index                                                           
**        queue_name                                                            
**        access_type                                                           
**    Output:                                                                   
**        Return Code                                                           
**        QUEUE_CONNECTION                                                      
**                                                                              
**==========================================================================    
*/                                                                              
static int open_queue(                                                          
              CONNECTION_HANDLE conn_handle,                                    
              unsigned short index,                                             
              char *queue_name,                                                 
              int access_type                                                   
              )                                                                 
{                                                                               
  MQOD        queue_od = {MQOD_DEFAULT};                                        
  MQLONG    O_Options;                                                          
  MQHOBJ    Hobj;                                                               
  QUEUE_CONNECTION    *local_queue_connection;                                  
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "open_queue";                                        
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry, queue_name = %.*s, access_type = %08x",                             
    QM_QUEUE_LEN, queue_name, access_type );                                    
                                                                                
  /*******************************************************************          
   **    set MQOPEN options                                                     
   ******************************************************************/          
                                                                                
  O_Options = MQOO_FAIL_IF_QUIESCING | MQOO_INQUIRE;                            
                                                                                
  if ( access_type & QM_INPUT_EXCLUSIVE )                                       
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Exclusive Input" );                                  
      O_Options |= MQOO_INPUT_EXCLUSIVE;                                        
    }                                                                           
  if ( access_type & QM_INPUT_SHARED )                                          
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Shared Input" );                                     
      O_Options |= MQOO_INPUT_SHARED;                                           
    }                                                                           
  if ( access_type & QM_INPUT_AS_Q_DEF )                                        
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Input as Queue Default" );                           
      O_Options |= MQOO_INPUT_AS_Q_DEF;                                         
    }                                                                           
  if ( access_type & QM_OUTPUT )                                                
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Output" );                                           
      O_Options |= MQOO_OUTPUT;                                                 
    }                                                                           
  if ( access_type & QM_BROWSE )                                                
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Browse" );                                           
      O_Options |= MQOO_BROWSE;                                                 
    }                                                                           
  if ( access_type & QM_INQUIRE )                                               
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Inquire" );                                          
      O_Options |= MQOO_INQUIRE;                                                
    }                                                                           
  if ( access_type & QM_SET )                                                   
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Opening Queue for Set" );                                              
      O_Options |= MQOO_SET;                                                    
    }                                                                           
                                                                                
  /***********************************************************************      
   **    Open the queue                                                         
   **********************************************************************/      
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      strncpy( queue_od.ObjectName, queue_name, MQ_Q_NAME_LENGTH );             
      MQOPEN( conn_handle->Hcon,                                                
         &queue_od,                                                             
         O_Options,                                                             
         &Hobj,                                                                 
         &(conn_handle->CompCode),                                              
         &(conn_handle->Reason) );                                              
      rv = handle_MQ_error( conn_handle, "MQOPEN" );                            
    }                                                                           
                                                                                
  /*******************************************************************          
   **    initialize QUEUE_CONNECTION and get max msg length                     
   ******************************************************************/          
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      local_queue_connection = (conn_handle->connection_list)+index;            
      local_queue_connection->Hobj = Hobj;                                      
      local_queue_connection->max_msglen = 0;                                   
      strncpy( local_queue_connection->queue_name, queue_name, QM_QUEUE_LEN );  
      rv = get_max_msglen( conn_handle, index );                                
    }                                                                           
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    get_max_msglen( )                                                         
**                                                                              
**    This internal function issues MQINQ to get the max msg length for a       
**    queue. This is only possible for a local queue. If the queue name is      
**    an alias, the real queue name is obtained and opened for inquire.         
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue index                                                           
**    Output:                                                                   
**        Return Code                                                           
**        Update QUEUE_CONNECTION                                               
**                                                                              
**==========================================================================    
*/                                                                              
static int get_max_msglen(                                                      
               CONNECTION_HANDLE    conn_handle,                                
               unsigned short    qh_index                                       
               )                                                                
{                                                                               
  MQOD        queue_od = {MQOD_DEFAULT};                                        
  MQLONG    O_Options;                                                          
  MQLONG    C_Options;                                                          
  QUEUE_CONNECTION *qconn;                                                      
  MQLONG        scount = 1;                                                     
  MQLONG    selectors??(1??);                                                   
  MQLONG    intcount;                                                           
  MQLONG      intattr??(1??), queue_type;                                       
  MQLONG    charlen;                                                            
  MQCHAR    charattr??(MQ_Q_NAME_LENGTH??);                                     
  MQHOBJ    qHobj;                                                              
  int        base_q_opened = 0;                                                 
  int        rv;                                                                
  char        *func_name = "get_max_msglen";                                    
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  /*******************************************************************          
  **    Inquire about queue type                                                
  ******************************************************************/           
                                                                                
  selectors??(0??) = MQIA_Q_TYPE;                                               
  intcount = 1;                                                                 
  charlen = 0;                                                                  
  qconn = (conn_handle->connection_list)+qh_index;                              
  qHobj = qconn->Hobj;                                                          
  pdebug( DBUG_FINE, conn_handle, func_name,                                    
      "Issue MQINQ to get queue type" );                                        
  MQINQ( conn_handle->Hcon,                                                     
     qHobj,                                                                     
     scount,                                                                    
     selectors,                                                                 
     intcount,                                                                  
     intattr,                                                                   
     charlen,                                                                   
     charattr,                                                                  
     &(conn_handle->CompCode),                                                  
     &(conn_handle->Reason) );                                                  
  rv = handle_MQ_error( conn_handle, "MQINQ" );                                 
  if ( rv != QM_SUCCESS )                                                       
    pdebug( DBUG_NORMAL, conn_handle, func_name,                                
        "inquire queue type failed" );                                          
  else                                                                          
    {                                                                           
      queue_type = intattr??(0??);                                              
      switch( queue_type )                                                      
    {                                                                           
    default:                                                                    
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
          "unexpected queue type %d", queue_type );                             
      break;                                                                    
    case MQQT_REMOTE:                                                           
      pdebug( DBUG_MEDIUM, conn_handle, func_name,                              
          "queue type is remote, max msglen not available");                    
      break;                                                                    
    case MQQT_LOCAL:                                                            
      pdebug( DBUG_MEDIUM, conn_handle, func_name,                              
          "queue type is local" );                                              
      break;                                                                    
    case MQQT_ALIAS:                                                            
      pdebug( DBUG_MEDIUM, conn_handle, func_name,                              
          "queue type is alias" );                                              
      break;                                                                    
    }                                                                           
    }                                                                           
                                                                                
  if ( ( rv == QM_SUCCESS ) && ( queue_type == MQQT_ALIAS ) )                   
    {                                                                           
      /*******************************************************************      
      **    get base queue name for alias queue                                 
      ******************************************************************/       
      selectors??(0??) = MQCA_BASE_Q_NAME;                                      
      intcount = 0;                                                             
      charlen = MQ_Q_NAME_LENGTH;                                               
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "Issue MQINQ to get base queue name for alias queue" );               
      MQINQ( conn_handle->Hcon,                                                 
         qHobj,                                                                 
         scount,                                                                
         selectors,                                                             
         intcount,                                                              
         intattr,                                                               
         charlen,                                                               
         charattr,                                                              
         &(conn_handle->CompCode),                                              
         &(conn_handle->Reason) );                                              
      rv = handle_MQ_error( conn_handle, "MQINQ" );                             
      if ( rv != QM_SUCCESS )                                                   
    pdebug( DBUG_NORMAL, conn_handle, func_name,                                
        "inquire base queue name failed" );                                     
      else                                                                      
    {                                                                           
      pdebug( DBUG_MEDIUM, conn_handle, func_name,                              
          "base queue name is %.*s",                                            
          MQ_Q_NAME_LENGTH, charattr );                                         
      /****************************************************************         
      **    open base queue name for inqure                                     
      ****************************************************************/         
      queue_od.ObjectType = MQOT_Q;                                             
      strncpy( queue_od.ObjectName, charattr, MQ_Q_NAME_LENGTH );               
      O_Options = MQOO_FAIL_IF_QUIESCING | MQOO_INQUIRE;                        
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "opening base queue name for inquire" );                              
      MQOPEN( conn_handle->Hcon,                                                
          &queue_od,                                                            
          O_Options,                                                            
          &qHobj,                                                               
          &(conn_handle->CompCode),                                             
          &(conn_handle->Reason) );                                             
      rv = handle_MQ_error( conn_handle, "MQOPEN" );                            
      if ( rv != QM_SUCCESS )                                                   
        pdebug( DBUG_NORMAL, conn_handle, func_name,                            
            "open base queue failed" );                                         
      else                                                                      
        {                                                                       
          base_q_opened = 1;                                                    
          /*************************************************************        
          **    get queue_type for base queue                                   
          *************************************************************/        
          selectors??(0??) = MQIA_Q_TYPE;                                       
          intcount = 1;                                                         
          charlen = 0;                                                          
          pdebug( DBUG_FINE, conn_handle, func_name,                            
              "Issue MQINQ to get queue type for base queue" );                 
          MQINQ( conn_handle->Hcon,                                             
             qHobj,                                                             
             scount,                                                            
             selectors,                                                         
             intcount,                                                          
             intattr,                                                           
             charlen,                                                           
             charattr,                                                          
             &(conn_handle->CompCode),                                          
             &(conn_handle->Reason) );                                          
          rv = handle_MQ_error( conn_handle, "MQINQ" );                         
          if ( rv != QM_SUCCESS )                                               
        pdebug( DBUG_NORMAL, conn_handle, func_name,                            
            "inquire queue type for base queue failed" );                       
          else                                                                  
        {                                                                       
          queue_type = intattr??(0??);                                          
          switch( queue_type )                                                  
            {                                                                   
            default:                                                            
              pdebug( DBUG_NORMAL, conn_handle, func_name,                      
                  "unexpected base queue type %d",                              
                  queue_type );                                                 
              break;                                                            
            case MQQT_REMOTE:                                                   
              pdebug( DBUG_MEDIUM, conn_handle, func_name,                      
                  "queue type for base queue is remote, "                       
                  "max msglen not available");                                  
              break;                                                            
            case MQQT_LOCAL:                                                    
              pdebug( DBUG_MEDIUM, conn_handle, func_name,                      
                  "queue type for base queue is local" );                       
              break;                                                            
            }                                                                   
        }                                                                       
        }                                                                       
    }                                                                           
    }                                                                           
  if ( ( rv == QM_SUCCESS ) && ( queue_type == MQQT_LOCAL ) )                   
    {                                                                           
      /*******************************************************************      
      **    get max msg len for local queue                                     
      ******************************************************************/       
      selectors??(0??) = MQIA_MAX_MSG_LENGTH;                                   
      intcount = 1;                                                             
      charlen = 0;                                                              
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "Issue MQINQ to get max msg length" );                                
      MQINQ( conn_handle->Hcon,                                                 
         qHobj,                                                                 
         scount,                                                                
         selectors,                                                             
         intcount,                                                              
         intattr,                                                               
         charlen,                                                               
         charattr,                                                              
         &(conn_handle->CompCode),                                              
         &(conn_handle->Reason) );                                              
      rv = handle_MQ_error( conn_handle, "MQINQ" );                             
      if ( rv != QM_SUCCESS )                                                   
    pdebug( DBUG_NORMAL, conn_handle, func_name,                                
        "inquire max msg length failed" );                                      
      else                                                                      
    {                                                                           
      pdebug( DBUG_MEDIUM, conn_handle, func_name,                              
          "max msg length is %d", intattr??(0??) );                             
      qconn->max_msglen = intattr??(0??);                                       
    }                                                                           
    }                                                                           
  if ( base_q_opened && ( rv != QM_CONNECTION_BROKEN ) )                        
    {                                                                           
      /*******************************************************************      
      **    close base queue name                                               
      ******************************************************************/       
      pdebug( DBUG_FINE, conn_handle, func_name,                                
          "closing base queue" );                                               
      C_Options = MQCO_NONE;                                                    
      MQCLOSE( conn_handle->Hcon,                                               
           &qHobj,                                                              
           C_Options,                                                           
           &(conn_handle->CompCode),                                            
           &(conn_handle->Reason) );                                            
      rv = handle_MQ_error( conn_handle, "MQCLOSE" );                           
      if ( rv != QM_SUCCESS )                                                   
    pdebug( DBUG_NORMAL, conn_handle, func_name,                                
        "close base queue failed" );                                            
    }                                                                           
  if ( rv != QM_CONNECTION_BROKEN )                                             
    rv = QM_SUCCESS;                                                            
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    put_msg( )                                                                
**                                                                              
**    This internal function calls MQPUT or MQPUT1                              
**    to put a message on a queue                                               
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        put_type ( PUT_NORMAL, PUT1 or REPLYTO )                              
**        queue_index ( for PUT_NORMAL )                                        
**        queue_name ( for PUT1 )                                               
**        MSG_DESCRIPTOR                                                        
**        msg                                                                   
**        msg_len                                                               
**    Output:                                                                   
**        Return Code                                                           
**        updated MSG_DESCRIPTOR                                                
**                                                                              
**==========================================================================    
*/                                                                              
static int put_msg(                                                             
           CONNECTION_HANDLE    conn_handle,                                    
           int            put_type,                                             
           unsigned short          qh_index,                                    
           char            *queue_name,                                         
           MSG_DESCRIPTOR    *descriptor,                                       
           char            *msg,                                                
           long            msg_len                                              
           )                                                                    
{                                                                               
  MQOD        queue_od = {MQOD_DEFAULT};                                        
  MQMD        local_md  = {MQMD_DEFAULT};                                       
  MQMD        *md = NULL;                                                       
  MQPMO        pmo = {MQPMO_DEFAULT};                                           
  QUEUE_CONNECTION *qconn;                                                      
  char        function??(10??);                                                 
  MSGID_STRING    pstring;                                                      
  char        LUW_STARTED_BY_API_FLAG = 'N';                                    
  char        *squeue_name;                                                     
  QM_OPTION    options;                                                         
  int        rv;                                                                
  char        *func_name = "put_msg";                                           
                                                                                
  /******** for turning off safestore ************/                             
  QM_OPTION    no_safestore = ~QM_SAFESTORE;                                    
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry, msg_len = %d", msg_len );                                           
                                                                                
  options = descriptor->options;                                                
#if 0                                                                           
  /*******************************************************************          
  **    temporarily turn off safestore option                                   
  ******************************************************************/           
                                                                                
  options &= no_safestore;                                                      
  pdebug( DBUG_FINE, conn_handle, func_name,                                    
      "options after safestore turned off = %08lx", options );                  
#endif                                                                          
  /*********************************************************************        
   **    set up Message Descriptor                                              
   ********************************************************************/        
                                                                                
  if ( descriptor->version == QM_DESCRIPTOR_VERSION_1 ) {                       
    md = &local_md;                                                             
  }                                                                             
  else {                                                                        
    md = &(descriptor->msg_md);                                                 
    memcpy( (char *)md, (char *)&local_md, sizeof(MQMD) );                      
  }                                                                             
                                                                                
  if ( put_type == REPLYTO )                                                    
    {                                                                           
      md->MsgType = MQMT_REPLY;                                                 
      qmMsgidCpy( &(md->CorrelId), descriptor->msg_id );                        
    }                                                                           
  else                                                                          
    {                                                                           
      if ( descriptor->msg_type == QM_REQUEST )                                 
    md->MsgType = MQMT_REQUEST;                                                 
      else if ( descriptor->msg_type == QM_REPLY )                              
    md->MsgType = MQMT_REPLY;                                                   
      strncpy( md->ReplyToQ, descriptor->reply_to_queue, MQ_Q_NAME_LENGTH );    
      strncpy( md->ReplyToQMgr, descriptor->reply_to_qmgr, MQ_Q_NAME_LENGTH );  
      qmMsgidCpy( &(md->MsgId), descriptor->msg_id );                           
      qmMsgidCpy( &(md->CorrelId), descriptor->msg_correlation_id );            
    }                                                                           
                                                                                
  md->Priority = descriptor->priority;                                          
                                                                                
  if ( options & QM_PERSISTENT )                                                
    md->Persistence = MQPER_PERSISTENT;                                         
  else if ( options & QM_NOT_PERSISTENT )                                       
    md->Persistence = MQPER_NOT_PERSISTENT;                                     
                                                                                
  if ( !( options & QM_BINARY ) )                                               
    strncpy( md->Format, MQFMT_STRING, sizeof(md->Format) );                    
                                                                                
  /***********************************************************************      
  * This was a quick temporary fix that forces Expiry to -1                     
  * because there is a bug with MQ that drops packets with Expiry set.          
  * Until we change this condition, Expiry's of 7 days are forced to            
  * no expiry.  This allows App's that are compiled with MQlib.h R1.14(old)     
  * to work without being recompiled with MQlib.h R1.15(new) which this         
  * shared object uses.                                                         
  ***********************************************************************/      
  if ( (descriptor->retention < 0) ||                                           
       (descriptor->retention == QM_MAX_RETENTION ) )                           
    md->Expiry = MQEI_UNLIMITED;                                                
  else                                                                          
    md->Expiry = (descriptor->retention)*36000;                                 
                                                                                
  /*********************************************************************        
   **    set up Message options                                                 
   ********************************************************************/        
                                                                                
#if defined(MVSCICS)                                                            
  if ( options & QM_NO_SYNCPOINT )                                              
    pmo.Options |= MQPMO_NO_SYNCPOINT;                                          
  else                                                                          
    pmo.Options |= MQPMO_SYNCPOINT;                                             
#else                                                                           
  if ( ( LUW_STARTED( conn_handle) ) &&                                         
      ( !(options & QM_NO_SYNCPOINT) ) )                                        
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "luw in progress, setting SYNCPOINT option" );                          
      pmo.Options |= MQPMO_SYNCPOINT;                                           
    }                                                                           
  /* set luw_flag if safestore is turned on */                                  
  else if ( !(LUW_STARTED( conn_handle )) &&                                    
        (options & QM_SAFESTORE))                                               
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Safestore is on, start luw and SYNCPOINT option" );                    
      pmo.Options |= MQPMO_SYNCPOINT;                                           
      conn_handle->luw_flag = 1;                                                
      LUW_STARTED_BY_API_FLAG = 'Y';                                            
    }                                                                           
  else                                                                          
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "luw not in progress, setting NO_SYNCPOINT option" );                   
      pmo.Options = MQPMO_NO_SYNCPOINT;                                         
    }                                                                           
#endif                                                                          
  pmo.Options |= MQPMO_FAIL_IF_QUIESCING;                                       
                                                                                
  /*********************************************************************        
   **    put Message on Queue                                                   
   ********************************************************************/        
                                                                                
  if ( put_type == PUT_NORMAL )                                                 
    {                                                                           
      strcpy( function, "MQPUT" );                                              
      qconn = (conn_handle->connection_list)+qh_index;                          
                                                                                
      MQPUT( conn_handle->Hcon,                                                 
        qconn->Hobj,                                                            
        md,                                                                     
        &pmo,                                                                   
        (MQLONG)msg_len,                                                        
        (MQBYTE *)msg,                                                          
        &(conn_handle->CompCode),                                               
        &(conn_handle->Reason) );                                               
    }                                                                           
  else                                                                          
    {                                                                           
      if ( put_type == REPLYTO )                                                
    {                                                                           
      strncpy( queue_od.ObjectName,                                             
          descriptor->reply_to_queue,                                           
          MQ_Q_NAME_LENGTH );                                                   
      strncpy( queue_od.ObjectQMgrName,                                         
          descriptor->reply_to_qmgr,                                            
          MQ_Q_NAME_LENGTH );                                                   
    }                                                                           
      else                                                                      
    strncpy( queue_od.ObjectName, queue_name, MQ_Q_NAME_LENGTH );               
                                                                                
      strcpy( function, "MQPUT1" );                                             
                                                                                
      MQPUT1( conn_handle->Hcon,                                                
        &queue_od,                                                              
        md,                                                                     
        &pmo,                                                                   
        (MQLONG)msg_len,                                                        
        (MQBYTE *)msg,                                                          
        &(conn_handle->CompCode),                                               
        &(conn_handle->Reason) );                                               
    }                                                                           
                                                                                
  rv = handle_MQ_error( conn_handle, function );                                
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      /*******************************************************************      
       **    copy msg_id to MSG_DESCRIPTOR                                      
       ******************************************************************/      
                                                                                
      qmMsgidCpy( &(descriptor->msg_id), md->MsgId );                           
#if !defined(MVSCICS)                                                           
      if ( is_debug_level( DBUG_FINE ) )                                        
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
           "msg_id = %s",                                                       
           qmMsgidString( descriptor->msg_id, pstring ) );                      
#endif                                                                          
                                                                                
      /*******************************************************************      
       **    copy queue name to (version 2) descriptor                          
       *******************************************************************/     
                                                                                
      if ( put_type == PUT_NORMAL) {                                            
    squeue_name = qconn->queue_name;                                            
      }                                                                         
      else {                                                                    
    squeue_name = queue_od.ObjectName;                                          
      }                                                                         
      if ( descriptor->version == QM_DESCRIPTOR_VERSION_2 ) {                   
    strncpy( descriptor->msg_queue, squeue_name, QM_QUEUE_LEN );                
      }                                                                         
                                                                                
      /*******************************************************************      
       **    if safestore requested, archive message                            
       ******************************************************************/      
                                                                                
      if ( options & QM_SAFESTORE )                                             
    {                                                                           
      rv = safestore_message( conn_handle,                                      
                 descriptor,                                                    
                 md,                                                            
                 msg,                                                           
                 msg_len,                                                       
                 squeue_name );                                                 
      if ( rv == QM_SUCCESS )                                                   
        {                                                                       
          if (LUW_STARTED_BY_API_FLAG == 'Y')                                   
        {                                                                       
          rv = commit_luw( conn_handle );                                       
          if ( ( rv != QM_SUCCESS ) && ( rv != QM_CONNECTION_BROKEN ) )         
            rv = QM_SAFESTORE_ERROR;                                            
        }                                                                       
        }                                                                       
      else if ( rv != QM_CONNECTION_BROKEN )                                    
        {                                                                       
          if (LUW_STARTED_BY_API_FLAG == 'Y')                                   
        rv = backout_luw( conn_handle );                                        
          if ( rv != QM_CONNECTION_BROKEN )                                     
        rv = QM_SAFESTORE_ERROR;                                                
        }                                                                       
    }                                                                           
    }                                                                           
                                                                                
  if ( LUW_STARTED_BY_API_FLAG == 'Y' )                                         
    conn_handle->luw_flag = 0;                                                  
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    get_msg( )                                                                
**                                                                              
**    This internal function calls MQGET to get a message from a queue          
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue_index                                                           
**        MSG_DESCRIPTOR                                                        
**        buffer                                                                
**        buf_len                                                               
**    Output:                                                                   
**        Return Code                                                           
**        message ( in buffer )                                                 
**        updated MSG_DESCRIPTOR                                                
**                                                                              
**==========================================================================    
*/                                                                              
static int get_msg(                                                             
           CONNECTION_HANDLE    conn_handle,                                    
           unsigned short          qh_index,                                    
           MSG_DESCRIPTOR    *descriptor,                                       
           char            *buffer,                                             
           long            buf_len                                              
           )                                                                    
{                                                                               
  MQMD        local_md  = {MQMD_DEFAULT};                                       
  MQMD        *md = NULL;                                                       
  MQGMO        gmo = {MQGMO_DEFAULT};                                           
  MQLONG    mq_msg_len;                                                         
  QUEUE_CONNECTION *qconn;                                                      
  MSGID_STRING    pstring;                                                      
  QM_OPTION    options;                                                         
  int        rv, put_msg_len;                                                   
  char        *func_name = "get_msg";                                           
  char         LUW_STARTED_BY_API_FLAG = 'N';                                   
                                                                                
  /******** for turning off safestore ************/                             
  QM_OPTION    no_safestore = ~QM_SAFESTORE;                                    
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry, buf_len = %d", buf_len );                                           
                                                                                
  options = descriptor->options;                                                
#if 0                                                                           
  /*******************************************************************          
  **    temporarily turn off safestore option                                   
  ******************************************************************/           
                                                                                
  options &= no_safestore;                                                      
  pdebug( DBUG_FINE, conn_handle, func_name,                                    
      "options after safestore turned off = %08lx", options );                  
#endif                                                                          
  /*********************************************************************        
   **    set up Message Descriptor                                              
   ********************************************************************/        
                                                                                
  if ( descriptor->version == QM_DESCRIPTOR_VERSION_1 ) {                       
    md = &local_md;                                                             
  }                                                                             
  else {                                                                        
    md = &(descriptor->msg_md);                                                 
    memcpy( (char *)md, (char *)&local_md, sizeof(MQMD) );                      
  }                                                                             
                                                                                
  qmMsgidCpy( &(md->MsgId), descriptor->msg_id );                               
  qmMsgidCpy( &(md->CorrelId), descriptor->msg_correlation_id );                
                                                                                
  /*********************************************************************        
   **    set up Message options                                                 
   ********************************************************************/        
                                                                                
  if ( !MSG_BROWSED(descriptor) )                                               
#if defined(MVSCICS)                                                            
    if ( options & QM_NO_SYNCPOINT )                                            
      gmo.Options |= MQGMO_NO_SYNCPOINT;                                        
    else                                                                        
      gmo.Options |= MQGMO_SYNCPOINT;                                           
#else                                                                           
    if  ( ( LUW_STARTED( conn_handle ) && !(options & QM_NO_SYNCPOINT) ) ||     
      ( options & QM_BEGIN_TRANSACTION ) )                                      
      {                                                                         
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
        "luw started or requested, setting SYNCPOINT option" );                 
    gmo.Options |= MQGMO_SYNCPOINT;                                             
      }                                                                         
  /* set luw_flag if safestore is turned on */                                  
    else if ( !(LUW_STARTED( conn_handle )) && (options & QM_SAFESTORE) )       
      {                                                                         
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
        "Safestore is on, start luw and SYNCPOINT option" );                    
    gmo.Options |= MQPMO_SYNCPOINT;                                             
    conn_handle->luw_flag = 1;                                                  
    LUW_STARTED_BY_API_FLAG = 'Y';                                              
      }                                                                         
    else                                                                        
      {                                                                         
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
        "luw not started or requested, "                                        
        "setting NO_SYNCPOINT option" );                                        
    gmo.Options |= MQGMO_NO_SYNCPOINT;                                          
      }                                                                         
#endif                                                                          
                                                                                
  if ( descriptor->timeout == 0 )                                               
    gmo.Options |= MQGMO_NO_WAIT;                                               
  else                                                                          
    {                                                                           
      gmo.Options |= MQGMO_WAIT;                                                
      if ( descriptor->timeout > 0 )                                            
    gmo.WaitInterval = 1000 * (descriptor->timeout);                            
      else                                                                      
    gmo.WaitInterval = MQWI_UNLIMITED;                                          
    }                                                                           
  if ( options & QM_ACCEPT_TRUNCATED_MSG )                                      
    gmo.Options |= MQGMO_ACCEPT_TRUNCATED_MSG;                                  
                                                                                
  if ( options & QM_BROWSE_FIRST )                                              
    gmo.Options |= MQGMO_BROWSE_FIRST;                                          
  else if ( options & QM_BROWSE_NEXT )                                          
    gmo.Options |= MQGMO_BROWSE_NEXT;                                           
  else if ( options & QM_MSG_UNDER_CURSOR )                                     
    gmo.Options |= MQGMO_MSG_UNDER_CURSOR;                                      
                                                                                
  if ( !( options & QM_BINARY ) )                                               
    gmo.Options |= MQGMO_CONVERT;                                               
  gmo.Options |= MQGMO_FAIL_IF_QUIESCING;                                       
                                                                                
  /*******************************************************************          
   **    get Message from Queue                                                 
   ******************************************************************/          
                                                                                
  qconn = (conn_handle->connection_list)+qh_index;                              
  MQGET( conn_handle->Hcon,                                                     
    qconn->Hobj,                                                                
    md,                                                                         
    &gmo,                                                                       
    buf_len,                                                                    
    buffer,                                                                     
    &mq_msg_len,                                                                
    &(conn_handle->CompCode),                                                   
    &(conn_handle->Reason)                                                      
    );                                                                          
                                                                                
  rv = handle_MQ_error( conn_handle, "MQGET" );                                 
                                                                                
  /************************************************************************     
   **    update message descriptor with output values                           
   ***********************************************************************/     
                                                                                
  if ( MSG_RETRIEVED(rv) )                                                      
  {                                                                             
      descriptor->msg_len = mq_msg_len;                                         
      qmMsgidCpy( &(descriptor->msg_id), md->MsgId );                           
      qmMsgidCpy( &(descriptor->msg_correlation_id), md->CorrelId  );           
      strncpy( descriptor->reply_to_queue, md->ReplyToQ, QM_QUEUE_LEN );        
      strncpy( descriptor->reply_to_qmgr, md->ReplyToQMgr, QM_QUEUE_LEN );      
                                                                                
      switch( md->MsgType )                                                     
    {                                                                           
    case MQMT_DATAGRAM:                                                         
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "msg_type = DATAGRAM" );                                                
      descriptor->msg_type = QM_DATAGRAM;                                       
      break;                                                                    
    case MQMT_REQUEST:                                                          
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "msg_type = REQUEST" );                                                 
      descriptor->msg_type = QM_REQUEST;                                        
      break;                                                                    
    case MQMT_REPLY:                                                            
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "msg_type = REPLY" );                                                   
      descriptor->msg_type = QM_REPLY;                                          
      break;                                                                    
    case MQMT_REPORT:                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "msg_type = REPORT" );                                                  
      descriptor->msg_type = QM_REPORT;                                         
      break;                                                                    
    default:                                                                    
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "msg_type, &d, is UNKNOWN", md->MsgType );                              
      descriptor->msg_type = QM_UNKNOWN;                                        
      break;                                                                    
    }                                                                           
                                                                                
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "Message Parameters:" );                                                
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "    msg_type            = %ld", descriptor->msg_type );                
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "    msg_len             = %ld", descriptor->msg_len );                 
#if !defined(MVSCICS)                                                           
      if ( is_debug_level( DBUG_FINE ) )                                        
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "    msg_id              = %s",                                         
        qmMsgidString( descriptor->msg_id, pstring ) );                         
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "    msg_correlation_id  = %s",                                         
        qmMsgidString( descriptor->msg_correlation_id, pstring ) );             
    }                                                                           
#endif                                                                          
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "   reply_to_queue       = %.*s",                                       
        QM_QUEUE_LEN, descriptor->reply_to_queue );                             
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "   reply_to_qmgr        = %.*s",                                       
        QM_QUEUE_LEN, descriptor->reply_to_qmgr );                              
  }                                                                             
                                                                                
  if ( MSG_DEQUEUED( rv, descriptor ) )                                         
    {                                                                           
      /*********************************************************************    
       **    if requested, start unit of work                                   
       ********************************************************************/    
                                                                                
#if !defined(MVSCICS)                                                           
      if ( options & QM_BEGIN_TRANSACTION )                                     
    conn_handle->luw_flag = 1;                                                  
#endif                                                                          
                                                                                
      /*******************************************************************      
       **    copy queue name to (version 2) descriptor                          
       *******************************************************************/     
                                                                                
      if ( descriptor->version == QM_DESCRIPTOR_VERSION_2 ) {                   
    strncpy( descriptor->msg_queue, qconn->queue_name, QM_QUEUE_LEN );          
      }                                                                         
                                                                                
      /**********************************************************************   
       ** if Safestore is turned on then put the message on the archive queue   
       **********************************************************************/  
                                                                                
      if ( options & QM_SAFESTORE )                                             
    {                                                                           
      if (mq_msg_len > buf_len)                                                 
        put_msg_len = buf_len;                                                  
      else                                                                      
        put_msg_len = mq_msg_len;                                               
                                                                                
      rv = safestore_message( conn_handle,                                      
                 descriptor,                                                    
                 md,                                                            
                 buffer,                                                        
                 put_msg_len,                                                   
                 qconn->queue_name );                                           
      if ( rv == QM_SUCCESS )                                                   
        {                                                                       
          if (LUW_STARTED_BY_API_FLAG == 'Y')                                   
        {                                                                       
          rv = commit_luw( conn_handle );                                       
          if ( ( rv != QM_SUCCESS ) && ( rv != QM_CONNECTION_BROKEN ) )         
            rv = QM_SAFESTORE_ERROR;                                            
        }                                                                       
        }                                                                       
      else if ( rv != QM_CONNECTION_BROKEN )                                    
        {                                                                       
          if (LUW_STARTED_BY_API_FLAG == 'Y')                                   
        rv = backout_luw( conn_handle );                                        
          if ( rv != QM_CONNECTION_BROKEN )                                     
        rv = QM_SAFESTORE_ERROR;                                                
        }                                                                       
    }                                                                           
    }                                                                           
                                                                                
  if ( LUW_STARTED_BY_API_FLAG == 'Y' )                                         
    conn_handle->luw_flag = 0;                                                  
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*=========================================================================     
**                                                                              
**    safestore_message( )                                                      
**                                                                              
**    This internal function puts a message on the ARCHIVE queue.               
**    If necessary the archive queue is opened.                                 
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        MSG_DESCRIPTOR                                                        
**        MQMD                                                                  
**        message                                                               
**        message length                                                        
**        queue name (for the message, not ARCHIVE )                            
**    Output:                                                                   
**        return code                                                           
**        message on ARCHIVE queue                                              
**                                                                              
**=========================================================================     
*/                                                                              
static int safestore_message( CONNECTION_HANDLE    conn_handle,                 
                 MSG_DESCRIPTOR     *descriptor,                                
                 MQMD        *msg_md,                                           
                 char        *msg,                                              
                 long        msg_len,                                           
                 char        *queue_name )                                      
{                                                                               
  MQMD            md  = {MQMD_DEFAULT};                                         
  MQPMO            pmo = {MQPMO_DEFAULT};                                       
  char            *archive_msg;                                                 
  QUEUE_CONNECTION    *qconn;                                                   
  MSGID_STRING        pstring;                                                  
  int            rv = QM_SUCCESS;                                               
  int            header_size = get_arch_header_size( );                         
  char            safestoreInfo??(QM_INFO_LEN+1??);                             
  char            *func_name = "safestore_message";                             
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entering" );                                                               
                                                                                
  /*******************************************************************          
   **    if safestore requested and safestore queue is not open,                
   **       open safestore queue                                                
   ******************************************************************/          
                                                                                
  if ( (conn_handle->connection_list)??(0??).Hobj == MQHO_UNUSABLE_HOBJ)        
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
         "opening safestore queue" );                                           
      rv = open_safestore_queue( conn_handle );                                 
      if ( rv != QM_SUCCESS )                                                   
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
         "open of safestore queue failed" );                                    
    }                                                                           
    }                                                                           
                                                                                
  if ( rv == QM_SUCCESS )                                                       
    {                                                                           
      /* make sure that safestore info is nul terminated */                     
      strncpy( safestoreInfo, descriptor->safestore_info, QM_INFO_LEN );        
      safestoreInfo??(QM_INFO_LEN??) = '\0';                                    
                                                                                
      archive_msg = msg - header_size;                                          
      build_arch_header( archive_msg,                                           
             conn_handle->queue_manager_name,                                   
             queue_name,                                                        
             safestoreInfo,                                                     
             msg_len,                                                           
             msg_md,                                                            
             &md );                                                             
                                                                                
      /*                                                                        
      build_header(descriptor, md, conn_handle, msg_len,                        
           archive_msg, queue_name);                                            
      */                                                                        
                                                                                
      /*******************************************************************      
       **    set up some options for the archive message                        
       ******************************************************************/      
                                                                                
      md.Persistence    = MQPER_PERSISTENT;                                     
      md.Expiry     = MQEI_UNLIMITED;                                           
      pmo.Options     |= MQPMO_FAIL_IF_QUIESCING;                               
#if defined(MVSCICS)                                                            
      pmo.Options |= MQPMO_SYNCPOINT;                                           
#else                                                                           
      if ( LUW_STARTED( conn_handle) ) {                                        
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
        "luw in progress, setting SYNCPOINT option" );                          
    pmo.Options |= MQPMO_SYNCPOINT;                                             
      }                                                                         
      else {                                                                    
    pdebug( DBUG_FINE, conn_handle, func_name,                                  
        "luw not in progress, setting NO_SYNCPOINT option" );                   
    pmo.Options = MQPMO_NO_SYNCPOINT;                                           
      }                                                                         
#endif                                                                          
      /*******************************************************************      
       **    put message on archive queue                                       
       ******************************************************************/      
                                                                                
      qconn = ( conn_handle->connection_list );                                 
                                                                                
      MQPUT( conn_handle->Hcon,                                                 
        qconn->Hobj,                                                            
        &md,                                                                    
        &pmo,                                                                   
        (MQLONG)(msg_len + header_size),                                        
        (MQBYTE *)archive_msg,                                                  
        &(conn_handle->CompCode),                                               
        &(conn_handle->Reason) );                                               
                                                                                
      rv = handle_MQ_error( conn_handle, "MQPUT" );                             
                                                                                
      if ( rv == QM_SUCCESS )                                                   
    {                                                                           
      pdebug( DBUG_FINE, conn_handle, func_name,                                
         "successfully put the message on ARCHIVE queue, "                      
         "msg_id = %s",                                                         
         qmMsgidString( md.MsgId, pstring ) );                                  
    }                                                                           
      else                                                                      
    {                                                                           
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
         "Error putting message on ARCHIVE queue");                             
    }                                                                           
    }                                                                           
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    build_header( MSG_DESCRIPTOR *descriptor, MQMD &md,                       
**                    MQOD queue_od, long msg_len, char *archive_msg,           
**              char *queue_name )                                              
**                                                                              
**    This internal function builds the header for the archive message.         
**                                                                              
**    Input:                                                                    
**        debug_fp, descripttor, md, queue_od, msg_len,                         
**            archive_msg, queue_name                                           
**    Output:                                                                   
**        archive_msg                                                           
**                                                                              
**==========================================================================    
*/                                                                              
static void build_header( MSG_DESCRIPTOR *descriptor, MQMD *md,                 
             CONNECTION_HANDLE conn_handle, long msg_len,                       
                  char *archive_msg, char *queue_name)                          
{                                                                               
  char hostname??(64??);                                                        
  char chMsgLen??(10??);                                                        
  char tempFld??(33??);                                                         
  int i, bufIndex = 0;                                                          
  char        *func_name = "build_header";                                      
                                                                                
  /*********************************************************************        
   ** add key fields (queue manager, system name, userdata, appl data,          
   ** message length) and MQMD to original message in archive_msg               
   ********************************************************************/        
  pdebug( DBUG_FINE, conn_handle, func_name,                                    
    "building archive_msg" );                                                   
                                                                                
  /* skip over buffer eye catcher */                                            
  bufIndex += SSTOR_ID_LEN;                                                     
                                                                                
  /* add queue manager name */                                                  
  copy_field(archive_msg, &bufIndex, conn_handle->queue_manager_name, 48);      
                                                                                
  pdebug( DBUG_FINE, conn_handle, func_name,                                    
    "queue name = %s", queue_name);                                             
                                                                                
  /* add queue name */                                                          
  copy_field(archive_msg, &bufIndex, queue_name, 48);                           
                                                                                
  /* add system name */                                                         
  copy_field(archive_msg, &bufIndex, conn_handle->system_id, 12);               
                                                                                
  /* add user_data/safestore_info to archive_msg */                             
  copy_field(archive_msg, &bufIndex, descriptor->safestore_info, 60);           
                                                                                
  /* add application data */                                                    
  /* strcpy(tempFld, md->ApplIdentityData);        */                           
  /* rm_trailing_whitespace(tempFld);             */                            
  /* copy_field(archive_msg, &bufIndex, tempFld, 32);    */                     
  copy_field(archive_msg, &bufIndex, md->ApplIdentityData, 32);                 
                                                                                
  /* add msg_len */                                                             
  sprintf( chMsgLen, "%.9d", msg_len );                                         
  copy_field(archive_msg, &bufIndex, chMsgLen, 9);                              
                                                                                
  for (i=bufIndex; i<KEY_FIELDS_LEN; i++)                                       
      archive_msg??(i??) = '\0';                                                
                                                                                
  /* make sure the MQMD data starts after the KEYFIELDS */                      
  bufIndex = KEY_FIELDS_LEN;                                                    
                                                                                
  /* add MQMD to archive_msg */                                                 
  copy_MQMD( bufIndex, md, archive_msg );                                       
                                                                                
}                                                                               
                                                                                
                                                                                
/*==========================================================================    
**                                                                              
**    copy_MQMD( int bufIndex, MQMD *md, char *archive_msg )                    
**                                                                              
**    This internal function will convert all the MQMD fields into char,        
**      if needed, and copy them into archive_msg                               
**                                                                              
**    Input:                                                                    
**        bufIndex, MQMD, archive_msg                                           
**    Output:                                                                   
**        archive_msg                                                           
**                                                                              
**==========================================================================    
*/                                                                              
static void copy_MQMD( int bufIndex, MQMD *md, char *archive_msg )              
{                                                                               
   int  i, len;                                                                 
   char temp??(10??);                                                           
   char fldLength??(4??);                                                       
   char tempFld??(33??);                                                        
   BYTE_STRNG pstring;                                                          
                                                                                
   /* copy StrucId */                                                           
   copy_field(archive_msg, &bufIndex, md->StrucId, 4);                          
                                                                                
   /* copy Version */                                                           
   sprintf( temp, "%d", md->Version );                                          
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy Report */                                                            
   sprintf( temp, "%d", md->Report );                                           
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy MsgType */                                                           
   sprintf( temp, "%d", md->MsgType );                                          
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy Expiry */                                                            
   sprintf( temp, "%d", md->Expiry );                                           
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy Feedback */                                                          
   sprintf( temp, "%d", md->Feedback );                                         
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy Encoding */                                                          
   sprintf( temp, "%d", md->Encoding );                                         
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy CodedCharSetId */                                                    
   sprintf( temp, "%d", md->CodedCharSetId );                                   
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy Format */                                                            
   copy_field(archive_msg, &bufIndex, md->Format, 8);                           
                                                                                
   /* copy Priority */                                                          
   sprintf( temp, "%d", md->Priority );                                         
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy Persistence */                                                       
   sprintf( temp, "%d", md->Persistence );                                      
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy MsgId */                                                             
   convertByteFld( md->MsgId, pstring, sizeof( md->MsgId ) );                   
   copy_field(archive_msg, &bufIndex, pstring, 48);                             
                                                                                
   /* copy CorrelId */                                                          
   convertByteFld( md->CorrelId, pstring, sizeof( md->CorrelId ) );             
   copy_field(archive_msg, &bufIndex, pstring, 48);                             
                                                                                
   /* copy BackoutCount */                                                      
   sprintf( temp, "%d", md->BackoutCount );                                     
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy ReplyToQ */                                                          
   copy_field(archive_msg, &bufIndex, md->ReplyToQ, 48);                        
                                                                                
   /* copy ReplyToQMgr */                                                       
   copy_field(archive_msg, &bufIndex, md->ReplyToQMgr, 48);                     
                                                                                
   /* copy UserIdentifier */                                                    
   copy_field(archive_msg, &bufIndex, md->UserIdentifier, 12);                  
                                                                                
   /* copy AccountingToken */                                                   
   convertByteFld( md->AccountingToken, pstring,                                
           sizeof( md->AccountingToken ) );                                     
   copy_field(archive_msg, &bufIndex, pstring, 64);                             
                                                                                
   /* copy ApplIdentityData */                                                  
   copy_field(archive_msg, &bufIndex, md->ApplIdentityData, 32);                
                                                                                
   /* copy PutApplType */                                                       
   sprintf( temp, "%d", md->PutApplType );                                      
   copy_field(archive_msg, &bufIndex, temp, 9);                                 
                                                                                
   /* copy PutApplName */                                                       
   copy_field(archive_msg, &bufIndex, md->PutApplName, 28);                     
                                                                                
   /* copy  PutDate */                                                          
   copy_field(archive_msg, &bufIndex, md->PutDate, 8);                          
                                                                                
   /* copy PutTime */                                                           
   copy_field(archive_msg, &bufIndex, md->PutTime, 8);                          
                                                                                
   /* copy ApplOriginData */                                                    
   copy_field(archive_msg, &bufIndex, md->ApplOriginData, 4);                   
                                                                                
   for (i=bufIndex; i<(KEY_FIELDS_LEN+MQMD_LEN); i++)                           
       archive_msg??(i??) = '\0';                                               
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    copy_field( )                                                             
**                                                                              
**    This routine copies the input field to the archive message buffer         
**      with the length infront.                                                
**                                                                              
**    Input:                                                                    
**        archive_msg                                                           
**        bufIndex                                                              
**        field                                                                 
**    Output:                                                                   
**              archive_msg                                                     
**                                                                              
**==========================================================================    
*/                                                                              
static void copy_field(                                                         
    char *archive_msg,                                                          
    int  *bufIndex,                                                             
    char *in_field,                                                             
    int  max_field_len)                                                         
{                                                                               
    int len;                                                                    
    char str??(4??), field??(65??);                                             
    int             count, length;                                              
                                                                                
    /* there many fields that are passed in by the caller doesn't have the */   
    /* null terminator at the end of the field. So to avoid stepping over */    
    /* the boundary without modifying the origional field string, we need */    
    /* to copy the in_field into another string and terminate the string  */    
    /* at the max_field_len */                                                  
                                                                                
    field??(max_field_len??) = '\0';                                            
    strncpy(field, in_field, max_field_len);                                    
    /* rm_trailing_whitespace(field); */                                        
    if (field??(0??) != '\0')                                                   
      {                                                                         
    length = strlen(field) - 1;                                                 
    for (count = length; isspace(field??(count??)); count--)                    
      field??(count??) = '\0';                                                  
      }                                                                         
    len = strlen(field);                                                        
    sprintf(str, "%.3d", len);                                                  
    strncpy(archive_msg + *bufIndex, str, 3);                                   
    *bufIndex += 3;                                                             
    strncpy(archive_msg + *bufIndex, field, len);                               
    *bufIndex += len;                                                           
                                                                                
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    close_queue( )                                                            
**                                                                              
**    This internal function calls MQCLOSE to close a queue.                    
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        queue_index                                                           
**    Output:                                                                   
**        Return Code                                                           
**                                                                              
**===========================================================================   
*/                                                                              
static int close_queue(                                                         
               CONNECTION_HANDLE conn_handle,                                   
               unsigned short int qh_index                                      
               )                                                                
{                                                                               
  MQLONG    C_Options;                                                          
  QUEUE_CONNECTION *qconn;                                                      
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "close_queue";                                       
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  qconn = (conn_handle->connection_list)+qh_index;                              
                                                                                
  /*****************************************************************            
   **    Close the Queue                                                        
   ****************************************************************/            
                                                                                
  C_Options = MQCO_NONE;                                                        
  MQCLOSE( conn_handle->Hcon,                                                   
      &(qconn->Hobj),                                                           
      C_Options,                                                                
      &(conn_handle->CompCode),                                                 
      &(conn_handle->Reason) );                                                 
                                                                                
  rv = handle_MQ_error( conn_handle, "MQCLOSE" );                               
                                                                                
  /*******************************************************************          
  **    On MVS the Hobj is not reset to MQHO_UNUSABLE_HOBJ                      
  ******************************************************************/           
                                                                                
#if defined(MVS)                                                                
  if ( rv == QM_SUCCESS )                                                       
    qconn->Hobj = MQHO_UNUSABLE_HOBJ;                                           
#endif                                                                          
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
  return( rv );                                                                 
}                                                                               
                                                                                
                                                                                
/*===========================================================================   
**                                                                              
**    disconnect( )                                                             
**                                                                              
**    This internal function disconnects from the queue manager.                
**                                                                              
**    Input:    CONNECTION_HANDLE                                               
**    Output:    Return Code                                                    
**                                                                              
**===========================================================================   
*/                                                                              
static int disconnect( CONNECTION_HANDLE conn_handle )                          
{                                                                               
  int        rv = QM_SUCCESS;                                                   
  char        *func_name = "disconnect";                                        
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  MQDISC( &(conn_handle->Hcon),                                                 
     &(conn_handle->CompCode),                                                  
     &(conn_handle->Reason) );                                                  
                                                                                
  rv = handle_MQ_error( conn_handle, "MQDISC" );                                
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
  return( rv );                                                                 
                                                                                
}                                                                               
                                                                                
/*===========================================================================   
**                                                                              
**    handle_MQ_error( )                                                        
**                                                                              
**    This internal function converts a MQ reason code to a MQlib               
**    return code.                                                              
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**        function name                                                         
**    Output                                                                    
**        Return Code                                                           
**                                                                              
**===========================================================================   
*/                                                                              
static int handle_MQ_error(                                                     
               CONNECTION_HANDLE conn_handle,                                   
               char *function                                                   
               )                                                                
{                                                                               
  MQLONG     CompCode;                                                          
  MQLONG     Reason;                                                            
  int        stat, rv = QM_SUCCESS;                                             
  char        *func_name = "handle_MQ_error";                                   
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Entry" );                                                                  
                                                                                
  CompCode = MQCCODE( conn_handle );                                            
  Reason = MQREASON( conn_handle );                                             
                                                                                
  pdebug( DBUG_FINE, conn_handle, func_name,                                    
    "function = %s, CompCode = %d, Reason = %d",                                
    function, CompCode, Reason );                                               
                                                                                
  if ( ( CompCode != MQCC_OK ) && ( Reason != MQRC_NO_MSG_AVAILABLE ) )         
    pdebug( DBUG_ERRMSG, conn_handle, func_name,                                
       "MQS error: %s", MQr2string( Reason ) );                                 
                                                                                
  switch( CompCode )                                                            
    {                                                                           
    default:                                                                    
      pdebug( DBUG_ERRMSG, conn_handle, func_name,                              
        "%s returned Unexpected Completion Code %d",                            
        function, CompCode );                                                   
      rv = QM_FAIL;                                                             
      break;                                                                    
    case MQCC_OK:                                                               
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "%s succeeded", function );                                             
      break;                                                                    
    case MQCC_WARNING:                                                          
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
        "%s returned warning, Reason Code = %d", function, Reason );            
      switch( Reason )                                                          
    {                                                                           
    case MQRC_ALREADY_CONNECTED:                                                
      rv = QM_ALREADY_CONNECTED;                                                
      break;                                                                    
    case MQRC_BACKED_OUT:                                                       
      rv = QM_BACKED_OUT;                                                       
      break;                                                                    
    case MQRC_PRIORITY_EXCEEDS_MAXIMUM:                                         
      break;                                                                    
    case MQRC_TRUNCATED_MSG_ACCEPTED:                                           
    case MQRC_TRUNCATED_MSG_FAILED:                                             
      rv = QM_MSG_TOO_BIG;                                                      
      break;                                                                    
    case MQRC_NOT_CONVERTED:                                                    
      rv = QM_CONVERSION_ERROR;                                                 
      break;                                                                    
    case MQRC_NO_EXTERNAL_PARTICIPANTS:                                         
      rv = QM_XA_NOT_CONFIGURED;                                                
      break;                                                                    
    case MQRC_PARTICIPANT_NOT_AVAILABLE:                                        
      rv = QM_XA_NOT_AVAILABLE;                                                 
      break;                                                                    
    case MQRC_OUTCOME_PENDING:                                                  
      rv = QM_XA_PENDING_OUTCOME;                                               
    default:                                                                    
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
        "Unexpected Reason Code "                                               
        "%d", Reason );                                                         
    }                                                                           
      break;                                                                    
    case MQCC_FAILED:                                                           
      if ( Reason != MQRC_NO_MSG_AVAILABLE )                                    
    pdebug( DBUG_NORMAL, conn_handle, func_name,                                
          "%s failed, Reason Code = %d",                                        
          function, Reason );                                                   
      switch ( Reason )                                                         
    {                                                                           
    case MQRC_Q_MGR_NAME_ERROR:                                                 
      rv = QM_INVALID;                                                          
      break;                                                                    
    case MQRC_Q_MGR_NOT_AVAILABLE:                                              
      rv = QM_NOT_AVAILABLE;                                                    
      break;                                                                    
                                                                                
    case MQRC_NOT_AUTHORIZED:                                                   
      rv = QM_NOT_AUTHORIZED;                                                   
      break;                                                                    
    case MQRC_HOBJ_ERROR:                                                       
    case MQRC_NAME_NOT_VALID_FOR_TYPE:                                          
    case MQRC_UNKNOWN_OBJECT_NAME:                                              
    case MQRC_OPTIONS_ERROR:                                                    
    case MQRC_PRIORITY_ERROR:                                                   
      rv = QM_INVALID;                                                          
      break;                                                                    
    case MQRC_OBJECT_IN_USE:                                                    
      rv = QM_QUEUE_IN_USE;                                                     
      break;                                                                    
    case MQRC_MSG_TOO_BIG_FOR_Q:                                                
      rv = QM_MSG_TOO_BIG;                                                      
      break;                                                                    
    case MQRC_NOT_OPEN_FOR_OUTPUT:                                              
    case MQRC_NOT_OPEN_FOR_INPUT:                                               
    case MQRC_NOT_OPEN_FOR_BROWSE:                                              
      rv = QM_ACCESS;                                                           
      break;                                                                    
    case MQRC_Q_FULL:                                                           
      rv = QM_QUEUE_FULL;                                                       
      break;                                                                    
    case MQRC_Q_SPACE_NOT_AVAILABLE:                                            
#if defined(MVS)                                                                
    case MQRC_PAGESET_FULL:                                                     
#endif                                                                          
      rv = QM_SPACE_NOT_AVAILABLE;                                              
      break;                                                                    
    case MQRC_NO_MSG_AVAILABLE:                                                 
      pdebug( DBUG_FINE, conn_handle, func_name,                                
        "No message available" );                                               
      rv = QM_NO_MSG_AVAILABLE;                                                 
      break;                                                                    
    case MQRC_SYNCPOINT_LIMIT_REACHED:                                          
      rv = QM_SYNCPOINT_LIMIT;                                                  
      break;                                                                    
    case MQRC_OUTCOME_MIXED:                                                    
      rv = QM_XA_MIXED_OUTCOME;                                                 
      break;                                                                    
    case MQRC_UOW_IN_PROGRESS:                                                  
      rv = QM_ALREADY_STARTED;                                                  
      break;                                                                    
                                                                                
    /*******************************************************************        
     **    handle Queue Manager quiescing                                       
     ******************************************************************/        
                                                                                
    case MQRC_Q_MGR_QUIESCING:                                                  
                                                                                
      /*******************************************************************      
       **    the following prevents a loop in the backout_luw() and             
       **    disconnect() calls.                                                
       **    Note: This is probably not necessary.                              
       ******************************************************************/      
                                                                                
      if ( DISCONNECTING( conn_handle ) )                                       
        break;                                                                  
      conn_handle->state = QM_DISCONNECTING;                                    
                                                                                
      /*******************************************************************      
       **    if unit of work in progress, back it out                           
       ******************************************************************/      
#if !defined(MVSCICS)                                                           
      if ( LUW_STARTED( conn_handle ) )                                         
        {                                                                       
          pdebug( DBUG_FINE, conn_handle, func_name,                            
            "Queue Manager Quiescing - Backing Out Unit of Work" );             
          stat = backout_luw( conn_handle );                                    
        }                                                                       
#endif                                                                          
      /*******************************************************************      
       **    disconnect                                                         
       ******************************************************************/      
                                                                                
      if ( DISCONNECTING( conn_handle ) )                                       
        {                                                                       
          pdebug( DBUG_FINE, conn_handle, func_name,                            
            "Queue Manager Quiescing - Disconnecting" );                        
          stat = disconnect( conn_handle );                                     
        }                                                                       
                                                                                
    case MQRC_Q_MGR_STOPPING:                                                   
    case MQRC_CONNECTION_BROKEN:                                                
      conn_handle->state = QM_UNCONNECTED;                                      
      conn_handle->luw_flag = 0;                                                
      rv = QM_CONNECTION_BROKEN;                                                
      break;                                                                    
    default:                                                                    
      rv = QM_FAIL;                                                             
      break;                                                                    
    } /* switch( Reason ) */                                                    
      break;                                                                    
    } /* switch( CompCode ) */                                                  
                                                                                
  pdebug( DBUG_ENTRY_EXIT, conn_handle, func_name,                              
    "Exit, rv = %d", rv );                                                      
                                                                                
  return( rv );                                                                 
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    MQr2string( )                                                             
**                                                                              
**    This function converts a MQSeries Reason Code to an error message         
**    string.                                                                   
**                                                                              
**    Input:                                                                    
**        MQSeries Reason Code                                                  
**    Output:                                                                   
**        Error Message                                                         
**                                                                              
**==========================================================================    
*/                                                                              
static const char *MQr2string( int reason )                                     
{                                                                               
  struct errmsg_element                                                         
    {                                                                           
      int    reason;                                                            
      char    *errmsg;                                                          
    };                                                                          
                                                                                
  static const struct errmsg_element errmsg_list??( ??) =                       
      {                                                                         
         { 0, "NONE" },                                                         
         { 2001, "ALIAS_BASE_Q_TYPE_ERROR" },                                   
         { 2002, "ALREADY_CONNECTED" },                                         
         { 2003, "BACKED_OUT" },                                                
         { 2004, "BUFFER_ERROR" },                                              
         { 2005, "BUFFER_LENGTH_ERROR" },                                       
         { 2006, "CHAR_ATTR_LENGTH_ERROR" },                                    
         { 2007, "CHAR_ATTRS_ERROR" },                                          
         { 2008, "CHAR_ATTRS_TOO_SHORT" },                                      
         { 2009, "CONNECTION_BROKEN" },                                         
         { 2010, "DATA_LENGTH_ERROR" },                                         
         { 2011, "DYNAMIC_Q_NAME_ERROR" },                                      
         { 2012, "ENVIRONMENT_ERROR" },                                         
         { 2013, "EXPIRY_ERROR" },                                              
         { 2014, "FEEDBACK_ERROR" },                                            
         { 2016, "GET_INHIBITED" },                                             
         { 2017, "HANDLE_NOT_AVAILABLE" },                                      
         { 2018, "HCONN_ERROR" },                                               
         { 2019, "HOBJ_ERROR" },                                                
         { 2020, "INHIBIT_VALUE_ERROR" },                                       
         { 2021, "INT_ATTR_COUNT_ERROR" },                                      
         { 2022, "INT_ATTR_COUNT_TOO_SMALL" },                                  
         { 2023, "INT_ATTRS_ARRAY_ERROR" },                                     
         { 2024, "SYNCPOINT_LIMIT_REACHED" },                                   
         { 2025, "MAX_CONNS_LIMIT_REACHED" },                                   
         { 2026, "MD_ERROR" },                                                  
         { 2027, "MISSING_REPLY_TO_Q" },                                        
         { 2029, "MSG_TYPE_ERROR" },                                            
         { 2030, "MSG_TOO_BIG_FOR_Q" },                                         
         { 2031, "MSG_TOO_BIG_FOR_Q_MGR" },                                     
         { 2033, "NO_MSG_AVAILABLE" },                                          
         { 2034, "NO_MSG_UNDER_CURSOR" },                                       
         { 2035, "NOT_AUTHORIZED" },                                            
         { 2036, "NOT_OPEN_FOR_BROWSE" },                                       
         { 2037, "NOT_OPEN_FOR_INPUT" },                                        
         { 2038, "NOT_OPEN_FOR_INQUIRE" },                                      
         { 2039, "NOT_OPEN_FOR_OUTPUT" },                                       
         { 2040, "NOT_OPEN_FOR_SET" },                                          
         { 2041, "OBJECT_CHANGED" },                                            
         { 2042, "OBJECT_IN_USE" },                                             
         { 2043, "OBJECT_TYPE_ERROR" },                                         
         { 2044, "OD_ERROR" },                                                  
         { 2045, "OPTION_NOT_VALID_FOR_TYPE" },                                 
         { 2046, "OPTIONS_ERROR" },                                             
         { 2047, "PERSISTENCE_ERROR" },                                         
         { 2048, "PERSISTENT_NOT_ALLOWED" },                                    
         { 2049, "PRIORITY_EXCEEDS_MAXIMUM" },                                  
         { 2050, "PRIORITY_ERROR" },                                            
         { 2051, "PUT_INHIBITED" },                                             
         { 2052, "Q_DELETED" },                                                 
         { 2053, "Q_FULL" },                                                    
         { 2055, "Q_NOT_EMPTY" },                                               
         { 2056, "Q_SPACE_NOT_AVAILABLE" },                                     
         { 2057, "Q_TYPE_ERROR" },                                              
         { 2058, "Q_MGR_NAME_ERROR" },                                          
         { 2059, "Q_MGR_NOT_AVAILABLE" },                                       
         { 2061, "REPORT_OPTIONS_ERROR" },                                      
         { 2062, "SECOND_MARK_NOT_ALLOWED" },                                   
         { 2063, "SECURITY_ERROR" },                                            
         { 2065, "SELECTOR_COUNT_ERROR" },                                      
         { 2066, "SELECTOR_LIMIT_EXCEEDED" },                                   
         { 2067, "SELECTOR_ERROR" },                                            
         { 2068, "SELECTOR_NOT_FOR_TYPE" },                                     
         { 2069, "SIGNAL_OUTSTANDING" },                                        
         { 2070, "SIGNAL_REQUEST_ACCEPTED" },                                   
         { 2071, "STORAGE_NOT_AVAILABLE" },                                     
         { 2072, "SYNCPOINT_NOT_AVAILABLE" },                                   
         { 2075, "TRIGGER_CONTROL_ERROR" },                                     
         { 2076, "TRIGGER_DEPTH_ERROR" },                                       
         { 2077, "TRIGGER_MSG_PRIORITY_ERR" },                                  
         { 2078, "TRIGGER_TYPE_ERROR" },                                        
         { 2079, "TRUNCATED_MSG_ACCEPTED" },                                    
         { 2080, "TRUNCATED_MSG_FAILED" },                                      
         { 2082, "UNKNOWN_ALIAS_BASE_Q" },                                      
         { 2085, "UNKNOWN_OBJECT_NAME" },                                       
         { 2086, "UNKNOWN_OBJECT_Q_MGR" },                                      
         { 2087, "UNKNOWN_REMOTE_Q_MGR" },                                      
         { 2090, "WAIT_INTERVAL_ERROR" },                                       
         { 2091, "XMIT_Q_TYPE_ERROR" },                                         
         { 2092, "XMIT_Q_USAGE_ERROR" },                                        
         { 2093, "NOT_OPEN_FOR_PASS_ALL" },                                     
         { 2094, "NOT_OPEN_FOR_PASS_IDENT" },                                   
         { 2095, "NOT_OPEN_FOR_SET_ALL" },                                      
         { 2096, "NOT_OPEN_FOR_SET_IDENT" },                                    
         { 2097, "CONTEXT_HANDLE_ERROR" },                                      
         { 2098, "CONTEXT_NOT_AVAILABLE" },                                     
         { 2099, "SIGNAL1_ERROR" },                                             
         { 2100, "OBJECT_ALREADY_EXISTS" },                                     
         { 2101, "OBJECT_DAMAGED" },                                            
         { 2102, "RESOURCE_PROBLEM" },                                          
         { 2103, "ANOTHER_Q_MGR_CONNECTED" },                                   
         { 2104, "UNKNOWN_REPORT_OPTION" },                                     
         { 2105, "STORAGE_CLASS_ERROR" },                                       
         { 2106, "COD_NOT_VALID_FOR_XCF_Q" },                                   
         { 2107, "XWAIT_CANCELED" },                                            
         { 2108, "XWAIT_ERROR" },                                               
         { 2109, "SUPPRESSED_BY_EXIT" },                                        
         { 2110, "FORMAT_ERROR" },                                              
         { 2111, "SOURCE_CCSID_ERROR" },                                        
         { 2112, "SOURCE_INTEGER_ENC_ERROR" },                                  
         { 2113, "SOURCE_DECIMAL_ENC_ERROR" },                                  
         { 2114, "SOURCE_FLOAT_ENC_ERROR" },                                    
         { 2115, "TARGET_CCSID_ERROR" },                                        
         { 2116, "TARGET_INTEGER_ENC_ERROR" },                                  
         { 2117, "TARGET_DECIMAL_ENC_ERROR" },                                  
         { 2118, "TARGET_FLOAT_ENC_ERROR" },                                    
         { 2119, "NOT_CONVERTED" },                                             
         { 2120, "CONVERTED_MSG_TOO_BIG" },                                     
         { 2121, "NO_EXTERNAL_PARTICIPANTS" },                                  
         { 2122, "PARTICIPANT_NOT_AVAILABLE" },                                 
         { 2125, "BRIDGE_STARTED" },                                            
         { 2126, "BRIDGE_STOPPED" },                                            
         { 2127, "ADAPTER_STORAGE_SHORTAGE" },                                  
         { 2128, "UOW_IN_PROGRESS" },                                           
         { 2129, "ADAPTER_CONN_LOAD_ERROR" },                                   
         { 2130, "ADAPTER_SERV_LOAD_ERROR" },                                   
         { 2131, "ADAPTER_DEFS_ERROR" },                                        
         { 2132, "ADAPTER_DEFS_LOAD_ERROR" },                                   
         { 2133, "ADAPTER_CONV_LOAD_ERROR" },                                   
         { 2138, "ADAPTER_DISC_LOAD_ERROR" },                                   
         { 2140, "CICS_WAIT_FAILED" },                                          
         { 2143, "SOURCE_LENGTH_ERROR" },                                       
         { 2144, "TARGET_LENGTH_ERROR" },                                       
         { 2145, "SOURCE_BUFFER_ERROR" },                                       
         { 2146, "TARGET_BUFFER_ERROR" },                                       
         { 2150, "DBCS_ERROR" },                                                
         { 2151, "TRUNCATED" },                                                 
         { 2157, "ASID_MISMATCH" },                                             
         { 2160, "CONN_ID_IN_USE" },                                            
         { 2161, "Q_MGR_QUIESCING" },                                           
         { 2162, "Q_MGR_STOPPING" },                                            
         { 2163, "DUPLICATE_RECOV_COORD" },                                     
         { 2173, "PMO_ERROR" },                                                 
         { 2182, "API_EXIT_NOT_FOUND" },                                        
         { 2183, "API_EXIT_LOAD_ERROR" },                                       
         { 2184, "REMOTE_Q_NAME_ERROR" },                                       
         { 2186, "GMO_ERROR" },                                                 
         { 2192, "PAGESET_FULL" },                                              
         { 2193, "PAGESET_ERROR" },                                             
         { 2194, "NAME_NOT_VALID_FOR_TYPE" },                                   
         { 2195, "UNEXPECTED_ERROR" },                                          
         { 2196, "UNKNOWN_XMIT_Q" },                                            
         { 2197, "UNKNOWN_DEF_XMIT_Q" },                                        
         { 2198, "DEF_XMIT_Q_TYPE_ERROR" },                                     
         { 2199, "DEF_XMIT_Q_USAGE_ERROR" },                                    
         { 2201, "NAME_IN_USE" },                                               
         { 2202, "CONNECTION_QUIESCING" },                                      
         { 2203, "CONNECTION_STOPPING" },                                       
         { 2204, "ADAPTER_NOT_AVAILABLE" },                                     
         { 2206, "MSG_ID_ERROR" },                                              
         { 2207, "CORREL_ID_ERROR" },                                           
         { 2208, "FILE_SYSTEM_ERROR" },                                         
         { 2209, "NO_MSG_LOCKED" },                                             
         { 2216, "FILE_NOT_AUDITED" },                                          
         { 2217, "CONNECTION_NOT_AUTHORIZED" },                                 
         { 2218, "MSG_TOO_BIG_FOR_CHANNEL" },                                   
         { 2219, "CALL_IN_PROGRESS" },                                          
         { 2222, "Q_MGR_ACTIVE" },                                              
         { 2223, "Q_MGR_NOT_ACTIVE" },                                          
         { 2224, "Q_DEPTH_HIGH" },                                              
         { 2225, "Q_DEPTH_LOW" },                                               
         { 2226, "Q_SERVICE_INTERVAL_HIGH" },                                   
         { 2227, "Q_SERVICE_INTERVAL_OK" },                                     
         { 2280, "HCONFIG_ERROR" },                                             
         { 2281, "FUNCTION_ERROR" },                                            
         { 2282, "CHANNEL_STARTED" },                                           
         { 2283, "CHANNEL_STOPPED" },                                           
         { 2284, "CHANNEL_CONV_ERROR" },                                        
         { 2285, "SERVICE_NOT_AVAILABLE" },                                     
         { 2286, "INITIALIZATION_FAILED" },                                     
         { 2287, "TERMINATION_FAILED" },                                        
         { 2288, "UNKNOWN_Q_NAME" },                                            
         { 2289, "SERVICE_ERROR" },                                             
         { 2290, "Q_ALREADY_EXISTS" },                                          
         { 2291, "USER_ID_NOT_AVAILABLE" },                                     
         { 2292, "UNKNOWN_ENTITY" },                                            
         { 2293, "UNKNOWN_AUTH_ENTITY" },                                       
         { 2294, "UNKNOWN_REF_OBJECT" },                                        
         { 2295, "CHANNEL_ACTIVATED" },                                         
         { 2296, "CHANNEL_NOT_ACTIVATED" },                                     
         { 9999, "TABLE_END" }                                                  
      };                                                                        
                                                                                
  static const char *oops = "????";                                             
  int i;                                                                        
                                                                                
  for( i = 0; errmsg_list??(i??).reason < 9999; i++ )                           
    if ( errmsg_list??(i??).reason == reason )                                  
      return( errmsg_list??(i??).errmsg );                                      
                                                                                
  return( oops );                                                               
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    pdebug( )                                                                 
**                                                                              
**    This internal function prints an error message.                           
**    If the message has level DBUG_ERRMSG, the message is stored in the        
**    errmsg field of the CONNECTION_HANDLE.                                    
**    If debugging is turned on, the message is printed to the debug log        
**    file.                                                                     
**                                                                              
**    Input:                                                                    
**        level                                                                 
**        CONNECTION_HANDLE                                                     
**        function name for caller                                              
**        message in printf format                                              
**    Output:                                                                   
**        None                                                                  
**                                                                              
**============================================================================  
*/                                                                              
static void pdebug( int debug_level,                                            
           CONNECTION_HANDLE conn_handle,                                       
           char *func_name,                                                     
           char *format, ... )                                                  
{                                                                               
  char         errmsg??(2*QM_ERRMSG_LEN??);                                     
  va_list     plist;                                                            
  int         formatted = 0;                                                    
  FILE        *debug_fp;                                                        
                                                                                
  va_start( plist, format );                                                    
                                                                                
  if ( debug_level == DBUG_ERRMSG )                                             
    {                                                                           
      vsprintf( errmsg, format, plist );                                        
      formatted = 1;                                                            
      if ( conn_handle != NULL_CONNECTION_HANDLE )                              
    {                                                                           
      strncpy( conn_handle->errmsg,  errmsg, QM_ERRMSG_LEN-1 );                 
      (conn_handle->errmsg)??(QM_ERRMSG_LEN??) = '\0';                          
    }                                                                           
      else                                                                      
    fprintf( stderr, "%s\n", errmsg );                                          
      debug_level = DBUG_NORMAL;                                                
    }                                                                           
#if !defined(MVSCICS)                                                           
  if ( debugging )                                                              
    {                                                                           
      debug_fp = DBGFP(conn_handle);                                            
      if ( !formatted )                                                         
    vsprintf( errmsg, format, plist );                                          
      debug( debug_level, debug_fp,                                             
        "%s: %s\n", func_name, errmsg );                                        
    }                                                                           
#endif                                                                          
  va_end( plist );                                                              
  return;                                                                       
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    get_store()                                                               
**                                                                              
**    This internal subroutine gets storage. It includes environmental          
**    differences for MVSCICS.                                                  
**                                                                              
**    Input:                                                                    
**        size of storage to be allocated                                       
**    Output:                                                                   
**        pointer to allocated storage                                          
**                                                                              
**==========================================================================    
*/                                                                              
static char *get_store( int size )                                              
{                                                                               
  char    *store = (char *)NULL;                                                
                                                                                
#if defined(MVSCICS)                                                            
  int    cresp;                                                                 
                                                                                
  EXEC CICS GETMAIN SET(store) FLENGTH(size)                                    
    NOSUSPEND RESP(cresp);                                                      
  if ( cresp != DFHRESP(NORMAL) )                                               
    store = (char *)NULL;                                                       
#else                                                                           
  store = malloc( size );                                                       
#endif                                                                          
                                                                                
  return( store );                                                              
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    free_store()                                                              
**                                                                              
**    This internal subroutine frees storage. It includes environmental         
**    differences for MVSCICS.                                                  
**                                                                              
**    Input:                                                                    
**        area to be freed                                                      
**    Output:                                                                   
**        None                                                                  
**                                                                              
**==========================================================================    
*/                                                                              
static void free_store( char *store )                                           
{                                                                               
#if defined(MVSCICS)                                                            
  int    cresp;                                                                 
  EXEC CICS FREEMAIN DATA(store) RESP(cresp);                                   
#else                                                                           
  free( store );                                                                
#endif                                                                          
}                                                                               
                                                                                
/*==========================================================================    
**                                                                              
**    get_system_id( )                                                          
**                                                                              
**    This internal function gets the system id. This may vary according        
**    to platform                                                               
**                                                                              
**    Input:                                                                    
**        CONNECTION_HANDLE                                                     
**    Output:                                                                   
**        System id (in CONNECTION_HANDLE )                                     
**                                                                              
**==========================================================================    
*/                                                                              
static void get_system_id( CONNECTION_HANDLE conn_handle )                      
{                                                                               
  char hostname??(64??);                                                        
  char *func_name = "get_system_id";                                            
                                                                                
#if defined(MVS)                                                                
#if defined(MVSCICS)                                                            
  strncpy( conn_handle->system_id, "MVSCICS", QM_SYSID_LEN );                   
#else                                                                           
  strncpy( conn_handle->system_id, "MVSBATCH", QM_SYSID_LEN );                  
#endif                                                                          
#else                                                                           
  if (gethostname( hostname, sizeof( hostname )) < 0 )                          
    {                                                                           
      strncpy( conn_handle->system_id, "Unknown", QM_SYSID_LEN );               
      pdebug( DBUG_NORMAL, conn_handle, func_name,                              
         "gethostname() failed");                                               
    }                                                                           
  else                                                                          
    {                                                                           
      strncpy( conn_handle->system_id, hostname, QM_SYSID_LEN );                
      pdebug( DBUG_FINE, conn_handle, func_name,                                
         "system_id: %.*s", QM_SYSID_LEN, conn_handle->system_id );             
    }                                                                           
#endif                                                                          
}                                                                               
