/***********************************************************************
 * Copyright der Gesellschaften der Quelle Versicherungen
 * Dateiname:   MAG1016
 * Revision:    $Revision$
 * Autor:       Kociurzynski
 * Datum:       1999.10.25
 * Zeit:        $Modtime$
 *
 * Beschreibung: Bearbeitung der verketteten Listen
 *
 ***********************************************************************
 * Modification History:
 * Wann      GQV0 Aend    Was
 * xx.10.1999 019 000 Ersterstellung
 * xx.07.2000 019 001 Umstellung auf MAG2002
 * xx.07.2000 019 002 lst_adds: Bug behoben
 * 28.05.2001 019 000 LST_DBG statt dbg(...,<List>)
 * 11.11.2002 019 000 DBG in lst_find
 * 13.05.2003 019 003 lst_fput laesst FILE=NULL zu (Aufruf qv_printf)
 * 10.01.2004 019 000 Bug in LST_REV
 * 01.07.2004 019 000 lst_clear1, lst_clear2
 * 15.07.2004 019 000 lst_dbg als Funktion; LST_DBG=>LST_DBGADDR
 * 30.05.2011 019 000 lst_delfor - neue Version, lst_delsome,
 *                    lst_findshead
 * 01.06.2011 019 000 qsteps, getcmpq
**********************************************************************/

#define MAG1016_C
static const char *Modul = "MAG1016";
#ifdef _MSC_VER
#include "compat.h"
#endif


/*#include "XAG00QV"*/  /*ASSERT etc */
/*#include "IAG1002"    /*Diagnostik - Zusatz*/
#include "XAG1016"      /*Typedef: ts_list (Liste von void *) */


/* Zur Codeverkuerzung */

#define CMP(Funktion,Zeiger1,Zeiger2)\
     (Funktion? Funktion(Zeiger1,Zeiger2)\
              : (unsigned long)(Zeiger1)-(unsigned long)(Zeiger2))
#define MOVE(List)     List=(List)->Tail

/* Zur Performance-Analyse */
static t1_count  qsteps;

/*===lst_gethead========================================================
 * Synopsis: lst_gethead(List)
 * Parameter: Zeiger auf die Liste (I)
 * Return: Zeiger auf den Kopf der Liste oder NULL
 * Beschreibung: Holt 1. Satz der Liste.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void *lst_gethead(tp_list List)
{return List? List->Head: NULL;
} /*lst_gethead*/

/*===lst_gettail========================================================
 * Synopsis: lst_gettail (tp_list List)
 * Parameter: Zeiger auf die Liste (I)
 * Return: Zeiger auf den naechsten Knoten der Liste
 * Beschreibung: Holt den Schweif der Liste
 * Modifikation:      Datum       Grund         Novell-ID
 *====================================================================*/
tp_list lst_gettail(tp_list List)
{return List? List->Tail:NULL;
} /*lst_gettail*/


/*===lst_getlast========================================================
 * Synopsis: lst_getlast (tp_list List)
 * Parameter: Zeiger auf die Liste (I)
 * Return: Zeiger auf den letzten Knoten der Liste
 * Beschreibung: s. Return
 * Modifikation:      Datum       Grund         Novell-ID
 *====================================================================*/
tp_list lst_getlast (tp_list List)
{tp_list Node, Prev=NULL;
 for(Node=List;Node;MOVE(Node)) Prev=Node;
 return Prev;
} /*lst_getlast*/


/*===lst_vaget==========================================================
 * Synopsis: lst_vaget(List)
 * Parameter: List(I)  - Zeiger auf die Liste (I)
 *            QQ  (I)  - Anzahl der zu holenden Saetze
 *            ... (IO) - DOPPELZEIGER auf die zu holenden Saetze
 * Return:
 * Beschreibung: Holt QQ Saetze aus der Liste.
 *               Die Liste bleibt ungeaendert
 * Modifikation:      Datum       Grund         Novell-ID
 *====================================================================*/
void lst_vaget(tp_list List,unsigned QQ,...)
{va_list    Ap;
 void     **Item;
 unsigned   jj,QL;
 MSG_DCL("lst_vaget");
 QL= lst_count(List);
 ASSERT(QQ<=QL,ABBRUCH,101601,
        qv_msg("Versuch %u Saetze zu holen, %u vorhanden", QQ,QL));
 va_start(Ap,QQ);
 for(jj=0;jj<QQ;jj++)
   {Item=(void **)va_arg(Ap,void **);
    ASSERT(Item, ABBRUCH, EC_NullPointer,
           qv_msg("Parameter #%u ist NULL oder kein Doppelzeiger",
                  jj+2));
    *Item=List->Head;
    MOVE(List);
   }
 MSG_END();
 va_end(Ap);
} /*lst_vaget*/

/*===lst_push===========================================================
 * Synopsis: lst_push(tp_list *List,void *NewItem)
 * Parameter: List (*IO) - Doppelzeiger auf den 1. Knoten der Liste
 *            NewItem (I) - in die Liste einzugefuegende Benutzersdaten,
 * Return: neue Liste
 * Beschreibung: Fuegt NewItem neuen Satz in die Liste ein. Erstellt
 *               einen neuen Knoten, weist seinen Zeiger dem Parameter
 *               List zu und liefert den Zeiger zurueck.
 * Modifikation:      Datum       Grund         Novell-ID
 *====================================================================*/
tp_list lst_push(tp_list *List, void *NewItem)
{
 tp_list Node= NULL;
 MSG_DCL("lst_push");
 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");
 Node= malloc_msg(sizeof(ts_list), "neuen Knoten");
 Node->Head= NewItem;
 Node->Tail= *List;
 *List= Node;
 MSG_END();
 return Node;
} /*lst_push*/


/*===lst_vaformrev======================================================
 * Synopsis: lst_vaformrev(unsigned QQ,...)
 * Parameter: QQ  (I) - Anzahl von uebergebenen va-Parametern
 *            ... (I) - Zeiger auf in die Liste einzugefuegende
 *                      Benutzersdaten,
 * Return:   Zeiger auf die neue Liste.
 * Beschreibung: Verbindet angegebene Saetze in der Liste in umgekehrter
 *               Folge.
 * Modifikation:      Datum       Grund         Novell-ID
 *====================================================================*/
tp_list lst_vaformrev(unsigned QQ,...)
{tp_list  List=NULL;
 va_list  Ap;
 unsigned jj;
 void    *Item;
 MSG_DCL("lst_vaformrev");
 va_start(Ap,QQ);
 for(jj=0;jj<QQ;jj++)
   {Item=(void *)va_arg(Ap,void *);
    lst_push(&List,Item);
   }
 va_end(Ap);
 MSG_ENDD();
 return List;
} /*lst_vaformrev*/


/*===lst_vaform=========================================================
 * Synopsis: lst_vaform(unsigned QQ,...)
 * Parameter: QQ  (I) - Anzahl von uebergebenen va-Parametern
 *            ... (I) - Zeiger auf in die Liste einzugefuegende
 *                      Benutzersdaten,
 * Return:   Zeiger auf die neue Liste.
 * Beschreibung: Verbindet angegebene Saetze in der Liste in angegebener
 *               Folge.
 * Modifikation:      Datum       Grund         Novell-ID
 *====================================================================*/
tp_list  lst_vaform(unsigned QQ,...)
{tp_list  List0=NULL,
          List1=NULL;
 va_list  Ap;
 unsigned jj;
 void    *Item;
 MSG_DCL("lst_vaform");
 va_start(Ap,QQ);
 for(jj=0;jj<QQ;jj++)
   {Item=(void *)va_arg(Ap,void *);
    lst_push(&List0,Item);
   }
 if(List0)
   {List1=lst_rev(List0);
    lst_destroy(&List0);
   }
 va_end(Ap);
 MSG_ENDD();
 return List1;
} /*lst_vaform*/


/*===lst_adds===========================================================
 * Synopsis: lst_adds(tp_list *List, tp_list *NewOrFound, void *NewItem,
 *                    tp_fcmp Fcompare)
 * Parameter: List (*IO) - Doppelzeiger auf den 1. Knoten der
 *                         SORTIERTEN Liste
 *            NewOrFound (*O) - NULL oder Doppelzeiger auf den neuen
 *                              Knoten
 *            NewItem (I) - in die Liste einzugefuegende Benutzersdaten,
 *            Fcompare - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UserData,void *DatenSatzAusListe)
 *                 Return - s. Beschreibung der Systemfunktion strcmp
 *                 Wenn NULL, werden die Zeiger direkt verglichen
 * Return: TRUE - eingefuegt, FALSE - gefunden
 * Beschreibung: Findet den angegebenen Datensatz NewItem in der SOR-
 *               TIERTEN Liste oder, wenn nicht gefunden, fuegt ihn ein.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
bool lst_adds(tp_list *List, tp_list *FoundOrNew,
                  void *NewItem,tp_fcmp Fcompare)
{
 bool               Rt;
 tp_list           Node, Prev=NULL;
 int                diff=1;
 MSG_DCL("lst_adds");
 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");

 LST_DBGADDR(*List); dbg("NewItem=%lu",(ulong)NewItem);

 for(Node=*List;Node && (diff=CMP(Fcompare,NewItem,Node->Head))>0;)
    {Prev=Node;MOVE(Node);}

 DBG_D1(%d,diff);
 LST_DBGADDR(Prev); LST_DBGADDR(Node);

 if(!diff) Rt=FALSE;
 else
   {lst_push(&Node,NewItem);
    LST_DBGADDR(Node);
    if(Prev) Prev->Tail=Node;
    else /*if(!*List)*/ *List=Node;                    /*AEND002: UPD*/

    LST_DBGADDR(Prev);  if(Prev) LST_DBGADDR(Prev->Tail);
    LST_DBGADDR(*List); if(*List) LST_DBGADDR((*List)->Tail);

    Rt=TRUE;
   }
 if(FoundOrNew) *FoundOrNew=Node;
 dbg("Rt=%d",(int)Rt);
 MSG_ENDD();
 return Rt;
} /*lst_adds*/


/*===lst_ins============================================================
 * Synopsis:
 *      tp_list lst_ins(tp_list *List, void *NewItem, tp_fcmp Fcompare)
 * Parameter: List (*IO) - Doppelzeiger auf den 1. Knoten der
 *                         SORTIERTEN Liste
 *            NewItem (I) - in die Liste einzugefuegende Benutzersdaten,
 *            Fcompare - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UserData,void *DatenSatzAusListe)
 *                 Return - s. Beschreibung der Systemfunktion strcmp
 *                 Wenn NULL, werden die Zeiger direkt verglichen
 * Return:    Zeiger auf den neuen Knoten
 * Beschreibung: Fuegt den angegebenen Datensatz NewItem in die SOR-
 *               TIERTE Liste ein, auch wenn der gleiche Eintrag
 *               gefunden wird.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
tp_list lst_ins(tp_list *List, void *NewItem, tp_fcmp Fcompare)
{
 tp_list            Node, Prev=NULL;
 int                diff=1;
 MSG_DCL("lst_ins");
 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");

 LST_DBGADDR(*List); dbg("NewItem=%lu",(ulong)NewItem);

 for(Node=*List;Node && (diff=CMP(Fcompare,NewItem,Node->Head))>=0;)
    {Prev=Node;MOVE(Node);}

 DBG_D1(%d,diff);
 LST_DBGADDR(Prev); LST_DBGADDR(Node);

 lst_push(&Node,NewItem);
 LST_DBGADDR(Node);

 if(Prev) Prev->Tail=Node;
 else /*if(!*List) -BS!*/ *List=Node;

 LST_DBGADDR(Prev);  if(Prev) LST_DBGADDR(Prev->Tail);
 LST_DBGADDR(*List); if(*List) LST_DBGADDR((*List)->Tail);
 dbg("@Rt=%lu",(unsigned)Node);

 MSG_ENDD();
 return Node;
} /*lst_ins*/


/*===lst_append=========================================================
 * Synopsis:
 *      tp_list lst_append(tp_list List, void *NewItem)
 * Parameter: List (*IO) - Doppelzeiger auf den 1. Knoten der
 *                         SORTIERTEN Liste
 *            NewItem (I) - in die Liste einzugefuegende Benutzersdaten,
 * Return:    Zeiger auf den neuen Knoten
 * Beschreibung: Fuegt den angegebenen Datensatz NewItem ans ENDE der
 *               Liste hinzu
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
tp_list lst_append(tp_list List, void *NewItem)
{
 tp_list Node, Prev=NULL;
 MSG_DCL("lst_append");

 LST_DBGADDR(List); dbg("NewItem=%lu",(ulong)NewItem);

 for(Node=List;Node;)
    {Prev=Node;MOVE(Node);}

 LST_DBGADDR(Prev); LST_DBGADDR(Node);

 Node= malloc_msg(sizeof(ts_list), "neuen Knoten");
 Node->Head=NewItem;
 Node->Tail=NULL;
 if(Prev) Prev->Tail=Node;

 LST_DBGADDR(Prev); if(Prev) LST_DBGADDR(Prev->Tail);
 LST_DBGADDR(Node);
 dbg("RT=Node=%lu",(ulong)Node);
 MSG_END();
 return Node;
} /*lst_append*/


/*===lst_cat============================================================
 * Synopsis: tp_list lst_cat(tp_list *List, tp_list Add)
 * Parameter: List (*IO) - Doppelzeiger auf den 1. Knoten der
 *            Add    (I) - die hinzuzufuegende Liste
 * Return:    Zeiger auf den 1. Knoten der geaenderten Liste
 * Beschreibung: Konkateniert 2 Listen
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
tp_list lst_cat(tp_list *List, tp_list Add)
{
 tp_list Node, Prev=NULL;
 MSG_DCL("lst_cat");

 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");
 LST_DBGADDR(*List);

 for(Node=*List;Node;)
    {Prev=Node;MOVE(Node);}

 LST_DBGADDR(Prev);

 if(Prev) Prev->Tail=Add;
 else *List = Add;
 dbg("Rt=*List=%lu",(unsigned)*List);
 MSG_END();
 return *List;
} /*lst_cat*/


/*===lst_rev============================================================
 * Synopsis: lst_rev(tp_list List)
 * Parameter: Die Liste (I)
 * Return:    Zeiger auf die neue Liste
 * Beschreibung: Kehrt die Listensatzfolge um
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
tp_list lst_rev(tp_list List0)
{tp_list List1=NULL;
 MSG_DCL("lst_rev");
 while(List0)
   {lst_push(&List1,List0->Head);
    MOVE(List0);
   }
 MSG_ENDD();
 return List1;
} /*lst_rev*/


/*===lst_pop============================================================
 * Synopsis: lst_pop(tp_list *List)
 * Parameter: Doppelzeiger auf die Liste (IO)
 * Return: Zeiger auf den Kopf der Liste oder NULL
 * Beschreibung: Holt 1. Satz der Liste und kuppelt ihn ab.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void *lst_pop(tp_list *List)
{
 tp_list    Node;
 void      *Top=NULL;
 MSG_DCL("lst_pop");
 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");
 if(Node=*List)
   {Top=Node->Head;
    MOVE(*List);
    free(Node);
   }
 MSG_END();
 return Top;
} /*lst_pop*/


/*===lst_clear==========================================================
 * Synopsis: lst_clear(tp_list *List, tp_fclear Fclear)
 * Parameter: List (IO) - Doppelzeiger auf die Liste,
 *            Fclear - NULL oder von Benutzer definierte
 *                    "free"-aehnliche Funktion:
 *                   void Fclear(void *DatenSatz)
 *                     die interne Strukturen des DatenSatz frei gibt.
 *                     NULL wird als Dummy-Funktion betrachtet.
 * Bemerkung: ungeschickte Variante, die keine Verschaltelte Listen
 *            putzen kann, denn lst_clear und Fclear unterschiedliche
 *            Zeigersebenen des 1. Parameter verlangen
 * Return:
 * Beschreibung: Loescht Liste und raeumt Speicher auf.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void lst_clear(tp_list *List, tp_fclear Fclear)
{
 tp_list  CurrNode;
 MSG_DCL("lst_clear");
 while(*List)
   {CurrNode = *List;
    MOVE(*List);
    if(CurrNode->Head)
      {if(Fclear) Fclear(CurrNode->Head);
     /*free(CurrNode->Head);*/
      }
    free(CurrNode);
   }
 MSG_ENDD();
} /*lst_clear*/


/*===lst_clear1=========================================================
 * Synopsis: lst_clear1(tp_list *List, tp_fclear1 Fclear)
 * Parameter: List (IO) - Zeiger auf die Liste,
 *            Fclear - NULL oder von Benutzer definierte
 *                    "free"-aehnliche Funktion:
 *                   void Fclear(void *DatenSatz)
 *                     die interne Strukturen des DatenSatz frei gibt.
 *                     NULL wird als Dummy-Funktion betrachtet.
 * Return:
 * Beschreibung: Loescht Liste und raeumt Speicher auf.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void lst_clear1(tp_list List,tp_fclear1 Fclear)
{
 tp_list  CurrNode;
 MSG_DCL("lst_clear1");
 while(List)
   {CurrNode = List;
    MOVE(List);
    if(CurrNode->Head)
      {if(Fclear) Fclear(CurrNode->Head);
     /*free(CurrNode->Head);*/
      }
    free(CurrNode);
   }
 MSG_ENDD();
} /*lst_clear1*/


/*===lst_clear2=========================================================
 * Synopsis: lst_clear2(tp_list *List,tp_fclear2 Fclear)
 * Parameter: List (IO) - Doppelzeiger auf die Liste,
 *            Fclear - NULL oder von Benutzer definierte
 *                    "free"-aehnliche Funktion:
 *                   void Fclear(void **DatenSatz)
 *                     die interne Strukturen des DatenSatz frei gibt.
 *                     NULL wird als Dummy-Funktion betrachtet.
 * Return:
 * Beschreibung: Loescht Liste und raeumt Speicher auf.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void lst_clear2(tp_list *List, tp_fclear2 Fclear)
{
 tp_list  CurrNode;
 MSG_DCL("lst_clear2");

 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");
 while(*List)
   {CurrNode = *List;
    MOVE(*List);
    if(CurrNode->Head)
      {if(Fclear) Fclear(&CurrNode->Head);
     /*free(CurrNode->Head);*/
      }
    free(CurrNode);
   }
 MSG_ENDD();
} /*lst_clear2*/


/*===lst_destroy========================================================
 * Synopsis: lst_destroy(tp_list *List)
 * Parameter: List (I) - Doppelzeiger auf die Liste,
 * Return:
 * Beschreibung: Zerstoert die Liste. Wirkt wie lst_clear, aber die
 *               Saetze bleiben intakt
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void lst_destroy(tp_list *List)    /*Liste zerstoeren*/
{
 tp_list  CurrNode;
 MSG_DCL("lst_destroy");
 while(*List)
   {CurrNode=*List;
    MOVE(*List);
    free(CurrNode);
   }
 MSG_END();
} /*lst_destroy*/


/*===lst_delsome========================================================
 * Synopsis: lst_delsome(tp_list List, void UsD,
 *                       tp_fcmp FMark, tp_fclear1 FClear,
 *                       bool Once)
 * Beschreibung: s. lst_delfor
 * Parameter (extra zu lst_delfor):
 *              Once     - TRUE: nur das 1. passende Element loeschen,
 *                         FALSE: alle passenden
*=====================================================================*/
static t1_count lst_delsome(tp_list *List, void *UsD,
                            tp_fcmp FMark, tp_fclear1 FClear,
                            bool Once)
{t1_count   Rt=0;
 ts_list  *Node, *Prev=NULL, *List0;
 int       UsRet=0;
 MSG_DCL("lst_delsome");

 CHKPARM_NN(1, List);
 LST_DBGADDR(*List);

 for(List0=Node=*List; Node;)
   {if(FMark) UsRet=FMark(UsD,Node->Head);
    DBG_D1(%d,UsRet); LST_DBGADDR(Prev); LST_DBGADDR(Node);
    if(!UsRet)
      {
       if(Prev) Prev->Tail=Node->Tail;
       else *List=Node->Tail;
       if(FClear) FClear(Node->Head);
       /*if(Node->Head) free(Node->Head);*/
       free(Node);
       Node=Prev? Prev->Tail: *List;
       Rt++;
      }
    else {Prev=Node; MOVE(Node);}

    if(Rt && Once) break;
    if(Node==List0) break;        /*wg autorecursion*/
   }
 dbg("*List=%#x, Rt=%u", (unsigned)*List, Rt);
 MSG_ENDD();
 return Rt;
} /*lst_delsome*/


/*===lst_delfor=========================================================
 * Synopsis: lst_delfor(tp_list  *List,void UsD,
 *                      tp_fcmp FMark, tp_fclear1 FClear)
 * Parameter: List (*IO) - Doppelzeiger auf den 1. Knoten der
 *                         SORTIERTEN Liste
 *            UsD  (I)   - Benutzersdaten,
 *            FMark      - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UsD,void *DatenSatzAusListe)
 *                 Return:  0=LOESCHEN,<>0=BEHALTEN
 *                 Wenn NULL, werden alle Eintraege geloescht
 *            FClear - NULL oder von Benutzer definierte
 *                    "free"-aehnliche Funktion:
 *                   void Fclear(void *DatenSatz)
 *                     die interne Strukturen des DatenSatz frei gibt.
 *                     NULL wird als Dummy-Funktion betrachtet.
 * Return: Return: Anzahl der geloeshten Eintraege
 * Beschreibung: Loescht alle durch Benutzersfunktion markierte
 *               Eintraege
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
 t1_count lst_delfor(tp_list *List, void *UsD,
                     tp_fcmp FMark, tp_fclear1 FClear)
{t1_count  Rt= 0;
 MSG_DCL("lst_delfor");

 ASSERT(List, ABBRUCH, EC_NullPointer, "Parameter List ist NULL");
 Rt= lst_delsome(List, UsD, FMark, FClear, FALSE);
 MSG_ENDD();
 return Rt;
} /*lst_delfor*/


/*===lst_find===========================================================
 * Synopsis: lst_find(tp_list List, void *UserData, tp_fcmp Fcompare)
 * Parameter: List (I) - die Liste,
 *            UserData (I) - zu suchende Benutzersdaten
 *            Fcompare - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UserData,void *DatenSatzAusListe)
 *                 Return - s. Beschreibung der Systemfunktion strcmp
 *                 Wenn NULL, werden die Zeiger direkt verglichen
 * Return: Gefundenen Knoten  oder NULL
 * Beschreibung: Sucht die Liste direkt fuer den angegebenen Satz durch.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
tp_list lst_find(tp_list List, void *UserData, tp_fcmp Fcompare)
{
 tp_list  Node= NULL;
 MSG_DCL("lst_find");
 if(!List) {qsteps=0; dbg("Liste ist leer");}
 else
    for(Node=List, qsteps=1;
        Node && CMP(Fcompare,UserData,Node->Head);)
      {dbg("Node->Head(8)='%.8s, UserData(8)='%.8s'",
            (char *)Node->Head, (char *)UserData);
       MOVE(Node);
       qsteps++;
      } /*for()*/
 MSG_ENDD();
 return Node;
} /*lst_find*/


/*===lst_finds==========================================================
 * Synopsis: lst_finds(tp_list List, void *UserData, tp_fcmp Fcompare)
 * Parameter: List (I) - die sortierte Liste,
 *            UserData (I) - zu suchende Benutzersdaten
 *            Fcompare - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UserData,void *DatenSatzAusListe)
 *                 Return - s. Beschreibung der Systemfunktion strcmp
 *                 Wenn NULL, werden die Zeiger direkt verglichen
 * Return: Gefundenen Knoten  oder NULL
 * Beschreibung: Sucht die sortierte Liste direkt fuer den angegebenen
 *               Satz durch.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
tp_list lst_finds(tp_list List, void *UserData, tp_fcmp Fcompare)
{
 tp_list  Node;
 int      diff=1;
 MSG_DCL("lst_finds");

 if(!List) {qsteps=0; dbg("Liste ist leer");}
 else
    for(Node=List, qsteps=1;
        Node && (diff=CMP(Fcompare,UserData,Node->Head))>0;)
      {MOVE(Node);
       qsteps++;
      } /*for()*/
 MSG_ENDD();
 return diff? NULL: Node;
} /*lst_finds*/


/*===lst_findhead=======================================================
 * Synopsis: lst_findhead(tp_list List, void *UserData,
 *                        tp_fcmp Fcompare)
 * Parameter: List (I) - die Liste,
 *            UserData (I) - zu suchende Benutzersdaten
 *            Fcompare - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UserData,void *DatenSatzAusListe)
 *                 Return - s. Beschreibung der Systemfunktion strcmp
 *                 Wenn NULL, werden die Zeiger direkt verglichen
 * Return: Gefundenen Satz oder NULL
 * Beschreibung: Sucht die Liste direkt fuer den angegebenen Satz durch.
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
void *lst_findhead(tp_list List, void *UserData, tp_fcmp Fcompare)
{void    *Rt;
 tp_list  Node;
 MSG_DCL("lst_findhead");
 Rt = (Node=lst_find(List, UserData, Fcompare))? Node->Head: NULL;
 MSG_END();
 return Rt;
} /*lst_findhead*/


 /*===lst_findshead=====================================================
 * Synopsis: lst_findshead(tp_list List, void *UserData,
 *                         tp_fcmp Fcompare)
 * Parameter: List (I) - die sortierte Liste,
 *            UserData (I) - zu suchende Benutzersdaten
 *            Fcompare - NULL oder Benutzersvergleichsfunktion in Form
 *                 int Fcompare(void *UserData, void *DatenSatzAusListe)
 *                 Return - s. Beschreibung der Systemfunktion strcmp
 *                 Wenn NULL, werden die Zeiger direkt verglichen
 * Return: Gefundenen Satz oder NULL
 * Beschreibung: Sucht die Liste direkt fuer den angegebenen Satz durch.
*=====================================================================*/
void *lst_findshead(tp_list List, void *UserData, tp_fcmp Fcompare)
{void    *Rt;
 tp_list  Node;
 MSG_DCL("lst_findshead");
 Rt = (Node=lst_finds(List, UserData, Fcompare))? Node->Head: NULL;
 MSG_END();
 return Rt;
} /*lst_findshead*/


/*===lst_getcmpq======================================================
 * Synopsis: lst_getcmpq()
 * Return: Anzahl der Vergleichsvorgaenge bei der letzten Suche
*=====================================================================*/
t1_count lst_getcmpq()
{return qsteps;
} /*lst_getcmpq*/


/*===lst_fput===========================================================
 * Synopsis: lst_fput(FILE *FF, tp_list List,
 *                    tp_fput Fput, void *UserCtrlData)
 * Parameter: FF (I) - Ausgabedatei,
 *            List(I) - die Liste,
 *            Fput(I) - von Benutzer definierte Ausgabefunktion
 *                  int Fput(FF,DatenSatz,UserCtrlData),
 *                  die DatenSatz in FF schreibt.
 *                  Return: >0 = OK und naechster Satz ist
 *                               zu bearbeiten,
 *                          0 = OK und lst_fput verlassen
 *                          <0 = Fehler aufgetreten
 *                  Wenn Fput==NULL, erste sizeof(unsigned) bytes
 *                  werden als unsigned ausgegeben und UserCtrlData
 *                  wird als Format betrachtet
 *                  (char *) Trenner betrachtet (by default \n).
 * Return: letzter Return von Fput oder (wenn List leer ist) 0.
 * Beschreibung: gibt die Liste in die Datei aus
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
int lst_fput(FILE *FF, tp_list List, tp_fput Fput, void *UserCtrlData)
{int         Rt= 0;
 const char *Routine= "lst_fput";
 tp_list     Node;
 char       *Sep="\n";
 unsigned   *Uns;
 /*ASSERT(FF,ABBRUCH,EC_NullPointer,                      AEND003: DEL 2
  *       qv_msg("(%s) Parameter FILE ist NULL",Routine));*/
 if(List)
   {Rt=1;
    if(Fput)
       for(Node=List; Node && Rt>0; MOVE(Node))
          Rt=Fput(FF,UserCtrlData,Node->Head);
    else
      {if(UserCtrlData)
          Sep=UserCtrlData;
       for(Node=List; Node; MOVE(Node))
         {Uns=(unsigned *)Node->Head;
          if(!FF) qv_printf("%0x%s",*Uns,Sep);         /*AEND002:INS 2*/
          else
             fprintf(FF,"%0x%s",*Uns,Sep);
         } /*for(Node,...)*/
      } /*if(Fput) / else*/
   } /*if(List)*/
 return Rt;
} /*lst_fput*/

/*===lst_dbg===========================================================
 * Synopsis: lst_dbg(tp_lst *List, tp_put Fput,void *UserCtrlData)
 * Parameter: List(I) - die Liste,
 *            Fput(I) - von Benutzer definierte Ausgabefunktion
 *                  int Fput(DatenSatz,UserCtrlData),
 *                  die DatenSatz auf stdout schreibt.
 *                  Return: >0 = OK und naechster Satz ist
 *                               zu bearbeiten,
 *                          0 = OK und lst_fput verlassen
 *                          <0 = Fehler aufgetreten
 *                  Wenn Fput==NULL, erste sizeof(unsigned) bytes
 *                  werden als unsigned ausgegeben und UserCtrlData
 *                  wird als
 *                  (char *) Trenner betrachtet (by default \n).
 * Return: letzter Return von Fput oder (wenn List leer ist) 0.
 * Beschreibung: gibt die Liste in die Datei aus
*=====================================================================*/
int lst_dbg(tp_list List, tp_put Fput, void *UserCtrlData)
{int       Rt=0;
 FILE     *Fp;
 char     *Sep="\n";
 unsigned *Uns;
 if(List && dbg_isto(NULL,NULL))
   {Fp= stdout;
    fprintf(Fp, "List=%0x,List->Head=%0x ",
            (ulong)List, (ulong)List->Head);
    if(Fput) Rt=Fput(UserCtrlData,List->Head);
    else
      {if(UserCtrlData) Sep=UserCtrlData;
       Uns=(unsigned *)List->Head;
       fprintf(Fp, "%0x%s", *Uns, Sep);
       Rt++;
      } /*if(Fput) / else*/
    Rt+= lst_dbg(List->Tail, Fput, UserCtrlData);
   } /*if(List)*/
 return Rt;
} /*lst_dbg*/

/*===lst_count==========================================================
 * Synopsis: lst_count(tp_list List)
 * Parameter: List (I) - die Liste
 * Return: Anzahl der Saetze in der Liste
 * Beschreibung: Zaehlt Saetze in der Liste
 * Modifikation:      Datum       Grund         Novell-ID
*=====================================================================*/
ulong lst_count(tp_list List)
{
 return List? lst_count(List->Tail)+1: 0UL;  /*Saetze zaehlen*/
} /*lst_count*/

                                                       /*AEND001: APP */
/*===MAG1016_gate=====================================================*/
/* Common initializer. Called from MAG1016 & MAG1016_open */
static void *MAG1016_gate(void *EnvInp,const char *Mode)
{static const char *Routine="MAG1016_gate";
 void *EnvOut;
 QV_SETRUN(Modul,Routine,Mode);
 EnvOut = MAG2002(EnvInp, Mode);
 return EnvOut;
} /*MAG1016_gate*/


/*===MAG1016== statische openMAG =====================================*/
void *MAG1016(void *Env, const char *Mode)
{static const char *Routine="MAG1016";
 LINK_TEST(Modul,Routine,LINK_STAT)
 return MAG1016_gate(Env, Mode);
} /*MAG1016*/
